@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,27 +12,22 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/aspath_test.cc,v 1.15 2006/03/16 00:03:27 pavlin Exp $"
+#ident "$XORP: xorp/bgp/aspath_test.cc,v 1.17 2007/02/16 22:45:10 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "bgp_module.h"
 
 #include "libxorp/xorp.h"
 
-#include <stdio.h>
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-
-#include "bgp_module.h"
-#include "aspath.hh"
-
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "aspath.hh"
+
+
 int
 main(int argc, char* argv[])
 {
