@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,20 +13,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp.cc,v 1.77 2006/07/12 02:35:19 atanu Exp $"
+#ident "$XORP: xorp/bgp/bgp.cc,v 1.81 2007/02/16 22:45:10 pavlin Exp $"
 
 // #define DEBUG_MAXIMUM_DELAY
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
 #include "bgp_module.h"
 
 #include "libxorp/xorp.h"
@@ -34,6 +26,10 @@
 #include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
 #include "libcomm/comm_api.h"
 
 #include "bgp.hh"
@@ -42,6 +38,7 @@
 #include "xrl_target.hh"
 #include "profile_vars.hh"
 
+
 // ----------------------------------------------------------------------------
 // Static class members
 
@@ -1597,7 +1594,7 @@ BGPMain::get_peer_as(const Iptuple& iptuple, uint32_t& peer_as)
 
     // XXX is it appropriate to return an extended AS number in this
     // context?
-    peer_as = pd->as().as32();
+    peer_as = pd->as().as4();
 
     return true;
 }
@@ -1952,7 +1949,7 @@ BGPMain::set_parameter(const Iptuple& iptuple , const string& parameter,
     
     debug_msg("Peer %s. %s\n", iptuple.str().c_str(),toggle ? "true" :"false");
 
-    BGPPeerData *peerdata = const_cast<BGPPeerData *const>(peer->peerdata());
+    BGPPeerData *peerdata = const_cast<BGPPeerData *>(peer->peerdata());
     if (toggle) {
 	peerdata->add_sent_parameter(node);
     } else {
