@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp_varrw.cc,v 1.28 2006/05/01 16:49:05 zec Exp $"
+#ident "$XORP: xorp/bgp/bgp_varrw.cc,v 1.31 2007/02/16 22:45:10 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "libxorp/xorp.h"
@@ -325,8 +325,9 @@ BGPVarRW<A>::read_neighbor()
 {
     Element* e = NULL;
     const PeerHandler* ph = _orig_rtmsg->origin_peer();
-    if(ph)
-	e = _ef.create(ElemIPv4::id, ph->id().str().c_str());
+    if (ph != NULL) {
+	e = _ef.create(ElemIPv4::id, ph->get_peer_addr().c_str());
+    }
     return e;
 }
 
@@ -692,7 +693,7 @@ BGPVarRWCallbacks<A>::BGPVarRWCallbacks()
     init_rw(BGPVarRW<A>::VAR_ORIGIN, 
 	    &BGPVarRW<A>::read_origin, &BGPVarRW<A>::write_origin);
 
-    init_rw(BGPVarRW<A>::VAR_NEIGHBOR, &BGPVarRW<A>::read_neighbor, NULL);
+    init_rw(BGPVarRW<A>::VAR_NEIGHBOR, &BGPVarRW<A>::read_neighbor_base_cb, NULL);
 
     init_rw(BGPVarRW<A>::VAR_LOCALPREF, 
 	    &BGPVarRW<A>::read_localpref, &BGPVarRW<A>::write_localpref);
