@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,12 +13,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp_varrw_export.cc,v 1.4 2006/03/16 00:03:28 pavlin Exp $"
+#ident "$XORP: xorp/bgp/bgp_varrw_export.cc,v 1.6 2007/02/16 22:45:11 pavlin Exp $"
 
 #include "bgp_module.h"
-#include "config.h"
+
+#include "libxorp/xorp.h"
+
 #include "bgp_varrw_export.hh"
 
+
 template <class A>
 BGPVarRWExport<A>::BGPVarRWExport(const string& name,
 				  const string& neighbor)
