@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/bgppp.cc,v 1.11 2006/03/16 00:03:41 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/bgppp.cc,v 1.14 2007/02/16 22:45:25 pavlin Exp $"
 
 /*
 ** BGP Pretty Print
@@ -20,10 +20,7 @@
 
 #include "bgp/bgp_module.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 
@@ -34,12 +31,10 @@ string
 bgppp(const uint8_t *buf, const size_t len)
 {
     string result;
-
-    const fixed_header *header = 
-	reinterpret_cast<const struct fixed_header *>(buf);
+    uint8_t type = extract_8(buf + BGPPacket::TYPE_OFFSET);
 
     try {
-	switch(header->type) {
+	switch(type) {
 	case MESSAGETYPEOPEN: {
 	    OpenPacket pac(buf, len);
 	    result = pac.str().c_str();
@@ -64,7 +59,7 @@ bgppp(const uint8_t *buf, const size_t len)
 	    /*
 	    ** Send a notification to the peer. This is a bad message type.
 	    */
-	    result = c_format("Unknown packet type %d\n", header->type);
+	    result = c_format("Unknown packet type %d\n", type);
 	    XLOG_WARNING("%s", result.c_str());
 	}
     } catch(CorruptMessage& c) {
