@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,36 +12,32 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/peer.cc,v 1.75 2006/04/07 20:14:23 atanu Exp $"
+#ident "$XORP: xorp/bgp/harness/peer.cc,v 1.80 2007/02/16 22:45:25 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include <string>
-#include <string.h>
-#include <stdlib.h>
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "bgp/bgp_module.h"
 
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
-
 #include "libxorp/eventloop.hh"
 #include "libxorp/ipv4net.hh"
 #include "libxorp/utils.hh"
 
+#include "libproto/packet.hh"
+
+#include "libxipc/xrl_std_router.hh"
+
+#include "xrl/interfaces/test_peer_xif.hh"
+
 #include "bgp/local_data.hh"
 #include "bgp/packet.hh"
 
-#include "libxipc/xrl_std_router.hh"
-#include "xrl/interfaces/test_peer_xif.hh"
 #include "peer.hh"
 #include "bgppp.hh"
 
+
 Peer::~Peer()
 {
     debug_msg("XXX Deleting peer %p\n", this);
@@ -729,9 +725,8 @@ Peer::send_dump_callback(const XrlError& error, FILE *fp,
     const uint8_t *buf;
 
     while(0 != (buf = mrtd_traffic_file_read(fp, len))) {
-	const fixed_header *header = 
-	    reinterpret_cast<const struct fixed_header *>(buf);
-	if(MESSAGETYPEUPDATE == header->type) {
+	uint8_t type = extract_8(buf + BGPPacket::TYPE_OFFSET);
+	if(MESSAGETYPEUPDATE == type) {
 	    /*
 	    ** Save the update message in the sent trie.
 	    */
@@ -990,7 +985,7 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
     /*
     ** Due to alignment problems I can't use a structure overlay.
     */
-    uint8_t viewbuf[18 + A::addr_size()], *ptr;
+    uint8_t viewbuf[18 + A::addr_bytelen()], *ptr;
 
     mrt_header header;
     header.time = /*htonl(tv.sec())*/0;
@@ -1011,16 +1006,16 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
     ptr = &viewbuf[0];
 
     // View number
-    *reinterpret_cast<uint16_t *>(ptr) = 0;
+    embed_16(ptr, 0);
     ptr += 2;
 
     // Sequence number
-    *reinterpret_cast<uint16_t *>(ptr) = htons(sequence);
+    embed_16(ptr, sequence);
     ptr += 2;
 
     // Prefix
     net.masked_addr().copy_out(ptr);
-    ptr += A::addr_size();
+    ptr += A::addr_bytelen();
 
     // Prefix length
     *reinterpret_cast<uint8_t *>(ptr) = net.prefix_len();
@@ -1031,19 +1026,19 @@ mrtd_routview_dump(const  UpdatePacket* p, const IPNet<A>& net,
     ptr += 1;
 
     // Uptime
-    *reinterpret_cast<uint32_t *>(ptr) = htonl(tv.sec());
+    embed_32(ptr, tv.sec());
     ptr += 4;
 
     // Peer address
-    *reinterpret_cast<uint32_t *>(ptr) = 0;
+    embed_32(ptr, 0);
     ptr += 4;
 
     // Peer AS
-    *reinterpret_cast<uint16_t *>(ptr) = 0;
+    embed_16(ptr, 0);
     ptr += 2;
 
     // Attribute length
-    *reinterpret_cast<uint16_t *>(ptr) = htons(length);
+    embed_16(ptr, length);
     ptr += 2;
 
     XLOG_ASSERT(ptr == &viewbuf[sizeof(viewbuf)]);
@@ -1460,14 +1455,13 @@ Peer::datain(const bool& status, const TimeVal& tv,
     for(size_t i = 0; i < length; i++)
 	buf[i] = data[i];
 
-    const fixed_header *header = 
-	reinterpret_cast<const struct fixed_header *>(buf);
+    uint8_t type = extract_8(buf + BGPPacket::TYPE_OFFSET);
 
     if (!_traffic_recv.is_empty())
 	_traffic_recv->dispatch(buf, length, tv);
 
     try {
-	switch(header->type) {
+	switch(type) {
 	case MESSAGETYPEOPEN: {
 	    debug_msg("OPEN Packet RECEIVED\n");
 	    OpenPacket pac(buf, length);
@@ -1529,7 +1523,7 @@ Peer::datain(const bool& status, const TimeVal& tv,
 	    /*
 	    ** Send a notification to the peer. This is a bad message type.
 	    */
-	    XLOG_ERROR("Unknown packet type %d", header->type);
+	    XLOG_ERROR("Unknown packet type %d", type);
 	}
     } catch(CorruptMessage c) {
 	/*
