@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,36 +12,36 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.38 2006/04/10 19:34:33 bms Exp $"
+#ident "$XORP: xorp/bgp/harness/test_peer.cc,v 1.45 2007/02/16 22:45:26 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
 #include "bgp/bgp_module.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/status_codes.h"
 #include "libxorp/xorpfd.hh"
 #include "libxorp/eventloop.hh"
-#include "libxipc/xrl_std_router.hh"
-
-#include "libcomm/comm_api.h"
-
-#include "bgp/socket.hh"
-
-#include "xrl/interfaces/datain_xif.hh"
-#include "bgppp.hh"
-#include "test_peer.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "libcomm/comm_api.h"
+
+#include "libxipc/xrl_std_router.hh"
+
+#include "xrl/interfaces/datain_xif.hh"
+
+#include "bgp/socket.hh"
+
+#include "bgppp.hh"
+#include "test_peer.hh"
+
+
 static const char SERVER[] = "test_peer";/* This servers name */
 static const char SERVER_VERSION[] = "0.1";
 
@@ -187,7 +187,7 @@ XrlTestPeerTarget::test_peer_0_1_send(const vector<uint8_t>& data)
 {
     debug_msg("\n");
 
-    if(_trace && MESSAGETYPEOPEN == data[BGP_COMMON_HEADER_LEN - 1]) {
+    if(_trace && MESSAGETYPEOPEN == data[BGPPacket::COMMON_HEADER_LEN - 1]) {
 	printf("send() - open\n");
     }
 
@@ -324,9 +324,8 @@ TestPeer::connect(const string& host, const uint32_t& port,
 	return false;
     }
 
-    char buf[Socket::SOCKET_BUFFER_SIZE];
-    struct sockaddr *peer = reinterpret_cast<struct sockaddr *>(buf);
-    size_t len = sizeof(buf);
+    struct sockaddr_storage peer;
+    size_t len = sizeof(peer);
     try {
 	Socket::init_sockaddr(host, port, peer, len);
     } catch(UnresolvableHost e) {
@@ -334,7 +333,7 @@ TestPeer::connect(const string& host, const uint32_t& port,
 	return false;
     }
 
-    XorpFd s = comm_sock_open(peer->sa_family, SOCK_STREAM, 0,
+    XorpFd s = comm_sock_open(peer.ss_family, SOCK_STREAM, 0,
 			      COMM_SOCK_NONBLOCKING);
     if (!s.is_valid()) {
 	error_string = c_format("comm_sock_open failed: %s\n",
@@ -344,7 +343,8 @@ TestPeer::connect(const string& host, const uint32_t& port,
 
     // XXX
     int in_progress = 0;
-    (void)comm_sock_connect(s, peer, COMM_SOCK_NONBLOCKING, &in_progress);
+    (void)comm_sock_connect(s, reinterpret_cast<struct sockaddr *>(&peer),
+			    COMM_SOCK_NONBLOCKING, &in_progress);
 
    /*
    ** If there is a coordinator then add an I/O event callback.
@@ -392,10 +392,8 @@ TestPeer::listen(const string& host, const uint32_t& port,
 	return false;
     }
 
-
-    char buf[Socket::SOCKET_BUFFER_SIZE];
-    struct sockaddr *local = reinterpret_cast<struct sockaddr *>(buf);
-    size_t len = sizeof(buf);
+    struct sockaddr_storage local;
+    size_t len = sizeof(local);
     try {
 	Socket::init_sockaddr(host, port, local, len);
     } catch(UnresolvableHost e) {
@@ -403,7 +401,8 @@ TestPeer::listen(const string& host, const uint32_t& port,
 	return false;
     }
 
-    XorpFd s = comm_bind_tcp(local, COMM_SOCK_NONBLOCKING);
+    XorpFd s = comm_bind_tcp(reinterpret_cast<struct sockaddr *>(&local),
+			     COMM_SOCK_NONBLOCKING);
     if (!s.is_valid()) {
 	error_string = c_format("comm_bind_tcp() failed: %s\n",
 				comm_get_last_error_str());
@@ -444,9 +443,8 @@ TestPeer::bind(const string& host, const uint32_t& port,
 	return false;
     }
 
-    char buf[Socket::SOCKET_BUFFER_SIZE];
-    struct sockaddr *local = reinterpret_cast<struct sockaddr *>(buf);
-    size_t len = sizeof(buf);
+    struct sockaddr_storage local;
+    size_t len = sizeof(local);
     try {
 	Socket::init_sockaddr(host, port, local, len);
     } catch(UnresolvableHost e) {
@@ -454,7 +452,7 @@ TestPeer::bind(const string& host, const uint32_t& port,
 	return false;
     }
 
-    XorpFd s = comm_sock_open(local->sa_family, SOCK_STREAM, 0,
+    XorpFd s = comm_sock_open(local.ss_family, SOCK_STREAM, 0,
 			      COMM_SOCK_NONBLOCKING);
     if (!s.is_valid()) {
 	error_string = c_format("comm_sock_open failed: %s",
@@ -468,7 +466,8 @@ TestPeer::bind(const string& host, const uint32_t& port,
 	return false;
     }
 
-    if (comm_sock_bind(s, local) != XORP_OK) {
+    if (comm_sock_bind(s, reinterpret_cast<struct sockaddr *>(&local))
+	!= XORP_OK) {
 	comm_sock_close(s);
 	error_string = c_format("comm_sock_bind failed: %s",
 				comm_get_last_error_str());
@@ -689,12 +688,10 @@ TestPeer::receive(XorpFd fd, IoEventType type)
     ** Now doing BGP packetisation.
     */
     int get;
-    if(_bgp_bytes < BGP_COMMON_HEADER_LEN) {
-	get = BGP_COMMON_HEADER_LEN - _bgp_bytes;
+    if(_bgp_bytes < BGPPacket::COMMON_HEADER_LEN) {
+	get = BGPPacket::COMMON_HEADER_LEN - _bgp_bytes;
     } else {
-	const fixed_header *header =
-	    reinterpret_cast<const struct fixed_header *>(_bgp_buf);
-	u_short length = ntohs(header->length);
+	uint16_t length = extract_16(_bgp_buf + BGPPacket::LENGTH_OFFSET);
 	get = length - _bgp_bytes;
     }
 
@@ -722,11 +719,9 @@ TestPeer::receive(XorpFd fd, IoEventType type)
 
     _bgp_bytes += len;
 
-    if (_bgp_bytes >= BGP_COMMON_HEADER_LEN) {
-	const fixed_header *header =
-	    reinterpret_cast<const struct fixed_header *>(_bgp_buf);
-	u_short length = ntohs(header->length);
-	if(length < MINPACKETSIZE || length > sizeof(_bgp_buf)) {
+    if (_bgp_bytes >= BGPPacket::COMMON_HEADER_LEN) {
+	uint16_t length = extract_16(_bgp_buf + BGPPacket::LENGTH_OFFSET);
+	if (length < BGPPacket::MINPACKETSIZE || length > sizeof(_bgp_buf)) {
 	    string error = c_format("Illegal length value %d", length);
 	    XLOG_ERROR("%s", error.c_str());
 	    datain(ERROR, _bgp_buf, _bgp_bytes, error);
@@ -814,23 +809,30 @@ TestPeer::sendit()
 
     _flying++;
 
-    switch(q.len) {
-    case 0:
+    if (q.len == 0) {
+	//
+	// XXX: Note that we don't consider it an error even if
+	// "q.st == ERROR".
+	// This is needed to preserve the original logic that considers
+	// the case of a peer resetting immediately the TCP
+	// connection not an error.
+	//
 	datain.send_closed(_coordinator.c_str(), _server, _genid,
 			   callback(this, &TestPeer::xrl_callback));
-	break;
-    case -1:
+	return;
+    }
+
+    if (q.st == ERROR) {
 	datain.send_error(_coordinator.c_str(), _server, _genid, q.error,
 			  callback(this, &TestPeer::xrl_callback));
-	break;
-    default:
-	datain.send_receive(_coordinator.c_str(), _server, _genid,
-			    GOOD == q.st ? true : false,
-			    q.secs, q.micro,
-			    q.v,
-			    callback(this, &TestPeer::xrl_callback));
-	break;
+	return;
     }
+
+    datain.send_receive(_coordinator.c_str(), _server, _genid,
+			GOOD == q.st ? true : false,
+			q.secs, q.micro,
+			q.v,
+			callback(this, &TestPeer::xrl_callback));
 }
 
 void
