@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_routing2.sh,v 1.20 2006/04/04 12:07:45 bms Exp $
+# $XORP: xorp/bgp/harness/test_routing2.sh,v 1.21 2006/08/16 22:10:14 atanu Exp $
 #
 
 #
@@ -45,7 +45,7 @@ if [ "X${srcdir}" = "X" ] ; then srcdir=`dirname $0` ; fi
 . ${srcdir}/../xrl_shell_funcs.sh ""
 . ${srcdir}/../../rib/xrl_shell_funcs.sh ""
 
-HOST=localhost
+HOST=127.0.0.1
 PORT1=10001
 PORT2=10002
 PORT3=10003
