@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,19 +12,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/iptuple.cc,v 1.16 2006/04/04 09:41:38 bms Exp $"
+#ident "$XORP: xorp/bgp/iptuple.cc,v 1.22 2007/02/16 22:45:12 pavlin Exp $"
 
 // #define DEBUG_LOGGING 
 // #define DEBUG_PRINT_FUNCTION_NAME
 
 #include "bgp_module.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "libxorp/debug.h"
 #include "libxorp/xorp.h"
+#include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
 #include "libxorp/callback.hh"
@@ -32,6 +28,7 @@
 #include "socket.hh"
 #include "iptuple.hh"
 
+
 Iptuple::Iptuple()
 {}
 
@@ -43,12 +40,9 @@ Iptuple::Iptuple(const char *local_interface, uint16_t local_port,
       _local_port(local_port),
       _peer_port(peer_port)
 {
-    _local_sock = reinterpret_cast<struct sockaddr *>(_local_buffer);
-    _local_sock_len = sizeof(_local_buffer);
-    _bind_sock = reinterpret_cast<struct sockaddr *>(_bind_buffer);
-    _bind_sock_len = sizeof(_bind_buffer);
-    _peer_sock = reinterpret_cast<struct sockaddr *>(_peer_buffer);
-    _peer_sock_len = sizeof(_peer_buffer);
+    _local_sock_len = sizeof(_local_sock);
+    _bind_sock_len = sizeof(_bind_sock);
+    _peer_sock_len = sizeof(_peer_sock);
 
     fill_address(local_interface, local_port, _local_sock, _local_sock_len,
 		 _local_address);
@@ -59,9 +53,14 @@ Iptuple::Iptuple(const char *local_interface, uint16_t local_port,
 		 _peer_address);
 
     //  Test for an address family mismatch
-    if (_local_sock->sa_family != _peer_sock->sa_family)
+    if (_local_sock.ss_family != _peer_sock.ss_family)
 	xorp_throw(AddressFamilyMismatch,
-		   c_format("mismatch %s %s",local_interface, peer_interface));
+		   c_format("mismatch %s (%u) %s (%u)",
+			    local_interface, _local_sock.ss_family, 
+			    peer_interface, _peer_sock.ss_family));
+
+    _local_address_ipvx = IPvX(_local_address.c_str());
+    _peer_address_ipvx = IPvX(_peer_address.c_str());
 }
 
 Iptuple::Iptuple(const Iptuple& rhs)
@@ -86,20 +85,19 @@ Iptuple::copy(const Iptuple& rhs)
     _local_interface = rhs._local_interface;
     _peer_interface = rhs._peer_interface;
 
-    memcpy(_local_buffer, rhs._local_buffer, Socket::SOCKET_BUFFER_SIZE);
-    _local_sock = reinterpret_cast<struct sockaddr *>(_local_buffer);
+    memcpy(&_local_sock, &rhs._local_sock, sizeof(_local_sock));
     _local_sock_len = rhs._local_sock_len;
 
-    memcpy(_bind_buffer, rhs._bind_buffer, Socket::SOCKET_BUFFER_SIZE);
-    _bind_sock = reinterpret_cast<struct sockaddr *>(_bind_buffer);
+    memcpy(&_bind_sock, &rhs._bind_sock, sizeof(_bind_sock));
     _bind_sock_len = rhs._bind_sock_len;
 
-    memcpy(_peer_buffer, rhs._peer_buffer, Socket::SOCKET_BUFFER_SIZE);
-    _peer_sock = reinterpret_cast<struct sockaddr *>(_peer_buffer);
+    memcpy(&_peer_sock, &rhs._peer_sock, sizeof(_peer_sock));
     _peer_sock_len = rhs._peer_sock_len;
     
     _local_address = rhs._local_address;
+    _local_address_ipvx = rhs._local_address_ipvx;
     _peer_address = rhs._peer_address;
+    _peer_address_ipvx = rhs._peer_address_ipvx;
 
     _local_port = rhs._local_port;
     _peer_port = rhs._peer_port;
@@ -124,7 +122,7 @@ Iptuple::operator==(const Iptuple& rhs) const
 */
 void
 Iptuple::fill_address(const char *interface, uint16_t local_port,
-		      struct sockaddr *sin, size_t& len,
+		      struct sockaddr_storage& ss, size_t& len,
 		      string& numeric_interface)
     throw(UnresolvableHost)
 {
@@ -158,8 +156,8 @@ Iptuple::fill_address(const char *interface, uint16_t local_port,
     debug_msg("addrlen %u len %u\n", XORP_UINT_CAST(res0->ai_addrlen),
 	XORP_UINT_CAST(len));
     XLOG_ASSERT(res0->ai_addrlen <= len);
-    memcpy(sin, res0->ai_addr, res0->ai_addrlen);
-    debug_msg("family %d\n", sin->sa_family);
+    memcpy(&ss, res0->ai_addr, res0->ai_addrlen);
+    debug_msg("family %d\n", ss.ss_family);
 
     len = res0->ai_addrlen;
 
@@ -184,7 +182,7 @@ const struct sockaddr *
 Iptuple::get_local_socket(size_t& len) const
 {
     len = _local_sock_len;
-    return _local_sock;
+    return reinterpret_cast<const struct sockaddr *>(&_local_sock);
 }
 
 string
@@ -193,6 +191,28 @@ Iptuple::get_local_addr() const
     return _local_address;
 }
 
+bool
+Iptuple::get_local_addr(IPv4& addr) const
+{
+    if (!_local_address_ipvx.is_ipv4())
+	return false;
+
+    addr = _local_address_ipvx.get_ipv4();
+
+    return true;
+}
+
+bool
+Iptuple::get_local_addr(IPv6& addr) const
+{
+    if (!_local_address_ipvx.is_ipv6())
+	return false;
+
+    addr = _local_address_ipvx.get_ipv6();
+
+    return true;
+}
+
 uint16_t
 Iptuple::get_local_port() const
 {
@@ -203,14 +223,14 @@ const struct sockaddr *
 Iptuple::get_bind_socket(size_t& len) const
 {
     len = _bind_sock_len;
-    return _bind_sock;
+    return reinterpret_cast<const struct sockaddr *>(&_bind_sock);
 }
 
 const struct sockaddr *
 Iptuple::get_peer_socket(size_t& len) const
 {
     len = _peer_sock_len;
-    return _peer_sock;
+    return reinterpret_cast<const struct sockaddr *>(&_peer_sock);
 }
 
 string
@@ -219,6 +239,28 @@ Iptuple::get_peer_addr() const
     return _peer_address;
 }
 
+bool
+Iptuple::get_peer_addr(IPv4& addr) const
+{
+    if (!_peer_address_ipvx.is_ipv4())
+	return false;
+
+    addr = _peer_address_ipvx.get_ipv4();
+
+    return true;
+}
+
+bool
+Iptuple::get_peer_addr(IPv6& addr) const
+{
+    if (!_peer_address_ipvx.is_ipv6())
+	return false;
+
+    addr = _peer_address_ipvx.get_ipv6();
+
+    return true;
+}
+
 uint16_t
 Iptuple::get_peer_port() const
 {
