@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,17 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/next_hop_resolver.cc,v 1.46 2006/03/16 00:03:29 pavlin Exp $"
+#ident "$XORP: xorp/bgp/next_hop_resolver.cc,v 1.50 2007/02/16 22:45:12 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "bgp_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/timer.hh"
@@ -36,6 +33,7 @@
 #include "route_table_decision.hh"
 #include "profile_vars.hh"
 
+
 template <class A>
 NextHopResolver<A>::NextHopResolver(XrlStdRouter *xrl_router,
 				    EventLoop& eventloop,
@@ -180,16 +178,16 @@ NextHopResolver<A>::lookup(const A nexthop, bool& resolvable,
 template <class A>
 bool
 NextHopResolver<A>::rib_client_route_info_changed(const A& addr,
-						  const uint32_t& real_prefix_len,
+						  const uint32_t& prefix_len,
 						  const A& nexthop,
 						  const uint32_t& metric)
 {
     debug_msg("addr %s prefix_len %u nexthop %s metric %u\n",
-	      addr.str().c_str(), XORP_UINT_CAST(real_prefix_len),
+	      addr.str().c_str(), XORP_UINT_CAST(prefix_len),
 	      nexthop.str().c_str(), XORP_UINT_CAST(metric));
     XLOG_TRACE(_bgp.profile().enabled(trace_nexthop_resolution),
 	       "addr %s prefix_len %u nexthop %s metric %u\n",
-	       addr.str().c_str(), XORP_UINT_CAST(real_prefix_len),
+	       addr.str().c_str(), XORP_UINT_CAST(prefix_len),
 	       nexthop.str().c_str(), XORP_UINT_CAST(metric));
 
     UNUSED(nexthop);
@@ -198,7 +196,7 @@ NextHopResolver<A>::rib_client_route_info_changed(const A& addr,
     ** Change the entries in the cache and make an upcall notifying
     ** the decision code that the metrics have changed.
     */
-    map<A, int> m = _next_hop_cache.change_entry(addr, real_prefix_len, metric);
+    map<A, int> m = _next_hop_cache.change_entry(addr, prefix_len, metric);
     typename map<A, int>::iterator i;
     for (i = m.begin(); i != m.end(); i++)
 	next_hop_changed(i->first);
@@ -403,6 +401,7 @@ NextHopCache<A>::validate_entry(A addr, A nexthop, int prefix_len, int real_pref
     return true;
 }
 
+#if	0
 template<class A>
 map<A, int>
 NextHopCache<A>::change_entry(A addr, int real_prefix_len, uint32_t metric)
@@ -427,6 +426,33 @@ NextHopCache<A>::change_entry(A addr, int real_prefix_len, uint32_t metric)
 
     return ret;
 }
+#endif
+
+template<class A>
+map<A, int>
+NextHopCache<A>::change_entry(A addr, int prefix_len, uint32_t metric)
+{
+    debug_msg("addr %s prefix_len %d\n", addr.str().c_str(), prefix_len);
+
+    PrefixIterator pi =
+	_next_hop_by_prefix.lookup_node(IPNet<A>(addr, prefix_len));
+    XLOG_ASSERT(pi != _next_hop_by_prefix.end());
+
+    PrefixEntry *en = pi.payload();
+
+    XLOG_ASSERT(en);
+    XLOG_ASSERT(en->_address == addr);
+    XLOG_ASSERT(en->_prefix_len == prefix_len);
+
+    // Save all the next hops and reference counts that were covered
+    // by this entry.
+    map<A, int> ret = en->_nexthop_references;
+
+    // Update the metric
+    en->_metric = metric;
+
+    return ret;
+}
 
 template<class A>
 map<A, int>
@@ -635,7 +661,7 @@ NextHopCache<A>::rpe_to_pe_delete(RealPrefixEntry& rpe, A addr,
     debug_msg("addr %s real prefix_len %d\n", addr.str().c_str(), 
 	      real_prefix_len);
 
-    typename RealPrefixEntry::const_iterator si;
+    typename RealPrefixEntry::iterator si;
 
 #ifdef	DEBUG_LOGGING
     for (si = rpe.begin(); si != rpe.end(); si++)
