@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,18 +12,19 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/notification_packet.cc,v 1.27 2006/03/16 00:03:29 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/notification_packet.cc,v 1.31 2007/02/16 22:45:13 pavlin Exp $"
 
 #include "bgp_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 
+#include "libproto/packet.hh"
+
 #include "packet.hh"
+
+
 #define DEBUG_BGPNotificationPacket
 
 /* **************** BGPNotificationPacket *********************** */
@@ -41,7 +42,7 @@ NotificationPacket::NotificationPacket(uint8_t		ec,
     if (ed == 0)
 	elen = 0;
     // elen is the length of the error_data to carry
-    _Length = MINNOTIFICATIONPACKET + elen;
+    _Length = BGPPacket::MINNOTIFICATIONPACKET + elen;
 
     _Type = MESSAGETYPENOTIFICATION;
     _error_code = ec;
@@ -60,17 +61,17 @@ NotificationPacket::NotificationPacket(const uint8_t *d, uint16_t l)
     throw(CorruptMessage)
 {
     debug_msg("Data %p len %d\n", d, l);
-    if (l < MINNOTIFICATIONPACKET)
+    if (l < BGPPacket::MINNOTIFICATIONPACKET)
 	xorp_throw(CorruptMessage,
 		   c_format("Notification message too short %d", l),
-		   MSGHEADERERR, BADMESSLEN, d + MARKER_SIZE, 2);
+		   MSGHEADERERR, BADMESSLEN, d + BGPPacket::MARKER_SIZE, 2);
 
     _Length = l;
     _Type = MESSAGETYPENOTIFICATION;
-    d += BGP_COMMON_HEADER_LEN;	// skip header
+    d += BGPPacket::COMMON_HEADER_LEN;		// skip header
     _error_code = d[0];
     _error_subcode = d[1];
-    int error_data_len = _Length - MINNOTIFICATIONPACKET;
+    int error_data_len = _Length - BGPPacket::MINNOTIFICATIONPACKET;
     if (error_data_len > 0) {
 	uint8_t *ed = new uint8_t[error_data_len];
 	memcpy(ed, d + 2, error_data_len);
@@ -93,11 +94,11 @@ NotificationPacket::encode(size_t& len, uint8_t *buf) const
 
     // allocate buffer, set length, fill up header
     buf = basic_encode(len, buf);
-    buf[BGP_COMMON_HEADER_LEN] = _error_code;
-    buf[BGP_COMMON_HEADER_LEN+1] = _error_subcode;
+    buf[BGPPacket::COMMON_HEADER_LEN] = _error_code;
+    buf[BGPPacket::COMMON_HEADER_LEN + 1] = _error_subcode;
     if (_error_data != 0)
-	memcpy(buf + MINNOTIFICATIONPACKET, _error_data,
-		len - MINNOTIFICATIONPACKET);
+	memcpy(buf + BGPPacket::MINNOTIFICATIONPACKET, _error_data,
+		len - BGPPacket::MINNOTIFICATIONPACKET);
     return buf;
 }
 
@@ -176,12 +177,13 @@ NotificationPacket::pretty_print_error_code(const int error, const int subcode,
 	    s += c_format("Connection Not Synchronized(%d)", CONNNOTSYNC);
 	    break;
 	case BADMESSLEN:
-	    if (error_data != NULL)
+	    if (error_data != NULL) {
 		s += c_format("Bad Message Length(%d) - field: %d",
-			      BADMESSLEN, ntohs((uint16_t &)*error_data));
-	    else
+			      BADMESSLEN, extract_16(error_data));
+	    } else {
 		s += c_format("Bad Message Length(%d) - "
 			      "NO ERROR DATA SUPPLIED", BADMESSLEN);
+	    }
 	    break;
 	case BADMESSTYPE:
 	    if (error_data != NULL)
@@ -197,13 +199,14 @@ NotificationPacket::pretty_print_error_code(const int error, const int subcode,
 	s += c_format("OPEN Message Error(%d): ", OPENMSGERROR);
 	switch (subcode) {
 	case UNSUPVERNUM:
-	    if (error_data != NULL)
+	    if (error_data != NULL) {
 		s += c_format("Unsupported Version Number(%d) - "
 			      "Min supported Version is %d",
-			      UNSUPVERNUM, ntohs((uint16_t &)*error_data));
-	    else
+			      UNSUPVERNUM, extract_16(error_data));
+	    } else {
 		s += c_format("Unsupported Version Number(%d) - "
 			      "NO ERROR DATA SUPPLIED", UNSUPVERNUM);
+	    }
 	    break;
 	case BADASPEER:
 	    s += c_format("Bad Peer AS(%d)", BADASPEER);
@@ -296,7 +299,7 @@ NotificationPacket::str() const
 {
     return "Notification Packet: " +
 	pretty_print_error_code(_error_code, _error_subcode, _error_data,
-				_Length - MINNOTIFICATIONPACKET) +
+				_Length - BGPPacket::MINNOTIFICATIONPACKET) +
 	"\n";
 }
 
@@ -316,7 +319,7 @@ NotificationPacket::operator==(const NotificationPacket& him ) const
 	return false;
 
     if (0 != memcmp(_error_data, him.error_data(),
-		   _Length - MINNOTIFICATIONPACKET))
+		   _Length - BGPPacket::MINNOTIFICATIONPACKET))
 	return false;
 
     return true;
