@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,33 +13,28 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/path_attribute.cc,v 1.81 2006/05/09 02:56:28 pavlin Exp $"
+#ident "$XORP: xorp/bgp/path_attribute.cc,v 1.85 2007/02/16 22:45:13 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "bgp_module.h"
 
 #include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
-
-#include "path_attribute.hh"
-#include "packet.hh"
+#include "libxorp/ipv4.hh"
+#include "libxorp/ipv6.hh"
 
 #include <vector>
-#include <stdio.h>
 
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
 
-#include "libxorp/ipv4.hh"
-#include "libxorp/ipv6.hh"
+#include "path_attribute.hh"
+#include "packet.hh"
+
 
 #ifdef DEBUG_LOGGING
 inline static void
@@ -210,7 +205,7 @@ NextHopAttribute<A>::NextHopAttribute(const uint8_t* d)
 	xorp_throw(CorruptMessage,
 		   c_format("Bad Flags in NextHop attribute %#x", flags()),
 		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
-    if (length(d) != A::addr_size())
+    if (length(d) != A::addr_bytelen())
 	xorp_throw(CorruptMessage, "Bad size in NextHop address",
 		   UPDATEMSGERR, ATTRLEN);
 
@@ -228,7 +223,7 @@ template<class A>
 void
 NextHopAttribute<A>::encode()
 {
-    uint8_t *d = set_header(_next_hop.addr_size());
+    uint8_t *d = set_header(A::addr_bytelen());
     _next_hop.copy_out(d);
 }
 
@@ -660,7 +655,7 @@ MPReachNLRIAttribute<IPv6>::encode()
 
     XLOG_ASSERT(AFI_IPV6 == _afi);
     XLOG_ASSERT((SAFI_UNICAST  == _safi) || (SAFI_MULTICAST == _safi));
-    XLOG_ASSERT(16 == IPv6::addr_size());
+    XLOG_ASSERT(16 == IPv6::addr_bytelen());
 
     /*
     ** Figure out how many bytes we need to allocate.
@@ -669,9 +664,9 @@ MPReachNLRIAttribute<IPv6>::encode()
     len = 2;	// AFI
     len += 1;	// SAFI
     len += 1;	// Length of Next Hop Address
-    len += IPv6::addr_size();	// Next Hop
+    len += IPv6::addr_bytelen();	// Next Hop
     if (!_link_local_next_hop.is_zero())
-	len += IPv6::addr_size();
+	len += IPv6::addr_bytelen();
     len += 1;	// Number of SNPAs
 
     const_iterator i;
@@ -691,15 +686,15 @@ MPReachNLRIAttribute<IPv6>::encode()
     *d++ = _safi;		// SAFIs
 
     if (_link_local_next_hop.is_zero()) {
-	*d++ = IPv6::addr_size();
+	*d++ = IPv6::addr_bytelen();
 	nexthop().copy_out(d);
-	d += IPv6::addr_size();
+	d += IPv6::addr_bytelen();
     } else {
-	*d++ = IPv6::addr_size() * 2;
+	*d++ = IPv6::addr_bytelen() * 2;
 	nexthop().copy_out(d);
-	d += IPv6::addr_size();
+	d += IPv6::addr_bytelen();
 	_link_local_next_hop.copy_out(d);
-	d += IPv6::addr_size();
+	d += IPv6::addr_bytelen();
     }
     
     *d++ = 0;	// Number of SNPAs
@@ -707,7 +702,7 @@ MPReachNLRIAttribute<IPv6>::encode()
     for (i = _nlri.begin(); i != _nlri.end(); i++) {
 	int bytes = (i->prefix_len() + 7)/ 8;
 	debug_msg("encode %s bytes = %d\n", i->str().c_str(), bytes);
-	uint8_t buf[IPv6::addr_size()];
+	uint8_t buf[IPv6::addr_bytelen()];
 	i->masked_addr().copy_out(buf);
 
 	*d++ = i->prefix_len();
@@ -723,7 +718,7 @@ MPReachNLRIAttribute<IPv4>::encode()
     delete[] _data;	// Zap any old allocation.
 
     XLOG_ASSERT(AFI_IPV4 == _afi && SAFI_MULTICAST == _safi);
-    XLOG_ASSERT(4 == IPv4::addr_size());
+    XLOG_ASSERT(4 == IPv4::addr_bytelen());
 
     /*
     ** Figure out how many bytes we need to allocate.
@@ -732,7 +727,7 @@ MPReachNLRIAttribute<IPv4>::encode()
     len = 2;	// AFI
     len += 1;	// SAFI
     len += 1;	// Length of Next Hop Address
-    len += IPv4::addr_size();	// Next Hop
+    len += IPv4::addr_bytelen();	// Next Hop
     len += 1;	// Number of SNPAs
 
     const_iterator i;
@@ -751,16 +746,16 @@ MPReachNLRIAttribute<IPv4>::encode()
     
     *d++ = _safi;		// SAFIs
 
-    *d++ = IPv4::addr_size();
+    *d++ = IPv4::addr_bytelen();
     nexthop().copy_out(d);
-    d += IPv4::addr_size();
+    d += IPv4::addr_bytelen();
     
     *d++ = 0;	// Number of SNPAs
 
     for (i = _nlri.begin(); i != _nlri.end(); i++) {
 	int bytes = (i->prefix_len() + 7) / 8;
 	debug_msg("encode %s bytes = %d\n", i->str().c_str(), bytes);
-	uint8_t buf[IPv4::addr_size()];
+	uint8_t buf[IPv4::addr_bytelen()];
 	i->masked_addr().copy_out(buf);
 
 	*d++ = i->prefix_len();
@@ -851,21 +846,21 @@ MPReachNLRIAttribute<IPv6>::MPReachNLRIAttribute(const uint8_t* d)
     */
     uint8_t len = *data++;
 
-    XLOG_ASSERT(16 == IPv6::addr_size());
+    XLOG_ASSERT(16 == IPv6::addr_bytelen());
     IPv6 temp;
 
     switch(len) {
     case 16:
 	temp.copy_in(data);
 	set_nexthop(temp);
-	data += IPv6::addr_size();
+	data += IPv6::addr_bytelen();
 	break;
     case 32:
 	temp.copy_in(data);
 	set_nexthop(temp);
-	data += IPv6::addr_size();
+	data += IPv6::addr_bytelen();
 	_link_local_next_hop.copy_in(data);
-	data += IPv6::addr_size();
+	data += IPv6::addr_bytelen();
 	break;
     default:
 	xorp_throw(CorruptMessage,
@@ -902,11 +897,11 @@ MPReachNLRIAttribute<IPv6>::MPReachNLRIAttribute(const uint8_t* d)
     while(data < end) {
 	uint8_t prefix_length = *data++;
 	size_t bytes = (prefix_length + 7)/ 8;
-	if (bytes > IPv6::addr_size())
+	if (bytes > IPv6::addr_bytelen())
 	    xorp_throw(CorruptMessage,
 		       c_format("prefix length too long %d", prefix_length),
 		       UPDATEMSGERR, OPTATTR);
-	uint8_t buf[IPv6::addr_size()];
+	uint8_t buf[IPv6::addr_bytelen()];
 	memset(buf, 0, sizeof(buf));
 	memcpy(buf, data, bytes);
 	IPv6 nlri;
@@ -974,14 +969,14 @@ MPReachNLRIAttribute<IPv4>::MPReachNLRIAttribute(const uint8_t* d)
     */
     uint8_t len = *data++;
 
-    XLOG_ASSERT(4 == IPv4::addr_size());
+    XLOG_ASSERT(4 == IPv4::addr_bytelen());
     IPv4 temp;
 
     switch(len) {
     case 4:
 	temp.copy_in(data);
 	set_nexthop(temp);
-	data += IPv4::addr_size();
+	data += IPv4::addr_bytelen();
 	break;
     default:
 	xorp_throw(CorruptMessage,
@@ -1018,11 +1013,11 @@ MPReachNLRIAttribute<IPv4>::MPReachNLRIAttribute(const uint8_t* d)
     while(data < end) {
 	uint8_t prefix_length = *data++;
 	size_t bytes = (prefix_length + 7) / 8;
-	if (bytes > IPv4::addr_size())
+	if (bytes > IPv4::addr_bytelen())
 	    xorp_throw(CorruptMessage,
 		       c_format("prefix length too long %d", prefix_length),
 		       UPDATEMSGERR, OPTATTR);
-	uint8_t buf[IPv4::addr_size()];
+	uint8_t buf[IPv4::addr_bytelen()];
 	memset(buf, 0, sizeof(buf));
 	memcpy(buf, data, bytes);
 	IPv4 nlri;
@@ -1059,7 +1054,7 @@ MPUNReachNLRIAttribute<IPv6>::encode()
 
     XLOG_ASSERT(AFI_IPV6 == _afi);
     XLOG_ASSERT((SAFI_UNICAST  == _safi) || (SAFI_MULTICAST == _safi));
-    XLOG_ASSERT(16 == IPv6::addr_size());
+    XLOG_ASSERT(16 == IPv6::addr_bytelen());
 
     /*
     ** Figure out how many bytes we need to allocate.
@@ -1087,7 +1082,7 @@ MPUNReachNLRIAttribute<IPv6>::encode()
     for (i = _withdrawn.begin(); i != _withdrawn.end(); i++) {
 	int bytes = (i->prefix_len() + 7)/ 8;
 	debug_msg("encode %s bytes = %d\n", i->str().c_str(), bytes);
-	uint8_t buf[IPv6::addr_size()];
+	uint8_t buf[IPv6::addr_bytelen()];
 	i->masked_addr().copy_out(buf);
 
 	*d++ = i->prefix_len();
@@ -1103,7 +1098,7 @@ MPUNReachNLRIAttribute<IPv4>::encode()
     delete[] _data;	// Zap any old allocation.
 
     XLOG_ASSERT(AFI_IPV4 == _afi && SAFI_MULTICAST == _safi);
-    XLOG_ASSERT(4 == IPv4::addr_size());
+    XLOG_ASSERT(4 == IPv4::addr_bytelen());
 
     /*
     ** Figure out how many bytes we need to allocate.
@@ -1131,7 +1126,7 @@ MPUNReachNLRIAttribute<IPv4>::encode()
     for (i = _withdrawn.begin(); i != _withdrawn.end(); i++) {
 	int bytes = (i->prefix_len() + 7) / 8;
 	debug_msg("encode %s bytes = %d\n", i->str().c_str(), bytes);
-	uint8_t buf[IPv4::addr_size()];
+	uint8_t buf[IPv4::addr_bytelen()];
 	i->masked_addr().copy_out(buf);
 
 	*d++ = i->prefix_len();
@@ -1223,12 +1218,12 @@ MPUNReachNLRIAttribute<IPv6>::MPUNReachNLRIAttribute(const uint8_t* d)
 	uint8_t prefix_length = *data++;
 	debug_msg("decode prefix length = %d\n", prefix_length);
 	size_t bytes = (prefix_length + 7)/ 8;
-	if (bytes > IPv6::addr_size())
+	if (bytes > IPv6::addr_bytelen())
 	    xorp_throw(CorruptMessage,
 		       c_format("prefix length too long %d", prefix_length),
 		       UPDATEMSGERR, OPTATTR);
 	debug_msg("decode bytes = %u\n", XORP_UINT_CAST(bytes));
-	uint8_t buf[IPv6::addr_size()];
+	uint8_t buf[IPv6::addr_bytelen()];
 	memset(buf, 0, sizeof(buf));
 	memcpy(buf, data, bytes);
 	IPv6 nlri;
@@ -1296,12 +1291,12 @@ MPUNReachNLRIAttribute<IPv4>::MPUNReachNLRIAttribute(const uint8_t* d)
 	uint8_t prefix_length = *data++;
 	debug_msg("decode prefix length = %d\n", prefix_length);
 	size_t bytes = (prefix_length + 7)/ 8;
-	if (bytes > IPv4::addr_size())
+	if (bytes > IPv4::addr_bytelen())
 	    xorp_throw(CorruptMessage,
 		       c_format("prefix length too long %d", prefix_length),
 		       UPDATEMSGERR, OPTATTR);
 	debug_msg("decode bytes = %u\n", XORP_UINT_CAST(bytes));
-	uint8_t buf[IPv4::addr_size()];
+	uint8_t buf[IPv4::addr_bytelen()];
 	memset(buf, 0, sizeof(buf));
 	memcpy(buf, data, bytes);
 	IPv4 nlri;
@@ -1592,7 +1587,7 @@ PathAttributeList<A>::PathAttributeList()
 {
     debug_msg("%p\n", this);
 
-    memset(_hash, 0, 16);
+    memset(_hash, 0, sizeof(_hash));
 }
 
 template<class A>
@@ -1672,13 +1667,13 @@ PathAttributeList<A>::add_path_attribute(PathAttribute *a)
 	for (i = begin(); i != end(); i++)
 	    if ( *(*i) > *a) {
 		insert(i, a);
-		memset(_hash, 0, 16);
+		memset(_hash, 0, sizeof(_hash));
 		return;
 	    }
     }
     // list empty, or tail insertion:
     push_back(a);
-    memset(_hash, 0, 16);
+    memset(_hash, 0, sizeof(_hash));
 }
 
 template<class A>
@@ -1721,7 +1716,7 @@ operator< (const PathAttributeList<A> &him) const
     if (him.size() < size())
         return false;
 
-    //    return (memcmp(_hash, him.hash(), 16) < 0);
+    //    return (memcmp(_hash, him.hash(), sizeof(_hash)) < 0);
     const_iterator my_i = begin();
     const_iterator his_i = him.begin();
     for (;;) {
@@ -1755,7 +1750,7 @@ operator == (const PathAttributeList<A> &him) const
     debug_msg("PathAttributeList operator== %p %p\n", this, &him);
     assert_rehash();
     him.assert_rehash();
-    return (memcmp(_hash, him.hash(), 16) == 0);
+    return (memcmp(_hash, him.hash(), sizeof(_hash)) == 0);
 }
 
 template<class A>
@@ -1792,7 +1787,7 @@ PathAttributeList<A>::replace_attribute(PathAttribute* new_att)
 	    delete (*i);
 	    erase(i);
 	    debug_msg("After: \n%s\n", str().c_str());
-	    memset(_hash, 0, 16);
+	    memset(_hash, 0, sizeof(_hash));
 	    return;
 	}
     XLOG_UNREACHABLE();
@@ -1837,7 +1832,7 @@ PathAttributeList<A>::remove_attribute_by_type(PathAttType type)
 	if ((*i)->type() == type) {
 	    delete *i;
 	    erase(i);
-	    memset(_hash, 0, 16);
+	    memset(_hash, 0, sizeof(_hash));
 	    return;
 	}
     }
@@ -1854,7 +1849,7 @@ PathAttributeList<A>::remove_attribute_by_pointer(PathAttribute *p)
 	if ((*i) == p) {
 	    delete *i;
 	    erase(i);
-	    memset(_hash, 0, 16);
+	    memset(_hash, 0, sizeof(_hash));
 	    return;
 	}
     }
@@ -1878,7 +1873,7 @@ PathAttributeList<A>::process_unknown_attributes()
 		delete *tmp;
 		erase(tmp);
 	    }
-	    memset(_hash, 0, 16);
+	    memset(_hash, 0, sizeof(_hash));
 	}
     }
 }
@@ -1975,10 +1970,13 @@ void
 PathAttributeList<A>::assert_rehash() const
 {
 #ifdef PARANOID
-    const uint32_t *i = (const uint32_t *)_hash;
-    if (i[0] || i[1] || i[2] || i[3])
-	return;
-    XLOG_FATAL("Missing rehash - attempted to use modified PathAttributeList without first calling rehash()\n");
+    size_t i;
+    for (i = 0; i < sizeof(_hash); i++) {
+	if (_hash[i])
+	    return;
+    }
+    XLOG_FATAL("Missing rehash - attempted to use modified PathAttributeList "
+	       "without first calling rehash()");
 #endif
 }
 
