@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,26 +12,27 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer.cc,v 1.137 2006/05/01 09:17:10 pavlin Exp $"
+#ident "$XORP: xorp/bgp/peer.cc,v 1.144 2007/02/16 22:45:13 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 // #define SAVE_PACKETS
 #define CHECK_TIME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "bgp_module.h"
+#include "libxorp/xorp.h"
 
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/timer.hh"
-#include "xrl/interfaces/bgp_mib_traps_xif.hh"
 #include "libxorp/timespent.hh"
+
 #include "libcomm/comm_api.h"
 
+#include "libproto/packet.hh"
+
+#include "xrl/interfaces/bgp_mib_traps_xif.hh"
+
 #include "peer.hh"
 #include "bgp.hh"
 #include "profile_vars.hh"
@@ -99,8 +100,8 @@ BGPPeer::clear_last_error()
 
 /*
  * This call is dispatched by the reader after making sure that
- * we have a packet with at least a fixed_header and whose
- * length matches the one in the fixed_header.
+ * we have a packet with at least a BGP common header and whose
+ * length matches the one in that header.
  * Our job now is to decode the message and dispatch it to the
  * state machine.
  */
@@ -123,7 +124,8 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	break;
 
     case BGPPacket::ILLEGAL_MESSAGE_LENGTH:
-	notify_peer_of_error(MSGHEADERERR, BADMESSLEN, buf + MARKER_SIZE, 2);
+	notify_peer_of_error(MSGHEADERERR, BADMESSLEN,
+			     buf + BGPPacket::MARKER_SIZE, 2);
 // 	event_tranfatal();
 	TIMESPENT_CHECK();
 	return false;
@@ -224,9 +226,8 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
     */
     XLOG_ASSERT(0 != buf);
 
-    const fixed_header *header =
-	reinterpret_cast<const struct fixed_header *>(buf);
-
+    const uint8_t* marker = buf + BGPPacket::MARKER_OFFSET;
+    uint8_t type = extract_8(buf + BGPPacket::TYPE_OFFSET);
     try {
 
 	/*
@@ -234,11 +235,11 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	** anything of interest.
 	*/
 	if (0 != memcmp(const_cast<uint8_t *>(&BGPPacket::Marker[0]),
-			&header->marker[0], MARKER_SIZE)) {
+			marker, BGPPacket::MARKER_SIZE)) {
 	    xorp_throw(CorruptMessage,"Bad Marker", MSGHEADERERR, CONNNOTSYNC);
 	}
 	
-	switch (header->type) {
+	switch (type) {
 	case MESSAGETYPEOPEN: {
 	    debug_msg("OPEN Packet RECEIVED\n");
 	    OpenPacket pac(buf, length);
@@ -307,9 +308,9 @@ BGPPeer::get_message(BGPPacket::Status status, const uint8_t *buf,
 	    ** Send a notification to the peer. This is a bad message type.
 	    */
 	    XLOG_ERROR("%s Unknown packet type %d",
-		       this->str().c_str(), header->type);
+		       this->str().c_str(), type);
 	    notify_peer_of_error(MSGHEADERERR, BADMESSTYPE,
-				 const_cast<uint8_t *>(&header->type),1);
+				 buf + BGPPacket::TYPE_OFFSET, 1);
 // 	    event_tranfatal();
 	    TIMESPENT_CHECK();
 	    return false;
@@ -1101,6 +1102,7 @@ BGPPeer::event_recvupdate(const UpdatePacket& p) // EVENTRECUPDATEMESS
 	    set_state(STATESTOPPED);
 	    break;
 	}
+	restart_hold_timer();
 	if (!good_nexthop)
 	    return;
 	// ok the packet is correct.
@@ -1118,7 +1120,6 @@ BGPPeer::event_recvupdate(const UpdatePacket& p) // EVENTRECUPDATEMESS
 	    }
 	}
 
-	restart_hold_timer();
 	// process the packet...
 	debug_msg("Process the packet!!!\n");
 	XLOG_ASSERT(_handler);
@@ -1274,7 +1275,7 @@ BGPPeer::check_open_packet(const OpenPacket *p) throw(CorruptMessage)
 {
     if (p->Version() != BGPVERSION) {
 	static uint8_t data[2];
-	*reinterpret_cast<uint16_t *>(&data[0]) = htons(BGPVERSION);
+	embed_16(data, BGPVERSION);
 	xorp_throw(CorruptMessage,
 		   c_format("Unsupported BGPVERSION %d", p->Version()),
 		   OPENMSGERROR, UNSUPVERNUM, &data[0], sizeof(data));
@@ -2567,7 +2568,7 @@ AcceptSession::get_message_accept(BGPPacket::Status status,
 
     case BGPPacket::ILLEGAL_MESSAGE_LENGTH:
 	notify_peer_of_error_accept(MSGHEADERERR, BADMESSLEN,
-				    buf + MARKER_SIZE, 2);
+				    buf + BGPPacket::MARKER_SIZE, 2);
 // 	event_tranfatal_accept();
 	TIMESPENT_CHECK();
 	debug_msg("Returning false\n");
@@ -2586,20 +2587,19 @@ AcceptSession::get_message_accept(BGPPacket::Status status,
     */
     XLOG_ASSERT(0 != buf);
 
-    const fixed_header *header =
-	reinterpret_cast<const struct fixed_header *>(buf);
-
+    const uint8_t* marker = buf + BGPPacket::MARKER_OFFSET;
+    uint8_t type = extract_8(buf + BGPPacket::TYPE_OFFSET);
     try {
 	/*
 	** Check the Marker, total waste of time as it never contains
 	** anything of interest.
 	*/
 	if (0 != memcmp(const_cast<uint8_t *>(&BGPPacket::Marker[0]),
-			&header->marker[0], MARKER_SIZE)) {
+			marker, BGPPacket::MARKER_SIZE)) {
 	    xorp_throw(CorruptMessage,"Bad Marker", MSGHEADERERR, CONNNOTSYNC);
 	}
 	
-	switch (header->type) {
+	switch (type) {
 	case MESSAGETYPEOPEN: {
 	    debug_msg("OPEN Packet RECEIVED\n");
 	    OpenPacket pac(buf, length);
@@ -2667,10 +2667,9 @@ AcceptSession::get_message_accept(BGPPacket::Status status,
 	    ** Send a notification to the peer. This is a bad message type.
 	    */
 	    XLOG_ERROR("%s Unknown packet type %d",
-		       this->str().c_str(), header->type);
+		       this->str().c_str(), type);
 	    notify_peer_of_error_accept(MSGHEADERERR, BADMESSTYPE,
-					const_cast<uint8_t *>(&header->type),
-					1);
+					buf + BGPPacket::TYPE_OFFSET, 1);
 // 	    event_tranfatal_accept();
 	    TIMESPENT_CHECK();
 	    debug_msg("Returning false\n");
