@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,28 +13,26 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/plumbing.cc,v 1.94 2006/07/12 02:35:19 atanu Exp $"
+#ident "$XORP: xorp/bgp/plumbing.cc,v 1.100 2007/02/16 22:45:14 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 #define CHECK_TIME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "bgp_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/timer.hh"
-#include "route_table_reader.hh"
+#include "libxorp/timespent.hh"
 
+#include "route_table_reader.hh"
 #include "plumbing.hh"
 #include "bgp.hh"
 #include "profile_vars.hh"
 #include "dump_iterators.hh"
-#include "libxorp/timespent.hh"
+
 
 BGPPlumbing::BGPPlumbing(const Safi safi,
 			 RibIpcHandler* ribhandler,
@@ -321,11 +319,19 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 				      _master.main().eventloop());
     _decision_table->set_next_table(_policy_sourcematch_table);
 
+    CacheTable<A>* cache_decision = 
+	new CacheTable<A>(_ribname + "IpcChannelDecisionCache",
+			  _master.safi(),
+			  _policy_sourcematch_table,
+			  _master.rib_handler());
+    _policy_sourcematch_table->set_next_table(cache_decision);
+    _tables.insert(cache_decision);
+
     _aggregation_table =
 	new AggregationTable<A>(ribname + "AggregationTable",
 				_master,
-				_policy_sourcematch_table);
-    _policy_sourcematch_table->set_next_table(_aggregation_table);
+				cache_decision);
+    cache_decision->set_next_table(_aggregation_table);
 
     _fanout_table =
 	new FanoutTable<A>(ribname + "FanoutTable",
@@ -396,6 +402,13 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
     _tables.insert(filter_out);
 
     XLOG_ASSERT(_master.rib_handler());
+
+#if 0
+    //
+    // XXX: don't plug a policy table on the RIB output branch, because
+    // the routes send to the RIB shouldn't be modified by the export
+    // policy rules.
+    //
     PolicyTable<A>* policy_filter_out =
 	new PolicyTableExport<A>(ribname + "IpcChannelOutputPolicyFilter",
 			         _master.safi(),
@@ -404,6 +417,7 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 				 _master.rib_handler()->id().str());
     filter_out->set_next_table(policy_filter_out);
     _tables.insert(policy_filter_out);
+#endif // 0
 
     // Drop in an aggregation filter - beheave like an IBGP peering
     filter_out->add_aggregation_filter(true);
@@ -411,9 +425,9 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
     CacheTable<A> *cache_out =
 	new CacheTable<A>(ribname + "IpcChannelOutputCache",
 			  _master.safi(),
-			  policy_filter_out,
+			  filter_out,
 			  _master.rib_handler());
-    policy_filter_out->set_next_table(cache_out);
+    filter_out->set_next_table(cache_out);
     _tables.insert(cache_out);
 
     _ipc_rib_out_table =
@@ -733,7 +747,7 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
 			         _master.safi(),
 				 filter_out,
 				 _master.policy_filters(),
-				 peer_handler->id().str());
+				 peer_handler->get_peer_addr());
     filter_out->set_next_table(policy_filter_out);
    
     CacheTable<A>* cache_out = 
@@ -1335,6 +1349,30 @@ BGPPlumbingAF<A>::push_routes() {
     _fanout_table->peer_table_info(peer_list);
 
     _policy_sourcematch_table->push_routes(peer_list);
+
+    /*
+    ** It is possible that another peer was in the middle of going
+    ** down when this peering came up. So sweep through the peers and
+    ** look for the deletion tables. Deletion tables can be
+    ** nested. Treat them as if they have just sent a
+    ** peering_went_down.
+    */
+
+    typename map <PeerHandler*, RibInTable<A>* >::iterator iter2;
+    for (iter2 = _in_map.begin(); iter2 != _in_map.end(); iter2++) {
+	RibInTable<A> *rib_in = iter2->second;
+	debug_msg("<%s>\n", rib_in->next_table()->str().c_str());
+	DeletionTable<A> *deletion_table =
+	    dynamic_cast<DeletionTable<A> *>(rib_in->next_table());
+	while (0 != deletion_table) {
+	    debug_msg("Found a deletion table\n");
+	    _policy_sourcematch_table
+		->peering_is_down(iter2->first, deletion_table->genid());
+	    deletion_table = 
+		dynamic_cast<DeletionTable<A> *>(deletion_table->next_table());
+	}
+    }
+
 }
 
 template class BGPPlumbingAF<IPv4>;
