@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.73 2006/03/27 12:46:04 zec Exp $"
+#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.75 2007/03/06 23:44:20 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -753,7 +753,11 @@ XrlQueue<A>::route_command_done(const XrlError& error,
 	break;
 
     case BAD_ARGS:
+	XLOG_FATAL("callback: %s %s",  comment.c_str(), error.str().c_str());
+	break;
     case COMMAND_FAILED:
+	XLOG_ERROR("callback: %s %s",  comment.c_str(), error.str().c_str());
+	break;
     case INTERNAL_ERROR:
 	XLOG_FATAL("callback: %s %s",  comment.c_str(), error.str().c_str());
 	break;
