@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_fanout.cc,v 1.54 2006/03/16 00:03:33 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_fanout.cc,v 1.62 2007/02/16 22:45:17 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 //#define DEBUG_PRINT_FUNCTION_NAME
@@ -223,6 +223,13 @@ FanoutTable<A>::add_route(const InternalMessage<A> &rtmsg,
 
     const PeerHandler *origin_peer = rtmsg.origin_peer();
 
+    log("add_route rcvd, net: " + rtmsg.route()->net().str()
+	+ " peer: " + origin_peer->peername()
+	+ c_format(" filters: %p,%p,%p",
+		   rtmsg.route()->policyfilter(0).get(),
+		   rtmsg.route()->policyfilter(1).get(),
+		   rtmsg.route()->policyfilter(2).get()));
+
     typename NextTableMap<A>::iterator i = _next_tables.begin();
     list <PeerTableInfo<A>*> queued_peers;
     while (i != _next_tables.end()) {
@@ -289,6 +296,9 @@ FanoutTable<A>::replace_route(const InternalMessage<A> &old_rtmsg,
     const PeerHandler *origin_peer = old_rtmsg.origin_peer();
     XLOG_ASSERT(origin_peer == new_rtmsg.origin_peer());
 
+    log("replace_route rcvd, net: " + old_rtmsg.route()->net().str()
+	+ " peer: " + origin_peer->peername());
+
     list <PeerTableInfo<A>*> queued_peers;
     typename NextTableMap<A>::iterator i;
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
@@ -332,6 +342,14 @@ FanoutTable<A>::delete_route(const InternalMessage<A> &rtmsg,
 
     const PeerHandler *origin_peer = rtmsg.origin_peer();
 
+    log("delete_route rcvd, net: " + rtmsg.route()->net().str()
+	+ " peer: " + origin_peer->peername()
+	+ c_format(" filters: %p,%p,%p",
+		   rtmsg.route()->policyfilter(0).get(),
+		   rtmsg.route()->policyfilter(1).get(),
+		   rtmsg.route()->policyfilter(2).get()));
+
+
     list <PeerTableInfo<A>*> queued_peers;
     typename NextTableMap<A>::iterator i;
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
@@ -367,6 +385,9 @@ FanoutTable<A>::route_dump(const InternalMessage<A> &rtmsg,
     XLOG_ASSERT(caller == this->_parent);
     XLOG_ASSERT(rtmsg.route()->nexthop_resolved());
 
+    log("route_dump, net: " + rtmsg.route()->net().str()
+	+ " dump peer: " + dump_peer->peername());
+
     BGPRouteTable<A> *dump_child = 0;
     typename NextTableMap<A>::iterator i;
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
@@ -389,6 +410,7 @@ int
 FanoutTable<A>::push(BGPRouteTable<A> *caller) 
 {
     debug_msg("Push\n");
+    log("received push");
     XLOG_ASSERT(caller == this->_parent);
     list <PeerTableInfo<A>*> queued_peers;
     typename NextTableMap<A>::iterator i;
@@ -626,13 +648,16 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
     if (peer_info->has_queued_data() == false) {
 	debug_msg("no data queued\n");
 	peer_info->set_is_ready();
+	log("get next message (non available): " 
+	    + peer_info->peer_handler()->peername());
 	return false;
     }
 
+    log("get next message: " + peer_info->peer_handler()->peername());
+
     typename list<const RouteQueueEntry<A>*>::iterator queue_ptr;
     queue_ptr = peer_info->queue_position();
     bool discard_possible = false;
-    int skipped = 1;
 
     switch ((*queue_ptr)->op()) {
     case RTQUEUE_OP_ADD: {
@@ -642,6 +667,7 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 				 (*queue_ptr)->origin_peer(),
 				 (*queue_ptr)->genid());
 	if ((*queue_ptr)->push()) rtmsg.set_push();
+	log("sending add_route: " + (*queue_ptr)->route()->net().str());
 	next_table->add_route(rtmsg, (BGPRouteTable<A>*)this);
 	break;
     }
@@ -651,6 +677,7 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 				 (*queue_ptr)->origin_peer(),
 				 (*queue_ptr)->genid());
 	if ((*queue_ptr)->push()) rtmsg.set_push();
+	log("sending delete_route: " + (*queue_ptr)->route()->net().str());
 	next_table->delete_route(rtmsg, (BGPRouteTable<A>*)this);
 	break;
     }
@@ -667,9 +694,9 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 				     (*queue_ptr)->origin_peer(),
 				     (*queue_ptr)->genid());
 	if ((*queue_ptr)->push()) new_rtmsg.set_push();
+	log("sending replace_route: " + (*queue_ptr)->route()->net().str());
 	next_table->replace_route(old_rtmsg, new_rtmsg,
 				  (BGPRouteTable<A>*)this);
-	skipped = 2;
 	break;
     }
     case RTQUEUE_OP_REPLACE_NEW: {
@@ -682,6 +709,7 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
     }
     case RTQUEUE_OP_PUSH: {
 	debug_msg("OP_PUSH\n");
+	log("sending push");
 	next_table->push(this);
 	break;
     }
@@ -716,6 +744,15 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 	}
 	if ((*queue_ptr)->op() == RTQUEUE_OP_REPLACE_OLD)
 	    queue_ptr++;
+
+	// sanity check
+	if(queue_ptr == _output_queue.end()) {
+	    // this shouldn't ever happen, as a REPLACE_NEW should
+	    // follow a REPLACE_OLD
+	    crash_dump();
+	    XLOG_UNREACHABLE();
+	}
+
 	queue_ptr++;
     }
     if (queue_ptr == _output_queue.end()) {
@@ -740,17 +777,22 @@ FanoutTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 		// discard.
 		if (nti.second().queue_position() == _output_queue.begin())
 		    discard = false;
-		// if we're considering popping the first two entries,
-		// we need to check no-one needs the second one either
-		if (skipped == 2 &&
-		    nti.second().queue_position() == (_output_queue.begin())++ )
-		    discard = false;
 	    }
 	}
 	if (discard) {
+	    // check if the item to delete is a replace, in which case
+	    // we need to delete both replace items
+	    bool delete_two = false;
+	    if (_output_queue.front()->op() == RTQUEUE_OP_REPLACE_OLD)
+		delete_two = true;
+
 	    delete _output_queue.front();
 	    _output_queue.pop_front();
-	    if (skipped == 2) {
+
+	    if (delete_two) {
+		XLOG_ASSERT(_output_queue.front()->op() 
+			    == RTQUEUE_OP_REPLACE_NEW);
+		XLOG_ASSERT(!_output_queue.empty());
 		delete _output_queue.front();
 		_output_queue.pop_front();
 	    }
@@ -782,7 +824,6 @@ FanoutTable<A>::skip_entire_queue(BGPRouteTable<A> *next_table)
     bool more_queued_data = true;
     while (more_queued_data) {
 	bool discard_possible = false;
-	int skipped = 1;
 
 	switch ((*queue_ptr)->op())
 	    {
@@ -794,7 +835,6 @@ FanoutTable<A>::skip_entire_queue(BGPRouteTable<A> *next_table)
 		if (queue_ptr == _output_queue.begin())
 		    discard_possible = true;
 		queue_ptr++;
-		skipped = 2;
 		break;
 	    case RTQUEUE_OP_REPLACE_NEW:
 		XLOG_FATAL("illegal route queue state");
@@ -838,18 +878,21 @@ FanoutTable<A>::skip_entire_queue(BGPRouteTable<A> *next_table)
 		    // discard.
 		    if (nti.second().queue_position() == _output_queue.begin())
 			discard = false;
-		    // if we're considering popping the first two entries,
-		    // we need to check no-one needs the second one either
-		    if (skipped == 2 &&
-			(nti.second().queue_position() 
-			 == (_output_queue.begin())++ ))
-			discard = false;
 		}
 	    }
 	    if (discard) {
+		// check if the item to delete is a replace, in which case
+		// we need to delete both replace items
+		bool delete_two = false;
+		if (_output_queue.front()->op() == RTQUEUE_OP_REPLACE_OLD)
+		    delete_two = true;
+
 		delete _output_queue.front();
 		_output_queue.pop_front();
-		if (skipped == 2) {
+		if (delete_two) {
+		    XLOG_ASSERT(_output_queue.front()->op() 
+				== RTQUEUE_OP_REPLACE_NEW);
+		    XLOG_ASSERT(!_output_queue.empty());
 		    delete _output_queue.front();
 		    _output_queue.pop_front();
 		}
@@ -869,6 +912,7 @@ FanoutTable<A>::peering_went_down(const PeerHandler *peer, uint32_t genid,
 				  BGPRouteTable<A> *caller) 
 {
     XLOG_ASSERT(this->_parent == caller);
+    log("Peering went down: " + peer->peername());
     typename NextTableMap<A>::iterator i;
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
 	i.first()->peering_went_down(peer, genid, (BGPRouteTable<A>*)this);
@@ -882,6 +926,7 @@ FanoutTable<A>::peering_down_complete(const PeerHandler *peer,
 				      BGPRouteTable<A> *caller) 
 {
     XLOG_ASSERT(this->_parent == caller);
+    log("Peering down complete: " + peer->peername());
 
     debug_msg("peer: %s genid: %u caller: %s\n",
 	      peer->peername().c_str(), XORP_UINT_CAST(genid),
@@ -906,6 +951,7 @@ FanoutTable<A>::peering_came_up(const PeerHandler *peer, uint32_t genid,
 				BGPRouteTable<A> *caller) 
 {
     XLOG_ASSERT(this->_parent == caller);
+    log("Peering came up: " + peer->peername());
     typename NextTableMap<A>::iterator i;
     for (i = _next_tables.begin();  i != _next_tables.end();  i++) {
 	i.first()->peering_came_up(peer, genid, (BGPRouteTable<A>*)this);
@@ -927,6 +973,30 @@ FanoutTable<A>::print_queue()
 #endif
 }
 
+template<class A>
+string
+FanoutTable<A>::dump_state() const {
+    string s;
+    s  = "=================================================================\n";
+    s += "FanoutTable\n";
+    s += "=================================================================\n";
+    s += "Rate control queue:\n";
+    typename list <const RouteQueueEntry<A>*>::const_iterator i;
+    int ctr = 0;
+    for (i = _output_queue.begin(); i != _output_queue.end(); i++) {
+	ctr++;
+	s += c_format("%-5d %s\n", ctr, (*i)->str().c_str());
+	s += c_format("Parent now: %p\n", (*i)->route()->parent_route());
+	s += c_format("Filters now: %p,%p,%p\n",
+		      (*i)->route()->policyfilter(0).get(),
+		      (*i)->route()->policyfilter(1).get(),
+		      (*i)->route()->policyfilter(2).get());
+		      
+    }
+    s += CrashDumper::dump_state();
+    return s;
+} 
+
 template class PeerTableInfo<IPv4>;
 template class PeerTableInfo<IPv6>;
 template class FanoutTable<IPv4>;
