@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_policy.cc,v 1.20 2006/03/16 00:03:34 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_policy.cc,v 1.23 2007/02/16 22:45:18 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -33,12 +33,15 @@ PolicyTable<A>::PolicyTable(const string& tablename, const Safi& safi,
 			    BGPRouteTable<A>* parent,
 			    PolicyFilters& pfs, 
 			    const filter::Filter& type)
-    : BGPRouteTable<A>(tablename, safi), _filter_type(type),
+    : BGPRouteTable<A>(tablename, safi),
+      _filter_type(type),
+      _varrw(NULL),
       _policy_filters(pfs)
 {
     this->_parent = parent;
-    init_varrw();
-    XLOG_ASSERT(_varrw);
+    // XXX: For clarity, explicitly call the local virtual init_varrw()
+    PolicyTable<A>::init_varrw();
+    XLOG_ASSERT(_varrw != NULL);
 	
     // Performance optimization - suppress generation of trace strings
     _varrw->suppress_trace();
@@ -471,6 +474,9 @@ template <class A>
 void
 PolicyTable<A>::init_varrw()
 {
+    if (_varrw != NULL)
+	delete _varrw;
+
     _varrw = new BGPVarRW<A>(filter::filter2str(_filter_type));
 }
 
