@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_policy_im.cc,v 1.10 2006/03/16 00:03:34 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_policy_im.cc,v 1.16 2007/02/16 22:45:18 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -55,6 +55,40 @@ PolicyTableImport<A>::route_dump(const InternalMessage<A>& rtmsg,
     // "old" filter...
     const InternalMessage<A>* fmsg = do_filtering(rtmsg, false);
     bool was_filtered = (fmsg == NULL);
+    if (fmsg != NULL) {
+	if (fmsg->route() == rtmsg.route()) {
+	    // fmsg contains the original route, so we need to clone it,
+	    // or we'll get bad interactions when we set the policyfilter
+	    // on the new route below because the policyfilter is
+	    // propagated back to the parent.
+	    SubnetRoute<A>* copy_old_rt = new SubnetRoute<A>(*rtmsg.route());
+
+	    // clear the parent route, so noone downstream can mess up the
+	    // metadata on the route still stored in RibIn.
+	    copy_old_rt->set_parent_route(NULL);
+
+	    InternalMessage<A>* copy_fmsg
+		= new InternalMessage<A>(copy_old_rt, rtmsg.origin_peer(),
+					 rtmsg.genid());
+
+	    if (rtmsg.changed())
+		copy_fmsg->set_changed();
+
+	    if (rtmsg.push())
+		copy_fmsg->set_push();
+
+	    if (rtmsg.from_previous_peering())
+		copy_fmsg->set_from_previous_peering();
+
+	    fmsg = copy_fmsg;
+	    XLOG_ASSERT(fmsg->route() != rtmsg.route());
+	} else {
+	    // the route is already a copy, but we still need to clear the
+	    // parent route to prevent anyone downstream modifying the
+	    // metadata on the route still stored in RibIn
+	    const_cast<SubnetRoute<A>*>(fmsg->route())->set_parent_route(NULL);
+	}
+    }
 
     // we want current filter
     rtmsg.route()->set_policyfilter(0, RefPf());
@@ -62,13 +96,6 @@ PolicyTableImport<A>::route_dump(const InternalMessage<A>& rtmsg,
     // filter new message
     const InternalMessage<A>* new_msg = do_filtering(rtmsg, false);
     
-    // check if route was modified by our filters.  If it was, and it was
-    // modified by the static filters, then we need to free the new subnet route
-    // allocated by static filters...
-    if (rtmsg.changed() && fmsg != &rtmsg && new_msg != &rtmsg) {
-	rtmsg.route()->unref();
-    }
-
     bool accepted = (new_msg != NULL);
 
     debug_msg("[BGP] Policy route dump accepted: %d\n", accepted);
@@ -84,10 +111,12 @@ PolicyTableImport<A>::route_dump(const InternalMessage<A>& rtmsg,
 	    debug_msg("[BGP] Policy add_route [accepted, was filtered]");
 	    res = next->add_route(*new_msg, this);
 	} else {
+	    XLOG_ASSERT(fmsg != NULL);
 	    debug_msg("[BGP] Policy replace_route old=(%s) new=(%s)\n",
 		      fmsg->str().c_str(), new_msg->str().c_str());
 
-
+	    XLOG_ASSERT(fmsg->route() != new_msg->route());
+#if 0
 	    // we will delete and add the same subnetroute!
 	    // make new internal message to preserve route.
 	    if (rtmsg.changed() && fmsg->route() == new_msg->route()) {
@@ -114,7 +143,7 @@ PolicyTableImport<A>::route_dump(const InternalMessage<A>& rtmsg,
 	    
 		XLOG_ASSERT(fmsg != new_msg);
 	    }
-
+#endif
 
 	    // XXX don't check return of deleteroute!
 	    res = next->delete_route(*fmsg, this);
@@ -139,16 +168,37 @@ PolicyTableImport<A>::route_dump(const InternalMessage<A>& rtmsg,
 	// not accepted
 	if (was_filtered) {
 	} else {
+	    XLOG_ASSERT(fmsg != NULL);
+	    //
+	    // XXX: A hack propagating the "not-winner route" flag back to
+	    // the original route.
+	    // The reason we need this hack here is because of an earlier
+	    // hack where the fmsg parent was modified to be NULL.
+	    //
+	    rtmsg.route()->set_is_not_winner();
 	    next->delete_route(*fmsg, this);
 	}
 	res = ADD_FILTERED;
     }
 
-    if (fmsg != &rtmsg)
-	delete fmsg;
+    //
+    // Check if route was modified by our filters.  If it was, and it was
+    // modified by the static filters, then we need to free the new subnet
+    // route allocated by static filters.
+    //
+    if (rtmsg.changed() && fmsg != &rtmsg && new_msg != &rtmsg) {
+	rtmsg.route()->unref();
+    }
 
-    if (new_msg != &rtmsg)
-	delete new_msg;
+    if (fmsg != &rtmsg) {
+	if (fmsg != NULL)
+	    delete fmsg;
+    }
+
+    if (new_msg != &rtmsg) {
+	if (new_msg != NULL)
+	    delete new_msg;
+    }
 
     return res;
 }
