@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_policy_sm.cc,v 1.6 2006/03/16 00:03:34 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_policy_sm.cc,v 1.11 2007/02/16 22:45:18 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -56,7 +56,9 @@ PolicyTableSourceMatch<A>::push_routes(list<const PeerTableInfo<A>*>& peer_list)
 
     debug_msg("[BGP] Push routes\n");
 
-    do_background_dump();
+    _dump_task = eventloop().new_task(
+	callback(this, &PolicyTableSourceMatch<A>::do_background_dump),
+	XorpTask::PRIORITY_BACKGROUND, XorpTask::WEIGHT_DEFAULT);
 }
 
 template <class A>
@@ -90,7 +92,7 @@ PolicyTableSourceMatch<A>::end_route_dump()
     delete _dump_iter;
     _dump_iter = NULL;
     _pushing_routes = false;
-    _dump_timer.unschedule();
+    _dump_task.unschedule();
 }
 
 template <class A>
@@ -101,23 +103,29 @@ PolicyTableSourceMatch<A>::eventloop()
 }
 
 template <class A>
-void
+bool
 PolicyTableSourceMatch<A>::do_background_dump()
 {
     debug_msg("[BGP] doing a background dump step\n");
 
     // we are done...
     if (!_pushing_routes)
-	return;
+	return false;
 
     // do a dump
     do_next_route_dump();
 
     // continue in background...
-    _dump_timer = eventloop().
-		    new_oneoff_after_ms(0,
-		     callback(this,
-			      &PolicyTableSourceMatch<A>::do_background_dump));
+    return true;
+}
+
+template<class A>
+void
+PolicyTableSourceMatch<A>::peering_is_down(const PeerHandler *peer, 
+					   uint32_t genid)
+{
+    if (pushing_routes())
+        _dump_iter->peering_is_down(peer, genid);
 }
 
 template<class A>
