@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/socket.cc,v 1.40 2006/04/04 09:41:04 bms Exp $"
+#ident "$XORP: xorp/bgp/socket.cc,v 1.46 2007/02/16 22:45:19 pavlin Exp $"
 
 // #define DEBUG_LOGGING 
 // #define DEBUG_PRINT_FUNCTION_NAME 
@@ -95,10 +95,10 @@ Socket::create_socket(const struct sockaddr *sin, int is_blocking)
 
 void
 Socket::init_sockaddr(string addr, uint16_t local_port,
-		      struct sockaddr *sin, size_t& len)
+		      struct sockaddr_storage& ss, size_t& len)
 {
-    debug_msg("addr %s port %u len = %u\n", addr.c_str(), 
-	XORP_UINT_CAST(local_port), XORP_UINT_CAST(len));
+    debug_msg("addr %s port %u len = %u\n", addr.c_str(),
+	      XORP_UINT_CAST(local_port), XORP_UINT_CAST(len));
 
     string port = c_format("%d", local_port);
 
@@ -120,9 +120,9 @@ Socket::init_sockaddr(string addr, uint16_t local_port,
 		   error_string);
     }
 
-    XLOG_ASSERT(res0->ai_addrlen <= len);
-    memcpy(sin,res0->ai_addr, res0->ai_addrlen);
-
+    XLOG_ASSERT(res0->ai_addrlen <= sizeof(ss));
+    memset(&ss, 0, sizeof(ss));
+    memcpy(&ss, res0->ai_addr, res0->ai_addrlen);
     len = res0->ai_addrlen;
 
     freeaddrinfo(res0);
@@ -209,9 +209,9 @@ void
 SocketClient::connected(XorpFd s)
 {
 #ifdef	DEBUG_PEERNAME
-    char socket_buffer[SOCKET_BUFFER_SIZE];
-    struct sockaddr *sin = reinterpret_cast<struct sockaddr *>(socket_buffer);
-    socklen_t len = sizeof(socket_buffer);
+    sockaddr_storage ss;
+    struct sockaddr *sin = reinterpret_cast<struct sockaddr *>(ss);
+    socklen_t len = sizeof(ss);
     if (-1 == getpeername(s, sin, &len))
 	XLOG_FATAL("getpeername failed: %s", strerror(errno));
     char hostname[1024];
@@ -256,7 +256,7 @@ SocketClient::async_read_start(size_t cnt, size_t offset)
  * Handler for reading incoming data on a BGP connection.
  *
  * When a packet first comes in, we read the default amount of data,
- * which is MINPACKETSIZE (i.e. the minimum size of a BGP message).
+ * which is BGPPacket::MINPACKETSIZE (i.e. the minimum size of a BGP message).
  * This callback is then invoked a first time, and it can check the
  * actual message length. If more bytes are needed, we call again
  * async_read_start with the desired length (instructing it to skip
@@ -279,11 +279,10 @@ SocketClient::async_read_message(AsyncFileWriter::Event ev,
     case AsyncFileReader::DATA:
 	XLOG_ASSERT(offset <= buf_bytes);
 	if (offset == buf_bytes) {		// message complete so far
-	    const fixed_header *header =
-		reinterpret_cast<const struct fixed_header *>(buf);
-	    size_t fh_length = ntohs(header->length);
+	    size_t fh_length = extract_16(buf + BGPPacket::LENGTH_OFFSET);
 
-	    if (fh_length < MINPACKETSIZE || fh_length > sizeof(_read_buf)) {
+	    if (fh_length < BGPPacket::MINPACKETSIZE
+		|| fh_length > sizeof(_read_buf)) {
 		XLOG_ERROR("Illegal length value %u",
 			   XORP_UINT_CAST(fh_length));
 		if (!_callback->dispatch(BGPPacket::ILLEGAL_MESSAGE_LENGTH,
@@ -547,7 +546,10 @@ SocketClient::async_add(XorpFd sock)
 	XLOG_FATAL("Failed to go non-blocking");
 
     XLOG_ASSERT(0 == _async_writer);
-    _async_writer = new AsyncFileWriter(eventloop(), sock);
+    // we use high priority for the file writer so that we don't
+    // generally receive data in faster than we can send it back out
+    _async_writer = new AsyncFileWriter(eventloop(), sock,
+					XorpTask::PRIORITY_HIGH);
 
     XLOG_ASSERT(0 == _async_reader);
     _async_reader = new AsyncFileReader(eventloop(), sock);
