@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_decision.cc,v 1.37 2006/03/16 00:03:36 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/test_decision.cc,v 1.40 2007/02/16 22:45:20 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -26,6 +22,10 @@
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #include "libcomm/comm_api.h"
 
 #include "bgp.hh"
@@ -37,9 +37,6 @@
 #include "local_data.hh"
 #include "dummy_next_hop_resolver.hh"
 
-#ifndef HOST_OS_WINDOWS
-#include <pwd.h>
-#endif
 
 bool
 test_decision(TestInfo& /*info*/)
@@ -2140,7 +2137,7 @@ test_decision(TestInfo& /*info*/)
 				" BECOMES UNRESOLVABLE").c_str());
     next_hop_resolver.unset_nexthop_metric(nexthop3);
     decision_table->igp_nexthop_changed(nexthop3);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2196,7 +2193,7 @@ test_decision(TestInfo& /*info*/)
 				" BECOMES UNRESOLVABLE").c_str());
     next_hop_resolver.unset_nexthop_metric(nexthop2);
     decision_table->igp_nexthop_changed(nexthop2);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2254,7 +2251,7 @@ test_decision(TestInfo& /*info*/)
 				" BECOMES RESOLVABLE").c_str());
     next_hop_resolver.set_nexthop_metric(nexthop3, 27);
     decision_table->igp_nexthop_changed(nexthop3);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2324,7 +2321,7 @@ test_decision(TestInfo& /*info*/)
 				" BECOMES RESOLVABLE").c_str());
     next_hop_resolver.set_nexthop_metric(nexthop2, 27);
     decision_table->igp_nexthop_changed(nexthop2);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2395,7 +2392,7 @@ test_decision(TestInfo& /*info*/)
 				" BECOMES RESOLVABLE").c_str());
     next_hop_resolver.set_nexthop_metric(nexthop1, 27);
     decision_table->igp_nexthop_changed(nexthop1);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2452,7 +2449,7 @@ test_decision(TestInfo& /*info*/)
 				" BECOMES UNRESOLVABLE").c_str());
     next_hop_resolver.unset_nexthop_metric(nexthop1);
     decision_table->igp_nexthop_changed(nexthop1);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
