@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_deletion.cc,v 1.28 2006/03/16 00:03:36 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/test_deletion.cc,v 1.31 2007/02/16 22:45:20 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -26,6 +22,10 @@
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #include "bgp.hh"
 #include "route_table_base.hh"
 #include "route_table_ribin.hh"
@@ -34,9 +34,6 @@
 #include "local_data.hh"
 #include "dump_iterators.hh"
 
-#ifndef HOST_OS_WINDOWS
-#include <pwd.h>
-#endif
 
 bool
 test_deletion(TestInfo& /*info*/)
@@ -143,7 +140,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -190,7 +187,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -237,7 +234,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -292,7 +289,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -315,7 +312,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -325,7 +322,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
     debug_table->write_separator();
@@ -397,7 +394,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -411,7 +408,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -487,7 +484,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -501,7 +498,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -577,7 +574,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -591,7 +588,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -651,7 +648,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -665,7 +662,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -715,7 +712,7 @@ test_deletion(TestInfo& /*info*/)
 
     debug_table->write_separator();
     debug_table->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
