@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_dump.cc,v 1.52 2006/03/16 00:03:36 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/test_dump.cc,v 1.55 2007/02/16 22:45:20 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -26,6 +22,10 @@
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #include "bgp.hh"
 #include "route_table_base.hh"
 #include "route_table_filter.hh"
@@ -35,9 +35,6 @@
 #include "local_data.hh"
 #include "dummy_next_hop_resolver.hh"
 
-#ifndef HOST_OS_WINDOWS
-#include <pwd.h>
-#endif
 
 bool
 test_dump_create(TestInfo& /*info*/)
@@ -317,11 +314,11 @@ test_dump(TestInfo& /*info*/)
 
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
     while (debug_table3->parent()->get_next_message(debug_table3)) {}
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -408,11 +405,11 @@ test_dump(TestInfo& /*info*/)
 
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
     while (debug_table3->parent()->get_next_message(debug_table3)) {}
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -525,7 +522,7 @@ test_dump(TestInfo& /*info*/)
     ribin_table2->ribin_peering_went_down();
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -557,7 +554,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table2->genid());
     debug_table2->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table2, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
     debug_table1->write_separator();
@@ -617,7 +614,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -631,7 +628,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT DEL 1.0.2.0/24 RECEIVED BY PEER 1");
     debug_table1->write_comment("EXPECT DEL 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT DEL 1.0.3.0/24 RECEIVED BY PEER 1");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -655,7 +652,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table2->genid());
     debug_table2->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table2, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -712,7 +709,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT DEL 1.0.1.0/24 RECEIVED BY PEER 2");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -736,7 +733,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table1->genid());
     debug_table1->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table1, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -797,7 +794,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -810,7 +807,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT DEL 1.0.2.0/24 RECEIVED BY PEER 1");
     debug_table1->write_comment("EXPECT DEL 1.0.3.0/24 RECEIVED BY PEER 1");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -834,7 +831,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table2->genid());
     debug_table2->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table2, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -902,7 +899,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT DEL 1.0.2.0/24 RECEIVED BY PEER 1");
     debug_table1->write_comment("EXPECT DEL 1.0.3.0/24 RECEIVED BY PEER 1");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -926,7 +923,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table2->genid());
     debug_table2->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table2, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -996,7 +993,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT NO CHANGE");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1008,7 +1005,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table2->genid());
     debug_table2->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table2, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1020,7 +1017,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table1->genid());
     debug_table1->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table1, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1093,7 +1090,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1124,7 +1121,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.4.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.5.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1274,7 +1271,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1305,7 +1302,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT REPLACE 1.0.3.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.4.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1416,7 +1413,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT DEL 1.0.1.0/24 RECEIVED BY PEER 2");
     debug_table1->write_comment("EXPECT DEL 1.0.2.0/24 RECEIVED BY PEER 2");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1442,7 +1439,7 @@ test_dump(TestInfo& /*info*/)
 				 ribin_table1->genid());
     debug_table1->set_parent(fanout_table);
     fanout_table->dump_entire_table(debug_table1, SAFI_UNICAST, "ribname");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1505,12 +1502,12 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
     debug_table1->write_separator();
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1645,11 +1642,11 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.4.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
     debug_table1->write_separator();
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1725,7 +1722,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT (MOVES TO DUMPING NEXT PEER)");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1746,7 +1743,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1823,7 +1820,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_separator();
     debug_table1->write_comment("LET EVENT QUEUE DRAIN");
     debug_table1->write_comment("NO ROUTES IN 1st PEER (MOVES TO DUMPING NEXT PEER)");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1844,7 +1841,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.3.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -1936,7 +1933,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2035,7 +2032,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT DEL 1.0.1.0/24 NOT RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2134,7 +2131,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT DEL 1.0.3.0/24 NOT RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2258,7 +2255,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.5.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2347,7 +2344,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_separator();
     debug_table1->write_comment("STEP EVENT LOOP ONCE TO DUMP ONE ROUTE");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2384,7 +2381,7 @@ test_dump(TestInfo& /*info*/)
     debug_table1->write_comment("EXPECT ADD 1.0.2.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("EXPECT ADD 1.0.5.0/24 RECEIVED BY PEER 3");
 
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -2457,7 +2454,7 @@ test_dump(TestInfo& /*info*/)
     fanout_table->dump_entire_table(debug_table3, SAFI_UNICAST, "ribname");
     debug_table1->write_comment("EXPECT ADD 1.0.1.0/24 RECEIVED BY PEER 3");
     debug_table1->write_comment("**ONLY ONE COPY**");
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
