@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_fanout.cc,v 1.32 2006/03/16 00:03:36 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/test_fanout.cc,v 1.34 2007/02/16 22:45:20 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -26,6 +22,10 @@
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #include "bgp.hh"
 #include "route_table_base.hh"
 #include "route_table_filter.hh"
@@ -35,9 +35,6 @@
 #include "local_data.hh"
 #include "dump_iterators.hh"
 
-#ifndef HOST_OS_WINDOWS
-#include <pwd.h>
-#endif
 
 bool
 test_fanout(TestInfo& /*info*/)
