@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,24 +12,23 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_main.cc,v 1.18 2006/03/29 22:41:23 atanu Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <stdio.h>
+#ident "$XORP: xorp/bgp/test_main.cc,v 1.23 2007/02/16 22:45:20 pavlin Exp $"
 
 #include "bgp_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 
 #include "test_next_hop_resolver.hh"
 
+
 bool test_ribin(TestInfo& info);
 bool test_ribin_dump(TestInfo& info);
 bool test_deletion(TestInfo& info);
 bool test_filter(TestInfo& info);
+bool test_policy(TestInfo& info);
+bool test_policy_export(TestInfo& info);
+bool test_policy_dump(TestInfo& info);
 bool test_cache(TestInfo& info);
 bool test_nhlookup(TestInfo& info);
 bool test_decision(TestInfo& info);
@@ -143,6 +142,9 @@ main(int argc, char** argv)
 	    {"RibInDump", callback(test_ribin_dump)},
 	    {"Deletion", callback(test_deletion)},
 	    {"Filter", callback(test_filter)},
+	    {"PolicyExport", callback(test_policy_export)},
+	    {"Policy", callback(test_policy)},
+	    {"PolicyDump", callback(test_policy_dump)},
 	    {"Cache", callback(test_cache)},
 	    {"NhLookup", callback(test_nhlookup)},
 	    {"Decision", callback(test_decision)},
