@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_ribin.cc,v 1.33 2006/03/16 00:03:37 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/test_ribin.cc,v 1.37 2007/02/16 22:45:22 pavlin Exp $"
 
 #include "bgp_module.h"
 
@@ -26,6 +22,10 @@
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+
 #include "bgp.hh"
 #include "route_table_base.hh"
 #include "route_table_ribin.hh"
@@ -34,9 +34,6 @@
 #include "local_data.hh"
 #include "dump_iterators.hh"
 
-#ifndef HOST_OS_WINDOWS
-#include <pwd.h>
-#endif
 
 bool
 validate_reference_file(string reference_file, string output_file,
@@ -127,7 +124,7 @@ test_ribin_dump(TestInfo& /*info*/)
     sr3->unref();
 
     while (ribin->dump_next_route(*dump_iter)) {
-	while (bgpmain.eventloop().timers_pending()) {
+	while (bgpmain.eventloop().events_pending()) {
 	    bgpmain.eventloop().run();
 	}
     }
@@ -381,7 +378,7 @@ test_ribin(TestInfo& /*info*/)
     debug_table->write_comment("NOW DROP THE PEERING");
 
     ribin->ribin_peering_went_down();
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -437,7 +434,7 @@ test_ribin(TestInfo& /*info*/)
     DumpIterator<IPv4>* dump_iter 
 	= new DumpIterator<IPv4>(&handler2, peers_to_dump);
     while (ribin->dump_next_route(*dump_iter)) {
-	while (bgpmain.eventloop().timers_pending()) {
+	while (bgpmain.eventloop().events_pending()) {
 	    bgpmain.eventloop().run();
 	}
     }
@@ -506,7 +503,7 @@ test_ribin(TestInfo& /*info*/)
     debug_table->write_comment("NEXTHOP 2.0.0.2 CHANGES");
     //this should trigger a replace
     ribin->igp_nexthop_changed(nexthop2);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -514,7 +511,7 @@ test_ribin(TestInfo& /*info*/)
     debug_table->write_comment("NEXTHOP 2.0.0.1 CHANGES");
     //this should trigger a replace
     ribin->igp_nexthop_changed(nexthop1);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -523,7 +520,7 @@ test_ribin(TestInfo& /*info*/)
     debug_table->write_comment("NEXTHOP 1.0.0.1 CHANGES");
     //this should have no effect
     ribin->igp_nexthop_changed(nexthop3);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -532,7 +529,7 @@ test_ribin(TestInfo& /*info*/)
     //this should have no effect
     debug_table->write_comment("NEXTHOP 3.0.0.1 CHANGES");
     ribin->igp_nexthop_changed(nexthop4);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
@@ -577,7 +574,7 @@ test_ribin(TestInfo& /*info*/)
     debug_table->write_comment("NEXTHOP 2.0.0.1 CHANGES");
     //this should trigger three a replaces
     ribin->igp_nexthop_changed(nexthop1);
-    while (bgpmain.eventloop().timers_pending()) {
+    while (bgpmain.eventloop().events_pending()) {
 	bgpmain.eventloop().run();
     }
 
