@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_subnet_route.cc,v 1.7 2006/03/16 00:03:37 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/bgp/test_subnet_route.cc,v 1.9 2007/02/16 22:45:22 pavlin Exp $"
 
 #include "bgp_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/test_main.hh"
 #include "libxorp/exceptions.hh"
 #include "libxorp/ipv4.hh"
@@ -33,6 +30,7 @@
 #include "path_attribute.hh"
 #include "subnet_route.hh"
 
+
 template<>
 inline void
 RefTrieNode<IPv4, const SubnetRoute<IPv4> >
