@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/tools/print_routes_main.cc,v 1.10 2006/07/12 00:22:00 atanu Exp $"
+#ident "$XORP: xorp/bgp/tools/print_routes_main.cc,v 1.11 2007/02/16 22:45:27 pavlin Exp $"
 
 #include "print_routes.hh"
 #include "bgp/aspath.hh"
