@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,25 +13,23 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/xrl_target.cc,v 1.62 2006/07/12 02:35:19 atanu Exp $"
+#ident "$XORP: xorp/bgp/xrl_target.cc,v 1.64 2007/02/16 22:45:23 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 #define PROFILE_UTILS_REQUIRED
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "bgp_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/status_codes.h"
-
 #include "libxorp/eventloop.hh"
+
 #include "libxipc/xrl_std_router.hh"
 #include "libxipc/xrl_error.hh"
+
 #include "xrl/interfaces/profile_client_xif.hh"
 
 #include "bgp.hh"
@@ -39,6 +37,7 @@
 #include "xrl_target.hh"
 #include "profile_vars.hh"
 
+
 XrlBgpTarget::XrlBgpTarget(XrlRouter *r, BGPMain& bgp)
 	: XrlBgpTargetBase(r),
 	  _bgp(bgp),
