@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_client.cc,v 1.56 2006/05/24 06:15:44 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_client.cc,v 1.58 2007/02/16 22:45:28 pavlin Exp $"
 
 
 //
@@ -1169,7 +1169,7 @@ CliClient::post_process_command()
     // Process the pending input data (if any)
     //
     if (! _pending_input_data.empty()) {
-	process_input_data();
+	schedule_process_input_data();
     }
 }
 
@@ -1215,14 +1215,12 @@ CliClient::flush_process_command_output()
 // XXX: returning %XORP_ERROR is also an indication that the connection
 // has been closed.
 int
-CliClient::process_char(const string& line, uint8_t val, bool& stop_processing,
-			bool& ignore_current_character)
+CliClient::process_char(const string& line, uint8_t val, bool& stop_processing)
 {
     int gl_buff_curpos = gl_get_buff_curpos(gl());
     int ret_value = XORP_OK;
     
     stop_processing = false;
-    ignore_current_character = false;
 
     if ((val == '\n') || (val == '\r')) {
 	// New command
@@ -1237,7 +1235,6 @@ CliClient::process_char(const string& line, uint8_t val, bool& stop_processing,
 	//
 	if (is_waiting_for_data()) {
 	    stop_processing = true;
-	    ignore_current_character = true;
 	}
 
 	return (XORP_OK);
