@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.52 2006/07/24 21:25:42 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.56 2007/02/16 22:45:29 pavlin Exp $"
 
 
 //
@@ -42,6 +42,7 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 #include "libxorp/c_format.hh"
+#include "libxorp/time_slice.hh"
 #include "libxorp/token.hh"
 
 #include "libcomm/comm_api.h"
@@ -365,8 +366,10 @@ CliNode::delete_connection(CliClient *cli_client, string& error_msg)
 	// XXX: delete the client only if this was a network connection
 	_client_list.erase(iter);
 	delete cli_client;
+    } else {
+	eventloop().remove_ioevent_cb(cli_client->input_fd(), IOT_READ);
     }
-    
+
     return (XORP_OK);
 }
 
@@ -385,12 +388,12 @@ CliClient::start_connection(string& error_msg)
     // Setup the telnet options
     //
     if (is_telnet()) {
-	char will_echo_cmd[] = { IAC, WILL, TELOPT_ECHO, '\0' };
-	char will_sga_cmd[]  = { IAC, WILL, TELOPT_SGA, '\0'  };
-	char dont_linemode_cmd[] = { IAC, DONT, TELOPT_LINEMODE, '\0' };
-	char do_window_size_cmd[] = { IAC, DO, TELOPT_NAWS, '\0' };
-	char do_transmit_binary_cmd[] = { IAC, DO, TELOPT_BINARY, '\0' };
-	char will_transmit_binary_cmd[] = { IAC, WILL, TELOPT_BINARY, '\0' };
+	uint8_t will_echo_cmd[] = { IAC, WILL, TELOPT_ECHO, '\0' };
+	uint8_t will_sga_cmd[]  = { IAC, WILL, TELOPT_SGA, '\0'  };
+	uint8_t dont_linemode_cmd[] = { IAC, DONT, TELOPT_LINEMODE, '\0' };
+	uint8_t do_window_size_cmd[] = { IAC, DO, TELOPT_NAWS, '\0' };
+	uint8_t do_transmit_binary_cmd[] = { IAC, DO, TELOPT_BINARY, '\0' };
+	uint8_t will_transmit_binary_cmd[] = { IAC, WILL, TELOPT_BINARY, '\0' };
 
 	send(input_fd(), will_echo_cmd, sizeof(will_echo_cmd), 0);
 	send(input_fd(), will_sga_cmd, sizeof(will_sga_cmd), 0);
@@ -514,9 +517,9 @@ CliClient::start_connection(string& error_msg)
 
 	; // do nothing
 #else
-	term_name = getenv("TERM");
-	if (term_name.empty())
-	    term_name = DEFAULT_TERM_TYPE;	// Set to default
+	char *term = getenv("TERM");
+	if ((term != NULL) && (! string(term).empty()))
+	    term_name = string(term);
 #endif
     }
 
@@ -770,12 +773,12 @@ CliClient::process_input_data()
     //
     _pending_input_data.clear();
 
+    TimeSlice time_slice(1000000, 1); // 1s, test every iteration
+
     // Process the input data
     vector<uint8_t>::iterator iter;
     for (iter = input_data.begin(); iter != input_data.end(); ++iter) {
 	uint8_t val = *iter;
-	bool stop_processing_tmp = false;
-	bool save_input = false;
 	bool ignore_current_character = false;
 	
 	if (is_telnet()) {
@@ -797,12 +800,6 @@ CliClient::process_input_data()
 		continue;
 	    }
 	}
-	
-	preprocess_char(val, stop_processing_tmp, ignore_current_character);
-	if (stop_processing_tmp && (! stop_processing)) {
-	    stop_processing = true;
-	    save_input = true;
-	}
 
 	if (val == CHAR_TO_CTRL('c')) {
 	    //
@@ -813,6 +810,16 @@ CliClient::process_input_data()
 	    return;
 	}
 
+	if (stop_processing)
+	    continue;
+
+	preprocess_char(val, stop_processing);
+
+	if (is_waiting_for_data()) {
+	    stop_processing = true;
+	    ignore_current_character = true;
+	}
+
 	if (! stop_processing) {
 	    //
 	    // Get a character and process it
@@ -833,13 +840,7 @@ CliClient::process_input_data()
 		    ret_value = process_char_page_mode(val);
 		    break;
 		}
-		ret_value = process_char(string(line), val,
-					 stop_processing_tmp,
-					 ignore_current_character);
-		if (stop_processing_tmp && (! stop_processing)) {
-		    stop_processing = true;
-		    save_input = true;
-		}
+		ret_value = process_char(string(line), val, stop_processing);
 		break;
 	    } while (false);
 
@@ -851,7 +852,11 @@ CliClient::process_input_data()
 	    }
 	}
 
-	if (save_input) {
+	if (time_slice.is_expired()) {
+	    stop_processing = true;
+	}
+
+	if (stop_processing) {
 	    //
 	    // Stop processing and save the remaining input data for later
 	    // processing.
@@ -859,24 +864,38 @@ CliClient::process_input_data()
 	    // primarily to look for Ctrl-C input.
 	    //
 	    vector<uint8_t>::iterator iter2 = iter;
-	    if (ignore_current_character)
+
+	    if (! ignore_current_character)
 		++iter2;
 	    if (iter2 != input_data.end())
 		_pending_input_data.assign(iter2, input_data.end());
 	}
     }
+    if (! _pending_input_data.empty())
+	schedule_process_input_data();
+	
     cli_flush();		// Flush-out the output
 }
 
+//
+// Schedule a timer to process (pending) input data
+//
+void
+CliClient::schedule_process_input_data()
+{
+    EventLoop& eventloop = cli_node().eventloop();
+    OneoffTaskCallback cb = callback(this, &CliClient::process_input_data);
+
+    _process_pending_input_data_task = eventloop.new_oneoff_task(cb);
+}
+
 //
 // Preprocess a character before 'libtecla' get its hand on it
 //
 int
-CliClient::preprocess_char(uint8_t val, bool& stop_processing,
-			   bool& ignore_current_character)
+CliClient::preprocess_char(uint8_t val, bool& stop_processing)
 {
     stop_processing = false;
-    ignore_current_character = false;
 
     if ((val == '\n') || (val == '\r')) {
 	// New command
