@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_table_set_click.cc,v 1.10 2006/03/16 00:03:53 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_table_set_click.cc,v 1.12 2007/02/16 22:45:40 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -63,8 +63,11 @@ FtiConfigTableSetClick::start(string& error_msg)
     if (ClickSocket::start(error_msg) < 0)
 	return (XORP_ERROR);
 
-    delete_all_entries4();
-    delete_all_entries6();
+    // Cleanup any leftover entries from previously run XORP instance
+    if (! ftic().unicast_forwarding_entries_retain_on_startup4())
+	delete_all_entries4();
+    if (! ftic().unicast_forwarding_entries_retain_on_startup6())
+	delete_all_entries6();
 
     _is_running = true;
 
@@ -89,8 +92,11 @@ FtiConfigTableSetClick::stop(string& error_msg)
     if (! _is_running)
 	return (XORP_OK);
 
-    delete_all_entries4();
-    delete_all_entries6();
+    // Delete the XORP entries
+    if (! ftic().unicast_forwarding_entries_retain_on_shutdown4())
+	delete_all_entries4();
+    if (! ftic().unicast_forwarding_entries_retain_on_shutdown6())
+	delete_all_entries6();
 
     ret_value = ClickSocket::stop(error_msg);
 
