@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_parse_nlm.cc,v 1.21 2006/03/16 00:03:55 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_parse_nlm.cc,v 1.30 2007/02/16 22:45:43 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -52,7 +52,7 @@
 
 #ifndef HAVE_NETLINK_SOCKETS
 bool
-IfConfigGet::parse_buffer_nlm(IfTree& , const uint8_t* , size_t )
+IfConfigGet::parse_buffer_nlm(IfTree& , const vector<uint8_t>&)
 {
     return false;
 }
@@ -70,15 +70,17 @@ static void nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
 				      int rta_len, bool is_deleted);
 
 bool
-IfConfigGet::parse_buffer_nlm(IfTree& it, const uint8_t* buf, size_t buf_bytes)
+IfConfigGet::parse_buffer_nlm(IfTree& it, const vector<uint8_t>& buffer)
 {
+    size_t buffer_bytes = buffer.size();
+    AlignData<struct nlmsghdr> align_data(buffer);
     const struct nlmsghdr* nlh;
     bool recognized = false;
     
-    for (nlh = reinterpret_cast<const struct nlmsghdr*>(buf);
-	 NLMSG_OK(nlh, buf_bytes);
-	 nlh = NLMSG_NEXT(const_cast<struct nlmsghdr*>(nlh), buf_bytes)) {
-	caddr_t nlmsg_data = reinterpret_cast<caddr_t>(NLMSG_DATA(const_cast<struct nlmsghdr*>(nlh)));
+    for (nlh = align_data.payload();
+	 NLMSG_OK(nlh, buffer_bytes);
+	 nlh = XORP_NLMSG_NEXT(const_cast<struct nlmsghdr*>(nlh), buffer_bytes)) {
+	void* nlmsg_data = NLMSG_DATA(const_cast<struct nlmsghdr*>(nlh));
 	
 	switch (nlh->nlmsg_type) {
 	case NLMSG_ERROR:
@@ -170,7 +172,7 @@ nlm_newlink_to_fea_cfg(IfConfig& ifc, IfTree& it,
     
     // The attributes
     memset(rta_array, 0, sizeof(rta_array));
-    rtattr = IFLA_RTA(const_cast<struct ifinfomsg*>(ifinfomsg));
+    rtattr = XORP_IFLA_RTA(const_cast<struct ifinfomsg*>(ifinfomsg));
     NlmUtils::get_rtattr(rtattr, rta_len, rta_array,
 			 sizeof(rta_array) / sizeof(rta_array[0]));
     
@@ -313,7 +315,7 @@ nlm_dellink_to_fea_cfg(IfConfig& ifc, IfTree& it,
     
     // The attributes
     memset(rta_array, 0, sizeof(rta_array));
-    rtattr = IFLA_RTA(const_cast<struct ifinfomsg*>(ifinfomsg));
+    rtattr = XORP_IFLA_RTA(const_cast<struct ifinfomsg*>(ifinfomsg));
     NlmUtils::get_rtattr(rtattr, rta_len, rta_array,
 			 sizeof(rta_array) / sizeof(rta_array[0]));
     
@@ -388,7 +390,7 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
     
     // The attributes
     memset(rta_array, 0, sizeof(rta_array));
-    rtattr = IFA_RTA(const_cast<struct ifaddrmsg*>(ifaddrmsg));
+    rtattr = XORP_IFA_RTA(const_cast<struct ifaddrmsg*>(ifaddrmsg));
     NlmUtils::get_rtattr(rtattr, rta_len, rta_array,
 			 sizeof(rta_array) / sizeof(rta_array[0]));
 
@@ -471,15 +473,15 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
     // Get the IP address
     if (rta_array[IFA_LOCAL] != NULL) {
 	const uint8_t* data = reinterpret_cast<const uint8_t*>(RTA_DATA(const_cast<struct rtattr*>(rta_array[IFA_LOCAL])));
-	if (RTA_PAYLOAD(rta_array[IFA_LOCAL]) != IPvX::addr_size(family)) {
+	if (RTA_PAYLOAD(rta_array[IFA_LOCAL]) != IPvX::addr_bytelen(family)) {
 	    XLOG_FATAL("Invalid IFA_LOCAL address size payload: "
 		       "received %d expected %u",
 		       XORP_INT_CAST(RTA_PAYLOAD(rta_array[IFA_LOCAL])),
-		       XORP_UINT_CAST(IPvX::addr_size(family)));
+		       XORP_UINT_CAST(IPvX::addr_bytelen(family)));
 	}
 	lcl_addr.copy_in(family, data);
     }
-    lcl_addr = kernel_ipvx_adjust(lcl_addr);
+    lcl_addr = kernel_adjust_ipvx_recv(lcl_addr);
     debug_msg("IP address: %s\n", lcl_addr.str().c_str());
     
     // Get the netmask
@@ -493,11 +495,11 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
 	    if (rta_array[IFA_BROADCAST] != NULL) {
 		const uint8_t* data = reinterpret_cast<const uint8_t*>(RTA_DATA(const_cast<struct rtattr*>(rta_array[IFA_BROADCAST])));
 		if (RTA_PAYLOAD(rta_array[IFA_BROADCAST])
-		    != IPvX::addr_size(family)) {
+		    != IPvX::addr_bytelen(family)) {
 		    XLOG_FATAL("Invalid IFA_BROADCAST address size payload: "
 			       "received %d expected %u",
 			       XORP_INT_CAST(RTA_PAYLOAD(rta_array[IFA_BROADCAST])),
-			       XORP_UINT_CAST(IPvX::addr_size(family)));
+			       XORP_UINT_CAST(IPvX::addr_bytelen(family)));
 		}
 		broadcast_addr.copy_in(family, data);
 		has_broadcast_addr = true;
@@ -520,11 +522,11 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
 	if ((rta_array[IFA_ADDRESS] != NULL) && !is_ifa_address_reassigned) {
 	    const uint8_t* data = reinterpret_cast<const uint8_t*>(RTA_DATA(const_cast<struct rtattr*>(rta_array[IFA_ADDRESS])));
 	    if (RTA_PAYLOAD(rta_array[IFA_ADDRESS])
-		!= IPvX::addr_size(family)) {
+		!= IPvX::addr_bytelen(family)) {
 		XLOG_FATAL("Invalid IFA_ADDRESS address size payload: "
 			   "received %d expected %u",
 			   XORP_INT_CAST(RTA_PAYLOAD(rta_array[IFA_ADDRESS])),
-			   XORP_UINT_CAST(IPvX::addr_size(family)));
+			   XORP_UINT_CAST(IPvX::addr_bytelen(family)));
 	    }
 	    peer_addr.copy_in(family, data);
 	    has_peer_addr = true;
