@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_parse_rtm.cc,v 1.26 2006/03/16 00:03:56 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_parse_rtm.cc,v 1.31 2007/02/16 22:45:43 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -65,7 +65,7 @@
 
 #ifndef HAVE_ROUTING_SOCKETS
 bool
-IfConfigGet::parse_buffer_rtm(IfTree& , const uint8_t* , size_t )
+IfConfigGet::parse_buffer_rtm(IfTree& , const vector<uint8_t>& )
 {
     return false;
 }
@@ -119,16 +119,17 @@ ifm_get_link_status(const struct if_msghdr* ifm, const string& if_name,
 }
 
 bool
-IfConfigGet::parse_buffer_rtm(IfTree& it, const uint8_t* buf, size_t buf_bytes)
+IfConfigGet::parse_buffer_rtm(IfTree& it, const vector<uint8_t>& buffer)
 {
+    AlignData<struct if_msghdr> align_data(buffer);
     bool recognized = false;
     u_short if_index_hint = 0;
-    
-    const struct if_msghdr* ifm = reinterpret_cast<const struct if_msghdr*>(buf);
-    const uint8_t* last = buf + buf_bytes;
-    
-    for (const uint8_t* ptr = buf; ptr < last; ptr += ifm->ifm_msglen) {
-    	ifm = reinterpret_cast<const struct if_msghdr*>(ptr);
+    const struct if_msghdr* ifm;
+    size_t offset;
+
+    ifm = align_data.payload();
+    for (offset = 0; offset < buffer.size(); offset += ifm->ifm_msglen) {
+	ifm = align_data.payload_by_offset(offset);
 	if (ifm->ifm_version != RTM_VERSION) {
 	    XLOG_ERROR("RTM version mismatch: expected %d got %d",
 		       RTM_VERSION,
@@ -608,7 +609,7 @@ rtm_addr_to_fea_cfg(IfConfig& ifc, const struct if_msghdr* ifm, IfTree& it,
 #ifdef HAVE_IPV6
     if (rti_info[RTAX_IFA]->sa_family == AF_INET6) {
 	IPv6 lcl_addr(*rti_info[RTAX_IFA]);
-	lcl_addr = kernel_ipv6_adjust(lcl_addr);
+	lcl_addr = kernel_adjust_ipv6_recv(lcl_addr);
 	fv->add_addr(lcl_addr);
 	debug_msg("IP address: %s\n", lcl_addr.str().c_str());
 	
