@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_set_netlink.cc,v 1.27 2006/03/31 06:11:44 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_set_netlink.cc,v 1.35 2007/02/16 22:45:44 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -414,17 +414,21 @@ IfConfigSetNetlink::config_interface(const string& ifname,
 #ifndef HAVE_NETLINK_SOCKETS_SET_FLAGS_IS_BROKEN
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
-    uint8_t		buffer[buffer_size];
+    union {
+	uint8_t		data[buffer_size];
+	struct nlmsghdr	nlh;
+    } buffer;
+    struct nlmsghdr*	nlh = &buffer.nlh;
     struct sockaddr_nl	snl;
-    struct nlmsghdr*	nlh;
     struct ifinfomsg*	ifinfomsg;
     NetlinkSocket&	ns = *this;
+    int			last_errno = 0;
 
     UNUSED(ifname);
     UNUSED(is_up);
     UNUSED(is_deleted);
 
-    memset(buffer, 0, sizeof(buffer));
+    memset(&buffer, 0, sizeof(buffer));
 
     // Set the socket
     memset(&snl, 0, sizeof(snl));
@@ -435,13 +439,12 @@ IfConfigSetNetlink::config_interface(const string& ifname,
     //
     // Set the request
     //
-    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
     nlh->nlmsg_type = RTM_NEWLINK;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
     nlh->nlmsg_seq = ns.seqno();
     nlh->nlmsg_pid = ns.nl_pid();
-    ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
+    ifinfomsg = static_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
     ifinfomsg->ifi_family = AF_UNSPEC;
     ifinfomsg->ifi_type = IFLA_UNSPEC;
     ifinfomsg->ifi_index = if_index;
@@ -455,7 +458,7 @@ IfConfigSetNetlink::config_interface(const string& ifname,
     }
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len);
 
-    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+    if (ns.sendto(&buffer, nlh->nlmsg_len, 0,
 		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
@@ -463,7 +466,7 @@ IfConfigSetNetlink::config_interface(const string& ifname,
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
-					error_msg) < 0) {
+					last_errno, error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -553,17 +556,21 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 #ifdef RTM_SETLINK
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
-    uint8_t		buffer[buffer_size];
+    union {
+	uint8_t		data[buffer_size];
+	struct nlmsghdr	nlh;
+    } buffer;
+    struct nlmsghdr*	nlh = &buffer.nlh;
     struct sockaddr_nl	snl;
-    struct nlmsghdr*	nlh;
     struct ifinfomsg*	ifinfomsg;
     struct rtattr*	rtattr;
     int			rta_len;
     NetlinkSocket&	ns = *this;
+    int			last_errno = 0;
 
     UNUSED(ifname);
 
-    memset(buffer, 0, sizeof(buffer));
+    memset(&buffer, 0, sizeof(buffer));
 
     // Set the socket
     memset(&snl, 0, sizeof(snl));
@@ -574,13 +581,12 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     //
     // Set the request
     //
-    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
     nlh->nlmsg_type = RTM_SETLINK;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
     nlh->nlmsg_seq = ns.seqno();
     nlh->nlmsg_pid = ns.nl_pid();
-    ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
+    ifinfomsg = static_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
     ifinfomsg->ifi_family = AF_UNSPEC;
     ifinfomsg->ifi_type = IFLA_UNSPEC;	// TODO: set to ARPHRD_ETHER ??
     ifinfomsg->ifi_index = if_index;
@@ -594,13 +600,13 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 		   XORP_UINT_CAST(sizeof(buffer)),
 		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
-    rtattr = IFLA_RTA(ifinfomsg);
+    rtattr = XORP_IFLA_RTA(ifinfomsg);
     rtattr->rta_type = IFLA_ADDRESS;
     rtattr->rta_len = rta_len;
     memcpy(RTA_DATA(rtattr), &ether_addr, ETH_ALEN);
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
-    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+    if (ns.sendto(&buffer, nlh->nlmsg_len, 0,
 		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
@@ -608,7 +614,7 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
-					error_msg) < 0) {
+					last_errno, error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -662,17 +668,21 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 #ifndef HAVE_NETLINK_SOCKETS_SET_MTU_IS_BROKEN
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
-    uint8_t		buffer[buffer_size];
+    union {
+	uint8_t		data[buffer_size];
+	struct nlmsghdr	nlh;
+    } buffer;
+    struct nlmsghdr*	nlh = &buffer.nlh;
     struct sockaddr_nl	snl;
-    struct nlmsghdr*	nlh;
     struct ifinfomsg*	ifinfomsg;
     struct rtattr*	rtattr;
     int			rta_len;
     NetlinkSocket&	ns = *this;
+    int			last_errno = 0;
 
     UNUSED(ifname);
 
-    memset(buffer, 0, sizeof(buffer));
+    memset(&buffer, 0, sizeof(buffer));
 
     // Set the socket
     memset(&snl, 0, sizeof(snl));
@@ -683,13 +693,12 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     //
     // Set the request
     //
-    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
     nlh->nlmsg_type = RTM_NEWLINK;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
     nlh->nlmsg_seq = ns.seqno();
     nlh->nlmsg_pid = ns.nl_pid();
-    ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
+    ifinfomsg = static_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
     ifinfomsg->ifi_family = AF_UNSPEC;
     ifinfomsg->ifi_type = IFLA_UNSPEC;
     ifinfomsg->ifi_index = if_index;
@@ -704,13 +713,13 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 		   XORP_UINT_CAST(sizeof(buffer)),
 		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
-    rtattr = IFLA_RTA(ifinfomsg);
+    rtattr = XORP_IFLA_RTA(ifinfomsg);
     rtattr->rta_type = IFLA_MTU;
     rtattr->rta_len = rta_len;
     memcpy(RTA_DATA(rtattr), &uint_mtu, sizeof(uint_mtu));
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
-    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+    if (ns.sendto(&buffer, nlh->nlmsg_len, 0,
 		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
@@ -718,7 +727,7 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
-					error_msg) < 0) {
+					last_errno, error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -765,14 +774,19 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 {
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
-    uint8_t		buffer[buffer_size];
+    union {
+	uint8_t		data[buffer_size];
+	struct nlmsghdr	nlh;
+    } buffer;
+    struct nlmsghdr*	nlh = &buffer.nlh;
     struct sockaddr_nl	snl;
-    struct nlmsghdr*	nlh;
     struct ifaddrmsg*	ifaddrmsg;
     struct rtattr*	rtattr;
     int			rta_len;
     uint8_t*		data;
     NetlinkSocket&	ns = *this;
+    void*		rta_align_data;
+    int			last_errno = 0;
 
     debug_msg("add_vif_address "
 	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
@@ -808,7 +822,7 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 	break;
     }
 
-    memset(buffer, 0, sizeof(buffer));
+    memset(&buffer, 0, sizeof(buffer));
 
     // Set the socket
     memset(&snl, 0, sizeof(snl));
@@ -819,13 +833,12 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
     //
     // Set the request
     //
-    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifaddrmsg));
     nlh->nlmsg_type = RTM_NEWADDR;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
     nlh->nlmsg_seq = ns.seqno();
     nlh->nlmsg_pid = ns.nl_pid();
-    ifaddrmsg = reinterpret_cast<struct ifaddrmsg*>(NLMSG_DATA(nlh));
+    ifaddrmsg = static_cast<struct ifaddrmsg*>(NLMSG_DATA(nlh));
     ifaddrmsg->ifa_family = addr.af();
     ifaddrmsg->ifa_prefixlen = prefix_len;
     ifaddrmsg->ifa_flags = 0;	// TODO: XXX: PAVPAVPAV: OK if 0?
@@ -833,31 +846,33 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
     ifaddrmsg->ifa_index = if_index;
 
     // Add the address as an attribute
-    rta_len = RTA_LENGTH(addr.addr_size());
+    rta_len = RTA_LENGTH(addr.addr_bytelen());
     if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
 	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
 		   XORP_UINT_CAST(sizeof(buffer)),
 		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
-    rtattr = IFA_RTA(ifaddrmsg);
+    rtattr = XORP_IFA_RTA(ifaddrmsg);
     rtattr->rta_type = IFA_LOCAL;
     rtattr->rta_len = rta_len;
-    data = reinterpret_cast<uint8_t*>(RTA_DATA(rtattr));
+    data = static_cast<uint8_t*>(RTA_DATA(rtattr));
     addr.copy_out(data);
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
     if (is_broadcast || is_p2p) {
 	// Set the p2p or broadcast address	
-	rta_len = RTA_LENGTH(dst_or_bcast.addr_size());
+	rta_len = RTA_LENGTH(dst_or_bcast.addr_bytelen());
 	if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
 	    XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
 		       XORP_UINT_CAST(sizeof(buffer)),
 		       XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
 	}
-	rtattr = (struct rtattr*)(((char*)(rtattr)) + RTA_ALIGN((rtattr)->rta_len));
+	rta_align_data = reinterpret_cast<char*>(rtattr)
+	    + RTA_ALIGN(rtattr->rta_len);
+	rtattr = static_cast<struct rtattr*>(rta_align_data);
 	rtattr->rta_type = IFA_UNSPEC;
 	rtattr->rta_len = rta_len;
-	data = reinterpret_cast<uint8_t*>(RTA_DATA(rtattr));
+	data = static_cast<uint8_t*>(RTA_DATA(rtattr));
 	dst_or_bcast.copy_out(data);
 	nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 	if (is_p2p) {
@@ -867,7 +882,7 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 	}
     }
 
-    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+    if (ns.sendto(&buffer, nlh->nlmsg_len, 0,
 		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
@@ -875,7 +890,7 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
-					error_msg) < 0) {
+					last_errno, error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
@@ -891,14 +906,18 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
 {
     static const size_t	buffer_size = sizeof(struct nlmsghdr)
 	+ sizeof(struct ifinfomsg) + 2*sizeof(struct rtattr) + 512;
-    uint8_t		buffer[buffer_size];
+    union {
+	uint8_t		data[buffer_size];
+	struct nlmsghdr	nlh;
+    } buffer;
+    struct nlmsghdr*	nlh = &buffer.nlh;
     struct sockaddr_nl	snl;
-    struct nlmsghdr*	nlh;
     struct ifaddrmsg*	ifaddrmsg;
     struct rtattr*	rtattr;
     int			rta_len;
     uint8_t*		data;
     NetlinkSocket&	ns = *this;
+    int			last_errno = 0;
 
     debug_msg("delete_vif_address "
 	      "(ifname = %s vifname = %s if_index = %u addr = %s "
@@ -932,7 +951,7 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
 	break;
     }
 
-    memset(buffer, 0, sizeof(buffer));
+    memset(&buffer, 0, sizeof(buffer));
 
     // Set the socket
     memset(&snl, 0, sizeof(snl));
@@ -943,13 +962,12 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     //
     // Set the request
     //
-    nlh = reinterpret_cast<struct nlmsghdr*>(buffer);
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifaddrmsg));
     nlh->nlmsg_type = RTM_DELADDR;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
     nlh->nlmsg_seq = ns.seqno();
     nlh->nlmsg_pid = ns.nl_pid();
-    ifaddrmsg = reinterpret_cast<struct ifaddrmsg*>(NLMSG_DATA(nlh));
+    ifaddrmsg = static_cast<struct ifaddrmsg*>(NLMSG_DATA(nlh));
     ifaddrmsg->ifa_family = addr.af();
     ifaddrmsg->ifa_prefixlen = prefix_len;
     ifaddrmsg->ifa_flags = 0;	// TODO: XXX: PAVPAVPAV: OK if 0?
@@ -957,20 +975,20 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     ifaddrmsg->ifa_index = if_index;
 
     // Add the address as an attribute
-    rta_len = RTA_LENGTH(addr.addr_size());
+    rta_len = RTA_LENGTH(addr.addr_bytelen());
     if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
 	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
 		   XORP_UINT_CAST(sizeof(buffer)),
 		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
-    rtattr = IFA_RTA(ifaddrmsg);
+    rtattr = XORP_IFA_RTA(ifaddrmsg);
     rtattr->rta_type = IFA_LOCAL;
     rtattr->rta_len = rta_len;
-    data = reinterpret_cast<uint8_t*>(RTA_DATA(rtattr));
+    data = static_cast<uint8_t*>(RTA_DATA(rtattr));
     addr.copy_out(data);
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
-    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+    if (ns.sendto(&buffer, nlh->nlmsg_len, 0,
 		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
@@ -978,7 +996,7 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
 	return (XORP_ERROR);
     }
     if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
-					error_msg) < 0) {
+					last_errno, error_msg) < 0) {
 	return (XORP_ERROR);
     }
     return (XORP_OK);
