@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_config.cc,v 1.16 2006/07/03 23:33:36 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_config.cc,v 1.18 2007/02/16 22:45:45 pavlin Exp $"
 
 //
 // TODO: a temporary solution for various MFEA configuration
@@ -297,8 +297,8 @@ int
 MfeaNode::set_config_all_vifs_done(string& error_msg)
 {
     map<string, Vif>::iterator vif_iter;
-    string err;
     map<string, Vif>& configured_vifs = ProtoNode<MfeaVif>::configured_vifs();
+    string dummy_error_msg;
     
     //
     // Add new vifs, and update existing ones
@@ -313,7 +313,7 @@ MfeaNode::set_config_all_vifs_done(string& error_msg)
 	// Add a new vif
 	//
 	if (node_vif == NULL) {
-	    add_vif(*vif, err);
+	    add_vif(*vif, dummy_error_msg);
 	    continue;
 	}
 	
@@ -384,7 +384,7 @@ MfeaNode::set_config_all_vifs_done(string& error_msg)
 	if (configured_vifs.find(node_vif->name()) == configured_vifs.end()) {
 	    // Delete the interface
 	    string vif_name = node_vif->name();
-	    delete_vif(vif_name, err);
+	    delete_vif(vif_name, dummy_error_msg);
 	    continue;
 	}
     }
