@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_proto_comm.cc,v 1.58 2006/07/28 00:29:06 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_proto_comm.cc,v 1.70 2007/02/16 22:45:46 pavlin Exp $"
 
 //
 // Multicast-related raw protocol communications.
@@ -62,6 +62,8 @@
 
 #include "libcomm/comm_api.h"
 
+#include "libproto/packet.hh"
+
 #include "mrt/include/ip_mroute.h"
 #include "mrt/max_vifs.h"
 #include "mrt/multicast_defs.h"
@@ -74,6 +76,8 @@
 #include "mrt/include/netinet/pim.h"
 #endif
 
+#include "kernel_utils.hh"
+
 #include "mfea_node.hh"
 #include "mfea_osdep.hh"
 #include "mfea_proto_comm.hh"
@@ -181,8 +185,9 @@ static LPFN_WSARECVMSG lpWSARecvMsg = NULL;
 #define IPTOS_PREC_INTERNETCONTROL	0xc0
 #endif
 #ifndef IPOPT_RA
-#define IPOPT_RA			148	/* 0x94 */
+#define IPOPT_RA			148	// 0x94
 #endif
+static uint32_t		ra_opt4;
 
 // IPv6 Router Alert stuff
 #ifdef HAVE_IPV6
@@ -201,9 +206,9 @@ static LPFN_WSARECVMSG lpWSARecvMsg = NULL;
 #endif // HAVE_IPV6
 //
 #ifdef HAVE_IPV6
-static uint16_t		rtalert_code;
+static uint16_t		rtalert_code6;
 #ifndef HAVE_RFC3542
-static uint8_t		raopt[IP6OPT_RTALERT_LEN];
+static uint8_t		ra_opt6[IP6OPT_RTALERT_LEN];
 #endif
 #endif // HAVE_IPV6
 
@@ -227,35 +232,31 @@ ProtoComm::ProtoComm(MfeaNode& mfea_node, int ip_protocol,
       _mfea_node(mfea_node),
       _ip_protocol(ip_protocol),
       _module_id(module_id),
-      _is_ip_hdr_included(false)
+      _is_ip_hdr_included(false),
+      _ip_id(random())
 {
     // Init Router Alert related option stuff
+    ra_opt4 = htonl((IPOPT_RA << 24) | (0x04 << 16));
 #ifdef HAVE_IPV6
-    rtalert_code = htons(IP6OPT_RTALERT_MLD); // XXX: used by MLD only (?)
+    rtalert_code6 = htons(IP6OPT_RTALERT_MLD); // XXX: used by MLD only (?)
 #ifndef HAVE_RFC3542
-    raopt[0] = IP6OPT_ROUTER_ALERT;
-    raopt[1] = IP6OPT_RTALERT_LEN - 2;
-    memcpy(&raopt[2], (caddr_t)&rtalert_code, sizeof(rtalert_code));
+    ra_opt6[0] = IP6OPT_ROUTER_ALERT;
+    ra_opt6[1] = IP6OPT_RTALERT_LEN - 2;
+    memcpy(&ra_opt6[2], (caddr_t)&rtalert_code6, sizeof(rtalert_code6));
 #endif // ! HAVE_RFC3542
 #endif // HAVE_IPV6
     
     // Allocate the buffers
-    _rcvbuf0 = new uint8_t[IO_BUF_SIZE];
-    _sndbuf0 = new uint8_t[IO_BUF_SIZE];
-    _rcvbuf1 = new uint8_t[IO_BUF_SIZE];
-    _sndbuf1 = new uint8_t[IO_BUF_SIZE];
+    _rcvbuf = new uint8_t[IO_BUF_SIZE];
+    _sndbuf = new uint8_t[IO_BUF_SIZE];
     _rcvcmsgbuf = new uint8_t[CMSG_BUF_SIZE];
     _sndcmsgbuf = new uint8_t[CMSG_BUF_SIZE];
 
     // Scatter/gatter array initialization
-    _rcviov[0].iov_base		= (caddr_t)_rcvbuf0;
-    _rcviov[1].iov_base		= (caddr_t)_rcvbuf1;
+    _rcviov[0].iov_base		= (caddr_t)_rcvbuf;
     _rcviov[0].iov_len		= IO_BUF_SIZE;
-    _rcviov[1].iov_len		= IO_BUF_SIZE;
-    _sndiov[0].iov_base		= (caddr_t)_sndbuf0;
-    _sndiov[1].iov_base		= (caddr_t)_sndbuf1;
+    _sndiov[0].iov_base		= (caddr_t)_sndbuf;
     _sndiov[0].iov_len		= 0;
-    _sndiov[1].iov_len		= 0;
 
     // recvmsg() and sendmsg() related initialization
 #ifndef HOST_OS_WINDOWS
@@ -294,10 +295,8 @@ ProtoComm::~ProtoComm()
     stop();
     
     // Free the buffers
-    delete[] _rcvbuf0;
-    delete[] _sndbuf0;
-    delete[] _rcvbuf1;
-    delete[] _sndbuf1;
+    delete[] _rcvbuf;
+    delete[] _sndbuf;
     delete[] _rcvcmsgbuf;
     delete[] _sndcmsgbuf;
 }
@@ -1025,22 +1024,10 @@ ProtoComm::open_proto_sockets()
 	return (XORP_ERROR);
     }
     // Include IP header when sending (XXX: doesn't do anything for IPv6)
-#ifndef HOST_OS_LINUX
-    //
-    // XXX: In case of Linux don't include the IP header, but use socket
-    // options instead, because the IP_HDRINCL IP packets are not fragmented
-    // and are limited to the interface MTU. The raw(7) Linux manual is wrong
-    // by saying it is a limitation only in Linux 2.2. It is a limitation
-    // in 2.4 and 2.6, and there is no indication this is going to be fixed
-    // in the future.
-    // Fragmenting the packet in user space before transmition is very messy,
-    // especially because we cannot manage the ip_id field in the IP header.
-    //
     if (ip_hdr_include(true) < 0) {
 	close_proto_sockets();
 	return (XORP_ERROR);
     }
-#endif // ! HOST_OS_LINUX
     // Show interest in receiving information from IP header (XXX: IPv6 only)
     if (recv_pktinfo(true) < 0) {
 	close_proto_sockets();
@@ -1227,8 +1214,8 @@ void
 ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 {
     ssize_t	nbytes;
-    int		ip_hdr_len = 0;
-    int		ip_data_len = 0;
+    size_t	ip_hdr_len = 0;
+    size_t	ip_data_len = 0;
     IPvX	src(family());
     IPvX	dst(family());
     int		int_val;
@@ -1237,10 +1224,12 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
     bool	is_router_alert = false; // Router Alert option received
     int		pif_index = 0;
     MfeaVif	*mfea_vif = NULL;
+    void	*cmsg_data;	// XXX: CMSG_DATA() is aligned, hence void ptr
 
     UNUSED(fd);
     UNUSED(type);
     UNUSED(int_val);
+    UNUSED(cmsg_data);
 
 #ifndef HOST_OS_WINDOWS
     // Zero and reset various fields
@@ -1281,7 +1270,7 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 	struct sockaddr_storage from;
 	socklen_t from_len = sizeof(from);
 
-	nbytes = recvfrom(_proto_socket_in, XORP_BUF_CAST(_rcvbuf0),
+	nbytes = recvfrom(_proto_socket_in, XORP_BUF_CAST(_rcvbuf),
 			  IO_BUF_SIZE, 0,
 			  reinterpret_cast<struct sockaddr *>(&from),
 			  &from_len);
@@ -1338,22 +1327,21 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 		     "IGMP is unsupported on this platform");
 	return;		// Error
 #else
-	struct igmpmsg *igmpmsg;
-	
-	igmpmsg = reinterpret_cast<struct igmpmsg *>(_rcvbuf0);
-	if (nbytes < (ssize_t)sizeof(*igmpmsg)) {
+	if (nbytes < (ssize_t)sizeof(struct igmpmsg)) {
 	    XLOG_WARNING("proto_socket_read() failed: "
 			 "kernel signal packet size %d is smaller than minimum size %u",
 			 XORP_INT_CAST(nbytes),
-			 XORP_UINT_CAST(sizeof(*igmpmsg)));
+			 XORP_UINT_CAST(sizeof(struct igmpmsg)));
 	    return;		// Error
 	}
-	if (igmpmsg->im_mbz == 0) {
+	struct igmpmsg igmpmsg;
+	memcpy(&igmpmsg, _rcvbuf, sizeof(igmpmsg));
+	if (igmpmsg.im_mbz == 0) {
 	    //
 	    // XXX: Packets sent up from kernel to daemon have
-	    //      igmpmsg->im_mbz = ip->ip_p = 0
+	    //      igmpmsg.im_mbz = ip->ip_p = 0
 	    //
-	    mfea_node().mfea_mrouter().kernel_call_process(_rcvbuf0, nbytes);
+	    mfea_node().mfea_mrouter().kernel_call_process(_rcvbuf, nbytes);
 	    return;		// OK
 	}
 #endif // HAVE_IPV4_MULTICAST_ROUTING
@@ -1379,7 +1367,7 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 	    break;
 	}
 
-	mrt6msg = reinterpret_cast<struct mrt6msg *>(_rcvbuf0);
+	mrt6msg = reinterpret_cast<struct mrt6msg *>(_rcvbuf);
 	if ((mrt6msg->im6_mbz == 0) || (_rcvmh.msg_controllen == 0)) {
 	    //
 	    // XXX: Packets sent up from kernel to daemon have
@@ -1394,7 +1382,7 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 	    // April 2000, FreeBSD-4.0) which don't have the
 	    //     'icmp6_type = 0' mechanism.
 	    //
-	    mfea_node().mfea_mrouter().kernel_call_process(_rcvbuf0, nbytes);
+	    mfea_node().mfea_mrouter().kernel_call_process(_rcvbuf, nbytes);
 	    return;		// OK
 	}
 #endif // HAVE_IPV6_MULTICAST_ROUTING
@@ -1418,53 +1406,56 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
     switch (family()) {
     case AF_INET:
     {
-	struct ip *ip;
+	IpHeader4 ip4(_rcvbuf);
 	bool is_datalen_error = false;
 	
 	// Input check
-	ip = reinterpret_cast<struct ip *>(_rcvbuf0);
-	if (nbytes < (ssize_t)sizeof(*ip)) {
+	if (nbytes < (ssize_t)ip4.size()) {
 	    XLOG_WARNING("proto_socket_read() failed: "
 			 "packet size %d is smaller than minimum size %u",
 			 XORP_INT_CAST(nbytes),
-			 XORP_UINT_CAST(sizeof(*ip)));
+			 XORP_UINT_CAST(ip4.size()));
 	    return;		// Error
 	}
 #ifndef HOST_OS_WINDOWS
-	// TODO: get rid of this and always use ip->ip_src ??
+	// TODO: get rid of this and always use ip4.ip_src() ??
 	src.copy_in(_from4);
 #else
-	src.copy_in(ip->ip_src);
+	src = ip4.ip_src();
 #endif
-	dst.copy_in(ip->ip_dst);
-	ip_ttl = ip->ip_ttl;
-	ip_tos = ip->ip_tos;
+	dst = ip4.ip_dst();
+	ip_ttl = ip4.ip_ttl();
+	ip_tos = ip4.ip_tos();
 	
-	ip_hdr_len  = ip->ip_hl << 2;
+	ip_hdr_len  = ip4.ip_header_len();
 #ifdef IPV4_RAW_INPUT_IS_RAW
-	ip_data_len = ntohs(ip->ip_len) - ip_hdr_len;
+	ip_data_len = ip4.ip_len() - ip_hdr_len;
 #else
-	ip_data_len = ip->ip_len;
+	//
+	// XXX: The original value is in host order, and excludes the
+	// IPv4 header length.
+	//
+	ip_data_len = ip4.ip_len_host();
 #endif // ! IPV4_RAW_INPUT_IS_RAW
 	// Check length
 	is_datalen_error = false;
 	do {
-	    if (ip_hdr_len + ip_data_len == nbytes) {
+	    if (ip_hdr_len + ip_data_len == static_cast<size_t>(nbytes)) {
 		is_datalen_error = false;
 		break;		// OK
 	    }
-	    if (nbytes < ip_hdr_len) {
+	    if (nbytes < static_cast<ssize_t>(ip_hdr_len)) {
 		is_datalen_error = true;
 		break;
 	    }
-	    if (ip->ip_p == IPPROTO_PIM) {
-		struct pim *pim;
-		if (nbytes < ip_hdr_len + PIM_REG_MINLEN) {
+	    if (ip4.ip_p() == IPPROTO_PIM) {
+		if (nbytes < static_cast<ssize_t>(ip_hdr_len + PIM_REG_MINLEN)) {
 		    is_datalen_error = true;
 		    break;
 		}
-		pim = reinterpret_cast<struct pim *>((uint8_t *)ip + ip_hdr_len);
-		if (PIM_VT_T(pim->pim_vt) != PIM_REGISTER) {
+		struct pim pim;
+		memcpy(&pim, ip4.data() + ip_hdr_len, sizeof(pim));
+		if (PIM_VT_T(pim.pim_vt) != PIM_REGISTER) {
 		    is_datalen_error = true;
 		    break;
 		}
@@ -1481,10 +1472,12 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 	if (is_datalen_error) {
 	    XLOG_ERROR("proto_socket_read() failed: "
 		       "RX packet size from %s to %s with %d bytes instead of "
-		       "hdr+datalen=%d+%d=%d",
+		       "hdr+datalen=%u+%u=%u",
 		       cstring(src), cstring(dst),
 		       XORP_INT_CAST(nbytes),
-		       ip_hdr_len, ip_data_len, ip_hdr_len + ip_data_len);
+		       XORP_UINT_CAST(ip_hdr_len),
+		       XORP_UINT_CAST(ip_data_len),
+		       XORP_UINT_CAST(ip_hdr_len + ip_data_len));
 	    return;		// Error
 	}
 
@@ -1504,7 +1497,8 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 		struct sockaddr_dl *sdl = NULL;
 		if (cmsgp->cmsg_len < CMSG_LEN(sizeof(struct sockaddr_dl)))
 		    continue;
-		sdl = reinterpret_cast<struct sockaddr_dl *>(CMSG_DATA(cmsgp));
+		cmsg_data = CMSG_DATA(cmsgp);
+		sdl = reinterpret_cast<struct sockaddr_dl *>(cmsg_data);
 		pif_index = sdl->sdl_index;
 	    }
 	    break;
@@ -1516,7 +1510,8 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 		struct in_pktinfo *inp = NULL;
 		if (cmsgp->cmsg_len < CMSG_LEN(sizeof(struct in_pktinfo)))
 		    continue;
-		inp = reinterpret_cast<struct in_pktinfo *>(CMSG_DATA(cmsgp));
+		cmsg_data = CMSG_DATA(cmsgp);
+		inp = reinterpret_cast<struct in_pktinfo *>(cmsg_data);
 		pif_index = inp->ipi_ifindex;
 	    }
 	    break;
@@ -1532,9 +1527,9 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 	// Check for Router Alert option
 	//
 	do {
-	    uint8_t *option_p = (uint8_t *)(ip + 1);
+	    const uint8_t *option_p = ip4.data() + ip4.size();
 	    uint8_t option_value, option_len;
-	    uint32_t test_ip_options_len = ip_hdr_len - sizeof(*ip);
+	    uint32_t test_ip_options_len = ip_hdr_len - ip4.size();
 	    while (test_ip_options_len) {
 		if (test_ip_options_len < 4)
 		    break;
@@ -1590,79 +1585,87 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 	     cmsgp = reinterpret_cast<struct cmsghdr *>(CMSG_NXTHDR(&_rcvmh, cmsgp))) {
 	    if (cmsgp->cmsg_level != IPPROTO_IPV6)
 		continue;
+
 	    switch (cmsgp->cmsg_type) {
 	    case IPV6_PKTINFO:
+	    {
 		if (cmsgp->cmsg_len < CMSG_LEN(sizeof(struct in6_pktinfo)))
 		    continue;
-		pi = reinterpret_cast<struct in6_pktinfo *>(CMSG_DATA(cmsgp));
+		cmsg_data = CMSG_DATA(cmsgp);
+		pi = reinterpret_cast<struct in6_pktinfo *>(cmsg_data);
 		pif_index = pi->ipi6_ifindex;
 		dst.copy_in(pi->ipi6_addr);
-		break;
+	    }
+	    break;
+
 	    case IPV6_HOPLIMIT:
+	    {
 		if (cmsgp->cmsg_len < CMSG_LEN(sizeof(int)))
 		    continue;
-		int_val = *((int *)CMSG_DATA(cmsgp));
+		int_val = extract_host_int(CMSG_DATA(cmsgp));
 		ip_ttl = int_val;
-		break;
+	    }
+	    break;
+
 	    case IPV6_HOPOPTS:
-		{
-		    //
-		    // Check for Router Alert option
-		    //
+	    {
+		//
+		// Check for Router Alert option
+		//
 #ifdef HAVE_RFC3542
-		    {
-			struct ip6_hbh *ext;
-			int currentlen;
-			uint8_t type;
-			size_t extlen;
-			socklen_t len;
-			void *databuf;
-			
-			ext = reinterpret_cast<struct ip6_hbh *>(CMSG_DATA(cmsgp));
-			extlen = (ext->ip6h_len + 1) * 8;
-			currentlen = 0;
-			while (true) {
-			    currentlen = inet6_opt_next(ext, extlen,
-							currentlen,
-							&type, &len, &databuf);
-			    if (currentlen == -1)
-				break;
-			    if (type == IP6OPT_ROUTER_ALERT) {
-				is_router_alert = true;
-				break;
-			    }
-			}
+		struct ip6_hbh *ext;
+		int currentlen;
+		uint8_t type;
+		size_t extlen;
+		socklen_t len;
+		void *databuf;
+
+		cmsg_data = CMSG_DATA(cmsgp);
+		ext = reinterpret_cast<struct ip6_hbh *>(cmsg_data);
+		extlen = (ext->ip6h_len + 1) * 8;
+		currentlen = 0;
+		while (true) {
+		    currentlen = inet6_opt_next(ext, extlen, currentlen,
+						&type, &len, &databuf);
+		    if (currentlen == -1)
+			break;
+		    if (type == IP6OPT_ROUTER_ALERT) {
+			is_router_alert = true;
+			break;
 		    }
-#else // ! HAVE_RFC3542 (i.e., the old advanced API)
-		    {
-#ifdef HAVE_IPV6_MULTICAST_ROUTING
-			//
-			// TODO: XXX: temporary use HAVE_IPV6_MULTICAST_ROUTING
-			// to conditionally compile, because Linux doesn't
-			// have inet6_option_*
-			//
-			uint8_t  *tptr = NULL;
-			while (inet6_option_next(cmsgp, &tptr) == 0) {
-			    if (*tptr == IP6OPT_ROUTER_ALERT) {
-				is_router_alert = true;
-				break;
-			    }
-			}
-#endif // HAVE_IPV6_MULTICAST_ROUTING
-		    }
-#endif // ! HAVE_RFC3542
-		    
 		}
-		
-		break;
+
+#else // ! HAVE_RFC3542 (i.e., the old advanced API)
+
+#ifdef HAVE_IPV6_MULTICAST_ROUTING
+		//
+		// TODO: XXX: temporary use HAVE_IPV6_MULTICAST_ROUTING
+		// to conditionally compile, because Linux doesn't
+		// have inet6_option_*
+		//
+		uint8_t *tptr = NULL;
+		while (inet6_option_next(cmsgp, &tptr) == 0) {
+		    if (*tptr == IP6OPT_ROUTER_ALERT) {
+			is_router_alert = true;
+			break;
+		    }
+		}
+#endif // HAVE_IPV6_MULTICAST_ROUTING
+#endif // ! HAVE_RFC3542
+	    }
+	    break;
+
 #ifdef IPV6_TCLASS
 	    case IPV6_TCLASS:
+	    {
 		if (cmsgp->cmsg_len < CMSG_LEN(sizeof(int)))
 		    continue;
-		int_val = *((int *)CMSG_DATA(cmsgp));
+		int_val = extract_host_int(CMSG_DATA(cmsgp));
 		ip_tos = int_val;
-		break;
+	    }
+	    break;
 #endif // IPV6_TCLASS
+
 	    default:
 		break;
 	    }
@@ -1756,7 +1759,7 @@ ProtoComm::proto_socket_read(XorpFd fd, IoEventType type)
 				    mfea_vif->vif_index(),
 				    src, dst, ip_ttl, ip_tos,
 				    is_router_alert,
-				    _rcvbuf0 + ip_hdr_len, nbytes - ip_hdr_len)
+				    _rcvbuf + ip_hdr_len, nbytes - ip_hdr_len)
 	!= XORP_OK) {
 	return;			// Error
     }
@@ -1792,16 +1795,14 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 			      const uint8_t *databuf, size_t datalen,
 			      string& error_msg)
 {
-    struct ip	*ip;
-    uint32_t	ip_option;
-    uint32_t	*ip_option_p;
-    int		ip_hdr_len = 0;
+    size_t	ip_hdr_len = 0;
     int		int_val;
-    int		setloop = false;
-    int		ret;
+    int		ret_value = XORP_OK;
     MfeaVif	*mfea_vif = mfea_node().vif_find_by_vif_index(vif_index);
+    void	*cmsg_data;	// XXX: CMSG_DATA() is aligned, hence void ptr
 
     UNUSED(int_val);
+    UNUSED(cmsg_data);
 
     if (mfea_vif == NULL) {
 	error_msg = c_format("Cannot find vif with vif_index = %u", vif_index);
@@ -1877,7 +1878,7 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
     //
     // Setup the IP header (including the Router Alert option, if specified)
     // In case of IPv4, if the IP_HDRINCL socket option is enabled, the IP
-    // header and the data are specified as two entries to the sndiov[]
+    // header and the data are specified as a single entry in the sndiov[]
     // scatter/gatter array, otherwise we use socket options to set the IP
     // header information and a single sndiov[] entry with the payload.
     // In case of IPv6, the IP header information is specified as
@@ -1885,6 +1886,67 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
     //
     switch (family()) {
     case AF_INET:
+    {
+	//
+	// XXX: In case of Linux IP_HDRINCL IP packets are not fragmented and
+	// are limited to the interface MTU. The raw(7) Linux manual is wrong
+	// by saying it is a limitation only in Linux 2.2. It is a limitation
+	// in 2.4 and 2.6 and there is no indication this is going to be fixed
+	// in the future.
+	// Hence, in case of Linux we do the following:
+	//   - If the IP packet fits in the MTU, then send the packet using
+	//     IP_HDRINCL option.
+	//   - Otherwise, if the IP source address belongs to the outgoing
+	//     interface, then use various socket options to specify some of
+	//     the IP header options.
+	//   - Otherwise, use IP_HDRINCL and fragment the IP packet in user
+	//     space before transmitting it. Note that in this case we need
+	//     to manage the ip_id field in the IP header.
+	//
+	// The reasoning behind the above logic is: (1) If the IP source
+	// address doesn't belong to one of the router's IP addresses, then
+	// we must use IP_HDRINCL; (2) We want to avoid as much as possible
+	// user-level IP packet fragmentation, because managing the ip_id
+	// field in user space does not guarantee the same ip_id is reused
+	// by the kernel as well (for the same tuple of <src, dest, protocol>).
+	//
+	// Note that in case of all other OS-es we always use the IP_HDRINCL
+	// option to transmit the packets.
+	//
+	bool do_fragmentation = false;
+	bool do_ip_hdr_include = true;
+
+	//
+	// Decide whether we should use IP_HDRINCL and whether we should
+	// do IP packet fragmentation.
+	//
+	do {
+#ifndef HOST_OS_LINUX
+	    break;	// XXX: The extra processing below is for Linux only
+#endif
+	    // Calculate the final packet size and whether it fits in the MTU
+	    ip_hdr_len = IpHeader4::SIZE;
+	    if (is_router_alert)
+		ip_hdr_len += sizeof(ra_opt4);
+
+	    if (ip_hdr_len + datalen <= mfea_vif->mtu())
+		break;
+
+	    if (mfea_vif->is_my_addr(src)) {
+		do_ip_hdr_include = false;
+		break;
+	    }
+
+	    do_fragmentation = true;
+	    break;
+	} while (false);
+
+	if (do_ip_hdr_include != _is_ip_hdr_included) {
+	    if (ip_hdr_include(do_ip_hdr_include) < 0) {
+		return (XORP_ERROR);
+	    }
+	}
+
 	if (! _is_ip_hdr_included) {
 	    //
 	    // Use socket options to set the IP header information
@@ -1895,10 +1957,9 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	    //
 #ifdef IP_OPTIONS
 	    if (is_router_alert) {
-		ip_option = htonl((IPOPT_RA << 24) | (0x04 << 16));
 		if (setsockopt(_proto_socket_out, IPPROTO_IP, IP_OPTIONS,
-			       XORP_SOCKOPT_CAST(&ip_option),
-			       sizeof(ip_option))
+			       XORP_SOCKOPT_CAST(&ra_opt4),
+			       sizeof(ra_opt4))
 		    < 0) {
 		    error_msg = c_format("setsockopt(IP_OPTIONS, IPOPT_RA) "
 					 "failed: %s",
@@ -1940,7 +2001,7 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 #endif // IP_TOS
 
 	    //
-	    // XXX: bind to the source address so the outgoing IP packet
+	    // XXX: Bind to the source address so the outgoing IP packet
 	    // will use that address.
 	    //
 	    struct sockaddr_in sin;
@@ -1958,70 +2019,80 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	    //
 	    // Now hook the data
 	    //
-#ifndef HOST_OS_WINDOWS
-	    dst.copy_out(_to4);
-	    _sndmh.msg_namelen		= sizeof(_to4);
-	    _sndmh.msg_iovlen		= 1;
-	    _sndmh.msg_control		= NULL;
-	    _sndmh.msg_controllen	= 0;
-#endif // ! HOST_OS_WINDOWS
-	    memcpy(_sndbuf0, databuf, datalen); // XXX: goes to _sndiov[0].iov_base
-	    // _sndiov[0].iov_base	= (caddr_t)databuf;
-	    _sndiov[0].iov_len	= datalen;
+	    memcpy(_sndbuf, databuf, datalen);	// XXX: _sndiov[0].iov_base
+	    _sndiov[0].iov_len = datalen;
+
+	    // Transmit the packet
+	    ret_value = proto_socket_transmit(mfea_vif, src, dst, error_msg);
 	    break;
 	}
 
 	//
-	// Compose the IP header
+	// Set the IPv4 header
 	//
-	ip = reinterpret_cast<struct ip *>(_sndbuf0);
+	IpHeader4Writer ip4(_sndbuf);
 	if (is_router_alert) {
 	    // Include the Router Alert option
-	    ip_option	= htonl((IPOPT_RA << 24) | (0x04 << 16));
-	    ip_option_p	= (uint32_t *)(ip + 1);
-	    *ip_option_p = ip_option;
-	    ip_hdr_len	= (sizeof(*ip) + sizeof(*ip_option_p));
+	    uint8_t* ip_option_p = ip4.data() + ip4.size();
+	    //
+	    // XXX: ra_opt4 is in network order, hence we write it as-is
+	    // by using embed_host_32().
+	    //
+	    embed_host_32(ip_option_p, ra_opt4);
+	    ip_hdr_len	= ip4.size() + sizeof(ra_opt4);
 	} else {
-	    ip_hdr_len	= sizeof(*ip);
+	    ip_hdr_len	= ip4.size();
 	}
-	ip->ip_hl	= ip_hdr_len >> 2;
-	ip->ip_v	= IPVERSION;
-	ip->ip_id	= 0;				// Let kernel fill in
-	ip->ip_off	= 0x0;
-	ip->ip_p	= _ip_protocol;
-	ip->ip_ttl	= ip_ttl;
-	ip->ip_tos	= ip_tos;
-
-	//
-	// XXX: we need to use a temporary in_addr storage as a work-around
-	// if "struct ip" is __packed
-	//
-	struct in_addr in_addr_tmp;
-	src.copy_out(in_addr_tmp);
-	ip->ip_src = in_addr_tmp;
-	dst.copy_out(in_addr_tmp);
-	ip->ip_dst = in_addr_tmp;
-	ip->ip_len = (ip->ip_hl << 2) + datalen;
-#ifdef IPV4_RAW_OUTPUT_IS_RAW
-	ip->ip_len = htons(ip->ip_len);
-#endif
-	ip->ip_sum = 0;					// Let kernel fill in
+	ip4.set_ip_version(IPVERSION);
+	ip4.set_ip_header_len(ip_hdr_len);
+	ip4.set_ip_tos(ip_tos);
+	ip4.set_ip_id(0);				// Let kernel fill in
+	ip4.set_ip_off(0);
+	ip4.set_ip_ttl(ip_ttl);
+	ip4.set_ip_p(_ip_protocol);
+	ip4.set_ip_sum(0);				// Let kernel fill in
+	ip4.set_ip_src(src.get_ipv4());
+	ip4.set_ip_dst(dst.get_ipv4());
+	ip4.set_ip_len(ip_hdr_len + datalen);
 
 	//
 	// Now hook the data
 	//
-#ifndef HOST_OS_WINDOWS
-	dst.copy_out(_to4);
-	_sndmh.msg_namelen	= sizeof(_to4);
-	_sndmh.msg_iovlen	= 2;
-	_sndmh.msg_control	= NULL;
-	_sndmh.msg_controllen	= 0;
-#endif // ! HOST_OS_WINDOWS
-	_sndiov[0].iov_len	= ip_hdr_len;
-    	memcpy(_sndbuf1, databuf, datalen); // XXX: goes to _sndiov[1].iov_base
-	// _sndiov[1].iov_base	= (caddr_t)databuf;
-	_sndiov[1].iov_len	= datalen;
-	break;
+	memcpy(_sndbuf + ip_hdr_len, databuf, datalen);	// XXX: _sndiov[0].iov_base
+	_sndiov[0].iov_len = ip_hdr_len + datalen;
+
+	if (! do_fragmentation) {
+	    // Transmit the packet
+	    ret_value = proto_socket_transmit(mfea_vif, src, dst, error_msg);
+	    break;
+	}
+
+	//
+	// Perform fragmentation and transmit each fragment
+	//
+	list<vector<uint8_t> > fragments;
+	list<vector<uint8_t> >::iterator iter;
+
+	// Calculate and set the IPv4 packet ID
+	_ip_id++;
+	if (_ip_id == 0)
+	    _ip_id++;
+	ip4.set_ip_id(_ip_id);
+	if (ip4.fragment(mfea_vif->mtu(), fragments, false, error_msg)
+	    != XORP_OK) {
+	    return (XORP_ERROR);
+	}
+	XLOG_ASSERT(! fragments.empty());
+	for (iter = fragments.begin(); iter != fragments.end(); ++iter) {
+	    vector<uint8_t>& ip_fragment = *iter;
+	    _sndiov[0].iov_len = ip_fragment.size();
+	    memcpy(_sndbuf, &ip_fragment[0], ip_fragment.size());
+	    ret_value = proto_socket_transmit(mfea_vif, src, dst, error_msg);
+	    if (ret_value != XORP_OK)
+		break;
+	}
+    }
+    break;
 
 #ifdef HAVE_IPV6
     case AF_INET6:
@@ -2070,7 +2141,7 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	    // to conditionally compile, because Linux doesn't
 	    // have inet6_option_*
 	    //
-	    hbhlen = inet6_option_space(sizeof(raopt));
+	    hbhlen = inet6_option_space(sizeof(ra_opt6));
 	    ctllen += hbhlen;
 #else
 	    UNUSED(hbhlen);
@@ -2095,7 +2166,8 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	cmsgp->cmsg_len   = CMSG_LEN(sizeof(struct in6_pktinfo));
 	cmsgp->cmsg_level = IPPROTO_IPV6;
 	cmsgp->cmsg_type  = IPV6_PKTINFO;
-	sndpktinfo = reinterpret_cast<struct in6_pktinfo *>(CMSG_DATA(cmsgp));
+	cmsg_data = CMSG_DATA(cmsgp);
+	sndpktinfo = reinterpret_cast<struct in6_pktinfo *>(cmsg_data);
 	memset(sndpktinfo, 0, sizeof(*sndpktinfo));
 	if ((mfea_vif->pif_index() > 0)
 	    && !(dst.is_unicast() && !dst.is_linklocal_unicast())) {
@@ -2138,7 +2210,7 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 		XLOG_ERROR("%s", error_msg.c_str());
 		return (XORP_ERROR);
 	    }
-	    inet6_opt_set_val(optp, 0, &rtalert_code, sizeof(rtalert_code));
+	    inet6_opt_set_val(optp, 0, &rtalert_code6, sizeof(rtalert_code6));
 	    if (inet6_opt_finish(hbhbuf, hbhlen, currentlen) == -1) {
 		error_msg = c_format("inet6_opt_finish(len = %d) failed",
 				     currentlen);
@@ -2159,7 +2231,7 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 		XLOG_ERROR("%s", error_msg.c_str());
 		return (XORP_ERROR);
 	    }
-	    if (inet6_option_append(cmsgp, raopt, 4, 0)) {
+	    if (inet6_option_append(cmsgp, ra_opt6, 4, 0)) {
 		error_msg = c_format("inet6_option_append(Router Alert) failed");
 		XLOG_ERROR("%s", error_msg.c_str());
 		return (XORP_ERROR);
@@ -2178,7 +2250,7 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	cmsgp->cmsg_level = IPPROTO_IPV6;
 	cmsgp->cmsg_type = IPV6_HOPLIMIT;
 	int_val = ip_ttl;
-	*(int *)(CMSG_DATA(cmsgp)) = int_val;
+	embed_host_int(CMSG_DATA(cmsgp), int_val);
 	cmsgp = CMSG_NXTHDR(&_sndmh, cmsgp);
 	
 	//
@@ -2189,21 +2261,18 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	cmsgp->cmsg_level = IPPROTO_IPV6;
 	cmsgp->cmsg_type = IPV6_TCLASS;
 	int_val = ip_tos;
-	*(int *)(CMSG_DATA(cmsgp)) = int_val;
+	embed_host_int(CMSG_DATA(cmsgp), int_val);
 	cmsgp = CMSG_NXTHDR(&_sndmh, cmsgp);
 #endif // IPV6_TCLASS
 	
 	//
 	// Now hook the data
 	//
-#ifndef HOST_OS_WINDOWS
-	dst.copy_out(_to6);
-	_sndmh.msg_namelen  = sizeof(_to6);
-	_sndmh.msg_iovlen   = 1;
-#endif // ! HOST_OS_WINDOWS
-	memcpy(_sndbuf0, databuf, datalen); // XXX: goes to _sndiov[0].iov_base
-	// _sndiov[0].iov_base	= (caddr_t)databuf;
+	memcpy(_sndbuf, databuf, datalen);	// XXX: _sndiov[0].iov_base
 	_sndiov[0].iov_len  = datalen;
+
+	// Transmit the packet
+	ret_value = proto_socket_transmit(mfea_vif, src, dst, error_msg);
     }
     break;
 #endif // HAVE_IPV6
@@ -2213,67 +2282,59 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	error_msg = c_format("Invalid address family: %d", family());
 	return (XORP_ERROR);
     }
-    
-    setloop = false;
+
+    return (ret_value);
+}
+
+int
+ProtoComm::proto_socket_transmit(MfeaVif *mfea_vif, const IPvX& src,
+				 const IPvX& dst, string& error_msg)
+{
+    bool setloop = false;
+    int ret_value = XORP_OK;
+
+    //
+    // Adjust some IPv4 header fields
+    //
+#ifndef IPV4_RAW_OUTPUT_IS_RAW
+    if (_is_ip_hdr_included && src.is_ipv4()) {
+	//
+	// XXX: The stored value should be in host order, and should
+	// include the IPv4 header length.
+	//
+	IpHeader4Writer ip4(_sndbuf);
+	ip4.set_ip_len_host(ip4.ip_len());
+    }
+#endif // ! IPV4_RAW_INPUT_IS_RAW
+
+    //
+    // Multicast-related setting
+    //
     if (dst.is_multicast()) {
 	set_default_multicast_vif(mfea_vif->vif_index());
 	set_multicast_loop(true);
 	setloop = true;
     }
 
-#ifndef HOST_OS_WINDOWS    
-    ret = XORP_OK;
-    if (sendmsg(_proto_socket_out, &_sndmh, 0) < 0) {
-	ret = XORP_ERROR;
-	if (errno == ENETDOWN) {
-	    // TODO: check the interface status. E.g. vif_state_check(family);
-	} else {
-	    error_msg = c_format("sendmsg(proto %d size %u from %s to %s "
-				 "on vif %s) failed: %s",
-				 _ip_protocol, XORP_UINT_CAST(datalen),
-				 cstring(src), cstring(dst),
-				 mfea_vif->name().c_str(), strerror(errno));
-	    XLOG_ERROR("%s", error_msg.c_str());
-	}
-    }
+    //
+    // Transmit the packet
+    //
+#ifndef HOST_OS_WINDOWS
 
-#else // HOST_OS_WINDOWS
-
-    ret = XORP_OK;
+    // Set some sendmsg()-related fields
     switch (family()) {
     case AF_INET:
-    {
-	struct sockaddr_in to;
-	DWORD sent, error;
-
-	dst.copy_out(to);
-	error = WSASendTo(_proto_socket_out,
-			  reinterpret_cast<WSABUF *>(_sndiov),
-			  2, &sent, 0,
-			  reinterpret_cast<struct sockaddr *>(&to),
-			  sizeof(to), NULL, NULL);
-	if (error != 0) {
-	    ret = XORP_ERROR;
-	}
-    }
-    break;
+	dst.copy_out(_to4);
+	_sndmh.msg_namelen	= sizeof(_to4);
+	_sndmh.msg_control	= NULL;
+	_sndmh.msg_controllen	= 0;
+	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
-    {
-	struct sockaddr_in6 to;
-	DWORD sent, error;
-
-	dst.copy_out(to);
-	error = WSASendTo(_proto_socket_out,
-			  reinterpret_cast<WSABUF *>(_sndiov),
-			  1, &sent, 0,
-			  reinterpret_cast<struct sockaddr *>(&to),
-			  sizeof(to), NULL, NULL);
-	if (error != 0) {
-	    ret = XORP_ERROR;
-	}
-    }
-    break;
+	dst.copy_out(_to6);
+	kernel_adjust_sockaddr_in6_send(_to6, mfea_vif->pif_index());
+	_sndmh.msg_namelen  = sizeof(_to6);
+	break;
 #endif // HAVE_IPV6
     default:
 	XLOG_UNREACHABLE();
@@ -2281,11 +2342,71 @@ ProtoComm::proto_socket_write(uint32_t vif_index,
 	return (XORP_ERROR);
     }
 
+    if (sendmsg(_proto_socket_out, &_sndmh, 0) < 0) {
+	ret_value = XORP_ERROR;
+	if (errno == ENETDOWN) {
+	    // TODO: check the interface status. E.g. vif_state_check(family);
+	} else {
+	    error_msg = c_format("sendmsg(proto %d size %u from %s to %s "
+				 "on vif %s) failed: %s",
+				 _ip_protocol,
+				 XORP_UINT_CAST(_sndiov[0].iov_len),
+				 cstring(src), cstring(dst),
+				 mfea_vif->name().c_str(), strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	}
+    }
+
+#else // HOST_OS_WINDOWS
+    // XXX: We may use WSASendMsg() on Longhorn to support IPv6.
+
+    DWORD sent, error;
+    struct sockaddr_storage to;
+    DWORD buffer_count = 1;
+    int to_len = 0;
+
+    memset(&to, 0, sizeof(to));
+    dst.copy_out(reinterpret_cast<struct sockaddr&>(to));
+
+    // Set some family-specific arguments
+    switch (family()) {
+    case AF_INET:
+	to_len = sizeof(struct sockaddr_in);
+	break;
+#ifdef HAVE_IPV6
+    case AF_INET6:
+	to_len = sizeof(struct sockaddr_in6);
+	break;
+#endif // HAVE_IPV6
+    default:
+	XLOG_UNREACHABLE();
+	error_msg = c_format("Invalid address family %d", family());
+	return (XORP_ERROR);
+    }
+
+    error = WSASendTo(_proto_socket_out,
+		      reinterpret_cast<WSABUF *>(_sndiov),
+		      buffer_count, &sent, 0,
+		      reinterpret_cast<struct sockaddr *>(&to),
+		      to_len, NULL, NULL);
+    if (error != 0) {
+	ret_value = XORP_ERROR;
+	error_msg = c_format("WSASendTo(proto %d size %u from %s to %s "
+			     "on vif %s) failed: %s",
+			     _ip_protocol, XORP_UINT_CAST(_sndiov[0].iov_len),
+			     cstring(src), cstring(dst),
+			     mfea_vif->name().c_str(),
+			     win_strerror(GetLastError()));
+	XLOG_ERROR("%s", error_msg.c_str());
+    }
 #endif // HOST_OS_WINDOWS
 
+    //
+    // Restore some multicast related settings
+    //
     if (setloop) {
 	set_multicast_loop(false);
     }
-    
-    return (ret);
+
+    return (ret_value);
 }
