@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/netlink_socket.cc,v 1.38 2006/04/21 06:17:15 pavlin Exp $"
+#ident "$XORP: xorp/fea/netlink_socket.cc,v 1.43 2007/02/16 22:45:47 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -50,6 +50,7 @@
 #include "libcomm/comm_api.h"
 
 #include "netlink_socket.hh"
+#include "netlink_socket_utils.hh"
 
 
 uint16_t NetlinkSocket::_instance_cnt = 0;
@@ -315,10 +316,11 @@ NetlinkSocket::force_read(string& error_msg)
 	//
 	bool is_end_of_message = true;
 	size_t new_size = off - last_mh_off;
+	AlignData<struct nlmsghdr> align_data(message);
 	const struct nlmsghdr* mh;
-	for (mh = reinterpret_cast<const struct nlmsghdr*>(&message[last_mh_off]);
+	for (mh = align_data.payload_by_offset(last_mh_off);
 	     NLMSG_OK(mh, new_size);
-	     mh = NLMSG_NEXT(const_cast<struct nlmsghdr*>(mh), new_size)) {
+	     mh = XORP_NLMSG_NEXT(const_cast<struct nlmsghdr*>(mh), new_size)) {
 	    XLOG_ASSERT(mh->nlmsg_len <= buffer.size());
 	    if ((mh->nlmsg_flags & NLM_F_MULTI)
 		|| _is_multipart_message_read) {
@@ -337,7 +339,7 @@ NetlinkSocket::force_read(string& error_msg)
     // Notify observers
     //
     for (ObserverList::iterator i = _ol.begin(); i != _ol.end(); i++) {
-	(*i)->nlsock_data(&message[0], message.size());
+	(*i)->nlsock_data(message);
     }
 
     return (XORP_OK);
@@ -392,10 +394,11 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
 	//
 	bool is_end_of_message = true;
 	size_t new_size = off - last_mh_off;
+	AlignData<struct nlmsghdr> align_data(message);
 	const struct nlmsghdr* mh;
-	for (mh = reinterpret_cast<const struct nlmsghdr*>(&message[last_mh_off]);
+	for (mh = align_data.payload_by_offset(last_mh_off);
 	     NLMSG_OK(mh, new_size);
-	     mh = NLMSG_NEXT(const_cast<struct nlmsghdr*>(mh), new_size)) {
+	     mh = XORP_NLMSG_NEXT(const_cast<struct nlmsghdr*>(mh), new_size)) {
 	    XLOG_ASSERT(mh->nlmsg_len <= buffer.size());
 	    if ((mh->nlmsg_flags & NLM_F_MULTI)
 		|| _is_multipart_message_read) {
@@ -414,7 +417,7 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
     // Notify observers
     //
     for (ObserverList::iterator i = _ol.begin(); i != _ol.end(); i++) {
-	(*i)->nlsock_data(&message[0], message.size());
+	(*i)->nlsock_data(message);
     }
 
     return (XORP_OK);
@@ -505,10 +508,11 @@ NetlinkSocket::force_recvmsg(int flags, bool only_kernel_messages,
 	//
 	bool is_end_of_message = true;
 	size_t new_size = off - last_mh_off;
+	AlignData<struct nlmsghdr> align_data(message);
 	const struct nlmsghdr* mh;
-	for (mh = reinterpret_cast<const struct nlmsghdr*>(&message[last_mh_off]);
+	for (mh = align_data.payload_by_offset(last_mh_off);
 	     NLMSG_OK(mh, new_size);
-	     mh = NLMSG_NEXT(const_cast<struct nlmsghdr*>(mh), new_size)) {
+	     mh = XORP_NLMSG_NEXT(const_cast<struct nlmsghdr*>(mh), new_size)) {
 	    XLOG_ASSERT(mh->nlmsg_len <= buffer.size());
 	    if ((mh->nlmsg_flags & NLM_F_MULTI)
 		|| _is_multipart_message_read) {
@@ -527,7 +531,7 @@ NetlinkSocket::force_recvmsg(int flags, bool only_kernel_messages,
     // Notify observers
     //
     for (ObserverList::iterator i = _ol.begin(); i != _ol.end(); i++) {
-	(*i)->nlsock_data(&message[0], message.size());
+	(*i)->nlsock_data(message);
     }
 
     return (XORP_OK);
@@ -637,37 +641,39 @@ NetlinkSocketReader::receive_data(NetlinkSocket& ns, uint32_t seqno,
  * has data to receive, therefore it should never be called directly by
  * anything else except the netlink socket facility itself.
  *
- * @param data the buffer with the received data.
- * @param nbytes the number of bytes in the @param data buffer.
+ * @param buffer the buffer with the received data.
  */
 void
-NetlinkSocketReader::nlsock_data(const uint8_t* data, size_t nbytes)
+NetlinkSocketReader::nlsock_data(const vector<uint8_t>& buffer)
 {
 #ifndef HAVE_NETLINK_SOCKETS
-    UNUSED(data);
-    UNUSED(nbytes);
+    UNUSED(buffer);
 
     XLOG_UNREACHABLE();
 
 #else // HAVE_NETLINK_SOCKETS
 
     size_t d = 0, off = 0;
+    AlignData<struct nlmsghdr> align_data(buffer);
 
     //
     // Copy data that has been requested to be cached by setting _cache_seqno
     //
-    _cache_data.resize(nbytes);
-    while (d < nbytes) {
+    _cache_data.resize(buffer.size());
+    while (d < buffer.size()) {
 	const struct nlmsghdr* nlh;
-	nlh = reinterpret_cast<const struct nlmsghdr*>(data + d);
+	nlh = align_data.payload_by_offset(d);
 	if ((nlh->nlmsg_seq == _cache_seqno)
 	    && (nlh->nlmsg_pid == _ns.nl_pid())) {
-	    XLOG_ASSERT(nbytes - d >= nlh->nlmsg_len);
+	    XLOG_ASSERT(buffer.size() - d >= nlh->nlmsg_len);
 	    memcpy(&_cache_data[off], nlh, nlh->nlmsg_len);
 	    off += nlh->nlmsg_len;
 	    _cache_valid = true;
 	}
 	d += nlh->nlmsg_len;
     }
+
+    // XXX: shrink _cache_data to contain only the data copied to it
+    _cache_data.resize(off);
 #endif // HAVE_NETLINK_SOCKETS
 }
