@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.13 2006/06/21 23:36:33 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.16 2007/02/16 22:45:51 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -28,6 +24,10 @@
 
 #include <set>
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 #include "libxipc/sockutil.hh"
 #include "libxipc/xrl_std_router.hh"
 
@@ -37,9 +37,6 @@
 #include "xrl_socket_server.hh"
 #include "addr_table.hh"
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 
 static const uint8_t FILLER_VALUE = 0xe7;
 
@@ -156,12 +153,12 @@ public:
 	_r = 0;
     }
 
-    inline const uint32_t bytes_received() const	{ return _b_rcv; }
-    inline const uint32_t bytes_sent() const		{ return _b_snd; }
-    inline const uint32_t packets_received() const	{ return _p_rcv; }
-    inline const uint32_t packets_sent() const		{ return _p_snd; }
-    inline const uint32_t xrl_errors() const		{ return _x_err; }
-    inline const bool	  closed() const		{ return _closed; }
+    inline uint32_t bytes_received() const	{ return _b_rcv; }
+    inline uint32_t bytes_sent() const		{ return _b_snd; }
+    inline uint32_t packets_received() const	{ return _p_rcv; }
+    inline uint32_t packets_sent() const	{ return _p_snd; }
+    inline uint32_t xrl_errors() const		{ return _x_err; }
+    inline bool	    closed() const		{ return _closed; }
 
     /**
      * Bind to interface and port.
