@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_rawsock6.cc,v 1.10 2006/06/15 06:04:36 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_rawsock6.cc,v 1.12 2007/02/16 22:45:53 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -118,7 +118,7 @@ public:
 	XrlAtomList ext_headers_type_list, ext_headers_payload_list;
 	size_t i;
 	for (i = 0; i < header.ext_headers_type.size(); i++) {
-	    ext_headers_type_list.append(XrlAtom(header.ext_headers_type[i]));
+	    ext_headers_type_list.append(XrlAtom(static_cast<uint32_t>(header.ext_headers_type[i])));
 	    ext_headers_payload_list.append(XrlAtom(header.ext_headers_payload[i]));
 	}
 
