@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_target.cc,v 1.82 2006/06/15 06:04:36 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_target.cc,v 1.85 2007/02/16 22:45:54 pavlin Exp $"
 
 #define PROFILE_UTILS_REQUIRED
 
@@ -1411,7 +1411,6 @@ XrlFeaTarget::ifmgr_0_1_set_interface_enabled(
     return _xifmgr.add(tid, new SetInterfaceEnabled(it, ifname, enabled));
 }
 
-
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_set_interface_discard(
 					      // Input values,
@@ -1423,7 +1422,6 @@ XrlFeaTarget::ifmgr_0_1_set_interface_discard(
     return _xifmgr.add(tid, new SetInterfaceDiscard(it, ifname, discard));
 }
 
-
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_set_mac(
 				// Input values,
@@ -1435,6 +1433,28 @@ XrlFeaTarget::ifmgr_0_1_set_mac(
     return _xifmgr.add(tid, new SetInterfaceMAC(it, ifname, mac));
 }
 
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_restore_original_mac(
+    // Input values,
+    const uint32_t&	tid,
+    const string&	ifname)
+{
+    IfTree& it = _xifmgr.iftree();
+
+    // Find the original MAC address
+    IfConfig& ifc = _xifmgr.ifconfig();
+    const IfTree& original_it = ifc.original_config();
+    IfTree::IfMap::const_iterator ii = original_it.get_if(ifname);
+    if (ii == original_it.ifs().end()) {
+	string error_msg = c_format("Interface %s does not exist.",
+				    ifname.c_str());
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+    const IfTreeInterface& fi = ii->second;
+    const Mac& mac = fi.mac();
+
+    return _xifmgr.add(tid, new SetInterfaceMAC(it, ifname, mac));
+}
 
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_set_mtu(
@@ -1447,6 +1467,29 @@ XrlFeaTarget::ifmgr_0_1_set_mtu(
     return _xifmgr.add(tid, new SetInterfaceMTU(it, ifname, mtu));
 }
 
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_restore_original_mtu(
+    // Input values,
+    const uint32_t&	tid,
+    const string&	ifname)
+{
+    IfTree& it = _xifmgr.iftree();
+
+    // Find the original MTU
+    IfConfig& ifc = _xifmgr.ifconfig();
+    const IfTree& original_it = ifc.original_config();
+    IfTree::IfMap::const_iterator ii = original_it.get_if(ifname);
+    if (ii == original_it.ifs().end()) {
+	string error_msg = c_format("Interface %s does not exist.",
+				    ifname.c_str());
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+    const IfTreeInterface& fi = ii->second;
+    uint32_t mtu = fi.mtu();
+
+    return _xifmgr.add(tid, new SetInterfaceMTU(it, ifname, mtu));
+}
+
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_create_vif(
 				   // Input values,
@@ -2008,6 +2051,74 @@ XrlFeaTarget::fti_0_2_set_unicast_forwarding_enabled6(
     return XrlCmdError::OKAY();
 }
 
+XrlCmdError
+XrlFeaTarget::fti_0_2_set_unicast_forwarding_entries_retain_on_startup4(
+    // Input values,
+    const bool&	retain)
+{
+    string error_msg;
+
+    if (_xftm.ftic().set_unicast_forwarding_entries_retain_on_startup4(
+	    retain,
+	    error_msg)
+	!= XORP_OK) {
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlFeaTarget::fti_0_2_set_unicast_forwarding_entries_retain_on_shutdown4(
+    // Input values,
+    const bool&	retain)
+{
+    string error_msg;
+
+    if (_xftm.ftic().set_unicast_forwarding_entries_retain_on_shutdown4(
+	    retain,
+	    error_msg)
+	!= XORP_OK) {
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlFeaTarget::fti_0_2_set_unicast_forwarding_entries_retain_on_startup6(
+    // Input values,
+    const bool&	retain)
+{
+    string error_msg;
+
+    if (_xftm.ftic().set_unicast_forwarding_entries_retain_on_startup6(
+	    retain,
+	    error_msg)
+	!= XORP_OK) {
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlFeaTarget::fti_0_2_set_unicast_forwarding_entries_retain_on_shutdown6(
+    // Input values,
+    const bool&	retain)
+{
+    string error_msg;
+
+    if (_xftm.ftic().set_unicast_forwarding_entries_retain_on_shutdown6(
+	    retain,
+	    error_msg)
+	!= XORP_OK) {
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+
+    return XrlCmdError::OKAY();
+}
+
 //
 // RIB routes redistribution transaction-based XRL interface
 //
