@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libproto/proto_state.cc,v 1.12 2006/03/16 00:04:12 pavlin Exp $"
+#ident "$XORP: xorp/libproto/proto_state.cc,v 1.14 2007/02/16 22:46:03 pavlin Exp $"
 
 
 //
@@ -177,7 +177,7 @@ ProtoState::shutdown()
 void
 ProtoState::disable()
 {
-    ProtoState::shutdown();
+    (void)ProtoState::shutdown();
     _flags &= ~XORP_ENABLED;
 }
 
