@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_client.cc,v 1.31 2006/03/16 00:04:14 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_client.cc,v 1.33 2007/02/16 22:46:04 pavlin Exp $"
 
 #include <functional>
 #include <algorithm>
@@ -562,7 +562,7 @@ public:
     inline const string& instance_name() const		{ return _ins_name; }
     inline const string& class_name() const		{ return _cls_name; }
     inline const XrlDispatcher* dispatcher() const	{ return _dispatcher; }
-    inline const uint32_t id() const			{ return _id; }
+    inline uint32_t id() const				{ return _id; }
 
 private:
     string		 _ins_name;
