@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_client_observer.cc,v 1.8 2006/03/16 00:04:14 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_client_observer.cc,v 1.10 2007/02/16 22:46:05 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <string>
+#include "libxorp/xorp.h"
 
 #include "finder_client_observer.hh"
 
+
 FinderClientObserver::~FinderClientObserver()
 {
 }
