@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,19 +12,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder.cc,v 1.20 2006/03/16 00:04:18 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxipc/test_finder.cc,v 1.22 2007/02/16 22:46:07 pavlin Exp $"
 
 #include "finder_module.h"
 
-#include "libxorp/eventloop.hh"
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
-#include "sockutil.hh"
+#include "libxorp/eventloop.hh"
+
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 
 #include "finder_server.hh"
 #include "finder_xrl_target.hh"
@@ -33,10 +32,8 @@
 #include "finder_client_xrl_target.hh"
 #include "finder_tcp_messenger.hh"
 #include "permits.hh"
+#include "sockutil.hh"
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 //
