@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_tcp.cc,v 1.20 2006/03/16 00:04:18 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxipc/test_finder_tcp.cc,v 1.22 2007/02/16 22:46:08 pavlin Exp $"
 
 #include "finder_module.h"
 
@@ -24,6 +20,10 @@
 #include "libxorp/xlog.h"
 #include "libxorp/xorpfd.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 #include "libcomm/comm_api.h"
 
 #include "sockutil.hh"
@@ -31,9 +31,6 @@
 #include "finder_constants.hh"
 #include "permits.hh"
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 
 ///////////////////////////////////////////////////////////////////////////////
 // Constants
