@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP"
+#ident "$XORP: xorp/libxipc/test_finder_to.cc,v 1.13 2007/02/16 22:46:08 pavlin Exp $"
+
+#include "finder_module.h"
 
 #include "libxorp/xorp.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
 #endif
 
 #include "finder_client.hh"
@@ -25,9 +27,6 @@
 #include "finder_tcp_messenger.hh"
 #include "finder_constants.hh"
 
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 
 //
 // A client process to help test the finder keepalive and time out
