@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_header.cc,v 1.7 2006/03/16 00:04:18 pavlin Exp $"
-
-#include <string>
-#include <stdio.h>
+#ident "$XORP: xorp/libxipc/test_header.cc,v 1.9 2007/02/16 22:46:08 pavlin Exp $"
 
 #include "xrl_module.h"
+#include "libxorp/xorp.h"
+
 #include "libxorp/xlog.h"
+
 #include "header.hh"
 
 void
