@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,27 +12,24 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_lemming.cc,v 1.17 2006/06/21 23:36:33 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_lemming.cc,v 1.19 2007/02/16 22:46:08 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#define XORP_MODULE_NAME "lemming"
+#include "ipc_module.h"
 
 #include "libxorp/xorp.h"
-#include "libxorp/eventloop.hh"
-#include "libxorp/exceptions.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/random.h"
-
-#include "libxipc/finder_server.hh"
-#include "libxipc/xrl_std_router.hh"
+#include "libxorp/eventloop.hh"
+#include "libxorp/exceptions.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "libxipc/finder_server.hh"
+#include "libxipc/xrl_std_router.hh"
+
+
 class verbose_ostream {
 public:
     verbose_ostream() : _verbose(false), _out(cout) {}
