@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,23 +12,22 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_xrl_args.cc,v 1.12 2006/03/16 00:04:19 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_xrl_args.cc,v 1.14 2007/02/16 22:46:09 pavlin Exp $"
 
 // test_xrl_args: String Serialization Tests
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "xrl_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
-#include "xrl_args.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "xrl_args.hh"
+
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Constants
