@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_xrl_sender.cc,v 1.16 2006/03/16 00:04:19 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_xrl_sender.cc,v 1.18 2007/02/16 22:46:09 pavlin Exp $"
 
 //
 // Test XRLs sender.
 //
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "xrl_module.h"
 
 #include "libxorp/xorp.h"
@@ -33,15 +29,17 @@
 #include "libxorp/exceptions.hh"
 #include "libxorp/status_codes.h"
 
-#include "libxipc/xrl_std_router.hh"
-
-#include "xrl/interfaces/test_xrls_xif.hh"
-#include "xrl/targets/test_xrls_base.hh"
-
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "libxipc/xrl_std_router.hh"
+
+#include "xrl/interfaces/test_xrls_xif.hh"
+
+#include "xrl/targets/test_xrls_base.hh"
+
+
 //
 // This is a sender program for testing XRL performance. It is used
 // as a pair with the "test_xrl_receiver" program.
