@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -14,7 +14,7 @@
 
 //#define DEBUG_LOGGING
 
-#ident "$XORP: xorp/libxipc/xrl_pf_stcp.cc,v 1.47 2006/06/21 23:36:33 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/xrl_pf_stcp.cc,v 1.53 2007/02/16 22:46:12 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
@@ -167,49 +167,48 @@ STCPRequestHandler::read_event(BufferedAsyncReader*		/* source */,
     }
 
     for (u_int iters = 0; iters < MAX_XRLS_DISPATCHED; iters++) {
-	if (buffer_bytes < sizeof(STCPPacketHeader)) {
+	if (buffer_bytes < STCPPacketHeader::header_size()) {
 	    // Not enough data to even inspect the header
-	    size_t new_trigger_bytes = sizeof(STCPPacketHeader) - buffer_bytes;
+	    size_t new_trigger_bytes = STCPPacketHeader::header_size() - buffer_bytes;
 	    _reader.set_trigger_bytes(new_trigger_bytes);
 	    return;
 	}
 
-	const STCPPacketHeader* sph =
-	    reinterpret_cast<const STCPPacketHeader*>(buffer);
+	const STCPPacketHeader sph(buffer);
 
-	if (!sph->is_valid()) {
+	if (!sph.is_valid()) {
 	    die("bad header");
 	    return;
 	}
 
-	if (sph->type() == STCP_PT_HELO) {
+	if (sph.type() == STCP_PT_HELO) {
 	    debug_msg("Got helo\n");
-	    ack_helo(sph->seqno());
-	    _reader.dispose(sph->frame_bytes());
-	    _reader.set_trigger_bytes(sizeof(STCPPacketHeader));
+	    ack_helo(sph.seqno());
+	    _reader.dispose(sph.frame_bytes());
+	    _reader.set_trigger_bytes(STCPPacketHeader::header_size());
 	    return;
-	} else if (sph->type() != STCP_PT_REQUEST) {
+	} else if (sph.type() != STCP_PT_REQUEST) {
 	    die("Bad packet type");
 	    return;
 	}
 
-	if (buffer_bytes >= sph->frame_bytes()) {
+	if (buffer_bytes >= sph.frame_bytes()) {
 	    uint8_t* xrl_data = buffer;
-	    xrl_data += sizeof(STCPPacketHeader) + sph->error_note_bytes();
-	    size_t   xrl_data_bytes = sph->payload_bytes();
-	    dispatch_request(sph->seqno(), xrl_data, xrl_data_bytes);
-	    _reader.dispose(sph->frame_bytes());
-	    buffer += sph->frame_bytes();
-	    buffer_bytes -= sph->frame_bytes();
+	    xrl_data += STCPPacketHeader::header_size() + sph.error_note_bytes();
+	    size_t   xrl_data_bytes = sph.payload_bytes();
+	    dispatch_request(sph.seqno(), xrl_data, xrl_data_bytes);
+	    _reader.dispose(sph.frame_bytes());
+	    buffer += sph.frame_bytes();
+	    buffer_bytes -= sph.frame_bytes();
 	} else {
-	    if (sph->frame_bytes() > _reader.reserve_bytes()) {
-		_reader.set_reserve_bytes(sph->frame_bytes());
+	    if (sph.frame_bytes() > _reader.reserve_bytes()) {
+		_reader.set_reserve_bytes(sph.frame_bytes());
 	    }
-	    _reader.set_trigger_bytes(sph->frame_bytes());
+	    _reader.set_trigger_bytes(sph.frame_bytes());
 	    return;
 	}
     }
-    _reader.set_trigger_bytes(sizeof(STCPPacketHeader));
+    _reader.set_trigger_bytes(STCPPacketHeader::header_size());
 }
 
 void
@@ -241,20 +240,20 @@ STCPRequestHandler::dispatch_request(uint32_t 		seqno,
     size_t xrl_response_bytes = response.packed_bytes();
     size_t note_bytes = e.note().size();
 
-    _responses.push_back(ReplyPacket(sizeof(STCPPacketHeader) + note_bytes + xrl_response_bytes));
+    _responses.push_back(ReplyPacket(STCPPacketHeader::header_size() + note_bytes + xrl_response_bytes));
     _responses_size++;
     ReplyPacket& r = _responses.back();
 
-    STCPPacketHeader* sph = reinterpret_cast<STCPPacketHeader*>(&r[0]);
-    sph->initialize(seqno, STCP_PT_RESPONSE, e, xrl_response_bytes);
+    STCPPacketHeader sph(&r[0]);
+    sph.initialize(seqno, STCP_PT_RESPONSE, e, xrl_response_bytes);
 
     if (note_bytes != 0) {
-	memcpy(&r[0] + sizeof(STCPPacketHeader),
+	memcpy(&r[0] + STCPPacketHeader::header_size(),
 	       e.note().c_str(), note_bytes);
     }
 
     if (xrl_response_bytes != 0) {
-	response.pack(&r[0] + sizeof(STCPPacketHeader) + note_bytes,
+	response.pack(&r[0] + STCPPacketHeader::header_size() + note_bytes,
 		      xrl_response_bytes);
     }
 
@@ -270,12 +269,12 @@ STCPRequestHandler::dispatch_request(uint32_t 		seqno,
 void
 STCPRequestHandler::ack_helo(uint32_t seqno)
 {
-    _responses.push_back(ReplyPacket(sizeof(STCPPacketHeader)));
+    _responses.push_back(ReplyPacket(STCPPacketHeader::header_size()));
     _responses_size++;
     ReplyPacket& r = _responses.back();
 
-    STCPPacketHeader* sph = reinterpret_cast<STCPPacketHeader*>(&r[0]);
-    sph->initialize(seqno, STCP_PT_HELO_ACK, XrlError::OKAY(), 0);
+    STCPPacketHeader sph(&r[0]);
+    sph.initialize(seqno, STCP_PT_HELO_ACK, XrlError::OKAY(), 0);
     _writer.add_buffer(&r[0], r.size(),
 		       callback(this, &STCPRequestHandler::update_writer));
 
@@ -454,13 +453,13 @@ public:
 		 const Callback&  cb)
 	: _p(p), _sn(sn), _cb(cb), _keepalive(false)
     {
-	size_t header_bytes = sizeof(STCPPacketHeader);
+	size_t header_bytes = STCPPacketHeader::header_size();
 	size_t xrl_bytes = x.packed_bytes();
 	_b.resize(header_bytes + xrl_bytes);
 
 	// Prepare header
-	STCPPacketHeader* sph = reinterpret_cast<STCPPacketHeader*>(&_b[0]);
-	sph->initialize(_sn, STCP_PT_REQUEST, XrlError::OKAY(), xrl_bytes);
+	STCPPacketHeader sph(&_b[0]);
+	sph.initialize(_sn, STCP_PT_REQUEST, XrlError::OKAY(), xrl_bytes);
 
 	// Pack XRL data
 	x.pack(&_b[0] + header_bytes, xrl_bytes);
@@ -472,12 +471,12 @@ public:
     RequestState(XrlPFSTCPSender* p, uint32_t sn)
 	: _p(p), _sn(sn), _keepalive(true)
     {
-	size_t header_bytes = sizeof(STCPPacketHeader);
+	size_t header_bytes = STCPPacketHeader::header_size();
 	_b.resize(header_bytes);
 
 	// Prepare header
-	STCPPacketHeader* sph = reinterpret_cast<STCPPacketHeader*>(&_b[0]);
-	sph->initialize(_sn, STCP_PT_HELO, XrlError::OKAY(), 0);
+	STCPPacketHeader sph(&_b[0]);
+	sph.initialize(_sn, STCP_PT_HELO, XrlError::OKAY(), 0);
     }
 
     inline bool		    has_seqno(uint32_t n) const { return _sn == n; }
@@ -488,10 +487,10 @@ public:
 
     inline bool is_keepalive()
     {
-	if (_b.size() < sizeof(STCPPacketHeader))
+	if (_b.size() < STCPPacketHeader::header_size())
 	    return false;
-	STCPPacketHeader* sph = reinterpret_cast<STCPPacketHeader*>(&_b[0]);
-	STCPPacketType pt = sph->type();
+	const STCPPacketHeader sph(&_b[0]);
+	STCPPacketType pt = sph.type();
 	return pt == STCP_PT_HELO || pt == STCP_PT_HELO_ACK;
     }
 
@@ -597,7 +596,7 @@ XrlPFSTCPSender::XrlPFSTCPSender(EventLoop& e, const char* addr_slash_port)
 				      callback(this,
 					       &XrlPFSTCPSender::read_event));
 
-    _reader->set_trigger_bytes(sizeof(STCPPacketHeader));
+    _reader->set_trigger_bytes(STCPPacketHeader::header_size());
     _reader->start();
 
     _writer = new AsyncFileWriter(e, _sock, MAX_WRITES);
@@ -777,7 +776,7 @@ XrlPFSTCPSender::read_event(BufferedAsyncReader*	/* reader */,
 {
     debug_msg("read event %u (need at least %u)\n",
 		XORP_UINT_CAST(buffer_bytes),
-		XORP_UINT_CAST(sizeof(STCPPacketHeader)));
+		XORP_UINT_CAST(STCPPacketHeader::header_size()));
 
     if (ev == BufferedAsyncReader::OS_ERROR) {
 	XLOG_ERROR("Read failed (error = %d)\n", _reader->error());
@@ -792,59 +791,58 @@ XrlPFSTCPSender::read_event(BufferedAsyncReader*	/* reader */,
 
     defer_keepalives();
 
-    if (buffer_bytes < sizeof(STCPPacketHeader)) {
+    if (buffer_bytes < STCPPacketHeader::header_size()) {
 	// Not enough data to even inspect the header
-	size_t new_trigger_bytes = sizeof(STCPPacketHeader) - buffer_bytes;
+	size_t new_trigger_bytes = STCPPacketHeader::header_size() - buffer_bytes;
 	_reader->set_trigger_bytes(new_trigger_bytes);
 	return;
     }
 
-    const STCPPacketHeader* sph =
-	reinterpret_cast<const STCPPacketHeader*>(buffer);
+    const STCPPacketHeader sph(buffer);
 
-    if (sph->is_valid() == false) {
+    if (sph.is_valid() == false) {
 	die("bad header");
 	return;
     }
 
-    if (sph->seqno() != _requests_sent.front()->seqno()) {
+    if (sph.seqno() != _requests_sent.front()->seqno()) {
 	die("Bad sequence number");
 	return;
     }
 
-    if (sph->type() == STCP_PT_HELO_ACK) {
+    if (sph.type() == STCP_PT_HELO_ACK) {
 	debug_msg("Got keep alive ack\n");
 	_keepalive_sent = false;
 	dispose_request();
-	_reader->dispose(sph->frame_bytes());
-	_reader->set_trigger_bytes(sizeof(*sph));
+	_reader->dispose(sph.frame_bytes());
+	_reader->set_trigger_bytes(sph.header_size());
 	return;
     }
 
-    if (sph->type() != STCP_PT_RESPONSE) {
+    if (sph.type() != STCP_PT_RESPONSE) {
 	die("unexpected packet type - not a response");
     }
 
     debug_msg("Frame Bytes %u Available %u\n",
-	      XORP_UINT_CAST(sph->frame_bytes()),
+	      XORP_UINT_CAST(sph.frame_bytes()),
 	      XORP_UINT_CAST(buffer_bytes));
-    if (sph->frame_bytes() > buffer_bytes) {
-	if (_reader->reserve_bytes() < sph->frame_bytes())
-	    _reader->set_reserve_bytes(sph->frame_bytes());
-	_reader->set_trigger_bytes(sph->frame_bytes() - buffer_bytes);
+    if (sph.frame_bytes() > buffer_bytes) {
+	if (_reader->reserve_bytes() < sph.frame_bytes())
+	    _reader->set_reserve_bytes(sph.frame_bytes());
+	_reader->set_trigger_bytes(sph.frame_bytes() - buffer_bytes);
 	return;
     }
 
-    const uint8_t* xrl_data = buffer + sizeof(STCPPacketHeader);
+    const uint8_t* xrl_data = buffer + STCPPacketHeader::header_size();
 
     XrlError xrl_error;
-    if (sph->error_note_bytes()) {
-	xrl_error = XrlError(XrlErrorCode(sph->error_code()),
+    if (sph.error_note_bytes()) {
+	xrl_error = XrlError(XrlErrorCode(sph.error_code()),
 			     string((const char*)xrl_data,
-				    sph->error_note_bytes()));
-	xrl_data += sph->error_note_bytes();
+				    sph.error_note_bytes()));
+	xrl_data += sph.error_note_bytes();
     } else {
-	xrl_error = XrlError(XrlErrorCode(sph->error_code()));
+	xrl_error = XrlError(XrlErrorCode(sph.error_code()));
     }
 
     // Get ref_ptr to callback from request state and discard the rest
@@ -857,8 +855,8 @@ XrlPFSTCPSender::read_event(BufferedAsyncReader*	/* reader */,
     XrlArgs  xa;
     XrlArgs* xap = NULL;
     try {
-	if (sph->payload_bytes() > 0) {
-	    xa.unpack(xrl_data, sph->payload_bytes());
+	if (sph.payload_bytes() > 0) {
+	    xa.unpack(xrl_data, sph.payload_bytes());
 	    xap = &xa;
 	}
     } catch (...) {
@@ -870,8 +868,8 @@ XrlPFSTCPSender::read_event(BufferedAsyncReader*	/* reader */,
 
     // Update reader to say we're done with this block of data and
     // specify minimum reserve for next call.
-    _reader->dispose(sph->frame_bytes());
-    _reader->set_trigger_bytes(sizeof(STCPPacketHeader));
+    _reader->dispose(sph.frame_bytes());
+    _reader->set_trigger_bytes(STCPPacketHeader::header_size());
 
     // Dispatch Xrl and exit
     cb->dispatch(xrl_error, xap);
@@ -898,8 +896,10 @@ XrlPFSTCPSender::set_keepalive_ms(uint32_t t)
 void
 XrlPFSTCPSender::start_keepalives()
 {
-    _keepalive_timer = _eventloop.new_periodic_ms(_keepalive_ms,
-						  callback(this, &XrlPFSTCPSender::send_keepalive));
+    _keepalive_timer = _eventloop.new_periodic_ms(
+	_keepalive_ms,
+	callback(this, &XrlPFSTCPSender::send_keepalive),
+	XorpTask::PRIORITY_XRL_KEEPALIVE);
 }
 
 void
@@ -912,7 +912,8 @@ void
 XrlPFSTCPSender::defer_keepalives()
 {
     if (_keepalive_timer.scheduled()) {
-	_keepalive_timer.schedule_after_ms(_keepalive_ms);
+	_keepalive_timer.schedule_after_ms(_keepalive_ms,
+					   XorpTask::PRIORITY_XRL_KEEPALIVE);
     }
 }
 
