@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,37 +13,29 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/asyncio.cc,v 1.25 2006/03/16 00:04:25 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxorp/asyncio.cc,v 1.31 2007/02/16 22:46:15 pavlin Exp $"
 
 #include "libxorp_module.h"
-#include "xorp.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/debug.h"
+#include "libxorp/xlog.h"
+#include "libxorp/eventloop.hh"
 
 #include <signal.h>
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
+
 #ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
 #endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
-#include "libxorp/debug.h"
-#include "libxorp/eventloop.hh"
-#include "libxorp/xlog.h"
 #include "xorpfd.hh"
 #include "asyncio.hh"
 
 #ifdef HOST_OS_WINDOWS
- #define EDGE_TRIGGERED_READ_LATENCY	// IOT_READ may be delayed.
- #define EDGE_TRIGGERED_WRITES		// IOT_WRITE is edge triggered.
- #include "win_dispatcher.hh"
- #include "win_io.h"
+#   define EDGE_TRIGGERED_READ_LATENCY		// IOT_READ may be delayed.
+#   define EDGE_TRIGGERED_WRITES		// IOT_WRITE is edge triggered.
+#   include "win_dispatcher.hh"
+#   include "win_io.h"
 #endif
 
 
@@ -220,10 +212,9 @@ AsyncFileReader::read(XorpFd fd, IoEventType type)
 	u_long remaining = 0;
 	int result = ioctlsocket(_fd, FIONREAD, &remaining);
 	if (result != SOCKET_ERROR && remaining > 0) {
-	    _idle_timer = _eventloop.new_oneoff_after_ms(
-		0,
+	    _deferred_io_task = _eventloop.new_oneoff_task(
 		callback(this, &AsyncFileReader::read, _fd, IOT_READ));
-	    XLOG_ASSERT(_idle_timer.scheduled());
+	    XLOG_ASSERT(_deferred_io_task.scheduled());
 	}
    }
 #endif // EDGE_TRIGGERED_READ_LATENCY
@@ -282,7 +273,8 @@ AsyncFileReader::start()
 
     EventLoop& e = _eventloop;
     if (e.add_ioevent_cb(_fd, IOT_READ,
-		       callback(this, &AsyncFileReader::read)) == false) {
+			 callback(this, &AsyncFileReader::read),
+			 _priority) == false) {
 	XLOG_ERROR("AsyncFileReader: Failed to add ioevent callback.");
 	return false;
     }
@@ -296,7 +288,8 @@ AsyncFileReader::start()
 	_disconnect_added = e.add_ioevent_cb(
 	    _fd,
 	    IOT_DISCONNECT,
-	    callback(this, &AsyncFileReader::disconnect));
+	    callback(this, &AsyncFileReader::disconnect),
+	    _priority);
 	if (_disconnect_added == false) {
 	    XLOG_ERROR("AsyncFileReader: Failed to add ioevent callback.");
 	    _eventloop.remove_ioevent_cb(_fd, IOT_READ);
@@ -319,8 +312,7 @@ AsyncFileReader::stop()
     _eventloop.remove_ioevent_cb(_fd, IOT_READ);
 
 #ifdef EDGE_TRIGGERED_WRITES
-    _idle_timer.unschedule();
-    _idle_timer.clear();
+    _deferred_io_task.unschedule();
 #endif
 #ifdef HOST_OS_WINDOWS
     if (_disconnect_added == true) {
@@ -352,8 +344,9 @@ AsyncFileReader::flush_buffers()
 #define MAX_IOVEC 16
 #endif
 
-AsyncFileWriter::AsyncFileWriter(EventLoop& e, XorpFd fd, uint32_t coalesce)
-    : AsyncFileOperator(e, fd)
+AsyncFileWriter::AsyncFileWriter(EventLoop& e, XorpFd fd, uint32_t coalesce,
+				 int priority)
+    : AsyncFileOperator(e, fd, priority)
 {
     static const uint32_t max_coalesce = 16;
     _coalesce = (coalesce > MAX_IOVEC) ? MAX_IOVEC : coalesce;
@@ -380,10 +373,9 @@ AsyncFileWriter::add_buffer(const uint8_t*	b,
     _buffers.push_back(BufferInfo(b, b_bytes, cb));
 #ifdef EDGE_TRIGGERED_WRITES
     if (_running == true) {
-	_idle_timer = _eventloop.new_oneoff_after_ms(
-	    0,
+	_deferred_io_task = _eventloop.new_oneoff_task(
 	    callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
-	XLOG_ASSERT(_idle_timer.scheduled());
+	XLOG_ASSERT(_deferred_io_task.scheduled());
     }
 #endif // EDGE_TRIGGERED_WRITES
 }
@@ -398,10 +390,9 @@ AsyncFileWriter::add_buffer_with_offset(const uint8_t*	b,
     _buffers.push_back(BufferInfo(b, b_bytes, off, cb));
 #ifdef EDGE_TRIGGERED_WRITES
     if (_running == true) {
-	_idle_timer = _eventloop.new_oneoff_after_ms(
-	    0,
+	_deferred_io_task = _eventloop.new_oneoff_task(
 	    callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
-	XLOG_ASSERT(_idle_timer.scheduled());
+	XLOG_ASSERT(_deferred_io_task.scheduled());
     }
 #endif // EDGE_TRIGGERED_WRITES
 }
@@ -526,10 +517,9 @@ AsyncFileWriter::write(XorpFd fd, IoEventType type)
 
 #ifdef EDGE_TRIGGERED_WRITES
     if (_buffers.empty() == false) {
-	_idle_timer = _eventloop.new_oneoff_after_ms(
-	    0,
+	_deferred_io_task = _eventloop.new_oneoff_task(
 	    callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
-	XLOG_ASSERT(_idle_timer.scheduled());
+	XLOG_ASSERT(_deferred_io_task.scheduled());
     }
 #endif // EDGE_TRIGGERED_WRITES
 }
@@ -629,7 +619,8 @@ AsyncFileWriter::start()
 
     EventLoop& e = _eventloop;
     if (e.add_ioevent_cb(_fd, IOT_WRITE,
-			 callback(this, &AsyncFileWriter::write)) == false) {
+			 callback(this, &AsyncFileWriter::write),
+			 _priority) == false) {
 	XLOG_ERROR("AsyncFileWriter: Failed to add I/O event callback.");
 	return false;
     }
@@ -645,7 +636,8 @@ AsyncFileWriter::start()
 	_disconnect_added = e.add_ioevent_cb(
 	    _fd,
 	    IOT_DISCONNECT,
-	    callback(this, &AsyncFileWriter::disconnect));
+	    callback(this, &AsyncFileWriter::disconnect),
+	    _priority);
 #if 0
 	if (_disconnect_added == false) {
 	    XLOG_ERROR("AsyncFileWriter: Failed to add I/O event callback.");
@@ -658,8 +650,7 @@ AsyncFileWriter::start()
 #endif // HOST_OS_WINDOWS
 
 #ifdef EDGE_TRIGGERED_WRITES
-    _idle_timer = _eventloop.new_oneoff_after_ms(
-	0,
+    _deferred_io_task = _eventloop.new_oneoff_task(
 	callback(this, &AsyncFileWriter::write, _fd, IOT_WRITE));
 #endif // EDGE_TRIGGERED_WRITES
 
@@ -674,8 +665,7 @@ AsyncFileWriter::stop()
     debug_msg("%p stop\n", this);
 
 #ifdef EDGE_TRIGGERED_WRITES
-    _idle_timer.unschedule();
-    _idle_timer.clear();
+    _deferred_io_task.unschedule();
 #endif // EDGE_TRIGGERED_WRITES
     _eventloop.remove_ioevent_cb(_fd, IOT_WRITE);
 #ifdef HOST_OS_WINDOWS
