@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/callback.cc,v 1.8 2006/03/16 00:04:26 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/callback.cc,v 1.10 2007/02/16 22:46:16 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp_module.h"
+
+#include "libxorp/xorp.h"
 
 /*
  * This code is only compiled if DEBUG_CALLBACKS is defined in
@@ -30,16 +30,14 @@
 
 #ifdef DEBUG_CALLBACKS
 
+#include "libxorp/xlog.h"
+#include "libxorp/callback.hh"
+#include "libxorp/c_format.hh"
+#include "libxorp/timeval.hh"
+#include "libxorp/timer.hh"
+
 #include <stack>
 
-#include "libxorp_module.h"
-#include "xlog.h"
-#include "xorp.h"
-
-#include "callback.hh"
-#include "c_format.hh"
-#include "timeval.hh"
-#include "timer.hh"
 
 /**
  * Maximum callback dispatch duration.
