@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2006 International Computer Science Institute
+ * Copyright (c) 2001-2007 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,13 +15,9 @@
  */
 
 /*
- * $XORP: xorp/libxorp/gai_strerror.c,v 1.4 2006/03/16 00:04:29 pavlin Exp $
+ * $XORP: xorp/libxorp/gai_strerror.c,v 1.6 2007/02/16 22:46:19 pavlin Exp $
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "xorp.h"
 
 #ifdef HOST_OS_WINDOWS
