@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,27 +12,25 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/ipv4.cc,v 1.22 2006/06/06 01:40:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/ipv4.cc,v 1.29 2007/02/16 22:46:19 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include "xorp.h"
-#include "ipv4.hh"
+#include "libxorp/xorp.h"
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
+
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
 
+#include "ipv4.hh"
+
+
 IPv4::IPv4(const uint8_t *from_uint8)
 {
     memcpy(&_addr, from_uint8, sizeof(_addr));
@@ -73,8 +71,8 @@ IPv4::IPv4(const char *from_cstring) throw (InvalidString)
 size_t
 IPv4::copy_out(uint8_t *to_uint8) const
 {
-    memcpy(to_uint8, &_addr, addr_size());
-    return addr_size();
+    memcpy(to_uint8, &_addr, addr_bytelen());
+    return addr_bytelen();
 }
 
 /**
@@ -122,8 +120,8 @@ IPv4::copy_out(struct sockaddr_in& to_sockaddr_in) const
 size_t
 IPv4::copy_in(const uint8_t *from_uint8)
 {
-    memcpy(&_addr, from_uint8, addr_size());
-    return (addr_size());
+    memcpy(&_addr, from_uint8, addr_bytelen());
+    return (addr_bytelen());
 }
 
 /**
@@ -248,7 +246,7 @@ IPv4::is_unicast() const
 
     return (! (IN_MULTICAST(addr4)
 	       || IN_BADCLASS(addr4)
-	       || (addr4 & 0xff000000U) == 0));
+	       || (addr4 == 0)));
 }
 
 bool
@@ -259,6 +257,44 @@ IPv4::is_multicast() const
     return (IN_MULTICAST(addr4));
 }
 
+bool
+IPv4::is_class_a() const
+{
+    uint32_t addr4 = ntohl(_addr);
+
+    return (IN_CLASSA(addr4));
+}
+
+bool
+IPv4::is_class_b() const
+{
+    uint32_t addr4 = ntohl(_addr);
+
+    return (IN_CLASSB(addr4));
+}
+
+bool
+IPv4::is_class_c() const
+{
+    uint32_t addr4 = ntohl(_addr);
+
+    return (IN_CLASSC(addr4));
+}
+
+bool
+IPv4::is_experimental() const
+{
+    uint32_t addr4 = ntohl(_addr);
+
+    //
+    // XXX: We use IN_BADCLASS() instead of IN_EXPERIMENTAL(), because
+    // the definition of IN_EXPERIMENTAL() is broken in Linux's <netinet/in.h>
+    // (it covers all addresses that start with 0xe0000000, which includes
+    // multicast as well).
+    //
+    return (IN_BADCLASS(addr4));
+}
+
 // XXX: in IPv4 there is no link-local unicast scope, therefore
 // the return value is always false.
 bool
@@ -267,10 +303,10 @@ IPv4::is_linklocal_unicast() const
     return (false);
 }
 
-// XXX: in IPv4 there is no node-local multicast scope, therefore
+// XXX: in IPv4 there is no interface-local multicast scope, therefore
 // the return value is always false.
 bool
-IPv4::is_nodelocal_multicast() const
+IPv4::is_interfacelocal_multicast() const
 {
     return (false);
 }
@@ -311,3 +347,7 @@ const IPv4 IPv4Constants::ospfigp_designated_routers(IPv4(htonl_literal(0xe00000
 const IPv4 IPv4Constants::rip2_routers(IPv4(htonl_literal(0xe0000009U)));
 const IPv4 IPv4Constants::pim_routers(IPv4(htonl_literal(0xe000000dU)));
 const IPv4 IPv4Constants::ssm_routers(IPv4(htonl_literal(0xe0000016U)));
+const IPv4 IPv4Constants::class_a_base(IPv4(htonl_literal(0x00000000U)));
+const IPv4 IPv4Constants::class_b_base(IPv4(htonl_literal(0x80000000U)));
+const IPv4 IPv4Constants::class_c_base(IPv4(htonl_literal(0xc0000000U)));
+const IPv4 IPv4Constants::experimental_base(IPv4(htonl_literal(0xf0000000U)));
