@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/ipvx.cc,v 1.21 2006/06/06 01:40:24 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/ipvx.cc,v 1.28 2007/02/16 22:46:20 pavlin Exp $"
 
 #include "xorp.h"
 #include "ipvx.hh"
@@ -52,7 +52,7 @@ IPvX::IPvX(int family, const uint8_t *from_uint8) throw (InvalidFamily)
 
     _af = family;
     memset(_addr, 0, sizeof(_addr));
-    memcpy(_addr, from_uint8, addr_size());
+    memcpy(_addr, from_uint8, addr_bytelen());
 }
 
 IPvX::IPvX(const IPv4& ipv4)
@@ -279,8 +279,8 @@ IPvX::mask_len() const
 size_t
 IPvX::copy_out(uint8_t *to_uint8) const
 {
-    memcpy(to_uint8, _addr, addr_size());
-    return (addr_size());
+    memcpy(to_uint8, _addr, addr_bytelen());
+    return (addr_bytelen());
 }
 
 /**
@@ -365,7 +365,27 @@ IPvX::copy_out(struct sockaddr_in6& to_sockaddr_in6) const
 	to_sockaddr_in6.sin6_len = sizeof(to_sockaddr_in6);
 #endif
 	to_sockaddr_in6.sin6_family = _af;
+
+#ifdef HAVE_SIN6_SCOPE_ID
+#ifdef IPV6_STACK_KAME
+	//
+	// XXX: In case of KAME the local interface index (also the link-local
+	// scope_id) is encoded in the third and fourth octet of an IPv6
+	// address (for link-local unicast/multicast addresses or
+	// interface-local multicast addresses only).
+	//
+	if (is_linklocal_unicast()
+	    || is_linklocal_multicast()
+	    || is_interfacelocal_multicast()) {
+	    uint32_t addr0 = ntohl(_addr[0]);
+	    uint16_t zoneid = (addr0 & 0xffff);		// XXX: 16 bits only
+	    to_sockaddr_in6.sin6_scope_id = zoneid;
+	}
+#endif // IPV6_STACK_KAME
+#endif // HAVE_SIN6_SCOPE_ID
+
 	return (copy_out(to_sockaddr_in6.sin6_addr));
+
     default:
 	xorp_throw(InvalidFamily, _af);
     }
@@ -386,8 +406,8 @@ IPvX::copy_in(int family, const uint8_t *from_uint8) throw (InvalidFamily)
     case AF_INET:
 	// FALLTHROUGH
     case AF_INET6:
-	memcpy(_addr, from_uint8, addr_size());
-	return (addr_size());
+	memcpy(_addr, from_uint8, addr_bytelen());
+	return (addr_bytelen());
     default:
 	xorp_throw(InvalidFamily, _af);
     }
@@ -489,6 +509,38 @@ IPvX::is_multicast() const
     return get_ipv6().is_multicast();
 }
 
+bool
+IPvX::is_class_a() const
+{
+    if (_af == AF_INET)
+	return get_ipv4().is_class_a();
+    return (false);		// XXX: this method applies only for IPv4
+}
+
+bool
+IPvX::is_class_b() const
+{
+    if (_af == AF_INET)
+	return get_ipv4().is_class_b();
+    return (false);		// XXX: this method applies only for IPv4
+}
+
+bool
+IPvX::is_class_c() const
+{
+    if (_af == AF_INET)
+	return get_ipv4().is_class_c();
+    return (false);		// XXX: this method applies only for IPv4
+}
+
+bool
+IPvX::is_experimental() const
+{
+    if (_af == AF_INET)
+	return get_ipv4().is_experimental();
+    return (false);		// XXX: this method applies only for IPv4
+}
+
 bool
 IPvX::is_linklocal_unicast() const
 {
@@ -498,11 +550,11 @@ IPvX::is_linklocal_unicast() const
 }
 
 bool
-IPvX::is_nodelocal_multicast() const
+IPvX::is_interfacelocal_multicast() const
 {
     if (_af == AF_INET)
-	return get_ipv4().is_nodelocal_multicast();
-    return get_ipv6().is_nodelocal_multicast();
+	return get_ipv4().is_interfacelocal_multicast();
+    return get_ipv6().is_interfacelocal_multicast();
 }
 
 bool
@@ -550,12 +602,12 @@ IPvX::ip_version_str() const throw (InvalidFamily)
 }
 
 size_t
-IPvX::addr_size(int family) throw (InvalidFamily)
+IPvX::addr_bytelen(int family) throw (InvalidFamily)
 {
     if (family == AF_INET)
-	return (IPv4::addr_size());
+	return (IPv4::addr_bytelen());
     if (family == AF_INET6)
-	return (IPv6::addr_size());
+	return (IPv6::addr_bytelen());
     xorp_throw(InvalidFamily, family);
 
     return ((size_t)-1);
@@ -573,6 +625,50 @@ IPvX::ip_multicast_base_address_mask_len(int family) throw (InvalidFamily)
     return ((uint32_t)-1);
 }
 
+uint32_t
+IPvX::ip_class_a_base_address_mask_len(int family) throw (InvalidFamily)
+{
+    if (family == AF_INET)
+	return (IPv4::ip_class_a_base_address_mask_len());
+    // XXX: this method applies only for IPv4
+
+    xorp_throw(InvalidFamily, family);
+    return ((uint32_t)-1);
+}
+
+uint32_t
+IPvX::ip_class_b_base_address_mask_len(int family) throw (InvalidFamily)
+{
+    if (family == AF_INET)
+	return (IPv4::ip_class_b_base_address_mask_len());
+    // XXX: this method applies only for IPv4
+
+    xorp_throw(InvalidFamily, family);
+    return ((uint32_t)-1);
+}
+
+uint32_t
+IPvX::ip_class_c_base_address_mask_len(int family) throw (InvalidFamily)
+{
+    if (family == AF_INET)
+	return (IPv4::ip_class_c_base_address_mask_len());
+    // XXX: this method applies only for IPv4
+
+    xorp_throw(InvalidFamily, family);
+    return ((uint32_t)-1);
+}
+
+uint32_t
+IPvX::ip_experimental_base_address_mask_len(int family) throw (InvalidFamily)
+{
+    if (family == AF_INET)
+	return (IPv4::ip_experimental_base_address_mask_len());
+    // XXX: this method applies only for IPv4
+
+    xorp_throw(InvalidFamily, family);
+    return ((uint32_t)-1);
+}
+
 //
 // IPvX "Constants"
 //
@@ -588,6 +684,26 @@ const IPvX& IPvX::name(int family) throw (InvalidFamily)		      \
     xorp_throw(InvalidFamily, family);					      \
 }
 
+// IPvX "Constant" that applies only for IPv4
+#define IPVX_CONSTANT_ACCESSOR_IPV4(name)				      \
+const IPvX& IPvX::name(int family) throw (InvalidFamily)		      \
+{									      \
+    static const IPvX c4_##name (IPv4::name());	/* IPv4 constant */	      \
+    if (family == AF_INET)						      \
+	return c4_##name;						      \
+    xorp_throw(InvalidFamily, family);					      \
+}
+
+// IPvX "Constant" that applies only for IPv6
+#define IPVX_CONSTANT_ACCESSOR_IPV6(name)				      \
+const IPvX& IPvX::name(int family) throw (InvalidFamily)		      \
+{									      \
+    static const IPvX c6_##name (IPv6::name());	/* IPv6 constant */	      \
+    if (family == AF_INET6)						      \
+	return c6_##name;						      \
+    xorp_throw(InvalidFamily, family);					      \
+}
+
 IPVX_CONSTANT_ACCESSOR(ZERO);
 IPVX_CONSTANT_ACCESSOR(ANY);
 IPVX_CONSTANT_ACCESSOR(ALL_ONES);
@@ -601,3 +717,7 @@ IPVX_CONSTANT_ACCESSOR(OSPFIGP_DESIGNATED_ROUTERS);
 IPVX_CONSTANT_ACCESSOR(RIP2_ROUTERS);
 IPVX_CONSTANT_ACCESSOR(PIM_ROUTERS);
 IPVX_CONSTANT_ACCESSOR(SSM_ROUTERS);
+IPVX_CONSTANT_ACCESSOR_IPV4(CLASS_A_BASE);
+IPVX_CONSTANT_ACCESSOR_IPV4(CLASS_B_BASE);
+IPVX_CONSTANT_ACCESSOR_IPV4(CLASS_C_BASE);
+IPVX_CONSTANT_ACCESSOR_IPV4(EXPERIMENTAL_BASE);
