@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,18 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/ref_ptr.cc,v 1.11 2006/03/16 00:04:32 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/ref_ptr.cc,v 1.13 2007/02/16 22:46:22 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
-#include <assert.h>
-#include <iostream>
-
-#include "xorp.h"
 #include "ref_ptr.hh"
 
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Debugging macro's
