@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,37 +13,36 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-// $XORP: xorp/libxorp/run_command.cc,v 1.26 2006/04/02 04:31:14 pavlin Exp $
+// $XORP: xorp/libxorp/run_command.cc,v 1.31 2007/02/16 22:46:22 pavlin Exp $
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "libxorp_module.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/debug.h"
+#include "libxorp/xlog.h"
+#include "libxorp/eventloop.hh"
+#include "libxorp/exceptions.hh"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/asyncio.hh"
+#include "libxorp/popen.hh"
+#ifdef HOST_OS_WINDOWS
+#include "libxorp/win_io.h"
+#endif
+
+#include <map>
+
+#ifdef HAVE_PATHS_H
+#include <paths.h>
 #endif
 
 #ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
 #endif
 
-#ifdef HAVE_PATHS_H
-#include <paths.h>
-#endif
-
-#include <map>
-
-#include "libxorp_module.h"
-
-#include "libxorp/xorp.h"
-#include "libxorp/xorp.h"
-#include "libxorp/debug.h"
-#include "libxorp/eventloop.hh"
-#include "libxorp/xlog.h"
-#include "libxorp/xorpfd.hh"
-#include "libxorp/asyncio.hh"
-#include "libxorp/popen.hh"
 #include "run_command.hh"
 
-#ifdef HOST_OS_WINDOWS
 
-#include "libxorp/win_io.h"
+#ifdef HOST_OS_WINDOWS
 
 #ifndef _PATH_BSHELL
 #define _PATH_BSHELL "C:\\MSYS\\bin\\sh.exe"
@@ -186,6 +185,8 @@ RunCommandBase::RunCommandBase(EventLoop&		eventloop,
 {
     memset(_stdout_buffer, 0, BUF_SIZE);
     memset(_stderr_buffer, 0, BUF_SIZE);
+
+    _done_timer = _eventloop.new_timer(callback(this, &RunCommandBase::done));
 }
 
 RunCommandBase::~RunCommandBase()
@@ -203,7 +204,7 @@ RunCommandBase::cleanup()
 	pid2command.erase(_pid);
 	_pid = 0;
     }
-    _done_timer.clear();
+    _done_timer.unschedule();
     _is_running = false;
     unblock_child_signals();
 }
@@ -432,9 +433,15 @@ RunCommandBase::wait_status_changed(int wait_status)
     // XXX: Schedule a timer to complete the command so we can return
     // control to the caller.
     //
-    _done_timer = _eventloop.new_oneoff_after(
-	TimeVal::ZERO(),
-	callback(this, &RunCommandBase::done));
+    // TODO: Temporary print any errors and catch any exceptions
+    // (for debugging purpose).
+    try {
+	errno = 0;
+	_done_timer.schedule_now();
+    } catch(...) {
+	XLOG_ERROR("Error scheduling RunCommand::_done_timer: %d", errno);
+	xorp_catch_standard_exceptions();
+    }
 }
 
 void
@@ -646,14 +653,16 @@ RunCommandBase::io_done(AsyncFileOperator::Event event, int error_code)
 
     close_output();
 
-    done();
+    done(_done_timer);
 }
 
 void
-RunCommandBase::done()
+RunCommandBase::done(XorpTimer& done_timer)
 {
     string prefix, suffix, reason;
 
+    done_timer.unschedule();
+
     if (_stdout_stream != NULL)
 	return;		// XXX: I/O is not done yet
 
@@ -663,7 +672,7 @@ RunCommandBase::done()
     // Remove the entry from the pid map
     pid2command.erase(_pid);
     _pid = 0;
-    _done_timer.clear();
+    _done_timer.unschedule();
     _is_running = false;
 
     if (_error_msg.size()) {
