@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,28 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipnet.cc,v 1.13 2006/08/01 22:21:40 mjh Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "xorp.h"
+#ident "$XORP: xorp/libxorp/test_ipnet.cc,v 1.16 2007/02/16 22:46:24 pavlin Exp $"
 
 #include "libxorp_module.h"
-#include "exceptions.hh"
-#include "ipv4net.hh"
-#include "xlog.h"
 
-#include <stdio.h>
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/exceptions.hh"
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "ipv4net.hh"
+
+
 // ----------------------------------------------------------------------------
 // Verbose output
 
@@ -118,17 +111,13 @@ v4_less_than_test()
 static bool
 v4_is_unicast_test()
 {
-    IPv4Net uni("128.16.0.0/24"); /* regular unicast */
-    IPv4Net multi("224.0.0.0/24"); /* multicast, not valid */
-    IPv4Net odd1("128.0.0.0/1"); /* not valid, includes multicast addresses */
-                                 /* we return true anyway - see
-				    comments in ipnet.hh */
-    IPv4Net odd2("192.0.0.0/2"); /* not valid, includes multicast addresses */
-                                 /* we return true anyway - see
-				    comments in ipnet.hh */
-    IPv4Net odd3("0.0.0.0/1"); /* only unicast, should be valid */
-    IPv4Net odd4("128.0.0.0/2"); /* only unicast, should be valid */
-    IPv4Net deflt("0.0.0.0/0"); /* default route, is valid */
+    IPv4Net uni("128.16.0.0/24");	// regular unicast
+    IPv4Net multi("224.0.0.0/24");	// multicast, not valid
+    IPv4Net odd1("128.0.0.0/1");	// not valid, includes multicast
+    IPv4Net odd2("192.0.0.0/2");	// not valid, includes multicast
+    IPv4Net odd3("0.0.0.0/1");		// only unicast, should be valid
+    IPv4Net odd4("128.0.0.0/2");	// only unicast, should be valid
+    IPv4Net deflt("0.0.0.0/0");		// default route, is valid
     if (!uni.is_unicast()) {
 	verbose_log("%s failed is_unicast test.\n", uni.str().c_str());
 	return false;
@@ -137,11 +126,11 @@ v4_is_unicast_test()
 	verbose_log("%s failed is_unicast test.\n", multi.str().c_str());
 	return false;
     }
-    if (!odd1.is_unicast()) {
+    if (odd1.is_unicast()) {
 	verbose_log("%s failed is_unicast test.\n", odd1.str().c_str());
 	return false;
     }
-    if (!odd2.is_unicast()) {
+    if (odd2.is_unicast()) {
 	verbose_log("%s failed is_unicast test.\n", odd2.str().c_str());
 	return false;
     }
