@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,25 +12,23 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipv4net.cc,v 1.16 2006/04/05 08:02:25 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxorp/test_ipv4net.cc,v 1.21 2007/02/16 22:46:24 pavlin Exp $"
 
 #include "libxorp_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
-#include "libxorp/ipv4net.hh"
 #include "libxorp/timer.hh"
-
 #include "libxorp/test_main.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "ipv4net.hh"
+
+
 //
 // TODO: XXX: remove after the switch to the TestMain facility is completed
 //
@@ -327,14 +325,113 @@ test_ipv4net_address_type(TestInfo& test_info)
 {
     UNUSED(test_info);
 
-    IPv4Net ipnet_a("12.34.0.0/16");
-    
+    IPv4Net ipnet_default("0.0.0.0/0");		// Default route: unicast
+    IPv4Net ipnet_unicast1("0.0.0.0/1");	// Unicast
+    IPv4Net ipnet_unicast2("12.34.0.0/16");	// Unicast
+    IPv4Net ipnet_unicast3("128.0.0.0/2");	// Unicast
+    IPv4Net ipnet_unicast4("128.16.0.0/24");	// Unicast
+    IPv4Net ipnet_unicast5("192.0.0.0/3");	// Unicast
+    IPv4Net ipnet_multicast1("224.0.0.0/4");	// Multicast
+    IPv4Net ipnet_multicast2("224.0.0.0/24");	// Multicast
+    IPv4Net ipnet_multicast3("224.0.1.0/24");	// Multicast
+    IPv4Net ipnet_class_a1("0.0.0.0/1");	// Class A
+    IPv4Net ipnet_class_a2("12.34.0.0/16");	// Class A
+    IPv4Net ipnet_class_b1("128.0.0.0/2");	// Class B
+    IPv4Net ipnet_class_b2("130.2.3.0/24");	// Class B
+    IPv4Net ipnet_class_c1("192.0.0.0/3");	// Class C
+    IPv4Net ipnet_class_c2("192.2.3.4/32");	// Class C
+    IPv4Net ipnet_experimental1("240.0.0.0/4");	// Experimental
+    IPv4Net ipnet_experimental2("240.0.1.0/16"); // Experimental
+    IPv4Net ipnet_odd1("128.0.0.0/1");		// Odd: includes multicast
+    IPv4Net ipnet_odd2("192.0.0.0/2");		// Odd: includes multicast
+
     //
-    // Test if this subnet is within the multicast address range.
+    // Test if a subnet is within the unicast address range.
     //
-    IPv4Net ipnet1("224.0.1.0/24");
-    verbose_assert(ipnet_a.is_multicast() == false, "is_multicast()");
-    verbose_assert(ipnet1.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet_default.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_unicast1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_unicast2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_unicast3.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_unicast4.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_unicast5.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_multicast1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet_multicast2.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet_multicast3.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet_class_a1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_class_a2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_class_b1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_class_b2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_class_c1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_class_c2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet_experimental1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet_experimental2.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet_odd1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet_odd2.is_unicast() == false, "is_unicast()");
+
+    //
+    // Test if a subnet is within the multicast address range.
+    //
+    verbose_assert(ipnet_default.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_unicast1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_unicast2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_unicast3.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_unicast4.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_unicast5.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_multicast1.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet_multicast2.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet_multicast3.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet_class_a1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_class_a2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_class_b1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_class_b2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_class_c1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_class_c2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_experimental1.is_multicast() == false,
+		   "is_multicast()");
+    verbose_assert(ipnet_experimental2.is_multicast() == false,
+		   "is_multicast()");
+    verbose_assert(ipnet_odd1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet_odd2.is_multicast() == false, "is_multicast()");
+
+    //
+    // Test if a subnet is within the experimental address range.
+    //
+    verbose_assert(ipnet_default.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_unicast1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_unicast2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_unicast3.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_unicast4.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_unicast5.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_multicast1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_multicast2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_multicast3.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_class_a1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_class_a2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_class_b1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_class_b2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_class_c1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_class_c2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet_experimental1.is_experimental() == true,
+		   "is_experimental()");
+    verbose_assert(ipnet_experimental2.is_experimental() == true,
+		   "is_experimental()");
+    verbose_assert(ipnet_odd1.is_experimental() == false, "is_experimental()");
+    verbose_assert(ipnet_odd2.is_experimental() == false, "is_experimental()");
 
     return (! failures());
 }
@@ -463,13 +560,26 @@ test_ipv4net_address_const(TestInfo& test_info)
     // Return the subnet containing all multicast addresses.
     //
     verbose_match(IPv4Net::ip_multicast_base_prefix().str(), "224.0.0.0/4");
+
+    //
+    // Return the subnet containing all Class A addresses.
+    //
+    verbose_match(IPv4Net::ip_class_a_base_prefix().str(), "0.0.0.0/1");
+
+    //
+    // Return the subnet containing all Class B addresses.
+    //
+    verbose_match(IPv4Net::ip_class_b_base_prefix().str(), "128.0.0.0/2");
+
+    //
+    // Return the subnet containing all Class C addresses.
+    //
+    verbose_match(IPv4Net::ip_class_c_base_prefix().str(), "192.0.0.0/3");
     
     //
-    // Test if this subnet is within the multicast address range.
+    // Return the subnet containing all experimental addresses.
     //
-    IPv4Net ipnet1("224.0.1.0/24");
-    verbose_assert(ipnet_a.is_multicast() == false, "is_multicast()");
-    verbose_assert(ipnet1.is_multicast() == true, "is_multicast()");
+    verbose_match(IPv4Net::ip_experimental_base_prefix().str(), "240.0.0.0/4");
 
     return (! failures());
 }
