@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,25 +12,23 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ipvxnet.cc,v 1.15 2006/04/05 08:02:25 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxorp/test_ipvxnet.cc,v 1.20 2007/02/16 22:46:25 pavlin Exp $"
 
 #include "libxorp_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
-#include "libxorp/ipvxnet.hh"
 #include "libxorp/timer.hh"
-
 #include "libxorp/test_main.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "ipvxnet.hh"
+
+
 //
 // TODO: XXX: remove after the switch to the TestMain facility is completed
 //
@@ -487,22 +485,146 @@ test_ipvxnet_address_type(TestInfo& test_info)
 {
     UNUSED(test_info);
 
-    IPvXNet ipnet4_a("12.34.0.0/16");
-    IPvXNet ipnet6_a("1234:5678::/32");
-    
+    IPvXNet ipnet4_default("0.0.0.0/0");	// Default route: unicast
+    IPvXNet ipnet4_unicast1("0.0.0.0/1");	// Unicast
+    IPvXNet ipnet4_unicast2("12.34.0.0/16");	// Unicast
+    IPvXNet ipnet4_unicast3("128.0.0.0/2");	// Unicast
+    IPvXNet ipnet4_unicast4("128.16.0.0/24");	// Unicast
+    IPvXNet ipnet4_unicast5("192.0.0.0/3");	// Unicast
+    IPvXNet ipnet4_multicast1("224.0.0.0/4");	// Multicast
+    IPvXNet ipnet4_multicast2("224.0.0.0/24");	// Multicast
+    IPvXNet ipnet4_multicast3("224.0.1.0/24");	// Multicast
+    IPvXNet ipnet4_class_a1("0.0.0.0/1");	// Class A
+    IPvXNet ipnet4_class_a2("12.34.0.0/16");	// Class A
+    IPvXNet ipnet4_class_b1("128.0.0.0/2");	// Class B
+    IPvXNet ipnet4_class_b2("130.2.3.0/24");	// Class B
+    IPvXNet ipnet4_class_c1("192.0.0.0/3");	// Class C
+    IPvXNet ipnet4_class_c2("192.2.3.4/32");	// Class C
+    IPvXNet ipnet4_experimental1("240.0.0.0/4");  // Experimental
+    IPvXNet ipnet4_experimental2("240.0.1.0/16"); // Experimental
+    IPvXNet ipnet4_odd1("128.0.0.0/1");		// Odd: includes multicast
+    IPvXNet ipnet4_odd2("192.0.0.0/2");		// Odd: includes multicast
+
+    IPvXNet ipnet6_default("::/0");		// Default route: unicast
+    IPvXNet ipnet6_unicast1("::/1");		// Unicast
+    IPvXNet ipnet6_unicast2("1234:5678::/32");	// Unicast
+    IPvXNet ipnet6_multicast1("ff00::/8");	// Multicast
+    IPvXNet ipnet6_multicast2("ff00:1::/32");	// Multicast
+    IPvXNet ipnet6_odd1("8000::/1");		// Odd: includes multicast
+    IPvXNet ipnet6_odd2("fe00::/7");		// Odd: includes multicast
+
     //
-    // Test if this subnet is within the multicast address range: IPv4.
+    // Test if a subnet is within the unicast address range: IPv4.
     //
-    IPvXNet ipnet1("224.0.1.0/24");
-    verbose_assert(ipnet4_a.is_multicast() == false, "is_multicast()");
-    verbose_assert(ipnet1.is_multicast() == true, "is_multicast()");
-    
+    verbose_assert(ipnet4_default.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_unicast1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_unicast2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_unicast3.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_unicast4.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_unicast5.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_multicast1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet4_multicast2.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet4_multicast3.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet4_class_a1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_class_a2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_class_b1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_class_b2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_class_c1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_class_c2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet4_experimental1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet4_experimental2.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet4_odd1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet4_odd2.is_unicast() == false, "is_unicast()");
+
     //
-    // Test if this subnet is within the multicast address range: IPv6.
+    // Test if a subnet is within the unicast address range: IPv6.
     //
-    IPvXNet ipnet2("ff00:1::/32");
-    verbose_assert(ipnet6_a.is_multicast() == false, "is_multicast()");
-    verbose_assert(ipnet2.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet6_default.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet6_unicast1.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet6_unicast2.is_unicast() == true, "is_unicast()");
+    verbose_assert(ipnet6_multicast1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet6_multicast2.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet6_odd1.is_unicast() == false, "is_unicast()");
+    verbose_assert(ipnet6_odd2.is_unicast() == false, "is_unicast()");
+
+    //
+    // Test if a subnet is within the multicast address range: IPv4.
+    //
+    verbose_assert(ipnet4_default.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_unicast1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_unicast2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_unicast3.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_unicast4.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_unicast5.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_multicast1.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet4_multicast2.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet4_multicast3.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet4_class_a1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_class_a2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_class_b1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_class_b2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_class_c1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_class_c2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_experimental1.is_multicast() == false,
+		   "is_multicast()");
+    verbose_assert(ipnet4_experimental2.is_multicast() == false,
+		   "is_multicast()");
+    verbose_assert(ipnet4_odd1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet4_odd2.is_multicast() == false, "is_multicast()");
+
+    //
+    // Test if a subnet is within the multicast address range: IPv6.
+    //
+    verbose_assert(ipnet6_default.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet6_unicast1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet6_unicast2.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet6_multicast1.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet6_multicast2.is_multicast() == true, "is_multicast()");
+    verbose_assert(ipnet6_odd1.is_multicast() == false, "is_multicast()");
+    verbose_assert(ipnet6_odd2.is_multicast() == false, "is_multicast()");
+
+    //
+    // Test if a subnet is within the experimental address range: IPv4.
+    //
+    // XXX: This test applies only for IPv4.
+    verbose_assert(ipnet4_default.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_unicast1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_unicast2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_unicast3.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_unicast4.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_unicast5.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_multicast1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_multicast2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_multicast3.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_class_a1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_class_a2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_class_b1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_class_b2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_class_c1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_class_c2.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_experimental1.is_experimental() == true,
+		   "is_experimental()");
+    verbose_assert(ipnet4_experimental2.is_experimental() == true,
+		   "is_experimental()");
+    verbose_assert(ipnet4_odd1.is_experimental() == false,
+		   "is_experimental()");
+    verbose_assert(ipnet4_odd2.is_experimental() == false,
+		   "is_experimental()");
 
     return (! failures());
 }
@@ -711,18 +833,32 @@ test_ipvxnet_address_const(TestInfo& test_info)
 		  "ff00::/8");
 
     //
-    // Test if this subnet is within the multicast address range: IPv4.
+    // Return the subnet containing all Class A addresses.
     //
-    IPvXNet ipnet1("224.0.1.0/24");
-    verbose_assert(ipnet4_a.is_multicast() == false, "is_multicast()");
-    verbose_assert(ipnet1.is_multicast() == true, "is_multicast()");
-    
+    // XXX: This test applies only for IPv4.
+    verbose_match(IPvXNet::ip_class_a_base_prefix(AF_INET).str(),
+		  "0.0.0.0/1");
+
     //
-    // Test if this subnet is within the multicast address range: IPv6.
+    // Return the subnet containing all Class B addresses.
     //
-    IPvXNet ipnet2("ffff:1:2::/48");
-    verbose_assert(ipnet6_a.is_multicast() == false, "is_multicast()");
-    verbose_assert(ipnet2.is_multicast() == true, "is_multicast()");
+    // XXX: This test applies only for IPv4.
+    verbose_match(IPvXNet::ip_class_b_base_prefix(AF_INET).str(),
+		  "128.0.0.0/2");
+
+    //
+    // Return the subnet containing all Class C addresses.
+    //
+    // XXX: This test applies only for IPv4.
+    verbose_match(IPvXNet::ip_class_c_base_prefix(AF_INET).str(),
+		  "192.0.0.0/3");
+
+    //
+    // Return the subnet containing all experimental addresses.
+    //
+    // XXX: This test applies only for IPv4.
+    verbose_match(IPvXNet::ip_experimental_base_prefix(AF_INET).str(),
+		  "240.0.0.0/4");
 
     return (! failures());
 }
