@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,28 +12,26 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_observers.cc,v 1.9 2006/06/21 23:36:34 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/test_observers.cc,v 1.11 2007/02/16 22:46:25 pavlin Exp $"
 
 //
-// test program to the Observer classes for TimerList and SelectorList
+// Test program to the Observer classes for TimerList and SelectorList
 //
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "libxorp_module.h"
-#include "libxorp/xorpfd.hh"
-#include "libxorp/timer.hh"
-#include "libxorp/eventloop.hh"
-#include "libxorp/xlog.h"
 
-#include <stdio.h>
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/xorpfd.hh"
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
+#include "timer.hh"
+#include "eventloop.hh"
+
+
 #ifndef HOST_OS_WINDOWS
 
 static int fired(0);
