@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_profile.cc,v 1.7 2006/03/29 22:41:24 atanu Exp $"
+#ident "$XORP: xorp/libxorp/test_profile.cc,v 1.8 2007/02/16 22:46:25 pavlin Exp $"
 
 #include "libxorp_module.h"
 #include "xorp.h"
