@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,23 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_ref_ptr.cc,v 1.8 2006/03/16 00:04:34 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxorp/test_ref_ptr.cc,v 1.10 2007/02/16 22:46:25 pavlin Exp $"
 
 #include "libxorp_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
 
-#include "libxorp/ref_ptr.hh"
-
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "ref_ptr.hh"
+
+
 //
 // XXX: MODIFY FOR YOUR TEST PROGRAM
 //
