@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,22 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxorp/test_run_command.cc,v 1.15 2006/04/26 03:18:08 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/libxorp/test_run_command.cc,v 1.17 2007/02/16 22:46:25 pavlin Exp $"
 
 #include "libxorp_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
-#include "libxorp/run_command.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "run_command.hh"
+
+
 #ifdef HOST_OS_WINDOWS
 #define MSYS_DEFAULT_ROOT	"C:\\MinGW"
 #define SLEEP_PATH		"\\bin\\sleep.exe"
