@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,21 +13,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-// $XORP: xorp/libxorp/win_dispatcher.cc,v 1.13 2006/07/31 22:39:57 pavlin Exp $
+// $XORP: xorp/libxorp/win_dispatcher.cc,v 1.19 2007/02/16 22:46:28 pavlin Exp $
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef HOST_OS_WINDOWS	    // This entire file is for Windows only.
+#include "libxorp/libxorp_module.h"
 
 #include "libxorp/xorp.h"
 
-#include "libxorp/libxorp_module.h"
-#include "libxorp/xlog.h"
-#include "libxorp/debug.h"
-#include "libxorp/win_io.h"
+#ifdef HOST_OS_WINDOWS	    // This entire file is for Windows only.
 
+#include "libxorp/debug.h"
+#include "libxorp/xlog.h"
+#include "libxorp/win_io.h"
 #include "libxorp/xorpfd.hh"
 #include "libxorp/timeval.hh"
 #include "libxorp/clock.hh"
@@ -36,6 +32,7 @@
 #include "libxorp/eventloop.hh"
 #include "libxorp/win_dispatcher.hh"
 
+
 static inline int
 _wsa2ioe(const long wsaevent)
 {
@@ -106,8 +103,11 @@ WinDispatcher::~WinDispatcher()
 }
 
 bool
-WinDispatcher::add_socket_cb(XorpFd& fd, IoEventType type, const IoEventCb& cb)
+WinDispatcher::add_socket_cb(XorpFd& fd, IoEventType type, const IoEventCb& cb,
+			     int priority)
 {
+    UNUSED(priority);		// TODO: XXX: we should really use this
+
     // XXX: Currently we only support 1 callback per socket/event tuple.
     // Check that the socket does not already have a callback
     // registered for it.
@@ -200,8 +200,11 @@ WinDispatcher::add_socket_cb(XorpFd& fd, IoEventType type, const IoEventCb& cb)
 }
 
 bool
-WinDispatcher::add_handle_cb(XorpFd& fd, IoEventType type, const IoEventCb& cb)
+WinDispatcher::add_handle_cb(XorpFd& fd, IoEventType type, const IoEventCb& cb,
+			     int priority)
 {
+    UNUSED(priority);		// TODO: XXX: we should really use this
+
     // You cannot currently register for anything other
     // than an IOT_EXCEPTION event on a Windows object handle because
     // there is no way of telling why an object was signalled --
@@ -242,18 +245,19 @@ WinDispatcher::add_handle_cb(XorpFd& fd, IoEventType type, const IoEventCb& cb)
 }
 
 bool
-WinDispatcher::add_ioevent_cb(XorpFd fd, IoEventType type, const IoEventCb& cb)
+WinDispatcher::add_ioevent_cb(XorpFd fd, IoEventType type, const IoEventCb& cb,
+			     int priority)
 {
 debug_msg("Adding event %d to object %s\n", type, fd.str().c_str());
 
     switch (fd.type()) {
     case XorpFd::FDTYPE_SOCKET:
-	return add_socket_cb(fd, type, cb);
+	return add_socket_cb(fd, type, cb, priority);
 	break;
     case XorpFd::FDTYPE_PIPE:
     case XorpFd::FDTYPE_CONSOLE:
     case XorpFd::FDTYPE_PROCESS:
-	return add_handle_cb(fd, type, cb);
+	return add_handle_cb(fd, type, cb, priority);
 	break;
     default:
 	break;
@@ -402,6 +406,39 @@ WinDispatcher::remove_ioevent_cb(XorpFd fd, IoEventType type)
     return false;
 }
 
+int
+WinDispatcher::get_ready_priority()
+{
+    // TODO: XXX: THIS IS COMPLETELY BOGUS
+    return XorpTask::PRIORITY_DEFAULT;
+}
+
+bool
+WinDispatcher::ready()
+{
+    DWORD retval;
+
+    // TODO: XXX: THIS IS PROBABLY BOGUS
+
+    for (vector<HANDLE>::iterator ii = _polled_pipes.begin();
+	ii != _polled_pipes.end(); ++ii) {
+	ssize_t result = win_pipe_read(*ii, NULL, 0);
+	if (result == WINIO_ERROR_HASINPUT 
+	    || result == WINIO_ERROR_DISCONNECT) {
+	    return true;
+	}
+    }
+    if (_handles.empty()) {
+	return false;
+    }
+    retval = WaitForMultipleObjects(_handles.size(), &_handles[0],
+				    FALSE, 0);
+    if (retval == WAIT_TIMEOUT) {
+	return false;
+    }
+    return true;
+}
+
 void
 WinDispatcher::wait_and_dispatch(int ms)
 {
