@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
-/* Copyright (c) 2001-2006 International Computer Science Institute
+/* Copyright (c) 2001-2007 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -13,26 +13,22 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libxorp/xlog.c,v 1.18 2006/03/16 00:04:38 pavlin Exp $"
+#ident "$XORP: xorp/libxorp/xlog.c,v 1.22 2007/02/16 22:46:29 pavlin Exp $"
 
 /*
  * Message logging utility.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "libxorp_module.h"
-#include "xorp.h"
-#include "xlog.h"
 
-#include <time.h>
+#include "libxorp/xorp.h"
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
+
+#include "xlog.h"
+
 
 /*
  * Exported variables
@@ -126,7 +122,7 @@ int
 xlog_init(const char *argv0, const char *preamble_message)
 {
     const char* process_name;
-    size_t i;
+    xlog_level_t level;
 
     if (init_flag)
 	return (-1);
@@ -157,9 +153,9 @@ xlog_init(const char *argv0, const char *preamble_message)
     xlog_set_preamble(preamble_message);
 
     /* Enable all log messages by default, and set default verbose level */
-    for (i = 0; i < XLOG_LEVEL_MAX; i++) {
-	xlog_enable(i);
-	xlog_verbose_level[i] = XLOG_VERBOSE_LOW;		/* Default */
+    for (level = XLOG_LEVEL_MIN; level < XLOG_LEVEL_MAX; level++) {
+	xlog_enable(level);
+	xlog_verbose_level[level] = XLOG_VERBOSE_LOW;		/* Default */
     }
     xlog_verbose_level[XLOG_LEVEL_FATAL] = XLOG_VERBOSE_HIGH;	/* XXX */
 
@@ -183,6 +179,7 @@ int
 xlog_exit(void)
 {
     int i;
+    xlog_level_t level;
 
     if (! init_flag)
 	return (-1);
@@ -216,9 +213,9 @@ xlog_exit(void)
     xlog_output_func_count = 0;
     fp_default = 0;
 
-    for (i = 0; i < XLOG_LEVEL_MAX; i++) {
-	xlog_disable(i);
-	xlog_verbose_level[i] = XLOG_VERBOSE_LOW;		/* Default */
+    for (level = XLOG_LEVEL_MIN; level < XLOG_LEVEL_MAX; level++) {
+	xlog_disable(level);
+	xlog_verbose_level[level] = XLOG_VERBOSE_LOW;		/* Default */
     }
     xlog_verbose_level[XLOG_LEVEL_FATAL] = XLOG_VERBOSE_HIGH;	/* XXX */
 
