@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mibs/bgp4_mib_1657_bgp4pathattrtable.cc,v 1.18 2006/07/05 22:46:57 atanu Exp $"
+#ident "$XORP: xorp/mibs/bgp4_mib_1657_bgp4pathattrtable.cc,v 1.23 2007/02/16 22:46:32 pavlin Exp $"
 
 
 #include <stack>
@@ -33,10 +33,10 @@
 // Local classes and typedefs
 
 typedef struct bgp4PathAttrTable_context_s {
-    netsnmp_index index; // THIS MUST BE FIRST!!! /
-    unsigned long bgp4PathAttrPeer;
+    netsnmp_index index;		// THIS MUST BE FIRST!!!
+    unsigned long bgp4PathAttrPeer;	// IPv4 address in network order
     unsigned long bgp4PathAttrIpAddrPrefixLen;
-    unsigned long bgp4PathAttrIpAddrPrefix;
+    unsigned long bgp4PathAttrIpAddrPrefix;  // IPv4 address in network order
     long bgp4PathAttrOrigin;
     u_char * bgp4PathAttrASPathSegment;
     unsigned long bgp4PathAttrASPathSegmentLen;
@@ -497,13 +497,19 @@ bgp4PathAttrTable_extract_index(bgp4PathAttrTable_context * ctx,
         * copy components into the context structure
         */
 	ctx->bgp4PathAttrIpAddrPrefix = 
-	    ntohl(*var_bgp4PathAttrIpAddrPrefix.val.integer);
+	    *var_bgp4PathAttrIpAddrPrefix.val.integer;
+#ifdef HAVE_NETSNMP_ASN_IPADDRESS_IN_HOST_ORDER
+	ctx->bgp4PathAttrIpAddrPrefix = htonl(ctx->bgp4PathAttrIpAddrPrefix);
+#endif
    
 	ctx->bgp4PathAttrIpAddrPrefixLen = 
 	    *var_bgp4PathAttrIpAddrPrefixLen.val.integer;
-   
+
 	ctx->bgp4PathAttrPeer = 
-	    ntohl(*var_bgp4PathAttrPeer.val.integer);
+	    *var_bgp4PathAttrPeer.val.integer;
+#ifdef HAVE_NETSNMP_ASN_IPADDRESS_IN_HOST_ORDER
+	ctx->bgp4PathAttrPeer = htonl(ctx->bgp4PathAttrPeer);
+#endif
     }
 
     // parsing may have allocated memory. free it.
