@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_group_record.cc,v 1.29 2006/07/10 08:50:03 pavlin Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_group_record.cc,v 1.31 2007/02/16 22:46:36 pavlin Exp $"
 
 //
 // Multicast group record information used by
@@ -907,6 +907,13 @@ Mld6igmpGroupRecord::group_query_periodic_timeout()
     TimeVal max_resp_time = mld6igmp_vif().query_last_member_interval().get();
     bool do_send_group_query = true;
 
+    //
+    // XXX: Don't send Group-Specific or Group-and-Source-Specific Queries
+    // for entries that are in IGMPv1 mode.
+    //
+    if (is_igmpv1_mode())
+	return (false);
+
     //
     // XXX: The IGMPv3/MLDv2 spec doesn't say what to do here if we changed
     // from a Querier to a non-Querier.
