@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.78 2006/07/28 06:23:59 pavlin Exp $"
+#ident "$XORP: xorp/mld6igmp/mld6igmp_vif.cc,v 1.84 2007/02/16 22:46:37 pavlin Exp $"
 
 
 //
@@ -26,7 +26,7 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 
-#include "mrt/inet_cksum.h"
+#include "libproto/checksum.h"
 
 #include "mld6igmp_vif.hh"
 
@@ -497,7 +497,7 @@ Mld6igmpVif::mld6igmp_send(const IPvX& src,
     //
     // Compute the checksum
     //
-    cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
+    cksum = inet_checksum(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
 #ifdef HAVE_IPV6
     // Add the checksum for the IPv6 pseudo-header
     if (proto_is_mld6()) {
@@ -505,10 +505,10 @@ Mld6igmpVif::mld6igmp_send(const IPvX& src,
 	size_t ph_len = BUFFER_DATA_SIZE(buffer);
 	cksum2 = calculate_ipv6_pseudo_header_checksum(src, dst, ph_len,
 						       IPPROTO_ICMPV6);
-	cksum = INET_CKSUM_ADD(cksum, cksum2);
+	cksum = inet_checksum_add(cksum, cksum2);
     }
 #endif // HAVE_IPV6
-    BUFFER_COPYPUT_INET_CKSUM(cksum, buffer, 2);	// XXX: the ckecksum
+    BUFFER_COPYPUT_INET_CKSUM(cksum, buffer, 2);	// XXX: the checksum
     
     XLOG_TRACE(mld6igmp_node().is_log_trace(), "TX %s from %s to %s",
 	       proto_message_type2ascii(message_type),
@@ -786,7 +786,7 @@ Mld6igmpVif::mld6igmp_query_send(const IPvX& src,
 	    - 8		// IPv6 Hop-by-hop Ext. Header with Router Alert option
 	    - MLD_V2_QUERY_MINLEN;	// MLDv2 Query pre-source fields
     }
-    max_sources_n = min(max_sources_n, max_payload / IPvX::addr_size(family()));
+    max_sources_n = min(max_sources_n, max_payload / IPvX::addr_bytelen(family()));
 
     //
     // XXX: According to RFC 3810 (MLDv2), Section 8.3.2, the Querier
@@ -922,7 +922,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
     bool check_src_linklocal_unicast = false;
     bool allow_src_zero_address = false;
     bool check_dst_multicast = false;
-    bool check_group_nodelocal_multicast = false;
+    bool check_group_interfacelocal_multicast = false;
     bool decode_extra_fields = false;
     
     //
@@ -941,7 +941,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
     //
     // Checksum verification.
     //
-    cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
+    cksum = inet_checksum(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
 #ifdef HAVE_IPV6
     // Add the checksum for the IPv6 pseudo-header
     if (proto_is_mld6()) {
@@ -949,7 +949,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 	size_t ph_len = BUFFER_DATA_SIZE(buffer);
 	cksum2 = calculate_ipv6_pseudo_header_checksum(src, dst, ph_len,
 						       IPPROTO_ICMPV6);
-	cksum = INET_CKSUM_ADD(cksum, cksum2);
+	cksum = inet_checksum_add(cksum, cksum2);
     }
 #endif // HAVE_IPV6
     if (cksum) {
@@ -992,6 +992,66 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 	       cstring(src), cstring(dst),
 	       name().c_str());
 
+    //
+    // Ignore messages that are not recognized by older protocol version.
+    //
+    // XXX: Unrecognized message types MUST be silently ignored.
+    //
+    if (proto_is_igmp()) {
+	switch (message_type) {
+	case IGMP_MEMBERSHIP_QUERY:
+	    // Recognized by IGMPv1, IGMPv2, IGMPv3
+	    break;
+	case IGMP_V1_MEMBERSHIP_REPORT:
+	    // Recognized by IGMPv1, IGMPv2, IGMPv3
+	    break;
+	case IGMP_V2_MEMBERSHIP_REPORT:
+	    // Recognized by IGMPv2, IGMPv3
+	    if (is_igmpv1_mode())
+		return (XORP_ERROR);
+	    break;
+	case IGMP_V2_LEAVE_GROUP:
+	    // Recognized by IGMPv2, IGMPv3
+	    if (is_igmpv1_mode())
+		return (XORP_ERROR);
+	    break;
+	case IGMP_V3_MEMBERSHIP_REPORT:
+	    // Recognized by IGMPv3
+	    if (is_igmpv1_mode() || is_igmpv2_mode())
+		return (XORP_ERROR);
+	    break;
+	case IGMP_DVMRP:
+	case IGMP_MTRACE:
+	    break;
+	default:
+	    // Unrecognized message
+	    return (XORP_ERROR);
+	}
+    }
+    if (proto_is_mld6()) {
+	switch (message_type) {
+	case MLD_LISTENER_QUERY:
+	    // Recognized by MLDv1, MLDv2
+	    break;
+	case MLD_LISTENER_REPORT:
+	    // Recognized by MLDv1, MLDv2
+	    break;
+	case MLD_LISTENER_DONE:
+	    // Recognized by MLDv1, MLDv2
+	    break;
+	case MLDV2_LISTENER_REPORT:
+	    // Recognized by MLDv2
+	    if (is_mldv1_mode())
+		return (XORP_ERROR);
+	    break;
+	case MLD_MTRACE:
+	    break;
+	default:
+	    // Unrecognized message
+	    return (XORP_ERROR);
+	}
+    }
+
     //
     // Assign various flags what needs to be checked, based on the
     // message type:
@@ -999,7 +1059,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
     //  - check_src_linklocal_unicast
     //  - allow_src_zero_address
     //  - check_dst_multicast
-    //  - check_group_nodelocal_multicast
+    //  - check_group_interfacelocal_multicast
     //  - decode_extra_fields
     //
     if (proto_is_igmp()) {
@@ -1022,7 +1082,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 	    check_dst_multicast = true;
 	    if (is_igmpv3_mode())
 		check_dst_multicast = false;		// XXX: disable
-	    check_group_nodelocal_multicast = false;	// Not needed for IPv4
+	    check_group_interfacelocal_multicast = false;// Not needed for IPv4
 	    decode_extra_fields = true;
 	    if (message_type == IGMP_V3_MEMBERSHIP_REPORT)
 		decode_extra_fields = false;
@@ -1048,7 +1108,7 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 	    check_dst_multicast = true;
 	    if (is_mldv2_mode())
 		check_dst_multicast = false;		// XXX: disable
-	    check_group_nodelocal_multicast = true;
+	    check_group_interfacelocal_multicast = true;
 	    decode_extra_fields = true;
 	    if (message_type == MLDV2_LISTENER_REPORT)
 		decode_extra_fields = false;
@@ -1132,6 +1192,16 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
 		     name().c_str(),
 		     src.af(), family());
     }
+    // Source address must be directly connected
+    if (! mld6igmp_node().is_directly_connected(*this, src)) {
+	error_msg = c_format("RX %s from %s to %s on vif %s: "
+			     "source must be directly connected",
+			     proto_message_type2ascii(message_type),
+			     cstring(src), cstring(dst),
+			     name().c_str());
+	XLOG_WARNING("%s", error_msg.c_str());
+	return (XORP_ERROR);
+    }
     if (check_src_linklocal_unicast) {
 	if (src.is_linklocal_unicast()
 	    || (allow_src_zero_address && src.is_zero())) {
@@ -1176,10 +1246,10 @@ Mld6igmpVif::mld6igmp_process(const IPvX& src,
     //
     // Inner multicast address scope check.
     //
-    if (check_group_nodelocal_multicast
-	&& group_address.is_nodelocal_multicast()) {
+    if (check_group_interfacelocal_multicast
+	&& group_address.is_interfacelocal_multicast()) {
 	error_msg = c_format("RX %s from %s to %s on vif %s: "
-			     "invalid node-local scope of inner "
+			     "invalid interface-local scope of inner "
 			     "multicast address: %s",
 			     proto_message_type2ascii(message_type),
 			     cstring(src), cstring(dst),
@@ -1645,7 +1715,9 @@ Mld6igmpVif::calculate_ipv6_pseudo_header_checksum(const IPvX& src,
     ip6_pseudo_header.ph_zero[2] = 0;
     ip6_pseudo_header.ph_next = protocol;
     
-    uint16_t cksum = INET_CKSUM(&ip6_pseudo_header, sizeof(ip6_pseudo_header));
+    uint16_t cksum = inet_checksum(
+	reinterpret_cast<const uint8_t *>(&ip6_pseudo_header),
+	sizeof(ip6_pseudo_header));
     
     return (cksum);
 }
