@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,29 +13,28 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/lsa.cc,v 1.70 2006/03/28 03:06:53 atanu Exp $"
+#ident "$XORP: xorp/ospf/lsa.cc,v 1.103 2007/03/19 05:47:07 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "config.h"
-#include <map>
-#include <list>
-#include <set>
-
 #include "ospf_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/callback.hh"
-
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
 #include "libxorp/ipnet.hh"
-
 #include "libxorp/status_codes.h"
 #include "libxorp/service.hh"
 #include "libxorp/eventloop.hh"
+#include "libproto/packet.hh"
+
+#include <map>
+#include <list>
+#include <set>
 
 #include "ospf.hh"
 #include "lsa.hh"
@@ -75,70 +74,31 @@ compute_checksum(uint8_t *buf, size_t len, size_t offset)
 }
 
 /**
- * RFC 2470 A.4.1 IPv6 Prefix Representation
- * Given an IPv6 prefix how many bytes will be occupied.
+ * Get the length of this LSA and verify that the length is smaller
+ * than the buffer and large enough to be a valid LSA. Otherwise throw
+ * an exception. Don't modify the value if its greater than the
+ * buffer.
  */
 inline
 size_t
-bytes_per_prefix(uint8_t prefix)
-{
-    return ((prefix + 31) / 32) * 4;
-}
-
-/**
- * RFC 2470 A.4.1 IPv6 Prefix Representation
- */
-inline
-IPNet<IPv6>
-get_ipv6_net(const char *caller, uint8_t *ptr, uint8_t prefix) throw(BadPacket)
-{
-    uint8_t addr[IPv6::ADDR_BITLEN / 8];
-    uint32_t bytes = bytes_per_prefix(prefix);
-    if (bytes > sizeof(addr)) 
-	xorp_throw(BadPacket,
-		   c_format("%s prefix length %u larger than 16",
-			    caller,
-			    bytes));
-    
-    memset(&addr[0], 0, IPv6::ADDR_BITLEN / 8);
-    memcpy(&addr[0], ptr, bytes);
-    IPv6 v6;
-    v6.set_addr(&addr[0]);
-    IPNet<IPv6> v6net(v6, prefix);
-    
-    return v6net;
-}
-
-inline
-size_t
-put_ipv6_net(IPNet<IPv6>& v6net, uint8_t *ptr)
-{
-    IPv6 v6 = v6net.masked_addr();
-    uint8_t buf[IPv6::ADDR_BITLEN / 8];
-    v6.copy_out(&buf[0]);
-    size_t bytes = bytes_per_prefix(v6net.prefix_len());
-    memcpy(ptr, &buf[0], bytes);
-
-    return bytes;
-}
-
-/**
- * Get the length of this LSA and make sure that its less than the
- * provided buffer. Otherwise throw an exception. Don't modify the
- * value if its greater than the buffer.
- */
-inline
-size_t
-get_lsa_len_from_header(const char *caller, uint8_t *buf, size_t len)
-    throw(BadPacket)
+get_lsa_len_from_header(const char *caller, uint8_t *buf, size_t len,
+			size_t min_len)
+    throw(InvalidPacket)
 {
     size_t tlen = Lsa_header::get_lsa_len_from_buffer(buf);
     if (tlen > len) {
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("%s header len %u larger than buffer %u",
 			    caller,
 			    XORP_UINT_CAST(tlen),
 			    XORP_UINT_CAST(len)));
+    } else if(tlen < min_len) {
+	xorp_throw(InvalidPacket,
+		   c_format("%s header len %u smaller than minimum LSA "
+			    "of this type %u",
+			    caller,
+			    XORP_UINT_CAST(tlen),
+			    XORP_UINT_CAST(min_len)));
     } else {
 	len = tlen;
     }
@@ -153,7 +113,7 @@ Lsa_header::get_lsa_len_from_buffer(uint8_t *ptr)
 }
 
 void
-Lsa_header::decode(Lsa_header& header, uint8_t *ptr) const throw(BadPacket)
+Lsa_header::decode(Lsa_header& header, uint8_t *ptr) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -185,7 +145,7 @@ Lsa_header::decode(Lsa_header& header, uint8_t *ptr) const throw(BadPacket)
  * enough space by calling the length() method.
  */
 Lsa_header
-Lsa_header::decode(uint8_t *ptr) const throw(BadPacket)
+Lsa_header::decode(uint8_t *ptr) const throw(InvalidPacket)
 {
      Lsa_header header(get_version());
      decode(header, ptr);
@@ -194,7 +154,7 @@ Lsa_header::decode(uint8_t *ptr) const throw(BadPacket)
 }
 
 void
-Lsa_header::decode_inline(uint8_t *ptr) throw(BadPacket)
+Lsa_header::decode_inline(uint8_t *ptr) throw(InvalidPacket)
 {
     decode(*this, ptr);
 }
@@ -288,7 +248,7 @@ void
 Lsa::update_age_and_seqno(const TimeVal& now)
 {
     XLOG_ASSERT(get_self_originating());
-    XLOG_ASSERT(get_header().get_ls_age() != OspfTypes::MaxAge);
+//     XLOG_ASSERT(get_header().get_ls_age() != OspfTypes::MaxAge);
 
     // If this LSA has been transmitted then its okay to bump the
     // sequence number.
@@ -373,7 +333,7 @@ Lsa::set_ls_age(uint16_t age)
  * enough space by calling the length() method.
  */
 Ls_request
-Ls_request::decode(uint8_t *ptr) throw(BadPacket)
+Ls_request::decode(uint8_t *ptr) throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -439,6 +399,8 @@ LsaDecoder::~LsaDecoder()
 
     for(i = _lsa_decoders.begin(); i != _lsa_decoders.end(); i++)
 	delete i->second;
+
+    delete _unknown_lsa_decoder;
 }
 
 void
@@ -456,14 +418,29 @@ LsaDecoder::register_decoder(Lsa *lsa)
 	_min_lsa_length = lsa->min_length();
 }
 
+void
+LsaDecoder::register_unknown_decoder(Lsa *lsa)
+{
+    switch(get_version()) {
+    case OspfTypes::V2:
+	XLOG_FATAL("OSPFv2 does not have an Unknown-LSA decoder");
+	break;
+    case OspfTypes::V3:
+	break;
+    }
+    // Don't allow a registration to be overwritten.
+    XLOG_ASSERT(0 == _unknown_lsa_decoder);
+    _unknown_lsa_decoder = lsa;
+}
+
 Lsa::LsaRef
-LsaDecoder::decode(uint8_t *ptr, size_t& len) const throw(BadPacket)
+LsaDecoder::decode(uint8_t *ptr, size_t& len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
     Lsa_header header(version);
 
     if (len < header.length())
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("LSA too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(header.length())));
@@ -476,16 +453,102 @@ LsaDecoder::decode(uint8_t *ptr, size_t& len) const throw(BadPacket)
     map<uint16_t, Lsa *>::const_iterator i;
     uint16_t type = header.get_ls_type();
     i = _lsa_decoders.find(type);
-    if (i == _lsa_decoders.end())
-	xorp_throw(BadPacket,
+    if (i == _lsa_decoders.end()) {
+	if (0 != _unknown_lsa_decoder)
+	    return _unknown_lsa_decoder->decode(ptr, len);
+	xorp_throw(InvalidPacket,
 		   c_format("OSPF Version %u Unknown LSA Type %#x",
 			    version, type));
+    }
     
     Lsa *lsa = i->second;
 
     return lsa->decode(ptr, len);
 }
 
+/* IPv6Prefix */
+
+size_t
+IPv6Prefix::length() const
+{
+    XLOG_ASSERT(OspfTypes::V3 == get_version());
+
+    return bytes_per_prefix(get_network().prefix_len());
+}
+
+IPv6Prefix
+IPv6Prefix::decode(uint8_t *ptr, size_t& len, uint8_t prefixlen,
+		   uint8_t option) const
+    throw(InvalidPacket)
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    IPv6Prefix prefix(version, use_metric());
+    prefix.set_prefix_options(option);
+    
+    uint8_t addr[IPv6::ADDR_BYTELEN];
+    uint32_t bytes = bytes_per_prefix(prefixlen);
+    if (bytes > sizeof(addr)) 
+	xorp_throw(InvalidPacket,
+		   c_format("Prefix length %u larger than %u", bytes,
+			    XORP_UINT_CAST(sizeof(addr))));
+
+    if (bytes > len)
+	xorp_throw(InvalidPacket,
+		   c_format("Prefix length %u larger than packet %u", bytes,
+			    XORP_UINT_CAST(len)));
+
+    memset(&addr[0], 0, IPv6::ADDR_BYTELEN);
+    memcpy(&addr[0], ptr, bytes);
+    IPv6 v6;
+    v6.set_addr(&addr[0]);
+    IPNet<IPv6> v6net(v6, prefixlen);
+    
+    prefix.set_network(v6net);
+    len = bytes;
+
+    return prefix;
+}
+
+/**
+ * The caller should have called length() to pre-allocate the space
+ * required.
+ */
+size_t
+IPv6Prefix::copy_out(uint8_t *ptr) const
+{
+    XLOG_ASSERT(OspfTypes::V3 == get_version());
+
+    IPv6 v6 = get_network().masked_addr();
+    uint8_t buf[IPv6::ADDR_BYTELEN];
+    v6.copy_out(&buf[0]);
+    size_t bytes = bytes_per_prefix(get_network().prefix_len());
+    memcpy(ptr, &buf[0], bytes);
+
+    return bytes;
+}
+
+string
+IPv6Prefix::str() const
+{
+    XLOG_ASSERT(OspfTypes::V3 == get_version());
+
+    string output;
+
+    output = c_format("Options %#4x", get_prefix_options());
+    output += c_format(" DN-bit: %d", get_dn_bit());
+    output += c_format(" P-bit: %d", get_p_bit());
+    output += c_format(" MC-bit: %d", get_mc_bit());
+    output += c_format(" LA-bit: %d", get_la_bit());
+    output += c_format(" NU-bit: %d", get_nu_bit());
+    if (use_metric())
+	output += c_format(" Metric %u", get_metric());
+    output += c_format(" Address %s", cstring(get_network()));
+
+    return output;
+}
+
 /* RouterLink */
 
 size_t
@@ -504,10 +567,10 @@ RouterLink::length() const
 }
 
 RouterLink
-RouterLink::decode(uint8_t *ptr, size_t& len) const throw(BadPacket)
+RouterLink::decode(uint8_t *ptr, size_t& len) const throw(InvalidPacket)
 {
     if (len < length())
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("RouterLink too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(length())));
@@ -539,7 +602,7 @@ RouterLink::decode(uint8_t *ptr, size_t& len) const throw(BadPacket)
 	    link.set_type(vlink);
 	    break;
 	default:
-	    xorp_throw(BadPacket,
+	    xorp_throw(InvalidPacket,
 		   c_format("RouterLink illegal type should be 0..4 not %u",
 			    XORP_UINT_CAST(type)));
 		break;
@@ -564,7 +627,7 @@ RouterLink::decode(uint8_t *ptr, size_t& len) const throw(BadPacket)
 	    link.set_type(vlink);
 	    break;
 	default:
-	    xorp_throw(BadPacket,
+	    xorp_throw(InvalidPacket,
 	    c_format("RouterLink illegal type should be 1,2 or 4 not %u",
 			    XORP_UINT_CAST(type)));
 		break;
@@ -620,31 +683,42 @@ RouterLink::str() const
 
     output = c_format("Type %u", get_type());
 
+    switch(get_type()) {
+    case p2p:
+	output += c_format(" Point-to-point");
+	break;
+    case transit:
+	output += c_format(" Transit network");
+	break;
+    case stub:
+	output += c_format(" Stub network");
+	break;
+    case vlink:
+	output += c_format(" Virtual Link");
+	break;
+    }
+
     switch(get_version()) {
     case OspfTypes::V2:
 	switch(get_type()) {
 	case p2p:
-	    output += c_format(" Point-to-point");
 	    output += c_format(" Neighbours Router ID %s",
 			       pr_id(get_link_id()).c_str());
 	    output += c_format(" Routers interface address %s",
 			       pr_id(get_link_data()).c_str());
 	    break;
 	case transit:
-	    output += c_format(" Transit network");
 	    output += c_format(" IP address of Designated router %s",
 			       pr_id(get_link_id()).c_str());
 	    output += c_format(" Routers interface address %s",
 			       pr_id(get_link_data()).c_str());
 	    break;
 	case stub:
-	    output += c_format(" Stub network");
 	    output += c_format(" Subnet number %s",
 			       pr_id(get_link_id()).c_str());
 	    output += c_format(" Mask %s", pr_id(get_link_data()).c_str());
 	    break;
 	case vlink:
-	    output += c_format(" Virtual Link");
 	    output += c_format(" Neighbours Router ID %s",
 			       pr_id(get_link_id()).c_str());
 	    output += c_format(" Routers interface address %s",
@@ -653,11 +727,21 @@ RouterLink::str() const
 	}
 	break;
     case OspfTypes::V3:
-	output += c_format(" Interface ID %#x", get_interface_id());
-	output += c_format(" Neighbour Interface ID %#x",
-			   get_neighbour_interface_id());
-	output += c_format(" Neighbour Router ID %#x",
-			   get_neighbour_router_id());
+	output += c_format(" Interface ID %u", get_interface_id());
+	switch(get_type()) {
+	case transit:
+	    output += c_format(" Designated Router Interface ID %u",
+			       get_neighbour_interface_id());
+	    output += c_format(" Designated Router ID %s",
+			       pr_id(get_neighbour_router_id()).c_str());
+	    break;
+	default:
+	    output += c_format(" Neighbour Interface ID %u",
+			       get_neighbour_interface_id());
+	    output += c_format(" Neighbour Router ID %s",
+			       pr_id(get_neighbour_router_id()).c_str());
+	    break;
+	}
 	break;
     }
 
@@ -667,7 +751,7 @@ RouterLink::str() const
 }
 
 Lsa::LsaRef
-RouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
+UnknownLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -675,17 +759,74 @@ RouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
     size_t required = header_length + min_length();
 
     if (len < required)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
+		   c_format("Unknown-LSA too short %u, must be at least %u",
+			    XORP_UINT_CAST(len),
+			    XORP_UINT_CAST(required)));
+
+    // This guy throws an exception of there is a problem.
+    len = get_lsa_len_from_header("Unknown-LSA", buf, len, required);
+
+    // Verify the checksum.
+    if (!verify_checksum(buf + 2, len - 2, 16 - 2))
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
+
+    UnknownLsa *lsa = 0;
+    try {
+	lsa = new UnknownLsa(version, buf, len);
+
+	// Decode the LSA Header.
+	lsa->_header.decode_inline(buf);
+	
+    } catch(InvalidPacket& e) {
+	delete lsa;
+	throw e;
+    }
+
+    return Lsa::LsaRef(lsa);
+}
+
+bool
+UnknownLsa::encode()
+{
+    XLOG_FATAL("Can't encode an Unknown-LSA");
+
+    return true;
+}
+
+string
+UnknownLsa::str() const
+{
+    string output;
+
+    output += "Unknown-LSA:\n";
+    if (!valid())
+	output += "INVALID\n";
+    output += _header.str();
+
+    return output;
+}
+
+Lsa::LsaRef
+RouterLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
+{
+    OspfTypes::Version version = get_version();
+
+    size_t header_length = _header.length();
+    size_t required = header_length + min_length();
+
+    if (len < required)
+	xorp_throw(InvalidPacket,
 		   c_format("Router-LSA too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(required)));
 
     // This guy throws an exception of there is a problem.
-    len = get_lsa_len_from_header("Router-LSA", buf, len);
+    len = get_lsa_len_from_header("Router-LSA", buf, len, required);
 
     // Verify the checksum.
     if (!verify_checksum(buf + 2, len - 2, 16 - 2))
-	xorp_throw(BadPacket, c_format("LSA Checksum failed"));
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
 
     RouterLsa *lsa = 0;
     try {
@@ -710,7 +851,7 @@ RouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	    lsa->set_v_bit(flag & 0x4);
 	    lsa->set_e_bit(flag & 0x2);
 	    lsa->set_b_bit(flag & 0x1);
-	    lsa->set_options(extract_32(&buf[header_length]) & 0xffffff);
+	    lsa->set_options(extract_24(&buf[header_length + 1]));
 	    break;
 	}
 
@@ -728,7 +869,7 @@ RouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	switch(version) {
 	case OspfTypes::V2:
 	    if (nlinks != lsa->get_router_links().size())
-		xorp_throw(BadPacket,
+		xorp_throw(InvalidPacket,
 			   c_format(
 				    "Router-LSA mismatch in router links"
 				    " expected %u received %u",
@@ -740,7 +881,7 @@ RouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	    break;
 	}
 
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	delete lsa;
 	throw e;
     }
@@ -791,8 +932,7 @@ RouterLsa::encode()
 	    flag |= 0x2;
 	if (get_b_bit())
 	    flag |= 0x1;
-	// Careful Options occupy 3 bytes, four bytes are written out.
-	embed_32(&ptr[header_length], get_options());
+	embed_24(&ptr[header_length + 1], get_options());
 	break;
     }
     ptr[header_length] = flag;
@@ -863,7 +1003,7 @@ RouterLsa::str() const
 }
 
 Lsa::LsaRef
-NetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
+NetworkLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -871,17 +1011,17 @@ NetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
     size_t required = header_length + min_length();
 
     if (len < required)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Network-LSA too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(required)));
 
     // This guy throws an exception of there is a problem.
-    len = get_lsa_len_from_header("Network-LSA", buf, len);
+    len = get_lsa_len_from_header("Network-LSA", buf, len, required);
 
     // Verify the checksum.
     if (!verify_checksum(buf + 2, len - 2, 16 - 2))
-	xorp_throw(BadPacket, c_format("LSA Checksum failed"));
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
 
     NetworkLsa *lsa = 0;
     try {
@@ -896,7 +1036,7 @@ NetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	    start = &buf[header_length + 4];
 	    break;
 	case OspfTypes::V3:
-	    lsa->set_options(extract_32(&buf[header_length]) & 0xffffff);
+	    lsa->set_options(extract_24(&buf[header_length + 1]));
 	    start = &buf[header_length + 4];
 	    break;
 	}
@@ -904,12 +1044,12 @@ NetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	uint8_t *end = &buf[len];
 	while(start < end) {
 	    if (!(start < end))
-		xorp_throw(BadPacket, c_format("Network-LSA too short"));
+		xorp_throw(InvalidPacket, c_format("Network-LSA too short"));
 	    lsa->get_attached_routers().push_back(extract_32(start));
 	    start += 4;
 	}
 
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	delete lsa;
 	throw e;
     }
@@ -951,8 +1091,7 @@ NetworkLsa::encode()
 	index = header_length + 4;
 	break;
     case OspfTypes::V3:
-	// Careful Options occupy 3 bytes, four bytes are written out.
-	embed_32(&ptr[header_length], get_options());
+	embed_24(&ptr[header_length + 1], get_options());
 	index = header_length + 4;
 	break;
     }
@@ -1020,7 +1159,7 @@ NetworkLsa::str() const
 }
 
 Lsa::LsaRef
-SummaryNetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
+SummaryNetworkLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -1028,17 +1167,17 @@ SummaryNetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
     size_t required = header_length + min_length();
 
     if (len < required)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Summary-LSA too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(required)));
 
     // This guy throws an exception of there is a problem.
-    len = get_lsa_len_from_header("Summary-LSA", buf, len);
+    len = get_lsa_len_from_header("Summary-LSA", buf, len, required);
 
     // Verify the checksum.
     if (!verify_checksum(buf + 2, len - 2, 16 - 2))
-	xorp_throw(BadPacket, c_format("LSA Checksum failed"));
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
 
     SummaryNetworkLsa *lsa = 0;
     try {
@@ -1053,16 +1192,23 @@ SummaryNetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	    break;
 	case OspfTypes::V3:
 	    lsa->set_metric(extract_24(&buf[header_length + 1]));
- 	    lsa->set_prefix_options(buf[header_length + 5]);
-
-	    IPNet<IPv6> v6net = get_ipv6_net("Summary-LSA",
-					     &buf[header_length + 8],
-					     buf[header_length + 4]);
-	    lsa->set_network(v6net);
+	    IPv6Prefix prefix(version);
+	    size_t space = len - IPV6_PREFIX_OFFSET;
+	    IPv6Prefix prefix_decoder(version);
+	    prefix = prefix_decoder.decode(&buf[header_length + 8],
+					   space,
+					   buf[header_length + 4],
+					   buf[header_length + 5]);
+	    size_t space_left = (len - (IPV6_PREFIX_OFFSET + space));
+	    if (0 != space_left) 
+		xorp_throw(InvalidPacket,
+			   c_format("Space left in LSA %u bytes",
+				    XORP_UINT_CAST(space_left)));
+	    lsa->set_ipv6prefix(prefix);
 	    break;
 	}
 
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	delete lsa;
 	throw e;
     }
@@ -1070,7 +1216,6 @@ SummaryNetworkLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
     return Lsa::LsaRef(lsa);
 }
 
-
 bool
 SummaryNetworkLsa::encode()
 {
@@ -1083,8 +1228,7 @@ SummaryNetworkLsa::encode()
  	len = _header.length() + 8;
 	break;
     case OspfTypes::V3:
-	len = _header.length() + 8 + 
-	    bytes_per_prefix(get_network().prefix_len());
+	len = _header.length() + 8 + get_ipv6prefix().length();
 	break;
     }
 
@@ -1108,11 +1252,10 @@ SummaryNetworkLsa::encode()
 	break;
     case OspfTypes::V3:
 	embed_24(&ptr[header_length + 1], get_metric());
-	IPNet<IPv6> v6net = get_network();
-	ptr[header_length + 4] = v6net.prefix_len();
-	ptr[header_length + 5] = get_prefix_options();
-	uint32_t bytes = put_ipv6_net(v6net, &ptr[header_length + 8]);
-	index = header_length + 8 + bytes;
+	IPv6Prefix prefix = get_ipv6prefix();
+	ptr[header_length + 4] = prefix.get_network().prefix_len();
+	ptr[header_length + 5] = prefix.get_prefix_options();
+	index = header_length + 8 + prefix.copy_out(&ptr[header_length + 8]);
 	break;
     }
 
@@ -1152,8 +1295,7 @@ SummaryNetworkLsa::str() const
 	break;
     case OspfTypes::V3:
 	output += c_format("\n\tMetric %d", get_metric());
-	output += c_format("\n\tPrefixOptions %#x", get_prefix_options());
-	output += c_format("\n\tNetwork %s", cstring(get_network()));
+	output += c_format("\n\tIPv6Prefix %s", cstring(get_ipv6prefix()));
 	break;
     }
 
@@ -1161,7 +1303,7 @@ SummaryNetworkLsa::str() const
 }
 
 Lsa::LsaRef
-SummaryRouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
+SummaryRouterLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -1169,17 +1311,17 @@ SummaryRouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
     size_t required = header_length + min_length();
 
     if (len < required)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Summary-LSA too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(required)));
 
     // This guy throws an exception of there is a problem.
-    len = get_lsa_len_from_header("Summary-LSA", buf, len);
+    len = get_lsa_len_from_header("Summary-LSA", buf, len, required);
 
     // Verify the checksum.
     if (!verify_checksum(buf + 2, len - 2, 16 - 2))
-	xorp_throw(BadPacket, c_format("LSA Checksum failed"));
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
 
     SummaryRouterLsa *lsa = 0;
     try {
@@ -1199,7 +1341,7 @@ SummaryRouterLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	    break;
 	}
 
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	delete lsa;
 	throw e;
     }
@@ -1284,7 +1426,8 @@ SummaryRouterLsa::str() const
 	output += c_format("\n\tMetric %d", get_metric());
 	break;
     case OspfTypes::V3:
-	output += c_format("\n\tOptions %#x", get_options());
+	output += c_format("\n\tOptions %#x %s", get_options(),
+			   cstring(Options(get_version(), get_options())));
 	output += c_format("\n\tMetric %d", get_metric());
 	output += c_format("\n\tDestination Router ID %s",
 			   pr_id(get_destination_id()).c_str());
@@ -1295,7 +1438,7 @@ SummaryRouterLsa::str() const
 }
 
 Lsa::LsaRef
-ASExternalLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
+ASExternalLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -1303,18 +1446,18 @@ ASExternalLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
     size_t required = header_length + min_length();
 
     if (len < required)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("AS-External-LSA too short %u, "
 			    "must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(required)));
 
     // This guy throws an exception of there is a problem.
-    len = get_lsa_len_from_header("AS-External-LSA", buf, len);
+    len = get_lsa_len_from_header("AS-External-LSA", buf, len, required);
 
     // Verify the checksum.
     if (!verify_checksum(buf + 2, len - 2, 16 - 2))
-	xorp_throw(BadPacket, c_format("LSA Checksum failed"));
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
 
     ASExternalLsa *lsa = 0;
     try {
@@ -1342,32 +1485,44 @@ ASExternalLsa::decode(uint8_t *buf, size_t& len) const throw(BadPacket)
 	    lsa->set_f_bit(flag & 0x2);
 	    lsa->set_t_bit(flag & 0x1);
 	    lsa->set_metric(extract_24(&buf[header_length + 1]));
-	    lsa->set_prefix_options(buf[header_length + 5]);
-	    lsa->set_referenced_ls_type(extract_16(&buf[header_length + 6]));
-	    
-	    IPNet<IPv6> v6net = get_ipv6_net("AS-External-LSA",
-					     &buf[header_length + 8],
-					     buf[header_length + 4]);
-	    lsa->set_network(v6net);
-	    size_t index = header_length + 8 + 
-		bytes_per_prefix(buf[header_length + 4]);
+ 	    lsa->set_referenced_ls_type(extract_16(&buf[header_length + 6]));
+	    size_t space = len - IPV6_PREFIX_OFFSET;
+	    IPv6Prefix prefix_decoder(version);
+	    lsa->set_ipv6prefix(prefix_decoder.decode(&buf[header_length + 8],
+						      space,
+						      buf[header_length + 4],
+						      buf[header_length + 5]));
+	    size_t index = header_length + 8 + space;
 	    if (lsa->get_f_bit()) {
+		if (index + IPv6::ADDR_BYTELEN > len)
+		    xorp_throw(InvalidPacket,
+			       c_format("AS-External-LSA"
+					" bit F set, packet too short"));
 		IPv6 address;
 		address.copy_in(&buf[index]);
 		lsa->set_forwarding_address_ipv6(address);
-		index += IPv6::ADDR_BITLEN / 8;
+		index += IPv6::ADDR_BYTELEN;
 	    }
 	    if (lsa->get_t_bit()) {
+		if (index + 4 > len)
+		    xorp_throw(InvalidPacket,
+			       c_format("AS-External-LSA"
+					" bit T set, packet too short"));
 		lsa->set_external_route_tag(extract_32(&buf[index]));
 		index += 4;
 	    }
 	    if (0 != lsa->get_referenced_ls_type()) {
+		if (index + 4 > len)
+		    xorp_throw(InvalidPacket,
+			       c_format("AS-External-LSA"
+					" Referenced LS Type set, "
+					"packet too short"));
 		lsa->set_referenced_link_state_id(extract_32(&buf[index]));
 	    }
 	    break;
 	}
 
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	delete lsa;
 	throw e;
     }
@@ -1388,8 +1543,8 @@ ASExternalLsa::encode()
 	break;
     case OspfTypes::V3:
 	len = _header.length() + 8 +
-	    bytes_per_prefix(get_network().prefix_len()) +
-	    (get_f_bit() ? IPv6::ADDR_BITLEN / 8 : 0) +
+	    get_ipv6prefix().length() +
+	    (get_f_bit() ? IPv6::ADDR_BYTELEN : 0) +
 	    (get_t_bit() ? 4 : 0) +
 	    (0 != get_referenced_ls_type() ? 4 : 0);
 	break;
@@ -1430,15 +1585,15 @@ ASExternalLsa::encode()
 	    flag |= 0x1;
 	ptr[header_length] = flag;
 	embed_24(&ptr[header_length + 1], get_metric());
-	IPNet<IPv6> v6net = get_network();
-	ptr[header_length + 4] = v6net.prefix_len();
-	ptr[header_length + 5] = get_prefix_options();
-	embed_16(&ptr[header_length + 6], get_referenced_ls_type());
-	index = header_length +8+ put_ipv6_net(v6net, &ptr[header_length + 8]);
+  	embed_16(&ptr[header_length + 6], get_referenced_ls_type());
+	IPv6Prefix prefix = get_ipv6prefix();
+	ptr[header_length + 4] = prefix.get_network().prefix_len();
+	ptr[header_length + 5] = prefix.get_prefix_options();
+	index = header_length + 8 + prefix.copy_out(&ptr[header_length + 8]);
 	if (get_f_bit()) {
 	    IPv6 forwarding_address = get_forwarding_address_ipv6();
 	    forwarding_address.copy_out(&ptr[index]);
-	    index += IPv6::ADDR_BITLEN / 8;
+	    index += IPv6::ADDR_BYTELEN;
 	}
 	if (get_t_bit()) {
 	    embed_32(&ptr[index], get_external_route_tag());
@@ -1492,10 +1647,9 @@ ASExternalLsa::str() const
 	output += c_format("\n\tMetric %d %#x", get_metric(), get_metric());
 	if (get_metric() == OspfTypes::LSInfinity)
 	    output += c_format(" LSInfinity");
-	output += c_format("\n\tPrefixOptions %#x", get_prefix_options());
+	output += c_format("\n\tIPv6Prefix %s", cstring(get_ipv6prefix()));
 	output += c_format("\n\tReferenced LS Type %#x",
 			   get_referenced_ls_type());
-	output += c_format("\n\tNetwork %s", cstring(get_network()));
 	if (get_f_bit())
 	    output += c_format("\n\tForwarding address %s",
 			       cstring(get_forwarding_address_ipv6()));
@@ -1503,11 +1657,311 @@ ASExternalLsa::str() const
 	    output += c_format("\n\tExternal Route Tag %#x",
 			       get_external_route_tag());
 	if (0 != get_referenced_ls_type())
-	    output += c_format("\n\tReferenced Link State ID %#x",
-			       get_referenced_link_state_id());
+	    output += c_format("\n\tReferenced Link State ID %s",
+			       pr_id(get_referenced_link_state_id()).c_str());
 	    
 	break;
     }
 
     return output;
 }
+
+Lsa::LsaRef
+LinkLsa::decode(uint8_t *buf, size_t& len) const throw(InvalidPacket)
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    size_t header_length = _header.length();
+    size_t required = header_length + min_length();
+
+    if (len < required)
+	xorp_throw(InvalidPacket,
+		   c_format("Link-LSA too short %u, must be at least %u",
+			    XORP_UINT_CAST(len),
+			    XORP_UINT_CAST(required)));
+
+    // This guy throws an exception of there is a problem.
+    len = get_lsa_len_from_header("Link-LSA", buf, len, required);
+
+    // Verify the checksum.
+    if (!verify_checksum(buf + 2, len - 2, 16 - 2))
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
+
+    LinkLsa *lsa = 0;
+    try {
+	lsa = new LinkLsa(version, buf, len);
+
+	// Decode the LSA Header.
+	lsa->_header.decode_inline(buf);
+	uint8_t *start = 0;
+
+	lsa->set_rtr_priority(extract_8(&buf[header_length + 0]));
+	lsa->set_options(extract_24(&buf[header_length + 1]));
+	IPv6 address;
+	address.copy_in(&buf[header_length + 4]);
+	lsa->set_link_local_address(address);
+	size_t prefix_num = extract_32(&buf[header_length + 4 +
+					    IPv6::ADDR_BYTELEN]);
+
+	start = &buf[header_length + 4 + IPv6::ADDR_BYTELEN + 4];
+	uint8_t *end = &buf[len];
+	IPv6Prefix decoder(version);
+	while(start < end) {
+	    if (!(start + 2 < end))
+		xorp_throw(InvalidPacket, c_format("Link-LSA too short"));
+	    size_t space = end - (start + 4);
+	    IPv6Prefix prefix = decoder.decode(start + 4, space,
+					       extract_8(start),
+					       extract_8(start + 1));
+ 	    lsa->get_prefixes().push_back(prefix);
+	    start += (space + 4);
+	    if (0 == --prefix_num) {
+		if (start != end)
+		    xorp_throw(InvalidPacket,
+			       c_format("Link-LSA # prefixes read data left"));
+		break;
+	    }
+	}
+	if (0 != prefix_num)
+	    if (start != end)
+		xorp_throw(InvalidPacket,
+			   c_format("Link-LSA # %d left buffer depleted",
+				    XORP_UINT_CAST(prefix_num)));
+
+    } catch(InvalidPacket& e) {
+	delete lsa;
+	throw e;
+    }
+
+    return Lsa::LsaRef(lsa);
+}
+
+bool
+LinkLsa::encode()
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    size_t len = _header.length() + 4 + IPv6::ADDR_BYTELEN + 4;
+
+    list<IPv6Prefix> &ps = get_prefixes();
+    for(list<IPv6Prefix>::iterator i = ps.begin(); i != ps.end(); i++)
+	len += i->length() + 4;
+
+    _pkt.resize(len);
+    uint8_t *ptr = &_pkt[0];
+//     uint8_t *ptr = new uint8_t[len];
+    memset(ptr, 0, len);
+
+    // Copy the header into the packet
+    _header.set_ls_checksum(0);
+    _header.set_length(len);
+    size_t header_length = _header.copy_out(ptr);
+    XLOG_ASSERT(len > header_length);
+
+    size_t index = 0;
+
+    embed_8(&ptr[header_length], get_rtr_priority());
+    embed_24(&ptr[header_length + 1], get_options());
+    get_link_local_address().copy_out(&ptr[header_length + 4]);
+    embed_32(&ptr[header_length + 4 + IPv6::ADDR_BYTELEN], ps.size());
+
+    index = header_length + 4 + IPv6::ADDR_BYTELEN + 4;
+
+    // Copy out the IPv6 Prefixes.
+    for(list<IPv6Prefix>::iterator i = ps.begin(); i != ps.end(); i++) {
+	embed_8(&ptr[index], i->get_network().prefix_len());
+	embed_8(&ptr[index + 1], i->get_prefix_options());
+	index += i->copy_out(&ptr[index + 4]) + 4;
+    }
+
+    XLOG_ASSERT(index == len);
+
+    // Compute the checksum and write the whole header out again.
+    _header.set_ls_checksum(compute_checksum(ptr + 2, len - 2, 16 - 2));
+    _header.copy_out(ptr);
+
+    return true;
+}
+
+string
+LinkLsa::str() const
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    string output;
+
+    output += "Link-LSA:\n";
+    if (!valid())
+	output += "INVALID\n";
+    output += _header.str();
+
+    output += c_format("\n\tRtr Priority %d", get_rtr_priority());
+    output += c_format("\n\tOptions %#x %s", get_options(),
+		       cstring(Options(get_version(), get_options())));
+    output += c_format("\n\tLink-local Interface Address %s",
+		       cstring(get_link_local_address()));
+
+    list<IPv6Prefix> ps = _prefixes;
+    for(list<IPv6Prefix>::iterator i = ps.begin(); i != ps.end(); i++)
+	output += "\n\tIPv6 Prefix " + i->str();
+
+    return output;
+}
+
+Lsa::LsaRef
+IntraAreaPrefixLsa::decode(uint8_t *buf, size_t& len) const 
+    throw(InvalidPacket)
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    size_t header_length = _header.length();
+    size_t required = header_length + min_length();
+
+    if (len < required)
+	xorp_throw(InvalidPacket,
+		   c_format("Intra-Area-Prefix-LSA too short %u, "
+			    "must be at least %u",
+			    XORP_UINT_CAST(len),
+			    XORP_UINT_CAST(required)));
+
+    // This guy throws an exception of there is a problem.
+    len = get_lsa_len_from_header("Intra-Area-Prefix-LSA", buf, len, required);
+
+    // Verify the checksum.
+    if (!verify_checksum(buf + 2, len - 2, 16 - 2))
+	xorp_throw(InvalidPacket, c_format("LSA Checksum failed"));
+
+    IntraAreaPrefixLsa *lsa = 0;
+    try {
+	lsa = new IntraAreaPrefixLsa(version, buf, len);
+
+	// Decode the LSA Header.
+	lsa->_header.decode_inline(buf);
+	uint8_t *start = 0;
+
+	size_t prefix_num = extract_16(&buf[header_length]);
+	lsa->set_referenced_ls_type(extract_16(&buf[header_length + 2]));
+	lsa->set_referenced_link_state_id(extract_32(&buf[header_length + 4]));
+	lsa->set_referenced_advertising_router(extract_32(&buf[header_length +
+							       8]));
+
+	start = &buf[header_length + 2 + 2 + 4 + 4];
+	uint8_t *end = &buf[len];
+	IPv6Prefix decoder(version, true);
+	while(start < end) {
+	    if (!(start + 2 < end))
+		xorp_throw(InvalidPacket, c_format("Intra-Area-Prefix-LSA "
+						   "too short"));
+	    size_t space = end - (start + 4);
+	    IPv6Prefix prefix = decoder.decode(start + 4, space,
+					       extract_8(start),
+					       extract_8(start + 1));
+	    prefix.set_metric(extract_16(start + 2));
+ 	    lsa->get_prefixes().push_back(prefix);
+	    start += (space + 4);
+	    if (0 == --prefix_num) {
+		if (start != end)
+		    xorp_throw(InvalidPacket,
+			       c_format("Intra-Area-Prefix-LSA # prefixes "
+					"read data left"));
+		break;
+	    }
+	}
+	if (0 != prefix_num)
+	    if (start != end)
+		xorp_throw(InvalidPacket,
+			   c_format("Intra-Area-Prefix-LSA # %d left "
+				    "buffer depleted",
+				    XORP_UINT_CAST(prefix_num)));
+
+    } catch(InvalidPacket& e) {
+	delete lsa;
+	throw e;
+    }
+
+    return Lsa::LsaRef(lsa);
+}
+
+bool
+IntraAreaPrefixLsa::encode()
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    size_t len = _header.length() + 2 + 2 + 4 + 4;
+
+    list<IPv6Prefix> &ps = get_prefixes();
+    for(list<IPv6Prefix>::iterator i = ps.begin(); i != ps.end(); i++)
+	len += i->length() + 4;
+
+    _pkt.resize(len);
+    uint8_t *ptr = &_pkt[0];
+//     uint8_t *ptr = new uint8_t[len];
+    memset(ptr, 0, len);
+
+    // Copy the header into the packet
+    _header.set_ls_checksum(0);
+    _header.set_length(len);
+    size_t header_length = _header.copy_out(ptr);
+    XLOG_ASSERT(len > header_length);
+
+    embed_16(&ptr[header_length], ps.size());
+    embed_16(&ptr[header_length + 2], get_referenced_ls_type());
+    embed_32(&ptr[header_length + 4], get_referenced_link_state_id());
+    embed_32(&ptr[header_length + 8], get_referenced_advertising_router());
+
+    size_t index = header_length + 2 + 2 + 4 + 4;
+
+    // Copy out the IPv6 Prefixes.
+    for(list<IPv6Prefix>::iterator i = ps.begin(); i != ps.end(); i++) {
+	embed_8(&ptr[index], i->get_network().prefix_len());
+	embed_8(&ptr[index + 1], i->get_prefix_options());
+	embed_16(&ptr[index + 2], i->get_metric());
+	index += i->copy_out(&ptr[index + 4]) + 4;
+    }
+
+    XLOG_ASSERT(index == len);
+
+    // Compute the checksum and write the whole header out again.
+    _header.set_ls_checksum(compute_checksum(ptr + 2, len - 2, 16 - 2));
+    _header.copy_out(ptr);
+
+    return true;
+}
+
+string
+IntraAreaPrefixLsa::str() const
+{
+    OspfTypes::Version version = get_version();
+    XLOG_ASSERT(OspfTypes::V3 == version);
+
+    string output;
+
+    output += "Intra-Area-Prefix-LSA:\n";
+    if (!valid())
+	output += "INVALID\n";
+    output += _header.str();
+
+    output += c_format("\n\tReferenced LS type %#x", get_referenced_ls_type());
+    if (get_referenced_ls_type() == RouterLsa(version).get_ls_type()) {
+	output += c_format(" Router-LSA");
+    } else if (get_referenced_ls_type() == NetworkLsa(version).get_ls_type()) {
+	output += c_format(" Network-LSA");
+    } else {
+	output += c_format(" Unknown");
+    }
+    output += c_format("\n\tReferenced Link State ID %s",
+		       pr_id(get_referenced_link_state_id()).c_str());
+    output += c_format("\n\tReferenced Advertising Router %s",
+		       pr_id(get_referenced_advertising_router()).c_str());
+
+    list<IPv6Prefix> ps = _prefixes;
+    for(list<IPv6Prefix>::iterator i = ps.begin(); i != ps.end(); i++)
+	output += "\n\tIPv6 Prefix " + i->str();
+
+    return output;
+}
