@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,37 +13,37 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/ospf.cc,v 1.72 2006/03/28 03:06:54 atanu Exp $"
+#ident "$XORP: xorp/ospf/ospf.cc,v 1.91 2007/03/12 10:16:03 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "config.h"
-#include <map>
-#include <list>
-#include <set>
-
 #include "ospf_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/callback.hh"
-
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
 #include "libxorp/ipnet.hh"
-
 #include "libxorp/status_codes.h"
 #include "libxorp/service.hh"
 #include "libxorp/eventloop.hh"
 
+#include <map>
+#include <list>
+#include <set>
+
 #include "ospf.hh"
 
 template <typename A>
 Ospf<A>::Ospf(OspfTypes::Version version, EventLoop& eventloop, IO<A>* io)
     : _version(version), _eventloop(eventloop),
+      _testing(false),
       _io(io), _reason("Waiting for IO"), _process_status(PROC_STARTUP),
       _lsa_decoder(version), _peer_manager(*this), _routing_table(*this),
+      _instance_id(0), _router_id(0),
       _rfc1583_compatibility(false)
 {
     // Register the LSAs and packets with the associated decoder.
@@ -55,6 +55,9 @@ Ospf<A>::Ospf(OspfTypes::Version version, EventLoop& eventloop, IO<A>* io)
     _io->register_receive(callback(this,&Ospf<A>::receive));
 
     // The peer manager will solicit packets from the various interfaces.
+
+    // Make sure that this value can never be allocated as an interface ID.
+    _iidmap[""] = OspfTypes::UNUSED_INTERFACE_ID;
 }
 
 /**
@@ -77,8 +80,13 @@ Ospf<A>::receive(const string& interface, const string& vif,
 
     Packet *packet;
     try {
+	// If the transport is IPv6 then the checksum verification has
+	// to include the pseudo header. In the IPv4 case this
+	// function is a noop.
+	ipv6_checksum_verify<A>(src, dst, data, len, Packet::CHECKSUM_OFFSET,
+				_io->get_ip_protocol_number());
 	packet = _packet_decoder.decode(data, len);
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	XLOG_ERROR("%s", cstring(e));
 	return;
     }
@@ -132,6 +140,87 @@ Ospf<A>::enabled(const string& interface, const string& vif, A address)
     return _io->is_address_enabled(interface, vif, address);
 }
 
+template <typename A>
+bool
+Ospf<A>::get_addresses(const string& interface, const string& vif,
+		       list<A>& addresses) const
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    return _io->get_addresses(interface, vif, addresses);
+}
+
+template <typename A>
+bool
+Ospf<A>::get_link_local_address(const string& interface, const string& vif,
+				A& address)
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    return _io->get_link_local_address(interface, vif, address);
+}
+
+template <typename A>
+bool
+Ospf<A>::get_interface_id(const string& interface, const string& vif, 
+			  uint32_t& interface_id)
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    string concat = interface + "/" + vif;
+
+    if (0 == _iidmap.count(concat)) {
+	if (string(VLINK) == interface)
+	    interface_id = 100000;
+	else
+	    _io->get_interface_id(interface, interface_id);
+
+	bool match;
+	do {
+	    match = false;
+	    typename map<string, uint32_t>::iterator i;
+	    for(i = _iidmap.begin(); i != _iidmap.end(); i++)
+		if ((*i).second == interface_id) {
+		    interface_id++;
+		    match = true;
+		    break;
+		}
+	} while(match);
+	_iidmap[concat] = interface_id;
+    }
+
+    interface_id = _iidmap[concat];
+
+    XLOG_ASSERT(OspfTypes::UNUSED_INTERFACE_ID != interface_id);
+
+    debug_msg("Interface %s Vif %s ID = %u\n", interface.c_str(), vif.c_str(),
+	      interface_id);
+
+    _io->set_interface_mapping(interface_id, interface, vif);
+
+    return true;
+}
+
+template <typename A>
+bool
+Ospf<A>::get_interface_vif_by_interface_id(uint32_t interface_id,
+					   string& interface, string& vif)
+{
+    typename map<string, uint32_t>::iterator i;
+    for(i = _iidmap.begin(); i != _iidmap.end(); i++) {
+	if ((*i).second == interface_id) {
+	    string concat = (*i).first;
+	    interface = concat.substr(0, concat.find('/'));
+	    vif = concat.substr(concat.find('/') + 1, concat.size() - 1);
+// 	    fprintf(stderr, "interface <%s> vif <%s>\n", interface.c_str(),
+// 		    vif.c_str());
+	    return true;
+	}
+    }
+    
+    return false;
+}
+
 template <typename A>
 bool
 Ospf<A>::get_prefix_length(const string& interface, const string& vif,
@@ -191,13 +280,18 @@ Ospf<A>::transmit(const string& interface, const string& vif,
 {
     debug_msg("Interface %s Vif %s data %p len %u\n",
 	      interface.c_str(), vif.c_str(), data, len);
+
+    // If the transport is IPv6 then the checksum has to include the
+    // pseudo header. In the IPv4 case this function is a noop.
+    ipv6_checksum_apply<A>(src, dst, data, len, Packet::CHECKSUM_OFFSET,
+			   _io->get_ip_protocol_number());
 #ifdef	DEBUG_LOGGING
     try {
 	// Decode the packet in order to pretty print it.
 	Packet *packet = _packet_decoder.decode(data, len);
 	debug_msg("Transmit: %s\n", cstring(*packet));
 	delete packet;
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	debug_msg("Unable to decode packet\n");
     }
 #endif
@@ -205,22 +299,6 @@ Ospf<A>::transmit(const string& interface, const string& vif,
     return _io->send(interface, vif, dst, src, data, len);
 }
 
-template <typename A>
-bool
-Ospf<A>::set_interface_id(const string& interface, const string& vif,
-			  OspfTypes::AreaID area,
-			  uint32_t interface_id)
-{
-    try {
-	_peer_manager.set_interface_id(_peer_manager.get_peerid(interface,vif),
-				       area, interface_id);
-    } catch(BadPeer& e) {
-	XLOG_ERROR("%s", cstring(e));
-	return false;
-    }
-    return true;
-}
-
 template <typename A>
 bool
 Ospf<A>::set_hello_interval(const string& interface, const string& vif,
@@ -399,7 +477,7 @@ Ospf<A>::set_simple_authentication_key(const string&		interface,
 				       const string&		password,
 				       string&			error_msg)
 {
-    PeerID peerid;
+    OspfTypes::PeerID peerid;
 
     try {
 	peerid = _peer_manager.get_peerid(interface, vif);
@@ -426,7 +504,7 @@ Ospf<A>::delete_simple_authentication_key(const string&		interface,
 					  OspfTypes::AreaID	area,
 					  string&		error_msg)
 {
-    PeerID peerid;
+    OspfTypes::PeerID peerid;
 
     try {
 	peerid = _peer_manager.get_peerid(interface, vif);
@@ -458,7 +536,7 @@ Ospf<A>::set_md5_authentication_key(const string&	interface,
 				    string&		error_msg)
 
 {
-    PeerID peerid;
+    OspfTypes::PeerID peerid;
 
     try {
 	peerid = _peer_manager.get_peerid(interface, vif);
@@ -488,7 +566,7 @@ Ospf<A>::delete_md5_authentication_key(const string&		interface,
 				       uint8_t			key_id,
 				       string&			error_msg)
 {
-    PeerID peerid;
+    OspfTypes::PeerID peerid;
 
     try {
 	peerid = _peer_manager.get_peerid(interface, vif);
@@ -626,8 +704,9 @@ Ospf<A>::get_neighbour_info(OspfTypes::NeighbourID nid,
 
 template <typename A>
 bool
-Ospf<A>::add_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
-		   bool discard, const PolicyTags& policytags)
+Ospf<A>::add_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
+		   uint32_t metric, bool equal, bool discard,
+		   const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
 	      cstring(net), cstring(nexthop), metric, pb(equal),
@@ -639,13 +718,15 @@ Ospf<A>::add_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
 	      cstring(net), cstring(nexthop), metric, pb(equal),
 	      pb(discard), cstring(policytags));
 
-    return _io->add_route(net, nexthop, metric, equal, discard, policytags);
+    return _io->add_route(net, nexthop, nexthop_id, metric, equal, discard,
+			  policytags);
 }
 
 template <typename A>
 bool
-Ospf<A>::replace_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
-			bool discard, const PolicyTags& policytags)
+Ospf<A>::replace_route(IPNet<A> net, A nexthop, uint32_t nexthop_id, 
+		       uint32_t metric, bool equal, bool discard,
+		       const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
 	      cstring(net), cstring(nexthop), metric, pb(equal),
@@ -657,7 +738,7 @@ Ospf<A>::replace_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
 	      cstring(net), cstring(nexthop), metric, pb(equal),
 	      pb(discard), cstring(policytags));
 
-    return _io->replace_route(net, nexthop, metric, equal, discard,
+    return _io->replace_route(net, nexthop, nexthop_id, metric, equal, discard,
 			      policytags);
 }
 
