@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,69 +13,159 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/packet.cc,v 1.32 2006/03/28 03:06:54 atanu Exp $"
+#ident "$XORP: xorp/ospf/packet.cc,v 1.43 2007/02/16 22:46:41 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 // #define DEBUG_RAW_PACKETS
 
-#include "config.h"
+#include "ospf_module.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/debug.h"
+#include "libxorp/xlog.h"
+#include "libxorp/callback.hh"
+#include "libxorp/ipv4.hh"
+#include "libxorp/ipv6.hh"
+#include "libxorp/ipnet.hh"
+#include "libxorp/status_codes.h"
+#include "libxorp/service.hh"
+#include "libxorp/eventloop.hh"
+#include "libproto/checksum.h"
+#include "libproto/packet.hh"
+
 #include <map>
 #include <list>
 #include <set>
 #include <vector>
 
-#include "ospf_module.h"
-
-#include "libxorp/debug.h"
-#include "libxorp/xlog.h"
-#include "libxorp/callback.hh"
-
-#include "libxorp/ipv4.hh"
-#include "libxorp/ipv6.hh"
-#include "libxorp/ipnet.hh"
-
-#include "libxorp/status_codes.h"
-#include "libxorp/service.hh"
-#include "libxorp/eventloop.hh"
-
 #include "ospf.hh"
 #include "packet.hh"
 
 /**
- * XXX - Might make sense to move this into libxorp.
- *     - Rewrite this routine to not perform the intermediate copy and
- *       keep it portable to architectures that don't allow
- *       non-aligned accesses.
- *
- * Compute the IP checksum.
+ * Return the IP checksum in host order.
  */
 inline
 uint16_t
 checksum(uint8_t *ptr, size_t len)
 {
-    // Copy the buffer to be checksumed into a temporary buffer to
-    // deal with alignment and odd number of bytes issues.
-    size_t templen = (len + 1) / 2;
-    uint16_t temp[templen];
-    temp[templen - 1] = 0;// Put a zero at the end if the buffer to
-			  // deal with an odd number of bytes.
+    return ntohs(inet_checksum(ptr, len));
+}
 
-    memcpy(&temp[0], ptr, len);
+/**
+ * Return the IPv6 pseudo header checksum in host order.
+ */
+inline
+uint16_t
+ipv6_pseudo_header_checksum(const IPv6& src, const IPv6& dst, size_t len,
+			    uint8_t protocol)
+{
+    uint8_t pseudo_header[16	/* Source address */ 
+			  + 16	/* Destination address */
+			  + 4 	/* Upper-layer packet length */
+			  + 3 	/* Zero */
+			  + 1	/* Upper-layer protocol number */
+			  ];
 
-    uint32_t sum = 0;
-    for(size_t i = 0; i < templen; i++) {
-// 	printf("%u %#x\n", i * 2, temp[i]);
-	sum += temp[i];
-    }
+    src.copy_out(&pseudo_header[0]);
+    dst.copy_out(&pseudo_header[16]);
+    embed_32(&pseudo_header[16 + 16], len);
+    embed_24(&pseudo_header[16 + 16 + 4], 0);
+    pseudo_header[16 + 16 + 4 + 3] = protocol;
 
-    // Fold the carry back in.
-    sum += (sum >> 16) & 0xffff;
+    return checksum(&pseudo_header[0], sizeof(pseudo_header));
+}
 
-    uint16_t result = sum;
-    result ^= 0xffff;
+template <>
+void
+ipv6_checksum_verify<IPv6>(const IPv6& src, const IPv6& dst,
+			   const uint8_t *data, size_t len,
+			   size_t checksum_offset,
+			   uint8_t protocol) throw(InvalidPacket)
+{
+    debug_msg("src %s dst data %p %s len %u chsum offset %u protocol %u\n",
+	      cstring(src), cstring(dst),
+	      data, XORP_UINT_CAST(len), XORP_UINT_CAST(checksum_offset),
+	      protocol);
 
-    return htons(result);	// XXX - Fix the callers.
+    if (len < checksum_offset)
+	xorp_throw(InvalidPacket,
+		   c_format("Checksum offset %u greater than packet length %u",
+			    XORP_UINT_CAST(checksum_offset),
+			    XORP_UINT_CAST(len)));
+
+    if (0 == inet_checksum_add(ipv6_pseudo_header_checksum(src, dst, len,
+							   protocol),
+			       checksum(const_cast<uint8_t *>(data), len)))
+	return;
+
+    // If we get here there is a problem with the checksum. Compute
+    // the expected checksum to aid in debugging.
+    uint8_t *temp = new uint8_t[len];
+    memcpy(&temp[0], &data[0], len);
+    uint16_t checksum_inpacket = extract_16(&temp[checksum_offset]);
+    embed_16(&temp[checksum_offset], 0);
+    uint16_t checksum_computed = 
+	inet_checksum_add(ipv6_pseudo_header_checksum(src, dst, len,
+							   protocol),
+			  checksum(temp, len));
+    delete []temp;
+						 ;
+    if (checksum_inpacket != checksum_computed)
+	xorp_throw(InvalidPacket,
+		   c_format("Checksum mismatch expected %#x received %#x",
+			    checksum_computed,
+			    checksum_inpacket));
+}
+
+template <>
+void
+ipv6_checksum_verify<IPv4>(const IPv4& src, const IPv4& dst,
+			   const uint8_t *data, size_t len,
+			   size_t checksum_offset,
+			   uint8_t protocol) throw(InvalidPacket)
+{
+    debug_msg("src %s dst data %p %s len %u chsum offset %u protocol %u\n",
+	      cstring(src), cstring(dst),
+	      data, XORP_UINT_CAST(len), XORP_UINT_CAST(checksum_offset),
+	      protocol);
+}
+
+template <>
+void
+ipv6_checksum_apply<IPv6>(const IPv6& src, const IPv6& dst,
+			  uint8_t *data, size_t len,
+			  size_t checksum_offset,
+			  uint8_t protocol) throw(InvalidPacket)
+{
+    debug_msg("src %s dst data %p %s len %u chsum offset %u protocol %u\n",
+	      cstring(src), cstring(dst),
+	      data, XORP_UINT_CAST(len), XORP_UINT_CAST(checksum_offset),
+	      protocol);
+
+    if (len < checksum_offset)
+	xorp_throw(InvalidPacket,
+		   c_format("Checksum offset %u greater than packet length %u",
+			    XORP_UINT_CAST(checksum_offset),
+			    XORP_UINT_CAST(len)));
+
+    embed_16(&data[checksum_offset],
+	     inet_checksum_add(ipv6_pseudo_header_checksum(src, dst, len,
+							   protocol),
+			       checksum(data, len)));
+}
+
+template <>
+void
+ipv6_checksum_apply<IPv4>(const IPv4& src, const IPv4& dst,
+			  uint8_t *data, size_t len,
+			  size_t checksum_offset,
+			  uint8_t protocol) throw(InvalidPacket)
+{
+    debug_msg("src %s dst data %p %s len %u chsum offset %u protocol %u\n",
+	      cstring(src), cstring(dst),
+	      data, XORP_UINT_CAST(len), XORP_UINT_CAST(checksum_offset),
+	      protocol);
 }
 
 #ifdef	DEBUG_RAW_PACKETS
@@ -98,7 +188,7 @@ dump_packet(uint8_t *ptr, size_t len)
 /* Packet */
 
 size_t
-Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
+Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(InvalidPacket)
 {
     debug_msg("ptr %p len %u\n", ptr, XORP_UINT_CAST(len));
 #ifdef	DEBUG_RAW_PACKETS
@@ -112,7 +202,7 @@ Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
     // Make sure that at least two bytes have been extracted:
     // Version and Type fields.
     if (len < 2)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(2)));
@@ -126,7 +216,7 @@ Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
 	version = OspfTypes::V3;
 	break;
     default:
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Version mismatch expected %u received %u",
 			    get_version(),
 			    ptr[0] & 0xff));
@@ -134,7 +224,7 @@ Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
     }
 
     if (ptr[1] != get_type())
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Type mismatch expected %u received %u",
 			    get_type(),
 			    ptr[1]));
@@ -143,13 +233,13 @@ Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
     switch(version) {
     case OspfTypes::V2:
 	if (len < STANDARD_HEADER_V2)
-	    xorp_throw(BadPacket,
+	    xorp_throw(InvalidPacket,
 		       c_format("Packet too short %u, must be at least %u",
 				XORP_UINT_CAST(len),
 				XORP_UINT_CAST(STANDARD_HEADER_V2)));
     case OspfTypes::V3:
 	if (len < STANDARD_HEADER_V3)
-	    xorp_throw(BadPacket,
+	    xorp_throw(InvalidPacket,
 		       c_format("Packet too short %u, must be at least %u",
 				XORP_UINT_CAST(len),
 				XORP_UINT_CAST(STANDARD_HEADER_V3)));
@@ -161,7 +251,7 @@ Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
     if (packet_length != len) {
 	// If the frame is too small complain.
 	if (len < packet_length)
-	    xorp_throw(BadPacket,
+	    xorp_throw(InvalidPacket,
 		       c_format("Packet length expected %u received %u",
 			    packet_length,
 			    XORP_UINT_CAST(len)));
@@ -178,42 +268,44 @@ Packet::decode_standard_header(uint8_t *ptr, size_t& len) throw(BadPacket)
     case OspfTypes::V2:
 	// Verify the auth structure is the correct size.
 	static_assert(sizeof(_auth) == (64 / 8));
-	set_auth_type(extract_16(&ptr[14]));
-	memcpy(&_auth[0], &ptr[16], sizeof(_auth));
+	set_auth_type(extract_16(&ptr[Packet::AUTH_TYPE_OFFSET]));
+	memcpy(&_auth[0], &ptr[Packet::AUTH_PAYLOAD_OFFSET], sizeof(_auth));
 	// The authentication field is expected to be zero for the
 	// checksumming.
-	memset(&ptr[16], 0, sizeof(_auth));
+	memset(&ptr[Packet::AUTH_PAYLOAD_OFFSET], 0, sizeof(_auth));
 	break;
     case OspfTypes::V3:
 	set_instance_id(ptr[14]);
+	// For OSPFv3 the checksum has already been verified.
+	return get_standard_header_length();
 	break;
     }
 
     // Extract the checksum and check the packet.
-    uint16_t checksum_inpacket = extract_16(&ptr[12]);
+    uint16_t checksum_inpacket = extract_16(&ptr[Packet::CHECKSUM_OFFSET]);
     // Zero the checksum location.
-    embed_16(&ptr[12], 0);
+    embed_16(&ptr[Packet::CHECKSUM_OFFSET], 0);
     uint16_t checksum_actual = checksum(ptr, len);
 
     // Restore the zero'd fields.
     switch(version) {
     case OspfTypes::V2:
-	memcpy(&ptr[16], &_auth[0], sizeof(_auth));
+	memcpy(&ptr[Packet::AUTH_PAYLOAD_OFFSET], &_auth[0], sizeof(_auth));
 	break;
     case OspfTypes::V3:
 	break;
     }
-    embed_16(&ptr[12], checksum_inpacket);
+    embed_16(&ptr[Packet::CHECKSUM_OFFSET], checksum_inpacket);
 
     if (0 == checksum_inpacket &&
 	OspfTypes::CRYPTOGRAPHIC_AUTHENTICATION == get_auth_type())
 	return get_standard_header_length();
     
     if (checksum_inpacket != checksum_actual)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Checksum mismatch expected %#x received %#x",
-			    checksum_inpacket,
-			    checksum_actual));
+			    checksum_actual,
+			    checksum_inpacket));
 
     // Return the offset at which continued processing can take place.
     return get_standard_header_length();
@@ -244,19 +336,21 @@ Packet::encode_standard_header(uint8_t *ptr, size_t len)
     
     switch(version) {
     case OspfTypes::V2:
-	embed_16(&ptr[14], get_auth_type());
+	embed_16(&ptr[Packet::AUTH_TYPE_OFFSET], get_auth_type());
 	break;
     case OspfTypes::V3:
 	ptr[14] = get_instance_id();
+	// For OSPFv3 the checksum will be written later.
+	return get_standard_header_length();
 	break;
     }
 
-    embed_16(&ptr[12], checksum(ptr, len));
+    embed_16(&ptr[Packet::CHECKSUM_OFFSET], checksum(ptr, len));
 
     // OSPFv2 only copy the authentication data out.
     switch(version) {
     case OspfTypes::V2:
-	memcpy(&ptr[16], &_auth[0], sizeof(_auth));
+	memcpy(&ptr[Packet::AUTH_PAYLOAD_OFFSET], &_auth[0], sizeof(_auth));
 	break;
     case OspfTypes::V3:
 	break;
@@ -339,12 +433,12 @@ PacketDecoder::register_decoder(Packet *packet)
 }
 
 Packet *
-PacketDecoder::decode(uint8_t *ptr, size_t len) throw(BadPacket)
+PacketDecoder::decode(uint8_t *ptr, size_t len) throw(InvalidPacket)
 {
     // Make sure that at least two bytes have been extracted:
     // Version and Type fields.
     if (len < 2)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(2)));
@@ -358,7 +452,7 @@ PacketDecoder::decode(uint8_t *ptr, size_t len) throw(BadPacket)
 	version = OspfTypes::V3;
 	break;
     default:
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Unknown OSPF Version %u", ptr[0] & 0xff));
 	break;
     }
@@ -380,7 +474,7 @@ PacketDecoder::decode(uint8_t *ptr, size_t len) throw(BadPacket)
     }
     
     if (bad)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("OSPF Version %u Unknown Type %u", version, type));
     
     Packet *packet = i->second;
@@ -391,7 +485,7 @@ PacketDecoder::decode(uint8_t *ptr, size_t len) throw(BadPacket)
 /* HelloPacket */
 
 Packet *
-HelloPacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
+HelloPacket::decode(uint8_t *ptr, size_t len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -402,7 +496,7 @@ HelloPacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
     // Verify that this packet is large enough, up to but not including
     // any neighbours.
     if ((len - offset) < MINIMUM_LENGTH)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(offset + MINIMUM_LENGTH)));
@@ -539,7 +633,7 @@ HelloPacket::str() const
 /* Database Description packet */
 
 Packet *
-DataDescriptionPacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
+DataDescriptionPacket::decode(uint8_t *ptr, size_t len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -550,7 +644,7 @@ DataDescriptionPacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
     // Verify that this packet is large enough, up to but not including
     // any neighbours.
     if ((len - offset) < minimum_length())
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(offset + minimum_length())));
@@ -696,7 +790,7 @@ DataDescriptionPacket::str() const
 /* Link State Request Packet */
 
 Packet *
-LinkStateRequestPacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
+LinkStateRequestPacket::decode(uint8_t *ptr, size_t len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -709,7 +803,7 @@ LinkStateRequestPacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
     // Verify that this packet is large enough, a standard header plus
     // at least one request
     if ((len - offset) < ls.length())
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(offset + ls.length())));
@@ -784,7 +878,7 @@ LinkStateRequestPacket::str() const
 /* Link State Update Packet */
 
 Packet *
-LinkStateUpdatePacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
+LinkStateUpdatePacket::decode(uint8_t *ptr, size_t len) const throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -798,7 +892,7 @@ LinkStateUpdatePacket::decode(uint8_t *ptr, size_t len) const throw(BadPacket)
     size_t min_length = _lsa_decoder.min_length();
 
     if ((len - offset) < min_length)
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(offset + min_length)));
@@ -907,7 +1001,7 @@ LinkStateUpdatePacket::str() const
 
 Packet *
 LinkStateAcknowledgementPacket::decode(uint8_t *ptr, size_t len) const
-    throw(BadPacket)
+    throw(InvalidPacket)
 {
     OspfTypes::Version version = get_version();
 
@@ -919,7 +1013,7 @@ LinkStateAcknowledgementPacket::decode(uint8_t *ptr, size_t len) const
     // Verify that this packet is large enough to hold the at least
     // one LSA header.
     if ((len - offset) < Lsa_header::length())
-	xorp_throw(BadPacket,
+	xorp_throw(InvalidPacket,
 		   c_format("Packet too short %u, must be at least %u",
 			    XORP_UINT_CAST(len),
 			    XORP_UINT_CAST(offset + Lsa_header::length())));
