@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,31 +13,29 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/peer_manager.cc,v 1.121 2006/06/12 18:19:49 atanu Exp $"
+#ident "$XORP: xorp/ospf/peer_manager.cc,v 1.139 2007/03/13 18:25:51 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "config.h"
+#include "ospf_module.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/debug.h"
+#include "libxorp/xlog.h"
+#include "libxorp/callback.hh"
+#include "libxorp/ipv4.hh"
+#include "libxorp/ipv6.hh"
+#include "libxorp/ipnet.hh"
+#include "libxorp/status_codes.h"
+#include "libxorp/service.hh"
+#include "libxorp/eventloop.hh"
+
 #include <map>
 #include <list>
 #include <set>
 #include <queue>
 
-#include "ospf_module.h"
-
-#include "libxorp/debug.h"
-#include "libxorp/xlog.h"
-#include "libxorp/callback.hh"
-
-#include "libxorp/ipv4.hh"
-#include "libxorp/ipv6.hh"
-#include "libxorp/ipnet.hh"
-
-#include "libxorp/status_codes.h"
-#include "libxorp/service.hh"
-#include "libxorp/eventloop.hh"
-
 #include "libproto/spt.hh"
 
 #include "ospf.hh"
@@ -49,6 +47,7 @@
 #include "peer.hh"
 #include "peer_manager.hh"
 
+
 template <typename A>
 PeerManager<A>::~PeerManager()
 {
@@ -188,7 +187,7 @@ PeerManager<A>::change_area_router_type(OspfTypes::AreaID area,
     // area to peer mapping is not held so all peers must be notified.
     // When the correct area is found set the new options for the
     // hello packet.
-    typename map<PeerID, PeerOut<A> *>::iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++)
 	if ((*i).second->change_area_router_type(area, area_type))
 	    (*i).second->set_options(area, compute_options(area_type));
@@ -215,7 +214,7 @@ PeerManager<A>::destroy_area_router(OspfTypes::AreaID area)
     // Notify the peers that this area is being removed. If this is
     // the only area that the peer belonged to the peer can signify
     // this and the peer can be removed.
-    typename map<PeerID, PeerOut<A> *>::iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::iterator i;
     for(i = _peers.begin(); i != _peers.end();)
 	if ((*i).second->remove_area(area)) {
 	    delete_peer((*i).first);
@@ -333,7 +332,7 @@ bool
 PeerManager<A>::get_neighbour_list(list<OspfTypes::NeighbourID>& neighbours)
     const
 {
-    typename map<PeerID, PeerOut<A> *>::const_iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++)
 	(*i).second->get_neighbour_list(neighbours);
 
@@ -347,7 +346,7 @@ PeerManager<A>::get_neighbour_info(OspfTypes::NeighbourID nid,
 {
     list<OspfTypes::NeighbourID> neighbours;
 
-    typename map<PeerID, PeerOut<A> *>::const_iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++) {
 	(*i).second->get_neighbour_list(neighbours);
 	list<OspfTypes::NeighbourID>::const_iterator j;
@@ -363,7 +362,7 @@ PeerManager<A>::get_neighbour_info(OspfTypes::NeighbourID nid,
 }
 
 template <typename A>
-PeerID
+OspfTypes::PeerID
 PeerManager<A>::create_peerid(const string& interface, const string& vif)
     throw(BadPeer)
 {
@@ -374,14 +373,14 @@ PeerManager<A>::create_peerid(const string& interface, const string& vif)
 	xorp_throw(BadPeer,
 		   c_format("Mapping for %s already exists", concat.c_str()));
 			    
-    PeerID peerid = _next_peerid++;
+    OspfTypes::PeerID peerid = _next_peerid++;
     _pmap[concat] = peerid;
 
     return peerid;
 }
 
 template <typename A>
-PeerID
+OspfTypes::PeerID
 PeerManager<A>::get_peerid(const string& interface, const string& vif)
     throw(BadPeer)
 {
@@ -395,6 +394,27 @@ PeerManager<A>::get_peerid(const string& interface, const string& vif)
     return _pmap[concat];
 }
 
+template <typename A>
+bool
+PeerManager<A>::get_interface_vif_by_peerid(OspfTypes::PeerID peerid,
+					    string& interface, string& vif)
+    const
+{
+    debug_msg("PeerID %u\n", peerid);
+
+    typename map<string, OspfTypes::PeerID>::const_iterator pi;
+    for(pi = _pmap.begin(); pi != _pmap.end(); pi++) {
+	if ((*pi).second == peerid) {
+	    string concat = (*pi).first;
+	    interface = concat.substr(0, concat.find('/'));
+	    vif = concat.substr(concat.find('/') + 1, concat.size() - 1);
+	    return true;
+	}
+    }
+
+    return false;
+}
+
 template <typename A>
 void
 PeerManager<A>::destroy_peerid(const string& interface, const string& vif)
@@ -411,19 +431,16 @@ PeerManager<A>::destroy_peerid(const string& interface, const string& vif)
 }
 
 template <typename A>
-PeerID
+OspfTypes::PeerID
 PeerManager<A>::create_peer(const string& interface, const string& vif,
-			    const A source,
-			    uint16_t interface_prefix_length,
-			    uint16_t interface_mtu,
+			    A source,
 			    OspfTypes::LinkType linktype, 
 			    OspfTypes::AreaID area)
     throw(BadPeer)
 {
-    debug_msg("Interface %s Vif %s source net %s mtu %d linktype %u area %s\n",
+    debug_msg("Interface %s Vif %s source net %s linktype %u area %s\n",
 	      interface.c_str(), vif.c_str(),
-	      cstring(source),  interface_mtu,
-	      linktype, pr_id(area).c_str());
+	      cstring(source),linktype, pr_id(area).c_str());
 
     AreaRouter<A> *area_router = get_area_router(area);
 
@@ -432,9 +449,28 @@ PeerManager<A>::create_peer(const string& interface, const string& vif,
 	xorp_throw(BadPeer, 
 		   c_format("Unknown Area %s", pr_id(area).c_str()));
 
-    PeerID peerid = create_peerid(interface, vif);
+    OspfTypes::PeerID peerid = create_peerid(interface, vif);
+
+    switch (_ospf.get_version()) {
+    case OspfTypes::V2:
+	break;
+    case OspfTypes::V3:
+	if (OspfTypes::VirtualLink != linktype) {
+	    // Note that the source address is going to be replaced with
+	    // the link local address, unless this is a virtual link.
+	    if (!_ospf.get_link_local_address(interface, vif, source)) {
+		destroy_peerid(interface, vif);
+		xorp_throw(BadPeer, 
+			   c_format("Unable to get link local address for "
+				    "%s/%s",
+				    interface.c_str(), vif.c_str()));
+	    }
+	}
+	break;
+    }
 
     // Get the prefix length.
+    uint16_t interface_prefix_length;
     if (!_ospf.get_prefix_length(interface, vif, source,
 				 interface_prefix_length)) {
 	destroy_peerid(interface, vif);
@@ -444,7 +480,7 @@ PeerManager<A>::create_peer(const string& interface, const string& vif,
     }
 
     // Get the MTU.
-    interface_mtu = _ospf.get_mtu(interface);
+    uint16_t interface_mtu = _ospf.get_mtu(interface);
     if (0 == interface_mtu) {
 	destroy_peerid(interface, vif);
 	xorp_throw(BadPeer, 
@@ -459,11 +495,41 @@ PeerManager<A>::create_peer(const string& interface, const string& vif,
 				    interface_mtu, linktype,
 				    area, area_router->get_area_type());
 
+    switch (_ospf.get_version()) {
+    case OspfTypes::V2:
+	break;
+    case OspfTypes::V3: {
+	uint32_t interface_id;
+	if (!_ospf.get_interface_id(interface, vif, interface_id)) {
+	    delete_peer(peerid);
+	    xorp_throw(BadPeer, 
+		       c_format("Unable to get interface ID for %s",
+				interface.c_str()));
+	}
+	_peers[peerid]->set_interface_id(interface_id);
+    }
+	break;
+    }
+
     // Pass in the option to be sent by the hello packet.
     _peers[peerid]->set_options(area,
 				compute_options(area_router->get_area_type()));
 
-    _peers[peerid]->set_link_status(_ospf.enabled(interface, vif, source));
+    switch (_ospf.get_version()) {
+    case OspfTypes::V2:
+	_peers[peerid]->set_link_status(_ospf.enabled(interface, vif, source));
+	break;
+    case OspfTypes::V3:
+#if	0
+	// If this is a virtual link bring it up now, otherwise
+	// wait for the call to activate.
+	if (OspfTypes::VirtualLink == linktype) {
+	    _peers[peerid]->set_link_status(_ospf.enabled(interface, vif,
+							  source));
+	}
+#endif
+	break;
+    }
 
     // This call needs to be made only once per invocation of OSPF but
     // at this point we know that the interface mirror is up and running.
@@ -473,12 +539,15 @@ PeerManager<A>::create_peer(const string& interface, const string& vif,
 
     area_router->add_peer(peerid);
 
+    // The peer has now been fully configured so initialise it.
+    _peers[peerid]->go(area);
+
     return peerid;
 }
 
 template <typename A>
 bool
-PeerManager<A>::delete_peer(const PeerID peerid)
+PeerManager<A>::delete_peer(const OspfTypes::PeerID peerid)
 {
     debug_msg("PeerID %u\n", peerid);
 
@@ -497,7 +566,7 @@ PeerManager<A>::delete_peer(const PeerID peerid)
 	(*i).second->delete_peer(peerid);
 
     // Remove the interface/vif to PeerID mapping
-    typename map<string, PeerID>::iterator pi;
+    typename map<string, OspfTypes::PeerID>::iterator pi;
     for(pi = _pmap.begin(); pi != _pmap.end(); pi++)
 	if ((*pi).second == peerid) {
 	    _pmap.erase(pi);
@@ -509,7 +578,7 @@ PeerManager<A>::delete_peer(const PeerID peerid)
 
 template <typename A>
 bool
-PeerManager<A>::set_state_peer(const PeerID peerid, bool state)
+PeerManager<A>::set_state_peer(const OspfTypes::PeerID peerid, bool state)
 {
     debug_msg("PeerID %u\n", peerid);
 
@@ -525,7 +594,8 @@ PeerManager<A>::set_state_peer(const PeerID peerid, bool state)
 
 template <typename A>
 bool
-PeerManager<A>::set_link_status_peer(const PeerID peerid, bool state)
+PeerManager<A>::set_link_status_peer(const OspfTypes::PeerID peerid,
+				     bool state)
 {
     debug_msg("PeerID %u\n", peerid);
 
@@ -539,6 +609,186 @@ PeerManager<A>::set_link_status_peer(const PeerID peerid, bool state)
     return true;
 }
 
+template <typename A>
+bool
+PeerManager<A>::add_address_peer(const string& interface, const string& vif,
+				 OspfTypes::AreaID area, A addr)
+{
+    debug_msg("interface %s vif %s area %s address %s\n", interface.c_str(),
+	      vif.c_str(), pr_id(area).c_str(), cstring(addr));
+
+    // Get the prefix length.
+    uint16_t prefix;
+    if (!_ospf.get_prefix_length(interface, vif, addr, prefix)) {
+	XLOG_WARNING("Unable to get prefix for %s ", cstring(addr));
+	return false;
+    }
+
+    // An exception will be thrown if there is a problem.
+    OspfTypes::PeerID peerid = get_peerid(interface, vif);
+
+    set<AddressInfo<A> >& info = _peers[peerid]->get_address_info(area);
+
+    info.insert(AddressInfo<A>(addr, prefix));
+
+    recompute_addresses_peer(peerid, area);
+
+    return true;
+}
+
+template <typename A>
+bool 
+PeerManager<A>::remove_address_peer(const OspfTypes::PeerID peerid,
+				    OspfTypes::AreaID area, A addr)
+{
+    debug_msg("PeerID %u, area %s address %s\n", peerid, pr_id(area).c_str(),
+	      cstring(addr));
+
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return false;
+    }
+
+    set<AddressInfo<A> >& info = _peers[peerid]->get_address_info(area);
+
+    info.erase(AddressInfo<A>(addr));
+
+    recompute_addresses_peer(peerid, area);
+
+    return true;
+}
+
+template <typename A>
+bool
+PeerManager<A>::set_address_state_peer(const OspfTypes::PeerID peerid,
+				       OspfTypes::AreaID area, A addr,
+				       bool enable)
+{
+    debug_msg("PeerID %u, area %s address %s enable %s\n",
+	      peerid, pr_id(area).c_str(), cstring(addr), pb(enable));
+
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return false;
+    }
+
+    set<AddressInfo<A> >& info = _peers[peerid]->get_address_info(area);
+    typename set<AddressInfo<A> >:: iterator i = 
+	info.find(AddressInfo<A>(addr));
+
+    if (i == info.end()) {
+	XLOG_ERROR("Couldn't find %s", cstring(addr));
+	return false;
+    }
+    
+    AddressInfo<A> naddr((*i)._address, (*i)._prefix, enable);
+
+    info.erase(i);
+    info.insert(naddr);
+
+    recompute_addresses_peer(peerid, area);
+
+    return true;
+}
+
+template <typename A>
+bool
+PeerManager<A>::activate_peer(const string& interface, const string& vif,
+			      OspfTypes::AreaID area)
+{
+    debug_msg("interface %s vif %s area %s\n", interface.c_str(),
+	      vif.c_str(), pr_id(area).c_str());
+
+    // An exception will be thrown if there is a problem.
+    OspfTypes::PeerID peerid = get_peerid(interface, vif);
+
+    recompute_addresses_peer(peerid, area);
+
+    A source = _peers[peerid]->get_interface_address();
+    _peers[peerid]->set_link_status(_ospf.enabled(interface, vif, source));
+
+    return true;
+}
+
+template <typename A>
+bool
+PeerManager<A>::update_peer(const string& interface, const string& vif,
+			    OspfTypes::AreaID area)
+{
+    debug_msg("interface %s vif %s area %s\n", interface.c_str(),
+	      vif.c_str(), pr_id(area).c_str());
+
+    // Not currently required.
+
+    return true;
+}
+
+template <typename A>
+bool
+PeerManager<A>::recompute_addresses_peer(const OspfTypes::PeerID peerid,
+					 OspfTypes::AreaID area)
+{
+    debug_msg("PeerID %u area %s\n", peerid, pr_id(area).c_str());
+
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return false;
+    }
+
+    set<AddressInfo<A> >& info = _peers[peerid]->get_address_info(area);
+    
+    // Unconditionally remove all the global addresses that are being
+    // advertised.
+    _peers[peerid]->remove_all_nets(area);
+
+    // If no addresses have been configured then advertise all the
+    // configured addresses.
+    if (info.empty()) {
+	string interface, vif;
+	if (!get_interface_vif_by_peerid(peerid, interface, vif)) {
+	    XLOG_ERROR("Unable to find interface/vif associated with "
+		       "PeerID %u", peerid);
+	    return false;
+	}
+	list<A> addresses;
+	if (!_ospf.get_addresses(interface, vif, addresses)) {
+	    XLOG_ERROR("Unable to find addresses on %s/%s ", interface.c_str(),
+		       vif.c_str());
+	    return false;
+	}
+	typename list<A>::iterator i;
+	for (i = addresses.begin(); i != addresses.end(); i++) {
+	    if ((*i).is_linklocal_unicast())
+		continue;
+	    uint16_t interface_prefix_length;
+	    if (!_ospf.get_prefix_length(interface, vif, *i,
+					 interface_prefix_length)) {
+		XLOG_ERROR("Unable to get prefix length for %s", cstring(*i));
+		continue;
+	    }
+	    if (!_peers[peerid]->add_advertise_net(area, (*i),
+						   interface_prefix_length)) {
+		XLOG_WARNING("Unable to advertise %s in Link-LSA\n",
+			     cstring(*i));
+	    }
+	}
+    } else {
+	typename set<AddressInfo<A> >::iterator i;
+	for (i = info.begin(); i != info.end(); i++) {
+	    if ((*i)._enabled) {
+		if (!_peers[peerid]->add_advertise_net(area, (*i)._address,
+						      (*i)._prefix)) {
+		    XLOG_WARNING("Unable to advertise %s in Link-LSA\n",
+				 cstring((*i)._address));
+		}
+	    }
+	}
+    }
+
+    // Force out a new Link-LSA.
+    return _peers[peerid]->update_nets(area);
+}
+
 template <typename A>
 void
 PeerManager<A>::address_status_change(const string& interface,
@@ -548,7 +798,7 @@ PeerManager<A>::address_status_change(const string& interface,
     debug_msg("interface %s vif %s address %s state %s\n",
 	      interface.c_str(), vif.c_str(), cstring(source), pb(state));
 
-    PeerID peerid;
+    OspfTypes::PeerID peerid;
 
     // All interface/vif/address changes on the host come through
     // here, ignore the changes that are not for OSPF.
@@ -563,6 +813,18 @@ PeerManager<A>::address_status_change(const string& interface,
 	return;
     }
 
+    switch(_ospf.get_version()) {
+    case OspfTypes::V2:
+	break;
+    case OspfTypes::V3:
+	list<OspfTypes::AreaID> areas;
+	_peers[peerid]->get_areas(areas);
+	list<OspfTypes::AreaID>::iterator i;
+	for (i = areas.begin(); i != areas.end(); i++)
+	    recompute_addresses_peer(peerid, *i);
+	break;
+    }
+
     _peers[peerid]->set_link_status(state);
 
     return;
@@ -570,7 +832,8 @@ PeerManager<A>::address_status_change(const string& interface,
 
 template <typename A>
 bool
-PeerManager<A>::add_neighbour(const PeerID peerid, OspfTypes::AreaID area,
+PeerManager<A>::add_neighbour(const OspfTypes::PeerID peerid,
+			      OspfTypes::AreaID area,
 			      A neighbour_address, OspfTypes::RouterID rid)
 {
     if (0 == _peers.count(peerid)) {
@@ -583,7 +846,8 @@ PeerManager<A>::add_neighbour(const PeerID peerid, OspfTypes::AreaID area,
 
 template <typename A>
 bool
-PeerManager<A>::remove_neighbour(const PeerID peerid, OspfTypes::AreaID area,
+PeerManager<A>::remove_neighbour(const OspfTypes::PeerID peerid,
+				 OspfTypes::AreaID area,
 				 A neighbour_address, OspfTypes::RouterID rid)
 {
     if (0 == _peers.count(peerid)) {
@@ -623,14 +887,15 @@ PeerManager<A>::receive(const string& interface, const string& vif,
     debug_msg("Interface %s Vif %s src %s dst %s %s\n", interface.c_str(),
 	      vif.c_str(), cstring(dst), cstring(src), cstring((*packet)));
 
-    PeerID peerid = get_peerid(interface, vif);
+    OspfTypes::PeerID peerid = get_peerid(interface, vif);
     XLOG_ASSERT(0 != _peers.count(peerid));
     return _peers[peerid]->receive(dst, src, packet);
 }
 
 template <typename A>
 bool
-PeerManager<A>::queue_lsa(const PeerID peerid, const PeerID peer,
+PeerManager<A>::queue_lsa(const OspfTypes::PeerID peerid,
+			  const OspfTypes::PeerID peer,
 			  OspfTypes::NeighbourID nid, Lsa::LsaRef lsar,
 			  bool &multicast_on_peer)
 {
@@ -644,7 +909,7 @@ PeerManager<A>::queue_lsa(const PeerID peerid, const PeerID peer,
 
 template <typename A>
 bool
-PeerManager<A>::push_lsas(const PeerID peerid)
+PeerManager<A>::push_lsas(const OspfTypes::PeerID peerid)
 {
     if (0 == _peers.count(peerid)) {
 	XLOG_ERROR("Unknown PeerID %u", peerid);
@@ -654,11 +919,23 @@ PeerManager<A>::push_lsas(const PeerID peerid)
     return _peers[peerid]->push_lsas();    
 }
 
+template <typename A>
+uint32_t
+PeerManager<A>::get_interface_id(const OspfTypes::PeerID peerid)
+{
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return 0;
+    }
+
+    return _peers[peerid]->get_interface_id();
+}
+
 template <typename A>
 bool
 PeerManager<A>::configured_network(const A address) const
 {
-    typename map<PeerID, PeerOut<A> *>::const_iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++) {
 	IPNet<A> net((*i).second->get_interface_address(),
 		     (*i).second->get_interface_prefix_length());
@@ -678,7 +955,7 @@ PeerManager<A>::known_interface_address(const A address) const
     // configured peers. We should be checking the interface addresses
     // with the FEA.
 
-    typename map<PeerID, PeerOut<A> *>::const_iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++)
 	if ((*i).second->get_interface_address() == address) 
 	    return true;
@@ -688,7 +965,7 @@ PeerManager<A>::known_interface_address(const A address) const
 
 template <typename A>
 bool
-PeerManager<A>::neighbours_exchange_or_loading(const PeerID peerid,
+PeerManager<A>::neighbours_exchange_or_loading(const OspfTypes::PeerID peerid,
 					       OspfTypes::AreaID area)
 {
     if (0 == _peers.count(peerid)) {
@@ -701,7 +978,39 @@ PeerManager<A>::neighbours_exchange_or_loading(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::on_link_state_request_list(const PeerID peerid,
+PeerManager<A>::neighbour_at_least_two_way(const OspfTypes::PeerID peerid,
+					   OspfTypes::AreaID area,
+					   OspfTypes::RouterID rid,
+					   bool& twoway)
+{
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return false;
+    }
+
+    return _peers[peerid]->neighbour_at_least_two_way(area, rid, twoway);
+}
+
+template <typename A>
+bool
+PeerManager<A>::get_neighbour_address(const OspfTypes::PeerID peerid,
+				      OspfTypes::AreaID area,
+				      OspfTypes::RouterID rid,
+				      uint32_t interface_id,
+				      A& neighbour_address)
+{
+    if (0 == _peers.count(peerid)) {
+	XLOG_ERROR("Unknown PeerID %u", peerid);
+	return false;
+    }
+
+    return _peers[peerid]->
+	get_neighbour_address(area, rid, interface_id, neighbour_address);
+}
+
+template <typename A>
+bool
+PeerManager<A>::on_link_state_request_list(const OspfTypes::PeerID peerid,
 					   OspfTypes::AreaID area,
 					   const OspfTypes::NeighbourID nid,
 					   Lsa::LsaRef lsar)
@@ -716,7 +1025,7 @@ PeerManager<A>::on_link_state_request_list(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::event_bad_link_state_request(const PeerID peerid,
+PeerManager<A>::event_bad_link_state_request(const OspfTypes::PeerID peerid,
 					     OspfTypes::AreaID area,
 					     const OspfTypes::NeighbourID nid)
 {
@@ -730,7 +1039,8 @@ PeerManager<A>::event_bad_link_state_request(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::send_lsa(const PeerID peerid, OspfTypes::AreaID area,
+PeerManager<A>::send_lsa(const OspfTypes::PeerID peerid,
+			 OspfTypes::AreaID area,
 			 const OspfTypes::NeighbourID nid, Lsa::LsaRef lsar)
 {
     if (0 == _peers.count(peerid)) {
@@ -743,7 +1053,7 @@ PeerManager<A>::send_lsa(const PeerID peerid, OspfTypes::AreaID area,
 
 template <typename A>
 void
-PeerManager<A>::adjacency_changed(const PeerID peerid,
+PeerManager<A>::adjacency_changed(const OspfTypes::PeerID peerid,
 				  OspfTypes::RouterID rid,
 				  bool up)
 {
@@ -764,8 +1074,8 @@ PeerManager<A>::adjacency_changed(const PeerID peerid,
     uint32_t fully_adjacent_virtual_links = 0;
     typename list<OspfTypes::RouterID>::const_iterator i;
     for(i = rids.begin(); i != rids.end(); i++) {
-	PeerID peerid = _vlink.get_peerid(*i);
-	typename map<PeerID, PeerOut<A> *>::const_iterator j;
+	OspfTypes::PeerID peerid = _vlink.get_peerid(*i);
+	typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator j;
 	j = _peers.find(peerid);
 	if(j == _peers.end()) {
 	    // A peerid can be removed and the vlink database is not notified.
@@ -825,17 +1135,15 @@ PeerManager<A>::create_virtual_peer(OspfTypes::RouterID rid)
 {
     string ifname;
     string vifname;
-    uint16_t prefix_length = 0;
-    uint16_t mtu = 0;
 
     if (!_vlink.get_interface_vif(rid, ifname, vifname)) {
 	XLOG_FATAL("Router ID not found %s", pr_id(rid).c_str());
 	return false;
     }
 
-    PeerID peerid;
+    OspfTypes::PeerID peerid;
     try {
-	peerid = create_peer(ifname, vifname, A::ZERO(), prefix_length, mtu,
+	peerid = create_peer(ifname, vifname, A::ZERO(),
 			     OspfTypes::VirtualLink, OspfTypes::BACKBONE);
     } catch(XorpException& e) {
 	XLOG_ERROR("%s", cstring(e));
@@ -854,8 +1162,8 @@ template <typename A>
 bool
 PeerManager<A>::delete_virtual_peer(OspfTypes::RouterID rid)
 {
-    PeerID peerid = _vlink.get_peerid(rid);
-    if (ALLPEERS != peerid) {
+    OspfTypes::PeerID peerid = _vlink.get_peerid(rid);
+    if (OspfTypes::ALLPEERS != peerid) {
 	try {
 	    delete_peer(peerid);
 	} catch(XorpException& e) {
@@ -866,7 +1174,7 @@ PeerManager<A>::delete_virtual_peer(OspfTypes::RouterID rid)
 	// this virtual link, but is is possible that in the future
 	// removing the virtual link from the area router may cause an
 	// upcall.
-	_vlink.add_peerid(rid, ALLPEERS);
+	_vlink.add_peerid(rid, OspfTypes::ALLPEERS);
     }
 
     return true;
@@ -881,8 +1189,8 @@ PeerManager<A>::virtual_link_endpoint(OspfTypes::AreaID area) const
 
     typename list<OspfTypes::RouterID>::const_iterator i;
     for(i = rids.begin(); i != rids.end(); i++) {
-	PeerID peerid = _vlink.get_peerid(*i);
-	typename map<PeerID, PeerOut<A> *>::const_iterator j;
+	OspfTypes::PeerID peerid = _vlink.get_peerid(*i);
+	typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator j;
 	j = _peers.find(peerid);
 	if(j == _peers.end()) {
 	    // A peerid can be removed and the vlink database is not notified.
@@ -1016,11 +1324,11 @@ PeerManager<A>::up_virtual_link(OspfTypes::RouterID rid, A source,
 	return;
     }
 
-    PeerID peerid = _vlink.get_peerid(rid);
+    OspfTypes::PeerID peerid = _vlink.get_peerid(rid);
 
     // Scan through the peers and find the interface and vif that
     // match the source address.
-    typename map<PeerID, PeerOut<A> *>::const_iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++) {
 	if ((*i).second->match(source, ifname, vifname)) {
 	    if (!_vlink.set_physical_interface_vif(rid, ifname, vifname))
@@ -1053,8 +1361,8 @@ PeerManager<A>::down_virtual_link(OspfTypes::RouterID rid)
     XLOG_TRACE(_ospf.trace()._virtual_link,
 	       "Virtual link down rid %s\n", pr_id(rid).c_str());
 
-    PeerID peerid = _vlink.get_peerid(rid);
-    if (ALLPEERS == peerid) {
+    OspfTypes::PeerID peerid = _vlink.get_peerid(rid);
+    if (OspfTypes::ALLPEERS == peerid) {
 	XLOG_WARNING("No peer found when dropping virtual link %s",
 		     pr_id(rid).c_str());
 	return;
@@ -1081,8 +1389,8 @@ PeerManager<A>::receive_virtual_link(A dst, A src, Packet *packet)
 	       "Virtual link receive dest %s src %s packet %s\n",
 	       cstring(dst), cstring(src), cstring(*packet));
 
-    PeerID peerid = _vlink.get_peerid(dst, src);
-    if (ALLPEERS == peerid)
+    OspfTypes::PeerID peerid = _vlink.get_peerid(dst, src);
+    if (OspfTypes::ALLPEERS == peerid)
 	return false;
     XLOG_ASSERT(0 != _peers.count(peerid));
     return _peers[peerid]->receive(dst, src, packet);
@@ -1116,6 +1424,11 @@ PeerManager<A>::compute_options(OspfTypes::AreaType area_type)
     case OspfTypes::V2:
 	break;
     case OspfTypes::V3:
+	// XXX - Unconditionally set the router bit which means that
+	// we want to participate in the routing, the passive and
+	// loopback settings should be taken into account, when
+	// setting this value.
+	options.set_r_bit(true);
 	options.set_v6_bit(true);
 	break;
     }
@@ -1126,7 +1439,8 @@ PeerManager<A>::compute_options(OspfTypes::AreaType area_type)
 #if	0
 template <typename A> 
 bool
-PeerManager<A>::set_options(const PeerID peerid, OspfTypes::AreaID area,
+PeerManager<A>::set_options(const OspfTypes::PeerID peerid,
+			    OspfTypes::AreaID area,
 			    uint32_t options)
 {
     if (0 == _peers.count(peerid)) {
@@ -1142,14 +1456,15 @@ template <typename A>
 void
 PeerManager<A>::router_id_changing()
 {
-    typename map<PeerID, PeerOut<A> *>::const_iterator i;
+    typename map<OspfTypes::PeerID, PeerOut<A> *>::const_iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++)
 	(*i).second->router_id_changing();
 }
 
 template <typename A> 
 bool
-PeerManager<A>::set_interface_address(const PeerID peerid, A address)
+PeerManager<A>::set_interface_address(const OspfTypes::PeerID peerid,
+				      A address)
 {
     if (0 == _peers.count(peerid)) {
 	XLOG_ERROR("Unknown PeerID %u", peerid);
@@ -1159,22 +1474,10 @@ PeerManager<A>::set_interface_address(const PeerID peerid, A address)
     return _peers[peerid]->set_interface_address(address);
 }
 
-template <typename A> 
-bool
-PeerManager<A>::set_interface_id(const PeerID peerid, OspfTypes::AreaID area,
-		 uint32_t interface_id)
-{
-    if (0 == _peers.count(peerid)) {
-	XLOG_ERROR("Unknown PeerID %u", peerid);
-	return false;
-    }
-
-    return _peers[peerid]->set_interface_id(area, interface_id);
-}
-
 template <typename A>
 bool
-PeerManager<A>::set_hello_interval(const PeerID peerid, OspfTypes::AreaID area,
+PeerManager<A>::set_hello_interval(const OspfTypes::PeerID peerid,
+				   OspfTypes::AreaID area,
 				   uint16_t hello_interval)
 {
     if (0 == _peers.count(peerid)) {
@@ -1185,10 +1488,9 @@ PeerManager<A>::set_hello_interval(const PeerID peerid, OspfTypes::AreaID area,
     return _peers[peerid]->set_hello_interval(area, hello_interval);
 }
 
-
 template <typename A> 
 bool
-PeerManager<A>::set_router_priority(const PeerID peerid,
+PeerManager<A>::set_router_priority(const OspfTypes::PeerID peerid,
 				    OspfTypes::AreaID area,
 				    uint8_t priority)
 {
@@ -1202,7 +1504,7 @@ PeerManager<A>::set_router_priority(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_router_dead_interval(const PeerID peerid,
+PeerManager<A>::set_router_dead_interval(const OspfTypes::PeerID peerid,
 					 OspfTypes::AreaID area, 
 					 uint32_t router_dead_interval)
 {
@@ -1217,7 +1519,7 @@ PeerManager<A>::set_router_dead_interval(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_interface_cost(const PeerID peerid, 
+PeerManager<A>::set_interface_cost(const OspfTypes::PeerID peerid, 
 				   OspfTypes::AreaID /*area*/,
 				   uint16_t interface_cost)
 {
@@ -1231,7 +1533,7 @@ PeerManager<A>::set_interface_cost(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_retransmit_interval(const PeerID peerid, 
+PeerManager<A>::set_retransmit_interval(const OspfTypes::PeerID peerid, 
 					OspfTypes::AreaID area,
 					uint16_t retransmit_interval)
 {
@@ -1245,7 +1547,7 @@ PeerManager<A>::set_retransmit_interval(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_inftransdelay(const PeerID peerid, 
+PeerManager<A>::set_inftransdelay(const OspfTypes::PeerID peerid, 
 				   OspfTypes::AreaID /*area*/,
 				   uint16_t inftransdelay)
 {
@@ -1259,7 +1561,7 @@ PeerManager<A>::set_inftransdelay(const PeerID peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_simple_authentication_key(const PeerID	peerid,
+PeerManager<A>::set_simple_authentication_key(const OspfTypes::PeerID peerid,
 					      OspfTypes::AreaID	area,
 					      const string&	password,
 					      string&		error_msg)
@@ -1275,7 +1577,8 @@ PeerManager<A>::set_simple_authentication_key(const PeerID	peerid,
 
 template <typename A>
 bool
-PeerManager<A>::delete_simple_authentication_key(const PeerID	peerid,
+PeerManager<A>::delete_simple_authentication_key(const OspfTypes::PeerID
+						 peerid,
 						 OspfTypes::AreaID area,
 						 string&	error_msg)
 {
@@ -1289,7 +1592,7 @@ PeerManager<A>::delete_simple_authentication_key(const PeerID	peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_md5_authentication_key(const PeerID		peerid,
+PeerManager<A>::set_md5_authentication_key(const OspfTypes::PeerID peerid,
 					   OspfTypes::AreaID	area,
 					   uint8_t		key_id,
 					   const string&	password,
@@ -1312,7 +1615,7 @@ PeerManager<A>::set_md5_authentication_key(const PeerID		peerid,
 
 template <typename A>
 bool
-PeerManager<A>::delete_md5_authentication_key(const PeerID	peerid,
+PeerManager<A>::delete_md5_authentication_key(const OspfTypes::PeerID peerid,
 					      OspfTypes::AreaID	area,
 					      uint8_t		key_id,
 					      string&		error_msg)
@@ -1328,7 +1631,8 @@ PeerManager<A>::delete_md5_authentication_key(const PeerID	peerid,
 
 template <typename A>
 bool
-PeerManager<A>::set_passive(const PeerID peerid, OspfTypes::AreaID area,
+PeerManager<A>::set_passive(const OspfTypes::PeerID peerid,
+			    OspfTypes::AreaID area,
 			    bool passive)
 {
     if (0 == _peers.count(peerid)) {
@@ -1480,24 +1784,29 @@ PeerManager<A>::routing_recompute_all_areas()
     typename map<OspfTypes::AreaID, AreaRouter<A> *>::const_iterator i;
     for (i = _areas.begin(); i != _areas.end(); i++)
 	if ((*i).first == BACKBONE) {
-	    (*i).second->testing_routing_total_recompute();
+	    (*i).second->routing_total_recompute();
 	    break;
 	}
 
-    // XXX
-    // Currently a call the recompute the backbone will cause a
-    // recompute of all areas so this call is not necessary.
-    // routing_recompute_all_areas_except_backbone();
+    // Once the backbone is recomputed any transit areas will also be
+    // recomputed; so they don't need to be computed again.
+    for (i = _areas.begin(); i != _areas.end(); i++)
+	if ((*i).first != BACKBONE) {
+	    if (!(*i).second->get_transit_capability())
+		(*i).second->routing_total_recompute();
+	    break;
+	}
 }
 
 template <typename A>
 void
-PeerManager<A>::routing_recompute_all_areas_except_backbone()
+PeerManager<A>::routing_recompute_all_transit_areas()
 {
     typename map<OspfTypes::AreaID, AreaRouter<A> *>::const_iterator i;
     for (i = _areas.begin(); i != _areas.end(); i++)
 	if ((*i).first != BACKBONE)
-	    (*i).second->testing_routing_total_recompute();
+	    if ((*i).second->get_transit_capability())
+		(*i).second->routing_total_recompute();
 }
 
 template <typename A>
