@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,31 +13,29 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/routing_table.cc,v 1.53 2006/03/18 19:08:24 pavlin Exp $"
+#ident "$XORP: xorp/ospf/routing_table.cc,v 1.60 2007/02/27 18:33:13 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "config.h"
+#include "ospf_module.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/debug.h"
+#include "libxorp/xlog.h"
+#include "libxorp/callback.hh"
+#include "libxorp/ipv4.hh"
+#include "libxorp/ipv6.hh"
+#include "libxorp/ipnet.hh"
+#include "libxorp/status_codes.h"
+#include "libxorp/service.hh"
+#include "libxorp/eventloop.hh"
+
 #include <map>
 #include <list>
 #include <set>
 #include <deque>
 
-#include "ospf_module.h"
-
-#include "libxorp/debug.h"
-#include "libxorp/xlog.h"
-#include "libxorp/callback.hh"
-
-#include "libxorp/ipv4.hh"
-#include "libxorp/ipv6.hh"
-#include "libxorp/ipnet.hh"
-
-#include "libxorp/status_codes.h"
-#include "libxorp/service.hh"
-#include "libxorp/eventloop.hh"
-
 #include "libproto/spt.hh"
 
 #include "ospf.hh"
@@ -45,6 +43,7 @@
 #include "vertex.hh"
 #include "area_router.hh"
 
+
 template <typename A>
 void
 RoutingTable<A>::begin(OspfTypes::AreaID area)
@@ -98,9 +97,23 @@ RoutingTable<A>::add_entry(OspfTypes::AreaID area, IPNet<A> net,
     XLOG_ASSERT(_in_transaction);
     XLOG_ASSERT(area == rt.get_area());
     XLOG_ASSERT(rt.get_directly_connected() || rt.get_nexthop() != A::ZERO());
+    bool status = true;
 
-    if (rt.get_destination_type() == OspfTypes::Router)
-	_adv.add_entry(area, rt.get_router_id(), rt);
+    if (rt.get_destination_type() == OspfTypes::Router) {
+	status = _adv.add_entry(area, rt.get_router_id(), rt);
+	switch(_ospf.get_version()) {
+	case OspfTypes::V2:
+	    break;
+	case OspfTypes::V3:
+// 	    XLOG_ASSERT(!net.is_valid());
+	    if (net.is_valid()) {
+		XLOG_WARNING("Net should be zero %s", cstring(net));
+		status = false;
+	    }
+	    return status;
+	    break;
+	}
+    }
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
     i = _current->lookup_node(net);
@@ -113,7 +126,7 @@ RoutingTable<A>::add_entry(OspfTypes::AreaID area, IPNet<A> net,
     InternalRouteEntry<A>& irentry = i.payload();
     irentry.add_entry(area, rt);
 
-    return true;
+    return status;
 }
 
 template <typename A>
@@ -123,6 +136,23 @@ RoutingTable<A>::replace_entry(OspfTypes::AreaID area, IPNet<A> net,
 {
     debug_msg("area %s %s\n", pr_id(area).c_str(), cstring(net));
     XLOG_ASSERT(_in_transaction);
+    bool status = true;
+
+    if (rt.get_destination_type() == OspfTypes::Router) {
+	status = _adv.replace_entry(area, rt.get_router_id(), rt);
+	switch(_ospf.get_version()) {
+	case OspfTypes::V2:
+	    break;
+	case OspfTypes::V3:
+// 	    XLOG_ASSERT(!net.is_valid());
+	    if (net.is_valid()) {
+		XLOG_WARNING("Net should be zero %s", cstring(net));
+		status = false;
+	    }
+	    return status;
+	    break;
+	}
+    }
 
     typename Trie<A, InternalRouteEntry<A> >::iterator i;
     i = _current->lookup_node(net);
@@ -133,7 +163,7 @@ RoutingTable<A>::replace_entry(OspfTypes::AreaID area, IPNet<A> net,
     InternalRouteEntry<A>& irentry = i.payload();
     irentry.replace_entry(area, rt);
 
-    return true;
+    return status;
 }
 
 template <typename A>
@@ -355,8 +385,8 @@ bool
 RoutingTable<A>::add_route(OspfTypes::AreaID area, IPNet<A> net, A nexthop,
 			   uint32_t metric, RouteEntry<A>& rt)
 {
-    debug_msg("ADD ROUTE area %s net %s nexthop %s\n", pr_id(area).c_str(),
-	      cstring(net), cstring(nexthop));
+    debug_msg("ADD ROUTE area %s net %s nexthop %s metric %u\n",
+	      pr_id(area).c_str(), cstring(net), cstring(nexthop), metric);
 
     bool result = true;
     if (!rt.get_discard()) {
@@ -364,7 +394,7 @@ RoutingTable<A>::add_route(OspfTypes::AreaID area, IPNet<A> net, A nexthop,
 	bool accepted = do_filtering(net, nexthop, metric, rt, policytags);
 	rt.set_filtered(!accepted);
 	if (accepted)
-	    result = _ospf.add_route(net, nexthop, metric,
+	    result = _ospf.add_route(net, nexthop, rt.get_nexthop_id(), metric,
 				     false /* equal */,
 				     false /* discard */,
 				     policytags);
@@ -435,18 +465,19 @@ RoutingTable<A>::push_routes()
 	PolicyTags policytags;
 	IPNet<A> net = tic.key();
 	A nexthop = rt.get_nexthop();
+	uint32_t nexthop_id = rt.get_nexthop_id();
 	uint32_t metric = rt.get_cost();
 	bool accepted = do_filtering(net, nexthop, metric, rt, policytags);
 
 	if (accepted) {
 	    if (!rt.get_filtered()) {
-		_ospf.replace_route(net, nexthop, metric,
+		_ospf.replace_route(net, nexthop, nexthop_id, metric,
 				    false /* equal */,
 				    false /* discard */,
 				    policytags);
 				    
 	    } else {
-		_ospf.add_route(net, nexthop, metric,
+		_ospf.add_route(net, nexthop, nexthop_id, metric,
 				false /* equal */,
 				false /* discard */,
 				policytags);
@@ -480,7 +511,9 @@ RoutingTable<A>::do_filtering(IPNet<A>& net, A& nexthop,
     try {
 	bool e_bit;
 	uint32_t tag;
-	OspfVarRW<A> varrw(net, nexthop, metric, e_bit, tag, policytags);
+	bool tag_set;
+	OspfVarRW<A> varrw(net, nexthop, metric, e_bit, tag, tag_set,
+			   policytags);
 
 	// Import filtering
 	bool accepted;
@@ -498,7 +531,8 @@ RoutingTable<A>::do_filtering(IPNet<A>& net, A& nexthop,
 	if (!accepted) 
 	    return accepted;
 
-	OspfVarRW<A> varrw2(net, nexthop, metric, e_bit, tag, policytags);
+	OspfVarRW<A> varrw2(net, nexthop, metric, e_bit, tag, tag_set,
+			    policytags);
 
 	// Export source-match filtering
 	debug_msg("[OSPF] Running filter: %s on route: %s\n",
@@ -649,7 +683,7 @@ Adv<A>::clear_area(OspfTypes::AreaID area)
 }
 
 template <typename A>
-void
+bool
 Adv<A>::add_entry(OspfTypes::AreaID area, uint32_t adv,
 		  const RouteEntry<A>& rt)
 {
@@ -663,25 +697,59 @@ Adv<A>::add_entry(OspfTypes::AreaID area, uint32_t adv,
 	AREA a;
 	a[adv] = rt;
 	_adv[area] = a;
-	return;
+	return true;
     }
 
     typename ADV::iterator i = _adv.find(area);
     XLOG_ASSERT(_adv.end() != i);
     typename AREA::iterator j = i->second.find(adv);
-    // In OSPFv2 each router should generate only one Router-LSA in
-    // OSPFv3 each router is allowed to generate more than one
-    // Router-LSA, in which case it may become necessary to store all
-    // the Router-LSAs. Another option may be to add all router links to a
-    // single Router-LSA (its never going to be transmitted).
     if (i->second.end() != j) {
-	XLOG_WARNING("More than one Router-LSA seen with same adv %s",
+	XLOG_WARNING("An entry with this advertising router already exists %s",
 		     cstring(*rt.get_lsa()));
-	return;
+	return false;
     }
 
     AREA& aref = _adv[area];
     aref[adv] = rt;
+
+    return true;
+}
+
+template <typename A>
+bool
+Adv<A>::replace_entry(OspfTypes::AreaID area, uint32_t adv,
+		      const RouteEntry<A>& rt)
+{
+    debug_msg("Add entry area %s adv %s\n", pr_id(area).c_str(),
+	   pr_id(adv).c_str());
+
+    XLOG_ASSERT(dynamic_cast<RouterLsa *>(rt.get_lsa().get())||
+		dynamic_cast<SummaryRouterLsa *>(rt.get_lsa().get()));
+
+    if (0 == _adv.count(area)) {
+	XLOG_WARNING("There should already be an entry for this area %s",
+		     cstring(*rt.get_lsa()));
+	AREA a;
+	a[adv] = rt;
+	_adv[area] = a;
+	return false;
+    }
+
+    bool status = true;
+
+    typename ADV::iterator i = _adv.find(area);
+    XLOG_ASSERT(_adv.end() != i);
+    typename AREA::iterator j = i->second.find(adv);
+    if (i->second.end() == j) {
+	XLOG_WARNING("There should already be an entry with this adv %s",
+		     cstring(*rt.get_lsa()));
+	status = false;
+    }
+
+    AREA& aref = _adv[area];
+    aref[adv] = rt;
+
+    return status;
 }
 
 template <typename A>
@@ -706,5 +774,11 @@ Adv<A>::lookup_entry(OspfTypes::AreaID area, uint32_t adv,
     return true;
 }
 
+template class RouteEntry<IPv4>;
+template class RouteEntry<IPv6>;
+template class InternalRouteEntry<IPv4>;
+template class InternalRouteEntry<IPv6>;
+template class Adv<IPv4>;
+template class Adv<IPv6>;
 template class RoutingTable<IPv4>;
 template class RoutingTable<IPv6>;
