@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,14 +13,10 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/test_packet.cc,v 1.39 2006/03/29 22:41:25 atanu Exp $"
-
-#include "config.h"
-#include <map>
-#include <list>
-#include <set>
+#ident "$XORP: xorp/ospf/test_packet.cc,v 1.55 2007/03/19 05:47:07 atanu Exp $"
 
 #include "ospf_module.h"
+
 #include "libxorp/xorp.h"
 #include "libxorp/test_main.hh"
 #include "libxorp/debug.h"
@@ -28,10 +24,14 @@
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
 #include "libxorp/ipnet.hh"
-
 #include "libxorp/status_codes.h"
 #include "libxorp/service.hh"
 #include "libxorp/eventloop.hh"
+#include "libxorp/tokenize.hh"
+
+#include <map>
+#include <list>
+#include <set>
 
 #include "ospf.hh"
 #include "packet.hh"
@@ -71,6 +71,44 @@ compare_packets(TestInfo& info, vector<uint8_t>& pkt1, vector<uint8_t>& pkt2)
     return true;
 }
 
+/*
+ * Compare the string renditions of packets
+ *
+ * @return true when they are equal
+ */
+inline
+bool
+compare_packet_strings(TestInfo& info, string str1, string str2)
+{
+    if (str1 != str2) {
+	if (info.verbose()) {
+	    vector<string> token1;
+	    vector<string> token2;
+	    tokenize(str1, token1, "\n");
+	    tokenize(str2, token2, "\n");
+	    vector<string>::iterator i1 = token1.begin();
+	    vector<string>::iterator i2 = token2.begin();
+	    for(;;) {
+		if (token1.end() == i1 || token2.end() == i2) {
+		    DOUT(info) << "Ran out of tokens\n";
+		    break;
+		}
+		if (*i1 != *i2) {
+		    DOUT(info) << *i1
+			       << " *** DOES NOT MATCH ***" 
+			       << *i2 << endl;
+		    break;
+		}
+		i1++;
+		i2++;
+	    }
+	}
+	return false;
+    }
+
+    return true;
+}
+
 /**
  * Modify the frame lengths to see if we can freak the decoder.
  */
@@ -93,7 +131,7 @@ packet_bad_length(TestInfo& info, OspfTypes::Version version,
 	packet = packet_decoder.decode(ptr, pkt.size() - 1);
 	DOUT(info) << "Accepted short frame (bad)\n";
 	return false;
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	DOUT(info) << "Rejected short frame (good): " << e.str() << endl;
     }
 
@@ -102,7 +140,7 @@ packet_bad_length(TestInfo& info, OspfTypes::Version version,
 	try {
 	    packet = packet_decoder.decode(ptr, pkt.size() + i);
 	    DOUT(info) << "Accepted large frame (good)\n";
-	} catch(BadPacket& e) {
+	} catch(InvalidPacket& e) {
 	    DOUT(info) << "Didn't accept large frame (bad): " << e.str() << 
 		endl;
 	    return false;
@@ -156,7 +194,7 @@ populate_helloV3(HelloPacket *hello)
     hello->set_area_id(set_id("4.3.2.1"));
     hello->set_instance_id(5);
 
-    hello->set_interface_id(0x12345678);
+    hello->set_interface_id(12345678);
     hello->set_hello_interval(9876);
     hello->set_options(compute_options(OspfTypes::V3, OspfTypes::NORMAL));
     hello->set_router_priority(42);
@@ -264,21 +302,66 @@ populate_router_lsa(RouterLsa *rlsa, OspfTypes::Version version)
 	break;
     }
     
-    RouterLink rl(version);
-    rl.set_type(RouterLink::p2p);
-    rl.set_metric(2);
+    RouterLink rl1(version);
+    rl1.set_type(RouterLink::p2p);
+    rl1.set_metric(2);
     switch(version) {
     case OspfTypes::V2:
-	rl.set_link_id(3);
-	rl.set_link_data(4);
+	rl1.set_link_id(3);
+	rl1.set_link_data(4);
 	break;
     case OspfTypes::V3:
-	rl.set_interface_id(3);
-	rl.set_neighbour_interface_id(4);
-	rl.set_neighbour_router_id(5);
+	rl1.set_interface_id(3);
+	rl1.set_neighbour_interface_id(4);
+	rl1.set_neighbour_router_id(5);
 	break;
     }
-    rlsa->get_router_links().push_back(rl);
+    rlsa->get_router_links().push_back(rl1);
+
+    RouterLink rl2(version);
+    rl2.set_type(RouterLink::transit);
+    rl2.set_metric(2);
+    switch(version) {
+    case OspfTypes::V2:
+	rl2.set_link_id(3);
+	rl2.set_link_data(4);
+	break;
+    case OspfTypes::V3:
+	rl2.set_interface_id(3);
+	rl2.set_neighbour_interface_id(4);
+	rl2.set_neighbour_router_id(5);
+	break;
+    }
+    rlsa->get_router_links().push_back(rl2);
+
+    RouterLink rl3(version);
+    switch(version) {
+    case OspfTypes::V2:
+	rl3.set_metric(2);
+	rl3.set_type(RouterLink::stub);
+	rl3.set_link_id(3);
+	rl3.set_link_data(4);
+	rlsa->get_router_links().push_back(rl3);
+	break;
+    case OspfTypes::V3:
+	break;
+    }
+
+    RouterLink rl4(version);
+    rl4.set_type(RouterLink::vlink);
+    rl4.set_metric(2);
+    switch(version) {
+    case OspfTypes::V2:
+	rl4.set_link_id(3);
+	rl4.set_link_data(4);
+	break;
+    case OspfTypes::V3:
+	rl4.set_interface_id(3);
+	rl4.set_neighbour_interface_id(4);
+	rl4.set_neighbour_router_id(5);
+	break;
+    }
+    rlsa->get_router_links().push_back(rl4);
 
     // This will set the checksum and the length.
     rlsa->encode();
@@ -326,8 +409,14 @@ populate_summary_network_lsa(SummaryNetworkLsa *snlsa,
 	break;
     case OspfTypes::V3:
 	IPNet<IPv6> net("2001:468:e21:c800:220:edff:fe61:f033", 64);
-	snlsa->set_network(net);
-	snlsa->set_prefix_options(0x5);
+	IPv6Prefix prefix(version);
+	prefix.set_network(net);
+	prefix.set_nu_bit(false);
+	prefix.set_la_bit(false);
+	prefix.set_mc_bit(false);
+	prefix.set_p_bit(false);
+	prefix.set_dn_bit(false);
+	snlsa->set_ipv6prefix(prefix);
 	break;
     }
 
@@ -377,10 +466,16 @@ populate_as_external_lsa(ASExternalLsa *aelsa,
 	aelsa->set_e_bit(true);
 	aelsa->set_f_bit(true);
 	aelsa->set_t_bit(true);
-	aelsa->set_prefix_options(0x5);
 	aelsa->set_referenced_ls_type(2);
 	IPNet<IPv6> net("2001:468:e21:c800:220:edff:fe61:f033", 64);
-	aelsa->set_network(net);
+	IPv6Prefix prefix(version);
+	prefix.set_network(net);
+	prefix.set_nu_bit(true);
+	prefix.set_la_bit(true);
+	prefix.set_mc_bit(true);
+	prefix.set_p_bit(true);
+	prefix.set_dn_bit(true);
+	aelsa->set_ipv6prefix(prefix);
 	aelsa->set_forwarding_address_ipv6(
 			      IPv6("2001:468:e21:c800:220:edff:fe61:f033"));
 	aelsa->set_referenced_link_state_id(0x10);
@@ -393,6 +488,67 @@ populate_as_external_lsa(ASExternalLsa *aelsa,
     aelsa->encode();
 }
 
+inline
+void
+populate_link_lsa(LinkLsa *llsa, OspfTypes::Version version)
+{
+    populate_lsa_header(llsa->get_header(), version);
+
+    llsa->set_rtr_priority(42);
+    llsa->set_options(compute_options(version, OspfTypes::NORMAL));
+    llsa->set_link_local_address(IPv6("fe80::202:b3ff:fe19:be47"));
+    IPNet<IPv6> net1("2001:468:e21:c800:220:edff:fe61:f033", 64);
+    IPv6Prefix prefix1(version);
+    prefix1.set_network(net1);
+    prefix1.set_nu_bit(true);
+    prefix1.set_la_bit(true);
+    prefix1.set_mc_bit(true);
+    prefix1.set_p_bit(true);
+    prefix1.set_dn_bit(true);
+    llsa->get_prefixes().push_back(prefix1);
+    IPNet<IPv6> net2("2001:700:0:fff1::2", 64);
+    IPv6Prefix prefix2(version);
+    prefix2.set_network(net2);
+    prefix2.set_nu_bit(false);
+    prefix2.set_la_bit(false);
+    prefix2.set_mc_bit(false);
+    prefix2.set_p_bit(false);
+    prefix2.set_dn_bit(false);
+    llsa->get_prefixes().push_back(prefix2);
+}
+
+inline
+void
+populate_intra_area_prefix_lsa(IntraAreaPrefixLsa *iaplsa,
+			       OspfTypes::Version version)
+{
+    populate_lsa_header(iaplsa->get_header(), version);
+
+    iaplsa->set_referenced_ls_type(0x2001);
+    iaplsa->set_referenced_link_state_id(1);
+    iaplsa->set_referenced_advertising_router(set_id("192.1.1.2"));
+    IPNet<IPv6> net1("2001:468:e21:c800:220:edff:fe61:f033", 64);
+    IPv6Prefix prefix1(version, true);
+    prefix1.set_network(net1);
+    prefix1.set_metric(256);
+    prefix1.set_nu_bit(true);
+    prefix1.set_la_bit(true);
+    prefix1.set_mc_bit(true);
+    prefix1.set_p_bit(true);
+    prefix1.set_dn_bit(true);
+    iaplsa->get_prefixes().push_back(prefix1);
+    IPNet<IPv6> net2("2001:700:0:fff1::2", 64);
+    IPv6Prefix prefix2(version, true);
+    prefix2.set_network(net2);
+    prefix2.set_metric(3841);
+    prefix2.set_nu_bit(false);
+    prefix2.set_la_bit(false);
+    prefix2.set_mc_bit(false);
+    prefix2.set_p_bit(false);
+    prefix2.set_dn_bit(false);
+    iaplsa->get_prefixes().push_back(prefix2);
+}
+
 inline
 void
 populate_link_state_update(LinkStateUpdatePacket *lsup,
@@ -502,6 +658,9 @@ hello_packet_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, hello1->str(), hello3->str()))
+	return false;
+
     delete hello1;
     delete hello2;
     delete hello3;
@@ -575,6 +734,9 @@ data_description_packet_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, ddp1->str(), ddp3->str()))
+	return false;
+
     delete ddp1;
     delete ddp2;
     delete ddp3;
@@ -656,6 +818,9 @@ link_state_update_packet_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, lsup1->str(), lsup3->str()))
+	return false;
+
     delete lsup1;
     delete lsup2;
     delete lsup3;
@@ -732,6 +897,9 @@ link_state_request_packet_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, lsrp1->str(), lsrp3->str()))
+	return false;
+
     delete lsrp1;
     delete lsrp2;
     delete lsrp3;
@@ -810,6 +978,9 @@ link_state_acknowledgement_packet_compare(TestInfo& info,
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, lsrp1->str(), lsrp3->str()))
+	return false;
+
     delete lsrp1;
     delete lsrp2;
     delete lsrp3;
@@ -850,7 +1021,7 @@ packet_decoder1(TestInfo& info, OspfTypes::Version version)
     // fail.
     try {
 	dec.decode(&pkt[0], pkt.size());
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	DOUT(info) << "Caught exception: " << e.str() << endl;
 	return true;
     }
@@ -882,6 +1053,43 @@ packet_decoder2(TestInfo& info, OspfTypes::Version version)
     return true;
 }
 
+uint8_t bad_packet1[] = {
+#include "packet1.data"
+};
+
+uint8_t bad_packet2[] = {
+#include "packet2.data"
+};
+
+uint8_t bad_packet3[] = {
+#include "packet3.data"
+};
+
+bool
+packet_decode_bad_packet(TestInfo& info, OspfTypes::Version version,
+			 uint8_t *ptr, size_t len)
+{
+    PacketDecoder packet_decoder;
+    LsaDecoder lsa_decoder(version);
+
+    initialise_lsa_decoder(version, lsa_decoder);
+    initialise_packet_decoder(version, packet_decoder, lsa_decoder);
+
+    Packet *packet;
+    try {
+	packet = packet_decoder.decode(ptr, len);
+	DOUT(info) << "Accepted bad packet (bad)\n";
+	return false;
+    } catch(InvalidPacket& e) {
+	DOUT(info) << "Rejected bad packet (good): " << e.str() << endl;
+	return true;
+    }
+    
+    XLOG_UNREACHABLE();
+
+    return false;
+}
+
 /* Packet stuff above - LSA stuff below */
 
 inline
@@ -907,6 +1115,10 @@ router_lsa_print(TestInfo& info)
     populate_router_lsa(rlsa, OspfTypes::V3);
 
     DOUT(info) << rlsa->str() << endl;
+    XLOG_ASSERT(rlsa->known());
+    XLOG_ASSERT(!rlsa->link_local_scope());
+    XLOG_ASSERT(rlsa->area_scope());
+    XLOG_ASSERT(!rlsa->as_scope());
 
     delete rlsa;
 
@@ -927,6 +1139,10 @@ network_lsa_print(TestInfo& info)
     populate_network_lsa(nlsa, OspfTypes::V3);
 
     DOUT(info) << nlsa->str() << endl;
+    XLOG_ASSERT(nlsa->known());
+    XLOG_ASSERT(!nlsa->link_local_scope());
+    XLOG_ASSERT(nlsa->area_scope());
+    XLOG_ASSERT(!nlsa->as_scope());
 
     delete nlsa;
 
@@ -947,6 +1163,10 @@ summary_network_lsa_print(TestInfo& info)
     populate_summary_network_lsa(snlsa, OspfTypes::V3);
 
     DOUT(info) << snlsa->str() << endl;
+    XLOG_ASSERT(snlsa->known());
+    XLOG_ASSERT(!snlsa->link_local_scope());
+    XLOG_ASSERT(snlsa->area_scope());
+    XLOG_ASSERT(!snlsa->as_scope());
 
     delete snlsa;
 
@@ -967,6 +1187,10 @@ summary_router_lsa_print(TestInfo& info)
     populate_summary_router_lsa(srlsa, OspfTypes::V3);
 
     DOUT(info) << srlsa->str() << endl;
+    XLOG_ASSERT(srlsa->known());
+    XLOG_ASSERT(!srlsa->link_local_scope());
+    XLOG_ASSERT(srlsa->area_scope());
+    XLOG_ASSERT(!srlsa->as_scope());
 
     delete srlsa;
 
@@ -987,6 +1211,10 @@ as_external_lsa_print(TestInfo& info)
     populate_as_external_lsa(aelsa, OspfTypes::V3);
 
     DOUT(info) << aelsa->str() << endl;
+    XLOG_ASSERT(aelsa->known());
+    XLOG_ASSERT(!aelsa->link_local_scope());
+    XLOG_ASSERT(!aelsa->area_scope());
+    XLOG_ASSERT(aelsa->as_scope());
 
     delete aelsa;
 
@@ -1007,12 +1235,46 @@ type7_lsa_print(TestInfo& info)
     populate_as_external_lsa(type7, OspfTypes::V3);
 
     DOUT(info) << type7->str() << endl;
+    XLOG_ASSERT(type7->known());
+    XLOG_ASSERT(!type7->link_local_scope());
+    XLOG_ASSERT(type7->area_scope());
+    XLOG_ASSERT(!type7->as_scope());
 
     delete type7;
 
     return true;
 }
 
+bool
+link_lsa_print(TestInfo& info)
+{
+    LinkLsa *llsa = new LinkLsa(OspfTypes::V3);
+    populate_link_lsa(llsa, OspfTypes::V3);
+
+    DOUT(info) << llsa->str() << endl;
+    XLOG_ASSERT(llsa->known());
+    XLOG_ASSERT(llsa->link_local_scope());
+    XLOG_ASSERT(!llsa->area_scope());
+    XLOG_ASSERT(!llsa->as_scope());
+
+    delete llsa;
+
+    return true;
+}
+
+bool
+intra_area_prefix_lsa_print(TestInfo& info)
+{
+    IntraAreaPrefixLsa *iaplsa = new IntraAreaPrefixLsa(OspfTypes::V3);
+    populate_intra_area_prefix_lsa(iaplsa, OspfTypes::V3);
+
+    DOUT(info) << iaplsa->str() << endl;
+
+    delete iaplsa;
+
+    return true;
+}
+
 bool
 router_lsa_compare(TestInfo& info, OspfTypes::Version version)
 {
@@ -1050,6 +1312,9 @@ router_lsa_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, rlsa1->str(), rlsa3->str()))
+	return false;
+
     delete rlsa1;
     delete rlsa2;
 
@@ -1093,6 +1358,9 @@ network_lsa_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, nlsa1->str(), nlsa3->str()))
+	return false;
+
     delete nlsa1;
     delete nlsa2;
 
@@ -1136,6 +1404,9 @@ summary_network_lsa_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, snlsa1->str(), snlsa3->str()))
+	return false;
+
     delete snlsa1;
     delete snlsa2;
 
@@ -1179,6 +1450,9 @@ summary_router_lsa_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, srlsa1->str(), srlsa3->str()))
+	return false;
+
     delete srlsa1;
     delete srlsa2;
 
@@ -1222,6 +1496,9 @@ as_external_lsa_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, aelsa1->str(), aelsa3->str()))
+	return false;
+
     delete aelsa1;
     delete aelsa2;
 
@@ -1265,12 +1542,107 @@ type7_lsa_compare(TestInfo& info, OspfTypes::Version version)
     if (!compare_packets(info, pkt1, pkt2))
 	return false;
 
+    if (!compare_packet_strings(info, type71->str(), type73->str()))
+	return false;
+
     delete type71;
     delete type72;
 
     return true;
 }
 
+bool
+link_lsa_compare(TestInfo& info, OspfTypes::Version version)
+{
+    LinkLsa *llsa1= new LinkLsa(version);
+    populate_link_lsa(llsa1, version);
+
+    DOUT(info) << llsa1->str() << endl;
+
+    // Encode the Link-LSA.
+    llsa1->encode();
+    size_t len1;
+    uint8_t *ptr1 = llsa1->lsa(len1);
+
+    // Now decode the packet.
+    // Create a new packet to provide the decoder.
+    LinkLsa *llsa2= new LinkLsa(version);
+
+    Lsa::LsaRef llsa3 = llsa2->decode(ptr1, len1);
+
+    DOUT(info) << llsa3->str() << endl;
+
+    // Encode the second packet and compare.
+    llsa3->encode();
+
+    DOUT(info) << llsa3->str() << endl;
+
+    size_t len2;
+    uint8_t *ptr2 = llsa3->lsa(len2);
+    
+    vector<uint8_t> pkt1;
+    fill_vector(pkt1, ptr1, len1);
+    vector<uint8_t> pkt2;
+    fill_vector(pkt2, ptr2, len2);
+
+    if (!compare_packets(info, pkt1, pkt2))
+	return false;
+
+    if (!compare_packet_strings(info, llsa1->str(), llsa3->str()))
+	return false;
+
+    delete llsa1;
+    delete llsa2;
+
+    return true;
+}
+
+bool
+intra_area_prefix_lsa_compare(TestInfo& info, OspfTypes::Version version)
+{
+    IntraAreaPrefixLsa *iaplsa1= new IntraAreaPrefixLsa(version);
+    populate_intra_area_prefix_lsa(iaplsa1, version);
+
+    DOUT(info) << iaplsa1->str() << endl;
+
+    // Encode the Link-LSA.
+    iaplsa1->encode();
+    size_t len1;
+    uint8_t *ptr1 = iaplsa1->lsa(len1);
+
+    // Now decode the packet.
+    // Create a new packet to provide the decoder.
+    IntraAreaPrefixLsa *iaplsa2= new IntraAreaPrefixLsa(version);
+
+    Lsa::LsaRef iaplsa3 = iaplsa2->decode(ptr1, len1);
+
+    DOUT(info) << iaplsa3->str() << endl;
+
+    // Encode the second packet and compare.
+    iaplsa3->encode();
+
+    DOUT(info) << iaplsa3->str() << endl;
+
+    size_t len2;
+    uint8_t *ptr2 = iaplsa3->lsa(len2);
+    
+    vector<uint8_t> pkt1;
+    fill_vector(pkt1, ptr1, len1);
+    vector<uint8_t> pkt2;
+    fill_vector(pkt2, ptr2, len2);
+
+    if (!compare_packets(info, pkt1, pkt2))
+	return false;
+
+    if (!compare_packet_strings(info, iaplsa1->str(), iaplsa3->str()))
+	return false;
+
+    delete iaplsa1;
+    delete iaplsa2;
+
+    return true;
+}
+
 bool
 lsa_decoder1(TestInfo& info, OspfTypes::Version version)
 {
@@ -1286,7 +1658,7 @@ lsa_decoder1(TestInfo& info, OspfTypes::Version version)
     // fail.
     try {
 	dec.decode(ptr, len);
-    } catch(BadPacket& e) {
+    } catch(InvalidPacket& e) {
 	DOUT(info) << "Caught exception: " << e.str() << endl;
 	return true;
     }
@@ -1316,6 +1688,29 @@ lsa_decoder2(TestInfo& info, OspfTypes::Version version)
     return true;
 }
 
+bool
+lsa_decoder3(TestInfo& info, OspfTypes::Version version)
+{
+    LsaDecoder dec(version);
+    // Install the unknown LSA decoder.
+    UnknownLsa *unknown_lsa_decoder = new UnknownLsa(version);
+    dec.register_unknown_decoder(unknown_lsa_decoder);
+
+    RouterLsa router_lsa(version);
+    populate_router_lsa(&router_lsa, version);
+    router_lsa.encode();
+    size_t len;
+    uint8_t *ptr = router_lsa.lsa(len);
+
+    // The only decoder installed is the unknown decoder which should
+    // be able to handle the Router-LSA.
+    Lsa::LsaRef lsaref = dec.decode(ptr, len);
+
+    DOUT(info) << lsaref->str() << endl;
+
+    return true;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1348,6 +1743,8 @@ main(int argc, char **argv)
 	{"summary_router_lsa_print", callback(summary_router_lsa_print)},
 	{"as_external_lsa_print", callback(as_external_lsa_print)},
 	{"type7_lsa_print", callback(type7_lsa_print)},
+	{"link_lsa_print", callback(link_lsa_print)},
+	{"intra_area_prefix_lsa_print", callback(intra_area_prefix_lsa_print)},
 
 	{"hello_compareV2", callback(hello_packet_compare, OspfTypes::V2)},
 	{"hello_compareV3", callback(hello_packet_compare, OspfTypes::V3)},
@@ -1395,6 +1792,16 @@ main(int argc, char **argv)
 	{"packet_decoder2V2", callback(packet_decoder2, OspfTypes::V2)},
 	{"packet_decoder2V3", callback(packet_decoder2, OspfTypes::V3)},
 
+	{"packet_decode_bad1V2", callback(packet_decode_bad_packet,
+					  OspfTypes::V2,
+					  bad_packet1, sizeof(bad_packet1))},
+	{"packet_decode_bad2V2", callback(packet_decode_bad_packet,
+					  OspfTypes::V2,
+					  bad_packet2, sizeof(bad_packet2))},
+	{"packet_decode_bad3V2", callback(packet_decode_bad_packet,
+					  OspfTypes::V2,
+					  bad_packet3, sizeof(bad_packet3))},
+
 	{"router_lsa_compareV2", callback(router_lsa_compare, OspfTypes::V2)},
 	{"router_lsa_compareV3", callback(router_lsa_compare, OspfTypes::V3)},
 	{"network_lsa_compareV2",callback(network_lsa_compare, OspfTypes::V2)},
@@ -1413,11 +1820,15 @@ main(int argc, char **argv)
 	 callback(as_external_lsa_compare, OspfTypes::V3)},
 	{"type7_lsa_compareV2",	 callback(type7_lsa_compare, OspfTypes::V2)},
 	{"type7_lsa_compareV3",	 callback(type7_lsa_compare, OspfTypes::V3)},
+	{"link_lsa_compareV3",	 callback(link_lsa_compare, OspfTypes::V3)},
+	{"intra_area_prefix_lsa_compareV3", 
+	 callback(intra_area_prefix_lsa_compare, OspfTypes::V3)},
 
 	{"lsa_decoder1V2", callback(lsa_decoder1, OspfTypes::V2)},
 	{"lsa_decoder1V3", callback(lsa_decoder1, OspfTypes::V3)},
 	{"lsa_decoder2V2", callback(lsa_decoder2, OspfTypes::V2)},
 	{"lsa_decoder2V3", callback(lsa_decoder2, OspfTypes::V3)},
+	{"lsa_decoder3V3", callback(lsa_decoder3, OspfTypes::V3)},
     };
 
     try {
