@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,30 +13,31 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/test_routing.cc,v 1.20 2006/03/29 22:41:25 atanu Exp $"
+#ident "$XORP: xorp/ospf/test_routing.cc,v 1.26 2007/02/16 22:46:43 pavlin Exp $"
 
-#include "config.h"
-#include <map>
-#include <list>
-#include <set>
-#include <deque>
+#define DEBUG_LOGGING
+#define DEBUG_PRINT_FUNCTION_NAME
 
 #include "ospf_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/test_main.hh"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/callback.hh"
 #include "libxorp/tlv.hh"
-
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
 #include "libxorp/ipnet.hh"
-
 #include "libxorp/status_codes.h"
 #include "libxorp/service.hh"
 #include "libxorp/eventloop.hh"
 
+#include <map>
+#include <list>
+#include <set>
+#include <deque>
+
 #include "libproto/spt.hh"
 
 #include "ospf.hh"
@@ -46,12 +47,6 @@
 #include "area_router.hh"
 #include "test_common.hh"
 
-#ifndef	DEBUG_LOGGING
-#define DEBUG_LOGGING
-#endif /* DEBUG_LOGGING */
-#ifndef	DEBUG_PRINT_FUNCTION_NAME
-#define DEBUG_PRINT_FUNCTION_NAME
-#endif /* DEBUG_PRINT_FUNCTION_NAME */
 
 // Make sure that all tests free up any memory that they use. This will
 // allow us to use the leak checker program.
@@ -201,7 +196,6 @@ create_summary_lsa(OspfTypes::Version version, uint32_t link_state_id,
 	break;
     case OspfTypes::V3:
 	XLOG_WARNING("TBD");
-	snlsa->set_prefix_options(0x0);
 	break;
     }
 
@@ -338,8 +332,6 @@ routing1(TestInfo& info, OspfTypes::Version version)
     ospf.set_router_id(set_id("0.0.0.6"));
 
     OspfTypes::AreaID area = set_id("128.16.64.16");
-    const uint16_t interface_prefix_length = 16;
-    const uint16_t interface_mtu = 1500;
 
     PeerManager<A>& pm = ospf.get_peer_manager();
 
@@ -363,11 +355,8 @@ routing1(TestInfo& info, OspfTypes::Version version)
 	break;
     }
 
-    PeerID peerid = pm.
-	create_peer(interface, vif, src, interface_prefix_length,
-		    interface_mtu,
-		    OspfTypes::BROADCAST,
-		    area);
+    OspfTypes::PeerID peerid = pm.
+	create_peer(interface, vif, src, OspfTypes::BROADCAST, area);
 
     // Bring the peering up
     if (!pm.set_state_peer(peerid, true)) {
@@ -459,8 +448,6 @@ routing2(TestInfo& info)
     ospf.set_router_id(rid);
     
     OspfTypes::AreaID area = set_id("0.0.0.0");
-    const uint16_t interface_prefix_length = 30;
-    const uint16_t interface_mtu = 1500;
 
     PeerManager<IPv4>& pm = ospf.get_peer_manager();
 
@@ -472,11 +459,8 @@ routing2(TestInfo& info)
     const string interface = "eth0";
     const string vif = "vif0";
 
-    PeerID peerid = pm.
-	create_peer(interface, vif, src, interface_prefix_length,
-		    interface_mtu,
-		    OspfTypes::BROADCAST,
-		    area);
+    OspfTypes::PeerID peerid = pm.
+	create_peer(interface, vif, src, OspfTypes::BROADCAST, area);
 
     // Bring the peering up
     if (!pm.set_state_peer(peerid, true)) {
@@ -1438,6 +1422,100 @@ routing9(TestInfo& info)
     return true;
 }
 
+bool
+routing10(TestInfo& info)
+{
+    OspfTypes::Version version = OspfTypes::V2;
+
+    EventLoop eventloop;
+    DebugIO<IPv4> io(info, version, eventloop);
+    io.startup();
+
+    Ospf<IPv4> ospf(version, eventloop, &io);
+    ospf.trace().all(info.verbose());
+    OspfTypes::AreaID area = set_id("0.0.0.0");
+
+    PeerManager<IPv4>& pm = ospf.get_peer_manager();
+    pm.create_area_router(area, OspfTypes::NORMAL);
+    AreaRouter<IPv4> *ar = pm.get_area_router(area);
+    XLOG_ASSERT(ar);
+
+    OspfTypes::RouterID prid = set_id("10.10.10.149");
+    ospf.set_router_id(prid);
+    Lsa::LsaRef lsar;
+    RouterLsa *rlsa;
+
+    // Set testing to true to force the 2-Way test to return true.
+    ospf.set_testing(true);
+
+    // Create the peer 1 (DR) Router-LSA
+    OspfTypes::RouterID rid = set_id("10.10.10.125");
+    lsar = create_router_lsa(version, rid, rid);
+    rlsa = dynamic_cast<RouterLsa *>(lsar.get());
+    XLOG_ASSERT(rlsa);
+    transit(version, rlsa, rid, rid, 1);
+    rlsa->set_e_bit(true);
+    rlsa->set_b_bit(true);
+    lsar->encode();
+    ar->testing_add_lsa(lsar);
+    
+    // Create this router's Router-LSA
+    lsar = create_router_lsa(version, prid, prid);
+    rlsa = dynamic_cast<RouterLsa *>(lsar.get());
+    XLOG_ASSERT(rlsa);
+    transit(version, rlsa, rid, prid, 1 /* metric */);
+    lsar->encode();
+    lsar->set_self_originating(true);
+    ar->testing_replace_router_lsa(lsar);
+
+    // Create the peer 2 Router-LSA
+    OspfTypes::RouterID pprid = set_id("10.10.10.18");
+    lsar = create_router_lsa(version, pprid, pprid);
+    rlsa = dynamic_cast<RouterLsa *>(lsar.get());
+    XLOG_ASSERT(rlsa);
+    transit(version, rlsa, rid, pprid, 1);
+    rlsa->set_e_bit(true);
+    rlsa->set_b_bit(true);
+    lsar->encode();
+    ar->testing_add_lsa(lsar);
+
+    // Create the Network-LSA that acts as the binding glue.
+    lsar = create_network_lsa(version, rid, rid, 0xffffff00);
+    NetworkLsa *nlsa;
+    nlsa = dynamic_cast<NetworkLsa *>(lsar.get());
+    XLOG_ASSERT(nlsa);
+    nlsa->get_attached_routers().push_back(rid);
+    nlsa->get_attached_routers().push_back(prid);
+    nlsa->get_attached_routers().push_back(pprid);
+    lsar->encode();
+    ar->testing_add_lsa(lsar);
+
+    // Peer 2 originates External LSA
+    lsar = create_external_lsa(version, set_id("10.20.30.0"), pprid);
+    ASExternalLsa *aselsa;
+    aselsa = dynamic_cast<ASExternalLsa *>(lsar.get());
+    XLOG_ASSERT(aselsa);
+    aselsa->set_network_mask(0xffffff00);
+    aselsa->set_metric(1);
+    aselsa->set_forwarding_address_ipv4(IPv4("0.0.0.0"));
+    lsar->encode();
+    ar->testing_add_lsa(lsar);
+
+    if (info.verbose())
+        ar->testing_print_link_state_database();
+    ar->testing_routing_total_recompute();
+
+    if (!verify_routes(info, __LINE__, io, 1))
+        return false;
+    if (!io.routing_table_verify(IPNet<IPv4>("10.20.30.0/24"),
+		                 IPv4("10.10.10.18"), 2, false, false)) {
+	DOUT(info) << "Mismatch in routing table\n";
+	return false;
+    }
+
+    return true;
+}
+
 int
 main(int argc, char **argv)
 {
@@ -1470,6 +1548,7 @@ main(int argc, char **argv)
 	{"r7", callback(routing7)},
  	{"r8", callback(routing8)},
  	{"r9", callback(routing9)},
+ 	{"r10", callback(routing10)},
     };
 
     try {
