@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,29 +13,27 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/ospf/xrl_io.cc,v 1.32 2006/06/15 06:07:27 pavlin Exp $"
+#ident "$XORP: xorp/ospf/xrl_io.cc,v 1.40 2007/03/19 11:03:12 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "config.h"
-#include <list>
-#include <set>
-
 #include "ospf_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/callback.hh"
-
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
 #include "libxorp/ipnet.hh"
-
 #include "libxorp/status_codes.h"
 #include "libxorp/service.hh"
 #include "libxorp/eventloop.hh"
 
+#include <list>
+#include <set>
+
 #include "xrl/interfaces/fea_rawpkt4_xif.hh"
 #include "xrl/interfaces/fea_rawpkt6_xif.hh"
 #include "xrl/interfaces/rib_xif.hh"
@@ -43,6 +41,7 @@
 #include "ospf.hh"
 #include "xrl_io.hh"
 
+
 template <typename A>
 void
 XrlIO<A>::recv(const string& interface,
@@ -140,8 +139,8 @@ XrlIO<IPv6>::send(const string& interface, const string& vif,
 	src,
 	dst,
 	get_ip_protocol_number(),
-	-1,		// XXX: let the FEA set it
-	-1,		// XXX: let the FEA set it
+	dst.is_multicast() ? 1 : -1,		// XXX: let the FEA set it
+	-1,					// XXX: let the FEA set it
 	get_ip_router_alert(),
 	ext_headers_type,
 	ext_headers_payload,
@@ -412,6 +411,101 @@ XrlIO<IPv6>::is_address_enabled(const string& interface, const string& vif,
     return (fa->enabled());
 }
 
+template <>
+bool
+XrlIO<IPv4>::get_addresses(const string& interface, const string& vif,
+			   list<IPv4>& addresses) const
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    const IfMgrVifAtom* fv = ifmgr_iftree().find_vif(interface, vif);
+    if (fv == NULL)
+	return false;
+
+    IfMgrVifAtom::V4Map::const_iterator i;
+    for (i = fv->ipv4addrs().begin(); i != fv->ipv4addrs().end(); i++)
+	addresses.push_back(i->second.addr());
+
+    return true;
+}
+
+template <>
+bool
+XrlIO<IPv6>::get_addresses(const string& interface, const string& vif,
+			   list<IPv6>& addresses) const
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    const IfMgrVifAtom* fv = ifmgr_iftree().find_vif(interface, vif);
+    if (fv == NULL)
+	return false;
+
+    IfMgrVifAtom::V6Map::const_iterator i;
+    for (i = fv->ipv6addrs().begin(); i != fv->ipv6addrs().end(); i++)
+	addresses.push_back(i->second.addr());
+
+    return true;
+}
+
+template <>
+bool
+XrlIO<IPv4>::get_link_local_address(const string& interface, const string& vif,
+				    IPv4& address)
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    const IfMgrVifAtom* fv = ifmgr_iftree().find_vif(interface, vif);
+    if (fv == NULL)
+	return false;
+
+    IfMgrVifAtom::V4Map::const_iterator i;
+    for (i = fv->ipv4addrs().begin(); i != fv->ipv4addrs().end(); i++) {
+	if (i->second.addr().is_linklocal_unicast()) {
+	    address = i->second.addr();
+	    return true;
+	}
+    }
+
+    return false;
+}
+
+template <>
+bool
+XrlIO<IPv6>::get_link_local_address(const string& interface, const string& vif,
+				    IPv6& address)
+{
+    debug_msg("Interface %s Vif %s\n", interface.c_str(), vif.c_str());
+
+    const IfMgrVifAtom* fv = ifmgr_iftree().find_vif(interface, vif);
+    if (fv == NULL)
+	return false;
+
+    IfMgrVifAtom::V6Map::const_iterator i;
+    for (i = fv->ipv6addrs().begin(); i != fv->ipv6addrs().end(); i++) {
+	if (i->second.addr().is_linklocal_unicast()) {
+	    address = i->second.addr();
+	    return true;
+	}
+    }
+
+    return false;
+}
+
+template <typename A>
+bool
+XrlIO<A>::get_interface_id(const string& interface, uint32_t& interface_id)
+{
+    debug_msg("Interface %s\n", interface.c_str());
+
+    const IfMgrIfAtom* fi = ifmgr_iftree().find_if(interface);
+    if (fi == NULL)
+	return false;
+
+    interface_id = fi->pif_index();
+
+    return true;
+}
+
 template <>
 uint32_t
 XrlIO<IPv4>::get_prefix_length(const string& interface, const string& vif,
@@ -730,22 +824,25 @@ XrlIO<A>::rib_command_done(const XrlError& error, bool up,
 
 template <typename A>
 bool
-XrlIO<A>::add_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
-		    bool discard, const PolicyTags& policytags)
+XrlIO<A>::add_route(IPNet<A> net, A nexthop, uint32_t nexthop_id, 
+		    uint32_t metric, bool equal, bool discard,
+		    const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
 	      cstring(net), cstring(nexthop), metric, equal ? "true" : "false",
 	      discard ? "true" : "false", cstring(policytags));
 
-    _rib_queue.queue_add_route(_ribname, net, nexthop, metric, policytags);
+    _rib_queue.queue_add_route(_ribname, net, nexthop, nexthop_id, metric,
+			       policytags);
 
     return true;
 }
 
 template <typename A>
 bool
-XrlIO<A>::replace_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
-			bool discard, const PolicyTags& policytags)
+XrlIO<A>::replace_route(IPNet<A> net, A nexthop, uint32_t nexthop_id,
+			uint32_t metric, bool equal, bool discard,
+			const PolicyTags& policytags)
 {
     debug_msg("Net %s Nexthop %s metric %d equal %s discard %s policy %s\n",
 	      cstring(net), cstring(nexthop), metric, equal ? "true" : "false",
@@ -753,7 +850,8 @@ XrlIO<A>::replace_route(IPNet<A> net, A nexthop, uint32_t metric, bool equal,
 
     // XXX - The queue should support replace see TODO 36.
     _rib_queue.queue_delete_route(_ribname, net);
-    _rib_queue.queue_add_route(_ribname, net, nexthop, metric, policytags);
+    _rib_queue.queue_add_route(_ribname, net, nexthop, nexthop_id, metric,
+			       policytags);
 
     return true;
 }
@@ -771,7 +869,7 @@ XrlIO<A>::delete_route(IPNet<A> net)
 
 template<class A>
 XrlQueue<A>::XrlQueue(EventLoop& eventloop, XrlRouter& xrl_router)
-    : _eventloop(eventloop), _xrl_router(xrl_router), _flying(0)
+    : _io(0), _eventloop(eventloop), _xrl_router(xrl_router), _flying(0)
 {
 }
 
@@ -785,8 +883,8 @@ XrlQueue<A>::eventloop() const
 template<class A>
 void
 XrlQueue<A>::queue_add_route(string ribname, const IPNet<A>& net,
-			     const A& nexthop, uint32_t metric,
-			     const PolicyTags& policytags)
+			     const A& nexthop, uint32_t nexthop_id,
+			     uint32_t metric, const PolicyTags& policytags)
 {
     Queued q;
 
@@ -800,6 +898,7 @@ XrlQueue<A>::queue_add_route(string ribname, const IPNet<A>& net,
     q.ribname = ribname;
     q.net = net;
     q.nexthop = nexthop;
+    q.nexthop_id = nexthop_id;
     q.metric = metric;
     q.comment = 
 	c_format("add_route: ribname %s net %s nexthop %s",
@@ -969,14 +1068,37 @@ XrlQueue<IPv6>::sendit_spec(Queued& q, const char *protocol)
 	    _bgp.profile().log(profile_route_rpc_out, 
 			       c_format("add %s", q.net.str().c_str()));
 #endif
-	sent = rib.
-	    send_add_route6(q.ribname.c_str(),
-			    protocol,
-			    unicast, multicast,
-			    q.net, q.nexthop, q.metric, 
-			    q.policytags.xrl_atomlist(),
-			    callback(this, &XrlQueue::route_command_done,
-				     q.comment));
+	if (OspfTypes::UNUSED_INTERFACE_ID == q.nexthop_id) {
+	    sent = rib.
+		send_add_route6(q.ribname.c_str(),
+				protocol,
+				unicast, multicast,
+				q.net, q.nexthop, q.metric, 
+				q.policytags.xrl_atomlist(),
+				callback(this, &XrlQueue::route_command_done,
+					 q.comment));
+	} else {
+	    string interface;
+	    string vif;
+	    XLOG_ASSERT(_io);
+	    if (!_io->get_interface_vif_by_interface_id(q.nexthop_id,
+							interface, vif)) {
+		XLOG_ERROR("Unable to find interface/vif associated with %u",
+			   q.nexthop_id);
+		return false;
+	    }
+	    sent = rib.
+		send_add_interface_route6(q.ribname.c_str(),
+					  protocol,
+					  unicast, multicast,
+					  q.net, q.nexthop,
+					  interface, vif,
+					  q.metric, 
+					  q.policytags.xrl_atomlist(),
+					  callback(this,
+					    &XrlQueue::route_command_done,
+						   q.comment));
+	}
 	if (!sent)
 	    XLOG_WARNING("scheduling add route %s failed",
 			 q.net.str().c_str());
