@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_config.cc,v 1.47 2006/07/03 23:33:39 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_config.cc,v 1.50 2007/02/16 22:46:46 pavlin Exp $"
 
 
 //
@@ -37,30 +37,12 @@ int
 PimNode::set_config_all_vifs_done(string& error_msg)
 {
     map<string, Vif>::iterator vif_iter;
-    string err;
     map<string, Vif>& configured_vifs = ProtoNode<PimVif>::configured_vifs();
     set<string> send_pim_hello_vifs;
     string dummy_error_msg;
 
     //
-    // Remove vifs that don't exist anymore
-    //
-    for (uint32_t i = 0; i < maxvifs(); i++) {
-	Vif* node_vif = vif_find_by_vif_index(i);
-	if (node_vif == NULL)
-	    continue;
-	if (node_vif->is_pim_register())
-	    continue;		// XXX: don't delete the PIM Register vif
-	if (configured_vifs.find(node_vif->name()) == configured_vifs.end()) {
-	    // Delete the interface
-	    string vif_name = node_vif->name();
-	    delete_vif(vif_name, err);
-	    continue;
-	}
-    }
-
-    //
-    // Add new vifs, update existing ones and remove old addresses
+    // Add new vifs and update existing ones
     //
     for (vif_iter = configured_vifs.begin();
 	 vif_iter != configured_vifs.end();
@@ -72,7 +54,7 @@ PimNode::set_config_all_vifs_done(string& error_msg)
 	// Add a new vif
 	//
 	if (node_vif == NULL) {
-	    add_vif(*vif, err);
+	    add_vif(*vif, dummy_error_msg);
 	    continue;
 	}
 	
@@ -82,14 +64,45 @@ PimNode::set_config_all_vifs_done(string& error_msg)
 	set_vif_flags(vif->name(), vif->is_pim_register(), vif->is_p2p(),
 		      vif->is_loopback(), vif->is_multicast_capable(),
 		      vif->is_broadcast_capable(), vif->is_underlying_vif_up(),
-		      vif->mtu(), err);
+		      vif->mtu(), dummy_error_msg);
+    }
+
+    //
+    // Add new vif addresses, update existing ones, and remove old addresses
+    //
+    for (vif_iter = configured_vifs.begin();
+	 vif_iter != configured_vifs.end();
+	 ++vif_iter) {
+	Vif* vif = &vif_iter->second;
+	Vif* node_vif = vif_find_by_name(vif->name());
+	list<VifAddr>::const_iterator vif_addr_iter;
+
+	if (node_vif == NULL)
+	    continue;
+
+	//
+	// Add new vif addresses and update existing ones
+	//
+	for (vif_addr_iter = vif->addr_list().begin();
+	     vif_addr_iter != vif->addr_list().end();
+	     ++vif_addr_iter) {
+	    const VifAddr& vif_addr = *vif_addr_iter;
+	    bool should_send_pim_hello = false;
+	    add_vif_addr(vif->name(), vif_addr.addr(),
+			 vif_addr.subnet_addr(),
+			 vif_addr.broadcast_addr(),
+			 vif_addr.peer_addr(),
+			 should_send_pim_hello,
+			 dummy_error_msg);
+	    if (should_send_pim_hello)
+		send_pim_hello_vifs.insert(vif->name());
+	}
 	
 	//
 	// Delete vif addresses that don't exist anymore
 	//
 	{
 	    list<IPvX> delete_addresses_list;
-	    list<VifAddr>::const_iterator vif_addr_iter;
 	    for (vif_addr_iter = node_vif->addr_list().begin();
 		 vif_addr_iter != node_vif->addr_list().end();
 		 ++vif_addr_iter) {
@@ -104,7 +117,8 @@ PimNode::set_config_all_vifs_done(string& error_msg)
 		 ++ipvx_iter) {
 		const IPvX& ipvx = *ipvx_iter;
 		bool should_send_pim_hello = false;
-		delete_vif_addr(vif->name(), ipvx, should_send_pim_hello, err);
+		delete_vif_addr(vif->name(), ipvx, should_send_pim_hello,
+				dummy_error_msg);
 		if (should_send_pim_hello)
 		    send_pim_hello_vifs.insert(vif->name());
 	    }
@@ -112,34 +126,22 @@ PimNode::set_config_all_vifs_done(string& error_msg)
     }
 
     //
-    // Add new vif addresses, and update existing ones
+    // Remove vifs that don't exist anymore
     //
-    for (vif_iter = configured_vifs.begin();
-	 vif_iter != configured_vifs.end();
-	 ++vif_iter) {
-	Vif* vif = &vif_iter->second;
-	Vif* node_vif = vif_find_by_name(vif->name());
-
+    for (uint32_t i = 0; i < maxvifs(); i++) {
+	Vif* node_vif = vif_find_by_vif_index(i);
 	if (node_vif == NULL)
 	    continue;
-
-	list<VifAddr>::const_iterator vif_addr_iter;
-	for (vif_addr_iter = vif->addr_list().begin();
-	     vif_addr_iter != vif->addr_list().end();
-	     ++vif_addr_iter) {
-	    const VifAddr& vif_addr = *vif_addr_iter;
-	    bool should_send_pim_hello = false;
-	    add_vif_addr(vif->name(), vif_addr.addr(),
-			 vif_addr.subnet_addr(),
-			 vif_addr.broadcast_addr(),
-			 vif_addr.peer_addr(),
-			 should_send_pim_hello,
-			 err);
-	    if (should_send_pim_hello)
-		send_pim_hello_vifs.insert(vif->name());
+	if (node_vif->is_pim_register())
+	    continue;		// XXX: don't delete the PIM Register vif
+	if (configured_vifs.find(node_vif->name()) == configured_vifs.end()) {
+	    // Delete the interface
+	    string vif_name = node_vif->name();
+	    delete_vif(vif_name, dummy_error_msg);
+	    continue;
 	}
     }
-    
+
     //
     // Spec:
     // "If an interface changes one of its secondary IP addresses,
