@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_proto_hello.cc,v 1.24 2006/03/16 00:04:54 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_proto_hello.cc,v 1.26 2007/02/16 22:46:49 pavlin Exp $"
 
 
 //
@@ -79,17 +79,6 @@ PimVif::pim_hello_stop()
     hello_holdtime().set(save_holdtime);
 }
 
-void
-PimVif::pim_hello_stop_dr()
-{
-    uint32_t save_dr_priority = dr_priority().get();
-    string dummy_error_msg;
-    
-    dr_priority().set(PIM_HELLO_DR_PRIORITY_LOWEST);		// XXX: lowest
-    pim_hello_send(dummy_error_msg);
-    dr_priority().set(save_dr_priority);
-}
-
 /**
  * PimVif::pim_hello_recv:
  * @pim_nbr: The PIM neighbor message originator (or NULL if not a neighbor).
