@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_proto_register.cc,v 1.29 2006/07/28 06:07:58 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_proto_register.cc,v 1.37 2007/02/16 22:46:50 pavlin Exp $"
 
 
 //
@@ -27,7 +27,8 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 
-#include "mrt/inet_cksum.h"
+#include "libproto/checksum.h"
+#include "libproto/packet.hh"
 
 #include "pim_mfc.hh"
 #include "pim_node.hh"
@@ -72,7 +73,6 @@ PimVif::pim_register_recv(PimNbr *pim_nbr,
 			  const IPvX& dst,
 			  buffer_t *buffer)
 {
-#ifndef HOST_OS_WINDOWS
     uint32_t pim_register_flags;
     bool is_null_register, is_border_register;
     IPvX inner_src(family()), inner_dst(family());
@@ -105,19 +105,19 @@ PimVif::pim_register_recv(PimNbr *pim_nbr,
     //
     switch (family()) {
     case AF_INET: {
-	struct ip ip4_header;
-	uint8_t *cp = (uint8_t *)&ip4_header;
+	uint8_t ip_header4_buffer[IpHeader4::SIZE];
+	IpHeader4 ip4(ip_header4_buffer);
 	
-	BUFFER_GET_DATA(cp, buffer, sizeof(ip4_header));
-	inner_src.copy_in(ip4_header.ip_src);
-	inner_dst.copy_in(ip4_header.ip_dst);
+	BUFFER_GET_DATA(ip_header4_buffer, buffer, IpHeader4::SIZE);
+	inner_src = IPvX(ip4.ip_src());
+	inner_dst = IPvX(ip4.ip_dst());
 	if (is_null_register) {
 	    //
 	    // If the inner header checksum is non-zero, then
 	    // check the checksum.
 	    //
-	    if (ip4_header.ip_sum != 0) {
-		uint16_t cksum = INET_CKSUM(&ip4_header, sizeof(ip4_header));
+	    if (ip4.ip_sum() != 0) {
+		uint16_t cksum = inet_checksum(ip4.data(), ip4.size());
 		if (cksum != 0) {
 		    XLOG_WARNING("RX %s%s from %s to %s: "
 				 "inner dummy IP header checksum error",
@@ -134,14 +134,14 @@ PimVif::pim_register_recv(PimNbr *pim_nbr,
     
 #ifdef HAVE_IPV6	
     case AF_INET6: {
-	struct ip6_hdr ip6_header;
-	uint8_t *cp = (uint8_t *)&ip6_header;
+	uint8_t ip_header6_buffer[IpHeader6::SIZE];
+	IpHeader6 ip6(ip_header6_buffer);
 	uint16_t inner_data_len;
 	
-	BUFFER_GET_DATA(cp, buffer, sizeof(ip6_header));
-	inner_src.copy_in(ip6_header.ip6_src);
-	inner_dst.copy_in(ip6_header.ip6_dst);
-	inner_data_len = ntohs(ip6_header.ip6_plen);
+	BUFFER_GET_DATA(ip_header6_buffer, buffer, IpHeader6::SIZE);
+	inner_src = IPvX(ip6.ip_src());
+	inner_dst = IPvX(ip6.ip_dst());
+	inner_data_len = ip6.ip_plen();
 	if (is_null_register) {
 	    //
 	    // If the dummy PIM header is present, then
@@ -149,15 +149,17 @@ PimVif::pim_register_recv(PimNbr *pim_nbr,
 	    //
 	    if (inner_data_len == sizeof(struct pim)) {
 		struct pim pim_header;
-		cp = (uint8_t *)&pim_header;
+		uint8_t *cp = (uint8_t *)&pim_header;
 		BUFFER_GET_DATA(cp, buffer, sizeof(pim_header));
 		uint16_t cksum, cksum2;
-		cksum = INET_CKSUM(&pim_header, sizeof(pim_header));
+		cksum = inet_checksum(
+		    reinterpret_cast<const uint8_t *>(&pim_header),
+		    sizeof(pim_header));
 		cksum2 = calculate_ipv6_pseudo_header_checksum(inner_src,
 							       inner_dst,
 							       sizeof(struct pim),
 							       IPPROTO_PIM);
-		cksum = INET_CKSUM_ADD(cksum, cksum2);
+		cksum = inet_checksum_add(cksum, cksum2);
 		if (cksum != 0) {
 		    XLOG_WARNING("RX %s%s from %s to %s: "
 				 "inner dummy IP header checksum error",
@@ -200,10 +202,10 @@ PimVif::pim_register_recv(PimNbr *pim_nbr,
 	return (XORP_ERROR);
     }
     if (inner_dst.is_linklocal_multicast()
-	|| inner_dst.is_nodelocal_multicast()) {
+	|| inner_dst.is_interfacelocal_multicast()) {
 	XLOG_WARNING("RX %s%s from %s to %s: "
 		     "inner destination address = %s must not be "
-		     "link or node-local multicast group",
+		     "link or interface-local multicast group",
 		     PIMTYPE2ASCII(PIM_REGISTER),
 		     (is_null_register)? "(Null)" : "",
 		     cstring(src), cstring(dst),
@@ -434,17 +436,6 @@ PimVif::pim_register_recv(PimNbr *pim_nbr,
 		 cstring(src), cstring(dst));
     ++_pimstat_rx_malformed_packet;
     return (XORP_ERROR);
-
-#else /* HOST_OS_WINDOWS */
-
-    XLOG_FATAL("WinSock2 lacks struct ip definition");
-    return (XORP_ERROR);
-
-    UNUSED(pim_nbr);
-    UNUSED(src);
-    UNUSED(dst);
-    UNUSED(buffer);
-#endif /* !HOST_OS_WINDOWS */
 }
 
 int
@@ -455,17 +446,18 @@ PimVif::pim_register_send(const IPvX& rp_addr,
 			  size_t rcvlen,
 			  string& error_msg)
 {
-#ifndef HOST_OS_WINDOWS
-    const struct ip *ip4 = (const struct ip *)rcvbuf;
+    IpHeader4 ip4(rcvbuf);
     buffer_t *buffer;
     uint32_t flags = 0;
     size_t mtu = 0;
     string dummy_error_msg;
     
-    if (ip4->ip_v != source_addr.ip_version()) {
+    UNUSED(group_addr);
+
+    if (ip4.ip_version() != source_addr.ip_version()) {
 	error_msg = c_format("Cannot encapsulate IP packet: "
 			     "inner IP version (%u) != expected IP version (%u)",
-			     XORP_UINT_CAST(ip4->ip_v),
+			     XORP_UINT_CAST(ip4.ip_version()),
 			     XORP_UINT_CAST(source_addr.ip_version()));
 	XLOG_WARNING("%s", error_msg.c_str());
 	return (XORP_ERROR);
@@ -523,178 +515,34 @@ PimVif::pim_register_send(const IPvX& rp_addr,
     // Fragment the inner packet, then encapsulate and send each fragment
     //
     if (family() == AF_INET) {
-	uint8_t frag_buf[rcvlen];	// The buffer for the fragments
-	struct ip *frag_ip4 = (struct ip *)frag_buf;
-	size_t frag_optlen = 0;		// The optlen to copy into fragments
-	size_t frag_ip_hl;
-	
-	if (ntohs(ip4->ip_off) & IP_DF) {
-	    // Fragmentation is forbidded
-	    error_msg = c_format("Cannot fragment encapsulated IP packet "
-				 "from %s to %s: "
-				 "fragmentation not allowed",
-				 cstring(source_addr), cstring(group_addr));
-	    XLOG_WARNING("%s", error_msg.c_str());
-	    // XXX: we don't send ICMP "fragmentation needed" back to the
-	    // sender, because in IPv4 ICMP error messages are not sent
-	    // back for datagrams destinated to an multicast address.
+	list<vector<uint8_t> > fragments;
+	list<vector<uint8_t> >::iterator iter;
+
+	if (ip4.fragment(mtu, fragments, true, error_msg) != XORP_OK) {
+	    //
+	    // XXX: If fragmentation is forbidded, we don't send
+	    // ICMP "fragmentation needed" back to the sender, because in
+	    // IPv4 ICMP error messages are not sent back for datagrams
+	    // destinated to an multicast address.
+	    //
 	    return (XORP_ERROR);
 	}
-	if (((mtu - (ip4->ip_hl << 2)) & ~7) < 8) {
-	    // Fragmentation is possible only if we can put at least
-	    // 8 octets per fragment (except the last one)
-	    error_msg = c_format("Cannot fragment encapsulated IP packet "
-				 "from %s to %s: "
-				 "cannot send fragment with size less than 8 octets",
-				 cstring(source_addr), cstring(group_addr));
-	    XLOG_WARNING("%s", error_msg.c_str());
-	    return (XORP_ERROR);
-	}
-	
+
 	//
-	// Copy the IP header and the options that should be copied during
-	// fragmentation.
-	// XXX: the code below is taken from FreeBSD's ip_optcopy()
-	// in netinet/ip_output.c
+	// XXX: we already checked that the data packet is larger than
+	// the MTU, so the packet must have been fragmented.
 	//
-	memcpy(frag_buf, rcvbuf, sizeof(struct ip));
-	{
-	    register const u_char *cp;
-	    register u_char *dp;
-	    int opt, optlen, cnt;
-	    
-	    cp = (const u_char *)(ip4 + 1);
-	    dp = (u_char *)(frag_ip4 + 1);
-	    cnt = (ip4->ip_hl << 2) - sizeof (struct ip);
-	    for (; cnt > 0; cnt -= optlen, cp += optlen) {
-                opt = cp[0];
-                if (opt == IPOPT_EOL)
-		    break;
-                if (opt == IPOPT_NOP) {
-		    // Preserve for IP mcast tunnel's LSRR alignment.
-		    *dp++ = IPOPT_NOP;
-		    optlen = 1;
-		    continue;
-                }
-		
-		//
-		// Check for bogus lengths
-		//
-		if ((size_t)cnt < IPOPT_OLEN + sizeof(*cp)) {
-		    error_msg = c_format("Cannot fragment encapsulated IP "
-					 "packet from %s to %s: "
-					 "malformed IPv4 option",
-					 cstring(source_addr),
-					 cstring(group_addr));
-		    XLOG_WARNING("%s", error_msg.c_str());
-		    return (XORP_ERROR);
-		}
-                optlen = cp[IPOPT_OLEN];
-		if (optlen < (int)(IPOPT_OLEN + sizeof(*cp)) || optlen > cnt) {
-		    error_msg = c_format("Cannot fragment encapsulated IP "
-					 "packet from %s to %s: "
-					 "malformed IPv4 option",
-					 cstring(source_addr),
-					 cstring(group_addr));
-		    XLOG_WARNING("%s", error_msg.c_str());
-		    return (XORP_ERROR);
-		}
-		
-                if (optlen > cnt)
-		    optlen = cnt;
-                if (IPOPT_COPIED(opt)) {
-		    memcpy(dp, cp, optlen);
-		    dp += optlen;
-                }
-	    }
-	    for (optlen = dp - (u_char *)(frag_ip4+1); optlen & 0x3; optlen++)
-                *dp++ = IPOPT_EOL;
-	    frag_optlen = optlen;	// return (optlen);
-	}
-	// The header length with the remaining IP options
-	frag_ip_hl = (sizeof(struct ip) + frag_optlen);
-	frag_ip4->ip_hl = frag_ip_hl >> 2;
-	
-	//
-	// Do the fragmentation
-	//
-	size_t data_start = 0;
-	size_t data_end = rcvlen;
-	// Send the first segment with all the options
-	{
-	    uint8_t first_frag[mtu];
-	    struct ip *first_ip = (struct ip *)first_frag;
-	    size_t first_ip_hl = ip4->ip_hl << 2;
-	    size_t nfb = (mtu - first_ip_hl) / 8;
-	    size_t first_frag_len = first_ip_hl + nfb*8;
-	    
-	    memcpy(first_frag, rcvbuf, first_frag_len);
-	    
-	    // Correct the IP header
-	    first_ip->ip_off = htons(ntohs(first_ip->ip_off) | IP_MF);
-	    first_ip->ip_len = htons(first_frag_len);
-	    first_ip->ip_sum = 0;
-	    first_ip->ip_sum = INET_CKSUM(first_ip, first_ip_hl);
-	    
-	    // Encapsulate and send the fragment
+	XLOG_ASSERT(! fragments.empty());
+
+	// Encapsulate and send the fragments
+	for (iter = fragments.begin(); iter != fragments.end(); ++iter) {
+	    vector<uint8_t>& ip_fragment = *iter;
+
 	    buffer = buffer_send_prepare();
 	    BUFFER_PUT_HOST_32(flags, buffer);
-	    BUFFER_PUT_DATA(first_frag, buffer, first_frag_len);
+	    BUFFER_PUT_DATA(&ip_fragment[0], buffer, ip_fragment.size());
 	    pim_send(domain_wide_addr(), rp_addr, PIM_REGISTER, buffer,
 		     dummy_error_msg);
-	    
-	    data_start += first_frag_len;
-	}
-	
-	//
-	// Create the remaining of the fragments
-	//
-	while (data_start < data_end) {
-	    size_t nfb = (mtu - frag_ip_hl) / 8;
-	    size_t frag_len = frag_ip_hl + nfb*8;
-	    size_t frag_data_len = nfb*8;
-	    bool is_last_fragment = false;
-	    
-	    // Compute the fragment length
-	    if (data_end - data_start <= frag_data_len) {
-		frag_data_len = data_end - data_start;
-		frag_len = frag_ip_hl + frag_data_len;
-		is_last_fragment = true;
-	    }
-	    
-	    // Copy the data
-	    memcpy(frag_buf + frag_ip_hl, rcvbuf + data_start, frag_data_len);
-	    
-	    // The IP packet total length
-	    frag_ip4->ip_len = htons(frag_len);
-	    
-	    // The IP fragment flags and offset
-	    {
-#ifndef IP_OFFMASK
-#define IP_OFFMASK 0x1fff	// Mask for fragmenting bits
-#endif
-		unsigned short ip_off_field = ntohs(ip4->ip_off);
-		unsigned short frag_ip_off_flags = ip_off_field & ~IP_OFFMASK;
-		unsigned short frag_ip_off = (ip_off_field & IP_OFFMASK);
-		
-		if (! is_last_fragment)
-		    frag_ip_off_flags |= IP_MF;
-		
-		frag_ip_off += (data_start - (ip4->ip_hl << 2)) / 8;	// XXX
-		frag_ip4->ip_off = htons(frag_ip_off_flags | frag_ip_off);
-	    }
-	    
-	    frag_ip4->ip_sum = 0;
-	    frag_ip4->ip_sum = INET_CKSUM(frag_ip4, frag_ip_hl);
-	    
-	    // Encapsulate and send the fragment
-	    buffer = buffer_send_prepare();
-	    BUFFER_PUT_HOST_32(flags, buffer);
-	    BUFFER_PUT_DATA(frag_buf, buffer, frag_len);
-	    pim_send(domain_wide_addr(), rp_addr, PIM_REGISTER, buffer,
-		     dummy_error_msg);
-	    
-	    data_start += frag_data_len;
 	}
     }
     
@@ -717,19 +565,6 @@ PimVif::pim_register_send(const IPvX& rp_addr,
 			 cstring(domain_wide_addr()), cstring(rp_addr));
     XLOG_ERROR("%s", error_msg.c_str());
     return (XORP_ERROR);
-
-#else /* HOST_OS_WINDOWS */
-
-    error_msg = c_format("WinSock2 lacks struct ip definition");
-    XLOG_FATAL("%s", error_msg.c_str());
-    return (XORP_ERROR);
-
-    UNUSED(rp_addr);
-    UNUSED(source_addr);
-    UNUSED(group_addr);
-    UNUSED(rcvbuf);
-    UNUSED(rcvlen);
-#endif /* !HOST_OS_WINDOWS */
 }
 
 int
@@ -738,7 +573,6 @@ PimVif::pim_register_null_send(const IPvX& rp_addr,
 			       const IPvX& group_addr,
 			       string& error_msg)
 {
-#ifndef HOST_OS_WINDOWS
     buffer_t *buffer = buffer_send_prepare();
     uint32_t flags = 0;
 
@@ -749,42 +583,25 @@ PimVif::pim_register_null_send(const IPvX& rp_addr,
     // Create the dummy IP header and write it to the buffer
     switch (family()) {
     case AF_INET: {
-	struct ip ip4_header;
-	uint8_t *cp = (uint8_t *)&ip4_header;
-	
-	memset(&ip4_header, 0, sizeof(ip4_header));
-	ip4_header.ip_v		= IPVERSION;
-	ip4_header.ip_hl	= (sizeof(ip4_header) >> 2);
-	ip4_header.ip_tos	= 0;
-	ip4_header.ip_id	= 0;
-	ip4_header.ip_off	= 0;
-	ip4_header.ip_p		= IPPROTO_PIM;
-	ip4_header.ip_len	= htons(sizeof(ip4_header));
-	ip4_header.ip_ttl	= 0;
+	uint8_t ip_header4_buffer[IpHeader4::SIZE];
+	memset(ip_header4_buffer, 0, sizeof(ip_header4_buffer));
 
-	//
-	// XXX: we need to use a temporary in_addr storage as a work-around
-	// if "struct ip" is __packed
-	//
-	struct in_addr in_addr_tmp;
-	source_addr.copy_out(in_addr_tmp);
-	ip4_header.ip_src = in_addr_tmp;
-	group_addr.copy_out(in_addr_tmp);
-	ip4_header.ip_dst = in_addr_tmp;
-
-	//
-	// XXX: on older Linux 'ip->ip_sum' was named 'ip->ip_csum'.
-	// Later someone has realized that it is not a smart move,
-	// so now Linux is in sync with the rest of the UNIX-es.
-	// If you got one of those old Linux-es, you better upgrade to
-	// something more decent. If you are stubborn and you don't want
-	// to upgrade, then change 'ip_sum' to 'ip_csum' below, but
-	// you should know that in the future you will get what you deserve :)
-	//
-	ip4_header.ip_sum	= 0;
-	ip4_header.ip_sum	= INET_CKSUM(&ip4_header, sizeof(ip4_header));
+	IpHeader4Writer ip4(ip_header4_buffer);
 	
-	BUFFER_PUT_DATA(cp, buffer, sizeof(ip4_header));
+	ip4.set_ip_version(IpHeader4::IP_VERSION);
+	ip4.set_ip_header_len(IpHeader4::SIZE);
+	ip4.set_ip_tos(0);
+	ip4.set_ip_id(0);
+	ip4.set_ip_off(0);
+	ip4.set_ip_p(IPPROTO_PIM);
+	ip4.set_ip_len(IpHeader4::SIZE);
+	ip4.set_ip_ttl(0);
+	ip4.set_ip_src(source_addr.get_ipv4());
+	ip4.set_ip_dst(group_addr.get_ipv4());
+	ip4.set_ip_sum(0);
+	ip4.set_ip_sum(ntohs(inet_checksum(ip4.data(), ip4.size())));
+	
+	BUFFER_PUT_DATA(ip_header4_buffer, buffer, IpHeader4::SIZE);
 	break;
     }
     
@@ -793,42 +610,35 @@ PimVif::pim_register_null_send(const IPvX& rp_addr,
 	//
 	// First generate the dummy IPv6 header, and then the dummy PIM header
 	//
-	
+	uint8_t ip_header6_buffer[IpHeader6::SIZE];
+	memset(ip_header6_buffer, 0, sizeof(ip_header6_buffer));
+
 	// Generate the dummy IPv6 header
-	struct ip6_hdr ip6_header;
-	uint8_t *cp = (uint8_t *)&ip6_header;
+	IpHeader6Writer ip6(ip_header6_buffer);
+
+	ip6.set_ip_vtc_flow(0);
+	ip6.set_ip_version(IpHeader6::IP_VERSION);
+	ip6.set_ip_plen(sizeof(struct pim));
+	ip6.set_ip_hlim(0);
+	ip6.set_ip_nxt(IPPROTO_PIM);
+	ip6.set_ip_src(source_addr.get_ipv6());
+	ip6.set_ip_dst(group_addr.get_ipv6());
 	
-	// XXX: conditionally define IPv6 header related values
-#ifndef IPV6_VERSION
-#define IPV6_VERSION		0x60
-#endif
-#ifndef IPV6_VERSION_MASK
-#define IPV6_VERSION_MASK	0xf0
-#endif
-	memset(&ip6_header, 0, sizeof(ip6_header));
-	ip6_header.ip6_plen	= htons(sizeof(struct pim));
-	ip6_header.ip6_flow	= 0;
-	ip6_header.ip6_vfc	&= ~IPV6_VERSION_MASK;
-	ip6_header.ip6_vfc	|= IPV6_VERSION;
-	ip6_header.ip6_hlim	= 0;
-	ip6_header.ip6_nxt	= IPPROTO_PIM;
-	source_addr.copy_out((uint8_t*)&(ip6_header.ip6_src));
-	group_addr.copy_out((uint8_t*)&(ip6_header.ip6_dst));
-	
-	BUFFER_PUT_DATA(cp, buffer, sizeof(ip6_header));
+	BUFFER_PUT_DATA(ip_header6_buffer, buffer, IpHeader6::SIZE);
 	
 	// Generate the dummy PIM header
 	uint16_t cksum, cksum2;
 	struct pim pim_header;
-	cp = (uint8_t *)&pim_header;
+	uint8_t *cp = (uint8_t *)&pim_header;
 	
 	memset(&pim_header, 0, sizeof(pim_header));
-	cksum = INET_CKSUM(&pim_header, sizeof(pim_header));	// XXX: no-op
+	cksum = inet_checksum(reinterpret_cast<const uint8_t *>(&pim_header),
+			      sizeof(pim_header));	// XXX: no-op
 	cksum2 = calculate_ipv6_pseudo_header_checksum(source_addr,
 						       group_addr,
 						       sizeof(struct pim),
 						       IPPROTO_PIM);
-	cksum = INET_CKSUM_ADD(cksum, cksum2);
+	cksum = inet_checksum_add(cksum, cksum2);
 	pim_header.pim_cksum = cksum;
 	
 	BUFFER_PUT_DATA(cp, buffer, sizeof(pim_header));
@@ -854,15 +664,4 @@ PimVif::pim_register_null_send(const IPvX& rp_addr,
 			 cstring(domain_wide_addr()), cstring(rp_addr));
     XLOG_ERROR("%s", error_msg.c_str());
     return (XORP_ERROR);
-
-#else /* HOST_OS_WINDOWS */
-
-    error_msg = c_format("WinSock2 lacks struct ip definition");
-    XLOG_FATAL("%s", error_msg.c_str());
-    return (XORP_ERROR);
-
-    UNUSED(rp_addr);
-    UNUSED(source_addr);
-    UNUSED(group_addr);
-#endif /* !HOST_OS_WINDOWS */
 }
