@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_rp.cc,v 1.16 2006/03/16 00:04:55 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_rp.cc,v 1.18 2007/02/16 22:46:50 pavlin Exp $"
 
 
 //
@@ -283,12 +283,12 @@ RpTable::compare_rp(const IPvX& group_addr, PimRp *rp1, PimRp *rp2) const
 uint32_t
 RpTable::derived_addr(const IPvX& addr) const
 {
-    size_t addr_size = addr.addr_size()/sizeof(uint32_t);
+    size_t addr_bytelen = addr.addr_bytelen()/sizeof(uint32_t);
     uint32_t addr_array[sizeof(addr)];	// XXX: give us plenty of extra space
     uint32_t result = 0;
 	    
     addr.copy_out((uint8_t *)addr_array);
-    for (size_t i = 0; i < addr_size; i++)
+    for (size_t i = 0; i < addr_bytelen; i++)
 	result ^= addr_array[i];
     
     return (result);
