@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/pim/pim_vif.cc,v 1.61 2006/07/28 06:07:58 pavlin Exp $"
+#ident "$XORP: xorp/pim/pim_vif.cc,v 1.65 2007/02/16 22:46:51 pavlin Exp $"
 
 
 //
@@ -26,7 +26,8 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 
-#include "mrt/inet_cksum.h"
+#include "libproto/checksum.h"
+
 #include "mrt/random.h"
 
 #include "pim_node.hh"
@@ -392,12 +393,6 @@ PimVif::stop(string& error_msg)
 	//
 	pim_node().delete_protocol_mld6igmp(vif_index());
 	
-	// XXX: if the DR-priority Hello option is not in use, then we cannot
-	// stop ourselves being the DR. In that case, there could be a
-	// period of "missing-DR" until we completely stop this
-	// vif (if it was the DR).
-	pim_hello_stop_dr();
-	
 	set_i_am_dr(false);
     }
     
@@ -543,7 +538,11 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     int ret_value;
     size_t datalen;
     int ttl = MINTTL;
-    bool is_router_alert = true;
+    //
+    // XXX: According to newer revisions of the PIM-SM spec, the PIM-SM control
+    // messages don't include the IP Router Alert option.
+    //
+    bool is_router_alert = false;
 
     if (! (is_up() || is_pending_down()))
 	return (XORP_ERROR);
@@ -683,16 +682,17 @@ PimVif::pim_send(const IPvX& src, const IPvX& dst,
     // XXX: The checksum for PIM_REGISTER excludes the encapsulated data packet
     switch (message_type) {
     case PIM_REGISTER:
-	cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer),
-			   PIM_REGISTER_HEADER_LENGTH);
+	cksum = inet_checksum(BUFFER_DATA_HEAD(buffer),
+			      PIM_REGISTER_HEADER_LENGTH);
 	break;
     default:
-	cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
+	cksum = inet_checksum(BUFFER_DATA_HEAD(buffer),
+			      BUFFER_DATA_SIZE(buffer));
 	break;
     }
     
-    cksum = INET_CKSUM_ADD(cksum, cksum2);
-    BUFFER_COPYPUT_INET_CKSUM(cksum, buffer, 2);	// XXX: the ckecksum
+    cksum = inet_checksum_add(cksum, cksum2);
+    BUFFER_COPYPUT_INET_CKSUM(cksum, buffer, 2);	// XXX: the checksum
 
     XLOG_TRACE(pim_node().is_log_trace(),
 	       "TX %s from %s to %s on vif %s",
@@ -911,9 +911,9 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
     
     switch (message_type) {
     case PIM_REGISTER:
-	cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer),
-			   PIM_REGISTER_HEADER_LENGTH);
-	cksum = INET_CKSUM_ADD(cksum, cksum2);
+	cksum = inet_checksum(BUFFER_DATA_HEAD(buffer),
+			      PIM_REGISTER_HEADER_LENGTH);
+	cksum = inet_checksum_add(cksum, cksum2);
 	if (cksum == 0)
 	    break;
 	//
@@ -926,8 +926,9 @@ PimVif::pim_process(const IPvX& src, const IPvX& dst,
 	// FALLTHROUGH
 	
     default:
-	cksum = INET_CKSUM(BUFFER_DATA_HEAD(buffer), BUFFER_DATA_SIZE(buffer));
-	cksum = INET_CKSUM_ADD(cksum, cksum2);
+	cksum = inet_checksum(BUFFER_DATA_HEAD(buffer),
+			      BUFFER_DATA_SIZE(buffer));
+	cksum = inet_checksum_add(cksum, cksum2);
 
 	if (cksum == 0)
 	    break;
@@ -1609,7 +1610,9 @@ PimVif::calculate_ipv6_pseudo_header_checksum(const IPvX& src, const IPvX& dst,
     ip6_pseudo_header.ph_zero[2] = 0;
     ip6_pseudo_header.ph_next = protocol;
     
-    uint16_t cksum = INET_CKSUM(&ip6_pseudo_header, sizeof(ip6_pseudo_header));
+    uint16_t cksum = inet_checksum(
+	reinterpret_cast<const uint8_t *>(&ip6_pseudo_header),
+	sizeof(ip6_pseudo_header));
     
     return (cksum);
 }
