@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,17 +13,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/iv_exec.cc,v 1.10 2006/05/25 05:50:00 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/iv_exec.cc,v 1.13 2007/02/16 22:46:58 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
-#include "iv_exec.hh"
 #include "policy/common/policy_utils.hh"
 #include "policy/common/elem_null.hh"
 #include "policy/common/element.hh"
 
+#include "iv_exec.hh"
+
+
 IvExec::IvExec() : 
 	       _policies(NULL), _policy_count(0), _sman(NULL), _varrw(NULL),
 	       _finished(false), _fa(DEFAULT), _trash(NULL), _trashc(0),
@@ -179,7 +179,7 @@ void
 IvExec::visit(OnFalseExit& /* x */)
 {
     if (_stackptr < _stack)
-	throw RuntimeError("Got empty stack on ON_FALSE_EXIT");
+	xorp_throw(RuntimeError, "Got empty stack on ON_FALSE_EXIT");
 
     // we expect a bool at the top.
     const ElemBool* t = dynamic_cast<const ElemBool*>(*_stackptr);
@@ -195,7 +195,7 @@ IvExec::visit(OnFalseExit& /* x */)
 
 	// if it is anything else, its an error
         else {
-	   throw RuntimeError("Expected bool on top of stack instead: ");
+	   xorp_throw(RuntimeError, "Expected bool on top of stack instead: ");
 	}
 	    
     }
@@ -232,7 +232,7 @@ void
 IvExec::visit(Store& s)
 {
     if (_stackptr < _stack)
-	throw RuntimeError("Stack empty on assign of " + s.var());
+	xorp_throw(RuntimeError, "Stack empty on assign of " + s.var());
 
     const Element* arg = *_stackptr;
     _stackptr--;
