@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,19 +13,20 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/policy_filter.cc,v 1.8 2006/03/16 00:05:09 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/backend/policy_filter.cc,v 1.11 2007/02/16 22:46:59 pavlin Exp $"
 
 #include "policy/policy_module.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+
+#include "policy/common/policy_utils.hh"
+
 #include "policy_filter.hh"
 #include "policy_backend_parser.hh"
 #include "set_manager.hh"
 #include "iv_exec.hh"
-#include "policy/common/policy_utils.hh"
-#include "libxorp/xlog.h"
+
 
 using namespace policy_utils;
 using policy_backend_parser::policy_backend_parse;
@@ -47,7 +48,7 @@ void PolicyFilter::configure(const string& str)
 	delete_vector(policies);
 	clear_map(*sets);
 	delete sets;
-	throw ConfError(err);
+	xorp_throw(ConfError, err);
     }
 
     // properly erase old conf
