@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/policy_filters.cc,v 1.6 2006/03/16 00:05:09 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/backend/policy_filters.cc,v 1.9 2007/02/16 22:46:59 pavlin Exp $"
 
+#include "libxorp/xorp.h"
 #include "policy_filters.hh"
 
+
 PolicyFilters::PolicyFilters()
 {
     _import_filter = new PolicyFilter();
@@ -73,6 +71,6 @@ PolicyFilters::whichFilter(const uint32_t& ftype)
 	    return *_export_filter;
 	
     }
-    throw PolicyFiltersErr("Unknown filter: " +
-			   policy_utils::to_str(ftype));
+    xorp_throw(PolicyFiltersErr, 
+	       "Unknown filter: " + policy_utils::to_str(ftype));
 }
