@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,11 +13,9 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/policytags.cc,v 1.7 2006/03/16 00:05:10 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/policytags.cc,v 1.12 2007/02/16 22:46:59 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
 #include "policytags.hh"
 #include "policy/common/elem_set.hh"
@@ -36,7 +34,7 @@ PolicyTags::PolicyTags(const XrlAtomList& alist)
 
 	// only support u32's
 	if(atom.type() != xrlatom_uint32)
-	    throw PolicyTagsError("XrlAtomList does not contain uint32's");
+	    xorp_throw(PolicyTagsError, "XrlAtomList does not contain uint32's");
 
 	// it's good, insert it
 	_tags.insert(atom.uint32());    
@@ -48,8 +46,8 @@ PolicyTags::PolicyTags(const Element& element)
     // we only support set elements
     const ElemSetU32* es = dynamic_cast<const ElemSetU32*>(&element);
     if(!es)
-	throw PolicyTagsError(string("Element is not a set: ") + 
-			      element.type());
+	xorp_throw(PolicyTagsError, 
+		   string("Element is not a set: ") + element.type());
 
     // go through all the set elements.
     for(ElemSetU32::const_iterator i = es->begin(); i != es->end(); ++i) {
@@ -66,7 +64,7 @@ PolicyTags::str() const
 {
     ostringstream oss;
 
-    for(Set::iterator i = _tags.begin(); i != _tags.end(); ++i)
+    for(Set::const_iterator i = _tags.begin(); i != _tags.end(); ++i)
 	oss << *i << ", ";
 
     string res = oss.str();
@@ -92,7 +90,7 @@ PolicyTags::xrl_atomlist() const
 {
     XrlAtomList alist;
 
-    for(Set::iterator i = _tags.begin(); i != _tags.end(); ++i) {
+    for(Set::const_iterator i = _tags.begin(); i != _tags.end(); ++i) {
 	uint32_t tag = *i;
 
 	alist.append(XrlAtom(tag));
@@ -105,9 +103,9 @@ Element*
 PolicyTags::element() const
 {
     ElemSetU32* s = new ElemSetU32;
-    for (Set::iterator i = _tags.begin(); i != _tags.end(); ++i) {
-	ElemU32* e = new ElemU32(*i);
-	s->insert(*e);
+    for (Set::const_iterator i = _tags.begin(); i != _tags.end(); ++i) {
+	ElemU32 e(*i);
+	s->insert(e);
     }
     return s;
 }
@@ -116,7 +114,7 @@ void
 PolicyTags::insert(const PolicyTags& ptags)
 {
     // go through all the elements in ptags and insert them.
-    for(Set::iterator i = ptags._tags.begin();
+    for(Set::const_iterator i = ptags._tags.begin();
 	i != ptags._tags.end(); ++i)
 
 	_tags.insert(*i);
