@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,9 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/set_manager.cc,v 1.4 2006/03/16 00:05:11 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/set_manager.cc,v 1.7 2007/02/16 22:46:59 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
 #include "set_manager.hh"
 #include "policy/common/policy_utils.hh"
@@ -31,11 +29,11 @@ SetManager::~SetManager() {
 const Element&
 SetManager::getSet(const string& setid) const {
     if(!_sets)
-	throw SetNotFound("No sets initialized");
+	xorp_throw(SetNotFound, "No sets initialized");
 
     SetMap::iterator i = _sets->find(setid);
     if(i == _sets->end())
-        throw SetNotFound("Set not found: " + setid);
+        xorp_throw(SetNotFound, "Set not found: " + setid);
 
     Element* e = (*i).second;
 
