@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,17 +13,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/single_varrw.cc,v 1.13 2006/05/25 05:50:00 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/backend/single_varrw.cc,v 1.16 2007/02/16 22:46:59 pavlin Exp $"
 
 #include "policy/policy_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
-#include "single_varrw.hh"
+
 #include "policy/common/elem_null.hh"
 
+#include "single_varrw.hh"
+
+
 SingleVarRW::SingleVarRW() : _trashc(0), _did_first_read(false) 
 {
     memset(&_elems, 0, sizeof(_elems));
@@ -68,7 +69,7 @@ SingleVarRW::read(const Id& id) {
 
 	// out of luck...
 	if(!e)
-	    throw SingleVarRWErr("Unable to read variable " + id);
+	    xorp_throw(SingleVarRWErr, "Unable to read variable " + id);
     }
 
     return *e;
