@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,12 +13,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/backend/version_filters.cc,v 1.3 2006/03/16 00:05:12 pavlin Exp $"
+#ident "$XORP: xorp/policy/backend/version_filters.cc,v 1.5 2007/02/16 22:47:00 pavlin Exp $"
+
+#include "libxorp/xorp.h"
 
-#include "config.h"
 #include "version_filters.hh"
 #include "version_filter.hh"
 
+
 VersionFilters::VersionFilters() : PolicyFilters(
 				    new VersionFilter(VarRW::VAR_FILTER_IM),
 				    new VersionFilter(VarRW::VAR_FILTER_SM),
