@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,15 +13,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/code_generator.cc,v 1.9 2006/04/06 04:33:23 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/code_generator.cc,v 1.13 2007/02/16 22:46:52 pavlin Exp $"
 
 #include "policy_module.h"
+
+#include "libxorp/xorp.h"
+
 #include "code_generator.hh"    
 
+
 CodeGenerator::CodeGenerator(const VarMap& varmap) : _varmap(varmap)
 {
 }
@@ -31,8 +31,8 @@ CodeGenerator::CodeGenerator(const string& proto,
 			     const VarMap& varmap) : _varmap(varmap)
 {
     _protocol = proto;
-    _code._target.protocol = proto;
-    _code._target.filter = filter;
+    _code.set_target_protocol(proto);
+    _code.set_target_filter(filter);
 }
 
 // constructor for import policies
@@ -40,8 +40,8 @@ CodeGenerator::CodeGenerator(const string& proto, const VarMap& varmap) :
 				_varmap(varmap)
 {
     _protocol = proto;
-    _code._target.protocol = proto;
-    _code._target.filter = filter::IMPORT;
+    _code.set_target_protocol(proto);
+    _code.set_target_filter(filter::IMPORT);
 }
 
 CodeGenerator::~CodeGenerator()
@@ -64,7 +64,7 @@ CodeGenerator::visit_policy(PolicyStatement& policy)
 
     _os << "POLICY_END\n";
 
-    _code._code = _os.str();
+    _code.set_code(_os.str());
     return NULL;
 }
 
@@ -87,7 +87,7 @@ CodeGenerator::visit_term(Term& term)
 
     // Import policies should not have a dest block
     if(!dest.empty()) {
-	throw CodeGeneratorErr("Term " + term.name() + " has a dest part!");
+	xorp_throw(CodeGeneratorErr, "Term " + term.name() + " has a dest part!");
     }
 
     //
@@ -161,7 +161,7 @@ const Element*
 CodeGenerator::visit(NodeSet& node)
 {
     _os << "PUSH_SET " << node.setid() << endl;
-    _code._sets.insert(node.setid());
+    _code.add_referenced_set_name(node.setid());
     return NULL;
 }
 
@@ -186,7 +186,7 @@ CodeGenerator::visit_proto(NodeProto& node)
 
     // import policies may not have protocol set.
     err << "INVALID protocol statement in line " << node.line() << endl;
-    throw CodeGeneratorErr(err.str());
+    xorp_throw(CodeGeneratorErr, err.str());
 }
 
 const Code&
