@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,84 +12,82 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/code_list.cc,v 1.4 2006/03/16 00:04:57 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/code_list.cc,v 1.10 2007/02/16 22:46:52 pavlin Exp $"
 
 #include "policy_module.h"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+
 #include "code_list.hh"
 #include "policy/common/policy_utils.hh"
 
-CodeList::CodeList(const string& p) : _policy(p) {
+CodeList::CodeList(const string& policy)
+    : _policy(policy)
+{
 }
 
-CodeList::~CodeList() {
+CodeList::~CodeList()
+{
     policy_utils::clear_container(_codes);
 }
 
-
-void 
-CodeList::push_back(Code* c) {
+void
+CodeList::push_back(Code* c)
+{
     _codes.push_back(c);
 }
 
-string 
-CodeList::str() const {
+string
+CodeList::str() const
+{
     string ret = "Policy: " + _policy + "\n";
 
-    for(ListCode::const_iterator i = _codes.begin();
-	i != _codes.end(); ++i)
-
+    for (ListCode::const_iterator i = _codes.begin(); i != _codes.end(); ++i) {
         ret += (*i)->str();
+    }
 
-    return ret;    
+    return ret;
 }
 
-void 
-CodeList::link_code(Code& c) const {
+void
+CodeList::link_code(Code& c) const
+{
     // go through all the code we have, and link it to c.
-    for(ListCode::const_iterator i = _codes.begin();
-        i != _codes.end(); ++i) {
-	
+    for (ListCode::const_iterator i = _codes.begin(); i != _codes.end(); ++i) {
 	const Code* tmp = *i;
 
 	// the += operator will check for target correctness.
 	c += *tmp;
-    }    
-}
-
-void 
-CodeList::get_targets(Code::TargetSet& targets) const {
-
-    // go through all our code and see what targets the code is for
-    for(ListCode::const_iterator i = _codes.begin(); 
-	i != _codes.end() ; ++i) {
-	
-	const Code* c = *i;
-
-	targets.insert(c->_target);
     }
 }
 
-void 
-CodeList::get_tags(const string& protocol, Code::TagSet& tagset) const {
-    
+void
+CodeList::get_targets(Code::TargetSet& targets) const
+{
+    // go through all our code and see what targets the code is for
+    for (ListCode::const_iterator i = _codes.begin(); i != _codes.end(); ++i) {
+	const Code* c = *i;
+	targets.insert(c->target());
+    }
+}
+
+void
+CodeList::get_redist_tags(const string& protocol, Code::TagSet& tagset) const
+{
     // go through all the code we have.
-    for(ListCode::const_iterator i = _codes.begin(); 
-	i != _codes.end(); ++i) {
-	
+    for (ListCode::const_iterator i = _codes.begin(); i != _codes.end(); ++i) {
 	const Code* c = *i;
 
 	// we only want tags for specific protocols.
-	if(c->_target.protocol != protocol)
+	if (c->target().protocol() != protocol)
 	    continue;
 
-	const Code::TagSet& ts = c->_tags;
+	const Code::TagSet& ts = c->redist_tags();
 
 	// insert the tags for this protocol.
-	for(Code::TagSet::iterator j = ts.begin(); j != ts.end(); ++j) 
+	for (Code::TagSet::const_iterator j = ts.begin(); j != ts.end(); ++j) {
 	    tagset.insert(*j);
+	}
     }
 }
