@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,16 +13,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/common/dispatcher.cc,v 1.9 2006/03/16 00:05:14 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/dispatcher.cc,v 1.12 2007/02/16 22:47:01 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
 #include <typeinfo>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "dispatcher.hh"
 #include "elem_null.hh"
 #include "policy_utils.hh"
@@ -30,6 +26,7 @@
 #include "element.hh"
 #include "register_operations.hh"
 
+
 // init static members
 Dispatcher::Value Dispatcher::_map[32768];
 RegisterOperations Dispatcher::_regops;
@@ -69,8 +66,9 @@ Dispatcher::run(const Oper& op, unsigned argc, const Element** argv) const
 	string arg1type = argv[1]->type();
 
 	if (arg1type != ElemStr::id)
-	    throw OpNotFound("First argument of ctr must be txt type, but is: " 
-			     + arg1type);
+	    xorp_throw(OpNotFound,
+		       "First argument of ctr must be txt type, but is: " 
+		       + arg1type);
 	
 	const ElemStr& es = dynamic_cast<const ElemStr&>(*argv[1]);
 
@@ -92,9 +90,9 @@ Dispatcher::run(const Oper& op, unsigned argc, const Element** argv) const
 
 	// the infrastructure is ready however.
 	default:
-	    throw OpNotFound("Operations of arity: " +
-			     policy_utils::to_str(argc) + 
-			     " not supported");
+	    xorp_throw(OpNotFound, "Operations of arity: " +
+		       policy_utils::to_str(argc) + 
+		       " not supported");
     }
     // unreach
 }
@@ -132,7 +130,8 @@ Dispatcher::assign_op_hash(const Oper& op)
 
     // XXX
     if (_ophash > 31) {
-	throw PolicyException("Too many operations for dispatcher---find a better hashing mechanism\n");
+	xorp_throw(PolicyException,
+		   "Too many operations for dispatcher---find a better hashing mechanism\n");
     }
 
     op.set_hash(_ophash);
@@ -150,7 +149,8 @@ Dispatcher::assign_elem_hash(Element& e)
 
     // XXX
     if (_elemhash > 31) {
-	throw PolicyException("Too many elems for dispatcher---find a better hashing mechanism\n");
+	xorp_throw(PolicyException, 
+		   "Too many elems for dispatcher---find a better hashing mechanism\n");
     }
 
     e.set_hash(_elemhash);
