@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,17 +13,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/common/elem_set.cc,v 1.10 2006/04/02 23:24:55 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/common/elem_set.cc,v 1.16 2007/02/16 22:47:01 pavlin Exp $"
 
 #include "libxorp/xorp.h"
 
+#include <algorithm>
+
 #include "elem_set.hh"
 #include "policy_utils.hh"
-#include <algorithm>
+
 
 template <class T>
 ElemSetAny<T>::ElemSetAny(const Set& val) : _val(val) 
@@ -33,7 +31,7 @@ ElemSetAny<T>::ElemSetAny(const Set& val) : _val(val)
 template <class T>
 ElemSetAny<T>::ElemSetAny(const char* c_str)
 {
-    if(!c_str)
+    if (!c_str)
 	return;
 
     // create each element in the list
@@ -57,10 +55,10 @@ ElemSetAny<T>::str() const
 {
     string s = "";
 
-    if(!_val.size())
+    if (!_val.size())
 	return s;
 
-    for(typename Set::iterator i = _val.begin(); i != _val.end(); ++i) {
+    for (typename Set::const_iterator i = _val.begin(); i != _val.end(); ++i) {
 	s += (*i).str();
         s += ",";
     }
@@ -106,14 +104,14 @@ ElemSetAny<T>::operator<(const ElemSetAny<T>& rhs) const
     const Set& rset = rhs._val;
 
     // left has to be smaller
-    if(_val.size() >= rset.size())
+    if (_val.size() >= rset.size())
 	return false;
 
     // for all elements on left to match, the intersection must be equal to
     // the left set.
     Set tmp;
-    set_intersection(_val.begin(),_val.end(),rset.begin(),rset.end(),
-        insert_iterator<Set>(tmp,tmp.begin()));
+    set_intersection(_val.begin(), _val.end(), rset.begin(), rset.end(),
+        insert_iterator<Set>(tmp, tmp.begin()));
 
     return tmp == _val;
 }
@@ -150,14 +148,14 @@ template <class T>
 bool 
 ElemSetAny<T>::operator>(const T& rhs) const 
 {
-    typename Set::iterator i = _val.find(rhs);
+    typename Set::const_iterator i = _val.find(rhs);
 
-    if(i == _val.end())
+    if (i == _val.end())
 	return false;
 
     // left has to have at least 2 elements 
     // [so it has more than 1 elements -- size of rhs].
-    if(_val.size() < 2)
+    if (_val.size() < 2)
 	return false;
 
     return true;
@@ -278,22 +276,22 @@ ElemSetAny<T>::type() const
 }
 
 // define the various sets
-template class ElemSetAny<ElemU32>;
 template <> const char* ElemSetU32::id = "set_u32";
 template <> Element::Hash ElemSetU32::_hash = 0;
+template class ElemSetAny<ElemU32>;
 
-template class ElemSetAny<ElemCom32>;
 template <> const char* ElemSetCom32::id = "set_com32";
 template <> Element::Hash ElemSetCom32::_hash = 0;
+template class ElemSetAny<ElemCom32>;
 
-template class ElemSetAny<ElemIPv4Net>;
 template <> const char* ElemSetIPv4Net::id = "set_ipv4net";
 template <> Element::Hash ElemSetIPv4Net::_hash = 0;
+template class ElemSetAny<ElemIPv4Net>;
 
-template class ElemSetAny<ElemIPv6Net>;
 template <> const char* ElemSetIPv6Net::id = "set_ipv6net";
 template <> Element::Hash ElemSetIPv6Net::_hash = 0;
+template class ElemSetAny<ElemIPv6Net>;
 
-template class ElemSetAny<ElemStr>;
 template <> const char* ElemSetStr::id = "set_str";
 template <> Element::Hash ElemSetStr::_hash = 0;
+template class ElemSetAny<ElemStr>;
