@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,20 +13,9 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/common/policy_utils.cc,v 1.9 2006/03/16 00:05:19 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/policy_utils.cc,v 1.12 2007/02/16 22:47:04 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "policy_utils.hh"
-
-#include <errno.h>
-#include <stdio.h>
-
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
+#include "libxorp/xorp.h"
 
 #if defined(HAVE_REGEX_H)
 #include <regex.h>
@@ -37,6 +26,9 @@
 #error "No header defining regex_t is present."
 #endif
 
+#include "policy_utils.hh"
+
+
 namespace policy_utils {
 
 void 
@@ -96,7 +88,7 @@ read_file(const string& fname, string& out)
         err += "Unable to open file " + fname + ": ";
         err += strerror(errno);
 
-        throw PolicyUtilsErr(err);
+        xorp_throw(PolicyUtilsErr, err);
     }
 
     buff[0] = 0;
@@ -111,7 +103,7 @@ read_file(const string& fname, string& out)
             err += strerror(errno);
 
             fclose(f);
-            throw PolicyUtilsErr(err);
+            xorp_throw(PolicyUtilsErr, err);
         }
 
 	// append to content of file to out
@@ -154,7 +146,7 @@ regex(const string& str, const string& reg)
 	err += "): ";
 	err += tmp;
 
-	throw PolicyUtilsErr(err);
+	xorp_throw(PolicyUtilsErr, err);
     }
 
     // execute the regex [XXX: check for errors!!]
