@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,11 +13,9 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/common/register_elements.cc,v 1.9 2006/03/16 00:05:20 pavlin Exp $"
+#ident "$XORP: xorp/policy/common/register_elements.cc,v 1.11 2007/02/16 22:47:04 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
 #include "register_elements.hh"
 #include "element_factory.hh"
@@ -26,6 +24,7 @@
 #include "elem_null.hh"
 #include "elem_bgp.hh"
 
+
 RegisterElements::RegisterElements()
 {
     register_element<ElemInt32>();
