@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,11 +13,9 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/configuration.cc,v 1.12 2006/05/12 02:21:38 pavlin Exp $"
+#ident "$XORP: xorp/policy/configuration.cc,v 1.21 2007/02/16 22:46:52 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
 #include "policy_module.h"
 #include "configuration.hh"
@@ -41,6 +39,13 @@ Configuration::~Configuration()
     clear_map(_export_filters);
 
     clear_map(_tagmap);
+
+    //
+    // XXX: Clear the _policies before the _sets goes out of scope.
+    // Otherwise, the _policies destructor might try to use a reference
+    // to _sets after _sets has been destroyed.
+    //
+    _policies.clear();
 }
   
 Term& 
@@ -61,7 +66,7 @@ Configuration::delete_term(const string& policy, const string& term)
        return;
     }   
 
-    throw ConfError("TERM NOT FOUND " + policy + " " + term);
+    xorp_throw(ConfError, "TERM NOT FOUND " + policy + " " + term);
 }
    
 void 
@@ -76,8 +81,8 @@ Configuration::update_term_block(const string& policy,
 	t.set_block(block, order, statement);
         _modified_policies.insert(policy); // recompile on commit.
     } catch(const Term::term_syntax_error& e) {
-        string err = "In policy " + policy + ": " + e.str();
-        throw ConfError(err);
+        string err = "In policy " + policy + ": " + e.why();
+        xorp_throw(ConfError, err);
     }
 } 
 
@@ -88,8 +93,8 @@ Configuration::create_term(const string& policy, const ConfigNodeId& order,
     PolicyStatement& ps = _policies.find(policy);
 
     if(ps.term_exists(term)) {
-	throw ConfError("Term " + term +
-			" exists already in policy " + policy);
+	xorp_throw(ConfError,
+		   "Term " + term + " exists already in policy " + policy);
     }
 
     Term* t = new Term(term);
@@ -135,7 +140,23 @@ Configuration::delete_set(const string& set)
     // necessary to filters / configuration.
     _sets.delete_set(set);
 }  
-   
+
+void
+Configuration::add_to_set(const string& type, const string& set,
+			  const string& element)
+{
+    // policies affected will be marked as modified.
+    _sets.add_to_set(type, set, element, _modified_policies);
+}
+
+void
+Configuration::delete_from_set(const string& type, const string& set,
+			       const string& element)
+{
+    // policies affected will be marked as modified.
+    _sets.delete_from_set(type, set, element, _modified_policies);
+}
+
 void 
 Configuration::update_imports(const string& protocol, 
 			      const list<string>& imports)
@@ -143,7 +164,7 @@ Configuration::update_imports(const string& protocol,
    
     // check if protocol exists
     if(!_varmap.protocol_known(protocol))
-	throw ConfError("imports: Protocol " + protocol + " unknown");
+	xorp_throw(ConfError, "imports: Protocol " + protocol + " unknown");
 
     update_ie(protocol,imports,_imports,PolicyList::IMPORT);
     _modified_targets.insert(Code::Target(protocol,filter::IMPORT));
@@ -155,7 +176,7 @@ Configuration::update_exports(const string& protocol,
 {
     // check if protocol exists
     if(!_varmap.protocol_known(protocol))
-	throw ConfError("exports: Protocol " + protocol + " unknown");
+	xorp_throw(ConfError, "exports: Protocol " + protocol + " unknown");
 
     // XXX: if conf fails we lost tagmap
     TagMap::iterator i = _tagmap.find(protocol);
@@ -218,7 +239,7 @@ return conf;
 
         conf << protocol << ":";
 
-        for(TagSet::iterator j = tagset.begin(); j != tagset.end(); ++j) {
+        for(TagSet::const_iterator j = tagset.begin(); j != tagset.end(); ++j) {
 	   conf << " " << *j;
 	}
 
@@ -336,7 +357,7 @@ Configuration::link_sourcematch_code(const Code::Target& target)
     // This will allow the += operator of Code to behave properly. [i.e. link
     // only stuff we really are interested in].
     Code* code = new Code();
-    code->_target = target;
+    code->set_target(target);
 
     // only export statements have source match code.
     // go through all of them and link.
@@ -347,7 +368,7 @@ Configuration::link_sourcematch_code(const Code::Target& target)
     }
 
     // kill previous
-    CodeMap::iterator i = _sourcematch_filters.find(target.protocol);
+    CodeMap::iterator i = _sourcematch_filters.find(target.protocol());
     if(i != _sourcematch_filters.end()) {
 	delete (*i).second;
 	_sourcematch_filters.erase(i);
@@ -355,10 +376,10 @@ Configuration::link_sourcematch_code(const Code::Target& target)
 
 
     // if there is nothing, keep it deleted and empty.
-    if(code->_code == "") 
+    if(code->code() == "") 
         delete code;
     else {
-        _sourcematch_filters[target.protocol] = code;
+        _sourcematch_filters[target.protocol()] = code;
     }	
 }
    
@@ -381,9 +402,9 @@ Configuration::update_tagmap(const string& protocol)
 
     PolicyList* pl = (*i).second;
 
-    // Get the policytags for the protocol
+    // Get the redist policytags for the protocol
     TagSet* tagset = new TagSet();
-    pl->get_tags(protocol, *tagset);
+    pl->get_redist_tags(protocol, *tagset);
 
     if(tagset->size()) {
 	_tagmap[protocol] = tagset;
@@ -402,7 +423,7 @@ Configuration::link_code()
 
         const Code::Target& t = *i;
 
-        switch(t.filter) {
+        switch(t.filter()) {
 	    case filter::IMPORT:
 		link_code(t,_imports,_import_filters);
 		break;
@@ -414,7 +435,7 @@ Configuration::link_code()
 	    case filter::EXPORT:
 		link_code(t,_exports,_export_filters);
 		// export policies produce tags, update them.
-		update_tagmap(t.protocol);
+		update_tagmap(t.protocol());
 		break;
 	}
 
@@ -457,8 +478,9 @@ Configuration::add_varmap(const string& protocol, const string& variable,
     else if (access == "w")
 	acc = VarMap::WRITE;
     else
-	throw PolicyException("Unknown access (" + access + ") for protocol: " 
-			      + protocol + " variable: " + variable);
+	xorp_throw(PolicyException,
+		   "Unknown access (" + access + ") for protocol: " 
+		   + protocol + " variable: " + variable);
 
     _varmap.add_protocol_variable(protocol, 
 		  new VarMap::Variable(variable, type, acc, id)); 
@@ -510,7 +532,7 @@ Configuration::link_code(const Code::Target& target,
 			 CodeMap& codemap)
 {
     // find the policy list for the target
-    IEMap::iterator i = iemap.find(target.protocol);
+    IEMap::iterator i = iemap.find(target.protocol());
 
     XLOG_ASSERT(i != iemap.end());
 
@@ -518,24 +540,24 @@ Configuration::link_code(const Code::Target& target,
 
     // create new code and set target, so code may be linked properly
     Code* code = new Code();
-    code->_target = target;
+    code->set_target(target);
 
     // link the code
     pl->link_code(*code);
 
 
     // erase previous code
-    CodeMap::iterator iter = codemap.find(target.protocol);
+    CodeMap::iterator iter = codemap.find(target.protocol());
     if(iter != codemap.end()) {
 	delete (*iter).second;
 	codemap.erase(iter);
     }	   
 
     // if code is empty [no-op filter] just erase it, and keep no entry.
-    if(code->_code == "") 
+    if(code->code() == "") 
         delete code;
     else 
-	codemap[target.protocol] = code;
+	codemap[target.protocol()] = code;
 }
     
 string 
@@ -574,6 +596,6 @@ Configuration::dump_state(uint32_t id)
 	    break;
 
 	default:
-	    throw PolicyException("Unknown state id: " + to_str(id));
+	    xorp_throw(PolicyException, "Unknown state id: " + to_str(id));
     }
 }
