@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,16 +13,16 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/export_code_generator.cc,v 1.8 2006/04/06 04:33:23 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/export_code_generator.cc,v 1.12 2007/02/16 22:46:53 pavlin Exp $"
 
 #include "policy_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
+
 #include "export_code_generator.hh"
 
+
 ExportCodeGenerator::ExportCodeGenerator(
 			const string& proto, 
 			const SourceMatchCodeGenerator::Tags& t,
@@ -53,9 +53,19 @@ ExportCodeGenerator::visit_term(Term& term)
         _os << "PUSH u32 " << (ti.second) << endl;
         _os << "<=\n";
         _os << "ONFALSE_EXIT" << endl;
-    
+
+	bool is_redist_tag = true;
+	if (term.from_protocol() == protocol()) {
+	    //
+	    // XXX: If we have an export policy that exports routes
+	    // from a protocol to itself, then don't tag those routes
+	    // for redistribution from the RIB back to the protocol.
+	    //
+	    is_redist_tag = false;
+	}
+
 	// update tags used by the code
-	_code._tags.insert(ti.second);
+	_code.add_tag(ti.second, is_redist_tag);
     }
 
     // do dest block
