@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,16 +13,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/filter_manager.cc,v 1.10 2006/03/17 23:53:01 pavlin Exp $"
+#ident "$XORP: xorp/policy/filter_manager.cc,v 1.15 2007/02/16 22:46:53 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "policy_module.h"
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
+
 #include "filter_manager.hh"
 
 FilterManager::FilterManager(const CodeMap& imp, 
@@ -49,17 +48,17 @@ FilterManager::FilterManager(const CodeMap& imp,
 void 
 FilterManager::update_filter(const Code::Target& t)
 {
-    switch (t.filter) {
+    switch (t.filter()) {
 	case filter::IMPORT:
-	    update_import_filter(t.protocol);
+	    update_import_filter(t.protocol());
 	    break;
 
 	case filter::EXPORT_SOURCEMATCH:
-	    update_sourcematch_filter(t.protocol);
+	    update_sourcematch_filter(t.protocol());
 	    break;
 
 	case filter::EXPORT:
-	    update_export_filter(t.protocol);
+	    update_export_filter(t.protocol());
 	    break;
     }
 }
@@ -96,7 +95,7 @@ FilterManager::update_tagmap(const string& protocol)
     // convert tags to xrlatomlist
     XrlAtomList al;
 
-    for(TagSet::iterator iter = ts->begin();
+    for(TagSet::const_iterator iter = ts->begin();
 	iter != ts->end(); ++iter) {
    
         al.append(XrlAtom(*iter));	    
@@ -120,7 +119,7 @@ FilterManager::policy_backend_cb(const XrlError& e)
 	debug_msg("[POLICY] XRL exception: %s\n", e.str().c_str());
 	error_msg = c_format("XRL policy_backend_cb() error: %s",
 			     e.str().c_str());
-	throw FMException(error_msg); // XXX: what else can we do ?
+	xorp_throw(FMException, error_msg); // XXX: what else can we do ?
     }	
 }
 
@@ -257,8 +256,8 @@ FilterManager::birth(const string& protocol)
 
 	const Code* export_code = (*cmi).second;
 
-	for(set<string>::iterator i = export_code->_source_protos.begin();
-	    i != export_code->_source_protos.end(); ++i) {
+	for(set<string>::const_iterator i = export_code->source_protocols().begin();
+	    i != export_code->source_protocols().end(); ++i) {
 
 	    const string& push_proto = *i;
 	
@@ -350,13 +349,13 @@ FilterManager::update_queue(const string& protocol,
     // get the code
     Code* code = (*i).second;
 
-    string conf = code->_code;
+    string conf = code->code();
 
-    set<string>& sets = code->_sets;
+    const set<string>& set_names = code->referenced_set_names();
 
-    // expand the sets
-    for(set<string>::iterator iter = sets.begin();
-        iter != sets.end(); ++iter) {
+    // expand the set names
+    for(set<string>::const_iterator iter = set_names.begin();
+        iter != set_names.end(); ++iter) {
 
 	const string& name = *iter;
         const Element& e = _sets.getSet(*iter);
