@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,15 +13,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/parser.cc,v 1.5 2006/03/16 00:04:58 pavlin Exp $"
+#ident "$XORP: xorp/policy/parser.cc,v 1.7 2007/02/16 22:46:54 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "policy_module.h"
+
+#include "libxorp/xorp.h"
+
+#include "policy/common/policy_utils.hh"
+
 #include "parser.hh"
 #include "policy_parser.hh"
-#include "policy/common/policy_utils.hh"
+
 
 Parser::Nodes* 
 Parser::parse(const Term::BLOCKS& block, const string& text)
