@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,13 +13,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/policy_list.cc,v 1.6 2006/03/16 00:04:58 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/policy_list.cc,v 1.10 2007/02/16 22:46:54 pavlin Exp $"
 
 #include "policy_module.h"
+#include "libxorp/xorp.h"
+
 #include "policy_list.hh"
 #include "visitor_semantic.hh"
 #include "export_code_generator.hh"
@@ -171,7 +169,7 @@ PolicyList::get_targets(Code::TargetSet& targets)
 }
 
 void 
-PolicyList::get_tags(const string& protocol, Code::TagSet& ts)
+PolicyList::get_redist_tags(const string& protocol, Code::TagSet& ts)
 {
     // go through all policies and return tags associated with the requested
     // protocol.
@@ -180,7 +178,7 @@ PolicyList::get_tags(const string& protocol, Code::TagSet& ts)
     
 	CodeList* cl = (*i).second;
 
-	cl->get_tags(protocol,ts);
+	cl->get_redist_tags(protocol,ts);
     }    
 }
 
@@ -227,7 +225,7 @@ PolicyList::compile_import(PolicyCodeList::iterator& iter,
     (*iter).second = cl;
 
     // target was modified
-    modified_targets.insert(code->_target);
+    modified_targets.insert(code->target());
 }
 
 void 
@@ -262,7 +260,7 @@ PolicyList::compile_export(PolicyCodeList::iterator& iter, PolicyStatement& ps,
     (*iter).second = cl;
 
     // export target modified
-    modified_targets.insert(code->_target);
+    modified_targets.insert(code->target());
 
     // we may get a lot of code fragments here:
     // consider a policy where each term has a different source protocol...
@@ -275,9 +273,9 @@ PolicyList::compile_export(PolicyCodeList::iterator& iter, PolicyStatement& ps,
         Code* c = *i;
         cl->push_back(c);
 
-        modified_targets.insert(c->_target);
+        modified_targets.insert(c->target());
 
         // keep track of source protocols in export policy code.
-        code->_source_protos.insert(c->_target.protocol);
+        code->add_source_protocol(c->target().protocol());
     }
 }
