@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,16 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/policy_map.cc,v 1.6 2006/03/16 00:04:59 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/policy_map.cc,v 1.9 2007/02/16 22:46:54 pavlin Exp $"
 
 #include "policy_module.h"
+
+#include "libxorp/xorp.h"
+
 #include "visitor_printer.hh"
 #include "policy_map.hh"
 
+
 PolicyStatement& 
 PolicyMap::find(const string& name) const
 {
@@ -41,8 +41,8 @@ PolicyMap::create(const string& name,SetMap& smap)
 
     if (!_deps.create(name,ps)) {
 	delete ps;
-	throw PolicyMapError("Can't create policy " + name + 
-			     " : already exists");
+	xorp_throw(PolicyMapError,
+		   "Can't create policy " + name + " : already exists");
     }
 }
 
