@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/policy_statement.cc,v 1.9 2006/05/12 02:21:38 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/policy_statement.cc,v 1.12 2007/02/16 22:46:54 pavlin Exp $"
 
 #include "policy_module.h"
-#include "policy_statement.hh"   
+
+#include "libxorp/xorp.h"
+
 #include "policy/common/policy_utils.hh"
 
+#include "policy_statement.hh"   
+
+
 using namespace policy_utils;
 
 PolicyStatement::PolicyStatement(const string& name, SetMap& smap) : 
@@ -47,8 +48,8 @@ PolicyStatement::add_term(const ConfigNodeId& order, Term* term)
 {
     if((_terms.find(order) != _terms.end())
        || (find_out_of_order_term(order) != _out_of_order_terms.end())) {
-	throw PolicyException("Term already present in position: " +
-			      order.str());
+	xorp_throw(PolicyException,
+		   "Term already present in position: " + order.str());
     }
 
     pair<TermContainer::iterator, bool> res;
@@ -131,8 +132,8 @@ PolicyStatement::find_term(const string& name) const
 	    return *t;
 	}
 
-	throw PolicyStatementErr("Term " + name + " not found in policy " + 
-				 _name);
+	xorp_throw(PolicyStatementErr,
+		   "Term " + name + " not found in policy " + _name);
     }
 
     Term* t = i->second;
