@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,16 +13,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/policy_target.cc,v 1.12 2006/07/28 06:34:31 pavlin Exp $"
+#ident "$XORP: xorp/policy/policy_target.cc,v 1.16 2007/02/16 22:46:55 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "policy_module.h"
-#include "policy_target.hh"
+
+#include "libxorp/xorp.h"
+
 #include "policy/common/policy_utils.hh"
 #include "policy/common/varrw.hh"
 
+#include "policy_target.hh"
+
+
 // static members
 string PolicyTarget::policy_target_name = "policy";
 
@@ -111,6 +113,20 @@ PolicyTarget::delete_set(const string& name)
     _conf.delete_set(name);
 }
 
+void
+PolicyTarget::add_to_set(const string& type, const string& name, 
+			 const string& element)
+{
+    _conf.add_to_set(type, name, element);
+}
+
+void
+PolicyTarget::delete_from_set(const string& type, const string& name, 
+			      const string& element)
+{
+    _conf.delete_from_set(type, name, element);
+}
+
 void
 PolicyTarget::update_import(const string& protocol,
 			    const string& policies)
@@ -172,8 +188,9 @@ PolicyTarget::death(const string& tclass, const string& /* tinstance */)
     list<string> no_policy;
 
     // Remove the "import" and "export" dependencies for the protocol
-    _conf.update_imports(tclass, no_policy);
-    _conf.update_exports(tclass, no_policy);
+    string protocol = _pmap.protocol(tclass);
+    _conf.update_imports(protocol, no_policy);
+    _conf.update_exports(protocol, no_policy);
 
     _process_watch.death(tclass);
 }
