@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,17 +13,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/process_watch.cc,v 1.6 2006/03/17 23:53:01 pavlin Exp $"
+#ident "$XORP: xorp/policy/process_watch.cc,v 1.9 2007/02/16 22:46:55 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "policy_module.h"
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
+
 #include "process_watch.hh"
 
 ProcessWatch::ProcessWatch(XrlStdRouter& rtr, ProtocolMap& pmap) :
@@ -42,7 +40,7 @@ ProcessWatch::register_cb(const XrlError& err)
 
     if (err != XrlError::OKAY()) {
 	error_msg = c_format("XRL register_cb() error: %s", err.str().c_str());
-	throw PWException(error_msg);
+	xorp_throw(PWException, error_msg);
     }
 }
 
@@ -90,7 +88,7 @@ bool
 ProcessWatch::alive(const string& proto) 
 {
     if (_watching.find(proto) == _watching.end())
-	throw PWException("Not watching protocol: " + proto);
+	xorp_throw(PWException, "Not watching protocol: " + proto);
 
     return _alive.find(proto) != _alive.end();
 }
