@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,16 +13,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/semantic_varrw.cc,v 1.8 2006/05/25 05:50:00 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/semantic_varrw.cc,v 1.11 2007/02/16 22:46:55 pavlin Exp $"
 
 #include "policy_module.h"
-#include "semantic_varrw.hh"
+
+#include "libxorp/xorp.h"
+
 #include "policy/common/policy_utils.hh"
 
+#include "semantic_varrw.hh"
+
+
 SemanticVarRW::SemanticVarRW(VarMap& vars) : 
 		    _vars(vars) 
 {
@@ -51,7 +52,7 @@ SemanticVarRW::write(const Id& id, const Element& elem) {
     if(!var.writable()) {
 	string error = "Trying to write on read-only variable ";
         error += id;
-        throw var_error(error);
+        xorp_throw(var_error, error);
     }    
 
     // type checking
@@ -61,7 +62,7 @@ SemanticVarRW::write(const Id& id, const Element& elem) {
         err << "Trying to assign value of type " << elem.type() << " to " <<
 	var.type << " variable " << id;
 
-        throw var_error(err.str());
+        xorp_throw(var_error, err.str());
     }
 }
 
