@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,13 +13,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/source_match_code_generator.cc,v 1.9 2006/04/06 00:27:57 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/source_match_code_generator.cc,v 1.15 2007/02/16 22:46:56 pavlin Exp $"
 
 #include "policy_module.h"
+#include "libxorp/xorp.h"
+
 #include "source_match_code_generator.hh"
 
 SourceMatchCodeGenerator::SourceMatchCodeGenerator(uint32_t tagstart,
@@ -40,7 +38,7 @@ SourceMatchCodeGenerator::visit_policy(PolicyStatement& policy)
     for (i = terms.begin(); i != terms.end(); ++i) {
 	// reset code and sets
 	_os.str("");
-	_code._sets.clear();
+	_code.clear_referenced_set_names();
 
 	Term* term = i->second;
 	// make sure the source of the term has something [non empty source]
@@ -60,7 +58,7 @@ SourceMatchCodeGenerator::visit_policy(PolicyStatement& policy)
  	 i != _codes.end(); ++i) {
 
         Code* c = (*i).second;
-        c->_code += "POLICY_END\n";
+        c->add_code("POLICY_END\n");
 
         _codes_vect.push_back(c);
     }
@@ -74,9 +72,9 @@ SourceMatchCodeGenerator::addTerm(const string& pname)
     // copy the code for the term
     Code* term = new Code();
 
-    term->_target.protocol = _protocol;
-    term->_target.filter = filter::EXPORT_SOURCEMATCH;
-    term->_sets = _code._sets;
+    term->set_target_protocol(_protocol);
+    term->set_target_filter(filter::EXPORT_SOURCEMATCH);
+    term->set_referenced_set_names(_code.referenced_set_names());
 
     // see if we have code for this target already
     CodeMap::iterator i = _codes.find(_protocol);
@@ -86,7 +84,7 @@ SourceMatchCodeGenerator::addTerm(const string& pname)
 	Code* existing = (*i).second;
 
 	// link "raw" code
-	term->_code = _os.str();
+	term->set_code(_os.str());
 	*existing += *term;
 
 	delete term;
@@ -94,7 +92,7 @@ SourceMatchCodeGenerator::addTerm(const string& pname)
     }
 
     // code for a new target, need to create policy start header.
-    term->_code = "POLICY_START " + pname + "\n" + _os.str();
+    term->set_code("POLICY_START " + pname + "\n" + _os.str());
     _codes[_protocol] = term;
 }
 
@@ -115,8 +113,10 @@ SourceMatchCodeGenerator::visit_term(Term& term)
     // statements.
     //
     for(i = source.begin(); i != source.end(); ++i) {
-	if ((i->second)->is_protocol_statement())
+	if ((i->second)->is_protocol_statement()) {
 	    (i->second)->accept(*this);
+	    term.set_from_protocol(_protocol);
+	}
     }
 
     //
@@ -140,8 +140,8 @@ SourceMatchCodeGenerator::visit_term(Term& term)
 
     // XXX: we can assume _protocol = PROTOCOL IN EXPORT STATEMENT
     if(_protocol == "") 
-        throw NoProtoSpec("No protocol specified in term " + term.name() + 
-			  " in export policy source match");
+        xorp_throw(NoProtoSpec, "No protocol specified in term " + term.name() + 
+		                " in export policy source match");
 
     // ignore any destination block [that is dealt with in the export code
     // generator]
@@ -150,7 +150,31 @@ SourceMatchCodeGenerator::visit_term(Term& term)
     // ignore actions too...
 
 
-    // as an action, store policy tags...
+    //
+    // As an action, store policy tags...
+    // XXX: Note that we store the policy tags only if the route
+    // doesn't carry some other protocol's tags.
+    //
+    _tags.push_back(Taginfo(true, _currtag));
+    _protocol_tags[_protocol].insert(_currtag);
+
+    // Create the set of the tags known (so far) to belong to this protocol
+    ElemSetU32 element_set;
+    const set<uint32_t>& protocol_tags = _protocol_tags[_protocol];
+    for (set<uint32_t>::const_iterator iter = protocol_tags.begin();
+	 iter != protocol_tags.end();
+	 ++iter) {
+	ElemU32 e(*iter);
+	element_set.insert(e);
+    }
+
+    // Check that the route's tags are subset of this protocol's tags
+    _os << "PUSH set_u32 " << element_set.str() << endl;
+    _os << "LOAD " << VarRW::VAR_POLICYTAGS << "\n";
+    _os << "<=\n";
+    _os << "ONFALSE_EXIT" << endl;
+
+    // Add another tag to the route
     _os << "PUSH u32 " << _currtag << endl;
     _os << "LOAD " << VarRW::VAR_POLICYTAGS << "\n";
     _os << "+\n";
@@ -159,8 +183,6 @@ SourceMatchCodeGenerator::visit_term(Term& term)
 
     _os << "TERM_END\n";
 
-    _tags.push_back(Taginfo(true, _currtag));
-
     // FIXME: integer overflow
     _currtag++;
 
@@ -175,7 +197,7 @@ SourceMatchCodeGenerator::visit_proto(NodeProto& node)
 	ostringstream err; 
         err << "PROTOCOL REDEFINED FROM " << _protocol << " TO " <<
 	    node.proto() << " AT LINE " << node.line();
-        throw ProtoRedefined(err.str());
+        xorp_throw(ProtoRedefined, err.str());
     }
 
     // define protocol
