@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/test/compilepolicy.cc,v 1.9 2006/03/16 00:05:23 pavlin Exp $"
+#ident "$XORP: xorp/policy/test/compilepolicy.cc,v 1.12 2007/02/16 22:47:05 pavlin Exp $"
 
 /*
  * EXIT CODES:
@@ -29,27 +29,24 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "policy/policy_module.h"
+
 #include "libxorp/xorp.h"
-#include "policy/common/policy_utils.hh"
-#include "policy/configuration.hh"
-#include "process_watch_fake.hh"
-#include "filter_manager_fake.hh"
-#include <string>
+
 #include <iostream>
 #include <fstream>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "policy/configuration.hh"
+#include "policy/common/policy_utils.hh"
+
+#include "process_watch_fake.hh"
+#include "filter_manager_fake.hh"
+
+
 using namespace policy_utils;
 
 ProcessWatchFake pw;
@@ -77,12 +74,12 @@ void print_code(Code& c) {
 	os = code_out;
 
     
-    *os << c._code;
+    *os << c.code();
 
-    set<string>& sets = c._sets;
+    const set<string>& set_names = c.referenced_set_names();
 
-    for(set<string>::iterator i = sets.begin();
-	i != sets.end(); ++i) {
+    for(set<string>::const_iterator i = set_names.begin();
+	i != set_names.end(); ++i) {
     
 	*os << "SET " << *i << " \"";
 
