@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/test/execpolicy.cc,v 1.6 2006/03/16 00:05:23 pavlin Exp $"
+#ident "$XORP: xorp/policy/test/execpolicy.cc,v 1.8 2007/02/16 22:47:05 pavlin Exp $"
 
 /*
  * EXIT CODES:
@@ -24,25 +24,22 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "policy/policy_module.h"
 
-#include <string>
-#include <iostream>
-
 #include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
 #include "libxorp/timeval.hh"
 #include "libxorp/clock.hh"
 #include "libxorp/timer.hh"
 
-#include "policy/backend/policy_filter.hh"
+#include <iostream>
+
 #include "policy/common/policy_utils.hh"
-#include "libxorp/xlog.h"
+#include "policy/backend/policy_filter.hh"
+
 #include "file_varrw.hh"
 
+
 using namespace policy_utils;
 
 int main(int argc, char *argv[]) {
