@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,15 +13,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/test/file_varrw.cc,v 1.8 2006/05/25 05:50:02 pavlin Exp $"
+#ident "$XORP: xorp/policy/test/file_varrw.cc,v 1.11 2007/02/16 22:47:05 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
 
 #include "policy/common/policy_utils.hh"
+
 #include "file_varrw.hh"
 
+
 FileVarRW::FileVarRW(const string& fname) {
     FILE* f = fopen(fname.c_str(),"r");
     if(!f) {
@@ -29,7 +29,7 @@ FileVarRW::FileVarRW(const string& fname) {
 	err += ": ";
 	err += strerror(errno);
 
-	throw Error(err);
+	xorp_throw(Error, err);
     }	
 
     char buff[1024];
@@ -44,7 +44,7 @@ FileVarRW::FileVarRW(const string& fname) {
 	ostringstream oss;
 
 	oss << "Parse error at line: " << lineno;
-	throw Error(oss.str());
+	xorp_throw(Error, oss.str());
     }
 
     fclose(f);
@@ -95,7 +95,7 @@ FileVarRW::doLine(const string& str) {
     char* err = 0;
     Id id = strtol(varname.c_str(), &err, 10);
     if (*err) {
-	throw Error(string("Varname must be ID [numeric]: ") + err);
+	xorp_throw(Error, string("Varname must be ID [numeric]: ") + err);
     }
 
     _map[id] = e;
@@ -108,7 +108,7 @@ FileVarRW::read(const Id& id) {
     Map::iterator i = _map.find(id);
 
     if(i == _map.end())
-	throw Error("Cannot read variable: " + id);
+	xorp_throw(Error, "Cannot read variable: " + id);
 
     const Element* e = (*i).second;
 
