@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/test/process_watch_fake.cc,v 1.4 2006/03/16 00:05:24 pavlin Exp $"
+#ident "$XORP: xorp/policy/test/process_watch_fake.cc,v 1.6 2007/02/16 22:47:06 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "libxorp/xorp.h"
+
+#include <iostream>
 
 #include "process_watch_fake.hh"
-#include <iostream>
+
 
 void
 ProcessWatchFake::add_interest(const string& id) {
