@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,21 +13,23 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/visitor_semantic.cc,v 1.10 2006/03/16 00:05:03 pavlin Exp $"
+#ident "$XORP: xorp/policy/visitor_semantic.cc,v 1.14 2007/02/16 22:46:57 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "policy_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
-#include "visitor_semantic.hh"
-#include "policy/common/elem_null.hh"
+
 #include <sstream>
 
+#include "policy/common/elem_null.hh"
+
+#include "visitor_semantic.hh"
+
+
 VisitorSemantic::VisitorSemantic(SemanticVarRW& varrw, 
 				 VarMap& varmap,
 				 SetMap& setmap,
@@ -91,14 +93,14 @@ VisitorSemantic::visit(Term& term)
 
 	    err += " in term: " + term.name();
 
-	    throw sem_error(err);
+	    xorp_throw(sem_error, err);
 	}
     }
 
     // import policies should not have dest blocks
     if (_ptype == IMPORT && !(dest.empty())) {
-	throw sem_error("Invalid use of dest in import policy in term " + 
-		       term.name());
+	xorp_throw(sem_error, "Invalid use of dest in import policy in term " + 
+		   term.name());
     }
 
     // check dest block
@@ -133,7 +135,7 @@ VisitorSemantic::visit(NodeUn& node)
 	ostringstream error;
 
 	error << "Invalid unop " << e.str() << " at line " << node.line();
-	throw sem_error(error.str());
+	xorp_throw(sem_error, error.str());
     }
 }
 
@@ -157,7 +159,7 @@ VisitorSemantic::visit(NodeBin& node)
 
         error << "Invalid binop " << e.str() << " at line " << node.line();
     
-        throw sem_error(error.str());
+        xorp_throw(sem_error, error.str());
     }
 }
 
@@ -169,14 +171,14 @@ VisitorSemantic::visit(NodeAssign& node)
 
     // try assignment
     try {
-	VarRW::Id id = _varmap.var2id(current_protocol(), node.varid());
+	VarRW::Id id = _varmap.var2id(semantic_protocol(), node.varid());
 	_varrw.write(id, *rvalue);
     } catch(SemanticVarRW::var_error e) {
         ostringstream error;
 
         error << e.str() << " at line " << node.line();
 
-        throw sem_error(error.str());
+        xorp_throw(sem_error, error.str());
     }
     return NULL;
 }
@@ -186,14 +188,14 @@ VisitorSemantic::visit(NodeVar& node)
 {
     // try reading a variable
     try {
-	VarRW::Id id = _varmap.var2id(current_protocol(), node.val());
+	VarRW::Id id = _varmap.var2id(semantic_protocol(), node.val());
 	return &_varrw.read(id);
     } catch(SemanticVarRW::var_error e) {
         ostringstream error;
 
         error << e.str() << " at line " << node.line();
     
-        throw sem_error(error.str());
+        xorp_throw(sem_error, error.str());
     }
 }
     
@@ -209,7 +211,7 @@ VisitorSemantic::visit(NodeSet& node)
         ostringstream error;
         error << "Set not found: " << node.setid() << " at line " << node.line();
     
-        throw sem_error(error.str());
+        xorp_throw(sem_error, error.str());
     }
 }
 
@@ -241,7 +243,7 @@ VisitorSemantic::visit(NodeProto& node)
 	err << "May not define protocol for import policy at line " <<
         node.line();
         
-        throw sem_error(err.str());
+        xorp_throw(sem_error, err.str());
     }
 
     string proto = node.proto();
@@ -252,7 +254,7 @@ VisitorSemantic::visit(NodeProto& node)
         err << "Redifinition of protocol from " << _current_protocol << 
 	    " to " << proto << " at line " << node.line();
     
-        throw sem_error(err.str());
+        xorp_throw(sem_error, err.str());
     }
 
     // do the switch
@@ -265,12 +267,12 @@ VisitorSemantic::visit(NodeProto& node)
 void
 VisitorSemantic::change_protocol(const string& proto)
 {
-    _semantic_proto = proto;
-    _varrw.set_protocol(_semantic_proto);
+    _semantic_protocol = proto;
+    _varrw.set_protocol(_semantic_protocol);
 }
 
 const string&
-VisitorSemantic::current_protocol()
+VisitorSemantic::semantic_protocol()
 {
-    return _semantic_proto;
+    return _semantic_protocol;
 }
