@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,15 +13,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/visitor_setdep.cc,v 1.6 2006/03/16 00:05:03 pavlin Exp $"
+#ident "$XORP: xorp/policy/visitor_setdep.cc,v 1.9 2007/02/16 22:46:57 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "policy_module.h"
-#include "visitor_setdep.hh"
+#include "libxorp/xorp.h"
+
 #include <vector>
-#include <string>
+
+#include "visitor_setdep.hh"
 
 VisitorSetDep::VisitorSetDep(SetMap& setmap) : _setmap(setmap) 
 {
@@ -115,7 +114,7 @@ VisitorSetDep::visit(NodeSet& node)
         ostringstream error;
         error << "Set not found: " << node.setid() << " at line " << node.line();
     
-        throw sem_error(error.str());
+        xorp_throw(sem_error, error.str());
     }
     return NULL;
 }
