@@ -1,6 +1,6 @@
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,20 +12,16 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/xorp_policy.cc,v 1.6 2006/03/16 00:05:03 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/xorp_policy.cc,v 1.8 2007/02/16 22:46:57 pavlin Exp $"
 
 #include "policy_module.h"
+#include "libxorp/xorp.h"
+
 #include "policy/common/policy_exception.hh"
 #include "policy/common/policy_utils.hh"
 
 #include "xrl_target.hh"
 
-#include <string>
-
 
 void go()
 {
