@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,17 +13,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/policy/xrl_target.cc,v 1.11 2006/03/16 00:05:03 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/policy/xrl_target.cc,v 1.14 2007/02/16 22:46:57 pavlin Exp $"
 
 #include "policy_module.h"
-#include "xrl_target.hh"
-#include "common/policy_utils.hh"
+
+#include "libxorp/xorp.h"
 #include "libxorp/status_codes.h"
 
+#include "policy/common/policy_utils.hh"
+
+#include "xrl_target.hh"
+
+
 using namespace policy_utils;
 
 XrlPolicyTarget::XrlPolicyTarget(XrlStdRouter* r, PolicyTarget& ptarget) : 
@@ -212,6 +213,38 @@ XrlPolicyTarget::policy_0_1_delete_set(const string&   set)
     return XrlCmdError::OKAY();
 }	
 
+XrlCmdError
+XrlPolicyTarget::policy_0_1_add_to_set(
+    // Input values,
+    const string&	type,
+    const string&	set,
+    const string&	element)
+{
+    try {
+	_policy_target.add_to_set(type, set, element);
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("add_to_set: " + e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlPolicyTarget::policy_0_1_delete_from_set(
+    // Input values,
+    const string&	type,
+    const string&	set,
+    const string&	element)
+{
+    try {
+	_policy_target.delete_from_set(type, set, element);
+    } catch(const PolicyException& e) {
+	return XrlCmdError::COMMAND_FAILED("delete_from_set: " + e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
 XrlCmdError 
 XrlPolicyTarget::policy_0_1_done_global_policy_conf()
 {
