@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,28 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/add_route.cc,v 1.7 2006/03/16 00:05:26 pavlin Exp $"
+#ident "$XORP: xorp/rib/add_route.cc,v 1.9 2007/02/16 22:47:06 pavlin Exp $"
 
 // Add routes to the RIB.
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include <stdio.h>
-#include <string.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <fstream>
-#include <cstdlib>
-#include <vector>
-
 #include "rib_module.h"
 
 #include "libxorp/xorp.h"
@@ -44,14 +29,19 @@
 #include "libxorp/timeval.hh"
 #include "libxorp/timer.hh"
 
-#include "libxipc/xrl_std_router.hh"
-
-#include "xrl/interfaces/rib_xif.hh"
+#include <fstream>
+#include <cstdlib>
+#include <vector>
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#include "libxipc/xrl_std_router.hh"
+
+#include "xrl/interfaces/rib_xif.hh"
+
+
 template <typename A>
 class Route {
  public:
