@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rib.cc,v 1.61 2006/07/03 23:33:40 pavlin Exp $"
+#ident "$XORP: xorp/rib/rib.cc,v 1.63 2007/02/16 22:47:08 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -437,7 +437,7 @@ RIB<A>::add_connected_route(const Vif& vif,
     //
     add_route("connected", net, nexthop_addr, "", "", 0, PolicyTags());
 
-    if (vif.is_p2p() && (peer_addr != A::ZERO())) {
+    if (vif.is_p2p() && (peer_addr != A::ZERO()) && (! net.contains(peer_addr))) {
 	add_route("connected", IPNet<A>(peer_addr, A::addr_bitlen()),
 		  peer_addr, "", "", 0, PolicyTags());
     }
@@ -452,7 +452,7 @@ RIB<A>::delete_connected_route(const Vif& vif, const IPNet<A>& net,
 {
     delete_route("connected", net);
 
-    if (vif.is_p2p() && (peer_addr != A::ZERO())) {
+    if (vif.is_p2p() && (peer_addr != A::ZERO()) && (! net.contains(peer_addr))) {
 	delete_route("connected", IPNet<A>(peer_addr, A::addr_bitlen()));
     }
 
