@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_pol_redist.cc,v 1.6 2006/03/16 00:05:38 pavlin Exp $"
+#ident "$XORP: xorp/rib/rt_tab_pol_redist.cc,v 1.8 2007/02/16 22:47:10 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -156,7 +156,7 @@ PolicyRedistTable<A>::add_redist(const IPRouteEntry<A>& route,
 				 const Set& protos)
 {
     // send a redistribution request for all protocols in the set.
-    for (Set::iterator i = protos.begin(); i != protos.end(); ++i)
+    for (Set::const_iterator i = protos.begin(); i != protos.end(); ++i)
 	add_redist(route, *i);
 }
 
@@ -167,7 +167,7 @@ PolicyRedistTable<A>::del_redist(const IPRouteEntry<A>& route,
 				 const Set& protos)
 {
     // stop redistribution for all protocols in set.
-    for (Set::iterator i = protos.begin(); i != protos.end(); ++i)
+    for (Set::const_iterator i = protos.begin(); i != protos.end(); ++i)
 	del_redist(route, *i);
 }
 
