@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/rt_tab_register.cc,v 1.29 2006/03/16 00:05:40 pavlin Exp $"
+#ident "$XORP: xorp/rib/rt_tab_register.cc,v 1.31 2007/02/16 22:47:11 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -58,7 +58,7 @@ RouteRegister<A>::str() const
     else
 	s += "RR Route: NONE \n";
 
-    set<const ModuleData*, ModuleCmp>::iterator mod_iter;
+    set<const ModuleData*, ModuleCmp>::const_iterator mod_iter;
     mod_iter = _modules.begin();
     while (mod_iter != _modules.end()) {
 	s += "RR Module: " + (*mod_iter)->str() + "\n";
