@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rib/test_register_xrls.cc,v 1.36 2006/03/16 00:05:41 pavlin Exp $"
+#ident "$XORP: xorp/rib/test_register_xrls.cc,v 1.38 2007/02/16 22:47:11 pavlin Exp $"
 
 #include "rib_module.h"
 
@@ -394,7 +394,7 @@ main(int /* argc */, char* argv[])
     }
 
     ribclienttarget.verify_invalidated("9.0.0.0/24");
-    ribclienttarget.verify_no_info();
+    (void)ribclienttarget.verify_no_info();
 
     printf("====================================================\n");
 
@@ -407,7 +407,7 @@ main(int /* argc */, char* argv[])
     }
 
     ribclienttarget.verify_invalidated("9.0.1.0/24");
-    ribclienttarget.verify_no_info();
+    (void)ribclienttarget.verify_no_info();
 
     //
     // Gracefully stop and exit xlog
