@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,19 +12,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rip/packet_queue.cc,v 1.9 2006/03/16 00:05:49 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#ident "$XORP: xorp/rip/packet_queue.cc,v 1.11 2007/02/16 22:47:14 pavlin Exp $"
 
 #include "rip_module.h"
 
+#include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
+
 #include "packet_queue.hh"
 
+
 template <typename A>
 PacketQueue<A>::PacketQueue()
     : _buffered_bytes(0), _max_buffered_bytes(64000), _drops(0)
