@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8: 
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,16 +13,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#include "libxorp/xorp.h"
-
 // #define DEBUG_LOGGING
 
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
+
 #include "libxipc/xrl_router.hh"
 
 #include "constants.hh"
@@ -33,6 +29,7 @@
 #include "xrl_target_ripng.hh"
 #include "xrl_target_common.hh"
 
+
 XrlRipngTarget::XrlRipngTarget(EventLoop&		el,
 			       XrlRouter&		xr,
 			       XrlProcessSpy&		xps,
