@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,17 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/master_conf_tree.cc,v 1.71 2006/03/16 00:05:59 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-#ifdef HAVE_GRP_H
-#include <grp.h>
-#endif
+#ident "$XORP: xorp/rtrmgr/master_conf_tree.cc,v 1.77 2007/03/15 07:39:53 pavlin Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -31,6 +21,14 @@
 #include "libxorp/debug.h"
 #include "libxorp/utils.hh"
 
+#ifdef HAVE_GRP_H
+#include <grp.h>
+#endif
+
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #include "master_conf_tree.hh"
 #include "module_command.hh"
 #include "rtrmgr_error.hh"
@@ -39,6 +37,7 @@
 #include "master_template_tree_node.hh"
 #include "util.hh"
 
+
 #ifdef HOST_OS_WINDOWS
 
 // XXX: Use unlink emulation from MS VC runtime.
@@ -415,29 +414,30 @@ MasterConfigTree::order_module_list(const set<string>& module_set,
     // Figure out the dependencies for all the additional modules
     set<string> additional_done;
     while (!additional_modules.empty()) {
-	iter = additional_modules.begin();
-	ModuleCommand* mc = _template_tree->find_module(*iter);
+	set<string>::iterator mod_iter;
+	mod_iter = additional_modules.begin();
+	ModuleCommand* mc = _template_tree->find_module(*mod_iter);
 	if (mc == NULL) {
-	    debug_msg("%s has no info\n", (*iter).c_str());
-	    additional_done.insert(*iter);
-	    additional_modules.erase(iter);
-	    ordered_modules.push_back(*iter);
-	    satisfied.insert(*iter);
+	    debug_msg("%s has no info\n", (*mod_iter).c_str());
+	    additional_done.insert(*mod_iter);
+	    additional_modules.erase(mod_iter);
+	    ordered_modules.push_back(*mod_iter);
+	    satisfied.insert(*mod_iter);
 	    continue;
 	}
 	if (mc->depends().empty()) {
-	    debug_msg("%s has no dependencies\n", (*iter).c_str());
-	    additional_done.insert(*iter);
-	    ordered_modules.push_back(*iter);
-	    satisfied.insert(*iter);
-	    additional_modules.erase(iter);
+	    debug_msg("%s has no dependencies\n", (*mod_iter).c_str());
+	    additional_done.insert(*mod_iter);
+	    ordered_modules.push_back(*mod_iter);
+	    satisfied.insert(*mod_iter);
+	    additional_modules.erase(mod_iter);
 	    continue;
 	}
 
 	list<string>::const_iterator di;
 	for (di = mc->depends().begin(); di != mc->depends().end(); ++di) {
-	    depends.insert(pair<string,string>(*iter, *di));
-	    debug_msg("%s depends on %s\n", iter->c_str(), di->c_str());
+	    depends.insert(pair<string,string>(*mod_iter, *di));
+	    debug_msg("%s depends on %s\n", mod_iter->c_str(), di->c_str());
 	    // Check that the dependency is already in our list of modules.
 	    if (module_set.find(*di) == module_set.end()
 		&& additional_modules.find(*di) == additional_modules.end()
@@ -446,8 +446,8 @@ MasterConfigTree::order_module_list(const set<string>& module_set,
 		additional_modules.insert(*di);
 	    }
 	}
-	additional_done.insert(*iter);
-	additional_modules.erase(iter);
+	additional_done.insert(*mod_iter);
+	additional_modules.erase(mod_iter);
     }
     debug_msg("done additional modules\n");
 
@@ -552,11 +552,13 @@ MasterConfigTree::commit_changes_pass1(CallBack cb)
 	}
     }
 
+    bool needs_activate = false;
     bool needs_update = false;
     if (master_root_node().commit_changes(*_task_manager,
 					  /* do_commit = */ false,
 					  0, 0,
 					  error_msg,
+					  needs_activate,
 					  needs_update)
 	== false) {
 	// Something went wrong - return the error message.
@@ -637,11 +639,13 @@ MasterConfigTree::commit_changes_pass2()
 	}
     }
 
+    bool needs_activate = false;
     bool needs_update = false;
     if (!master_root_node().commit_changes(*_task_manager,
 					   /* do_commit = */ true,
 					   0, 0,
 					   error_msg,
+					   needs_activate,
 					   needs_update)) {
 	// Abort the commit
 	XLOG_ERROR("Commit failed in config tree");
@@ -751,6 +755,7 @@ MasterConfigTree::save_to_file(const string& filename, uid_t user_id,
 {
     string dummy_error_msg;
     string full_filename = config_full_filename(filename);
+    FILE* file = NULL;
 
     error_msg = "";
 
@@ -774,8 +779,6 @@ MasterConfigTree::save_to_file(const string& filename, uid_t user_id,
 	return false;
     }
 
-    FILE* file = NULL;
-
 #ifdef HOST_OS_WINDOWS
     {
 #else
@@ -785,7 +788,7 @@ MasterConfigTree::save_to_file(const string& filename, uid_t user_id,
 
     struct stat sb;
     if (stat(full_filename.c_str(), &sb) == 0) {
-	if (sb.st_mode & S_IFREG == 0) {
+	if ((sb.st_mode & S_IFREG) == 0) {
 	    if (((sb.st_mode & S_IFMT) == S_IFCHR) ||
 		((sb.st_mode & S_IFMT) == S_IFBLK)) {
 		error_msg = c_format("File %s is a special device.\n",
@@ -829,6 +832,7 @@ MasterConfigTree::save_to_file(const string& filename, uid_t user_id,
 		return false;
 	    }
 	    fclose(file);
+	    file = NULL;
 	}
 	// It seems OK to overwrite this file
 	if (unlink(full_filename.c_str()) < 0 && errno != ENOENT) {
