@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,29 +12,25 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/module_manager.cc,v 1.62 2006/04/26 04:38:39 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/module_manager.cc,v 1.64 2007/02/16 22:47:23 pavlin Exp $"
 
 #include "rtrmgr_module.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/utils.hh"
 
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
-
 #ifdef HAVE_GLOB_H
 #include <glob.h>
 #elif defined(HOST_OS_WINDOWS)
 #include "glob_win32.h"
 #endif
 
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #include "module_manager.hh"
 #include "conf_tree_node.hh"
 #include "master_conf_tree.hh"
@@ -42,6 +38,7 @@
 #include "main_rtrmgr.hh"
 #include "util.hh"
 
+
 #ifdef HOST_OS_WINDOWS
 #define stat _stat
 #endif
