@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,39 +13,38 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/profiler.cc,v 1.6 2006/03/16 00:06:00 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/profiler.cc,v 1.8 2007/02/16 22:47:23 pavlin Exp $"
 
 // Access the profiling support in XORP processes.
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "rtrmgr_module.h"
 
 #include <stdio.h>
 
-#ifdef HAVE_SYSEXITS_H
-#include <sysexits.h>
-#endif
-
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
 #include "libxorp/status_codes.h"
 #include "libxorp/callback.hh"
-#include "libxipc/xrl_std_router.hh"
-
-#include "xrl/interfaces/profile_xif.hh"
-#include "xrl/targets/profiler_base.hh"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#ifdef HAVE_SYSEXITS_H
+#include <sysexits.h>
+#endif
+
+#include "libxipc/xrl_std_router.hh"
+
+#include "xrl/interfaces/profile_xif.hh"
+
+#include "xrl/targets/profiler_base.hh"
+
+
 class XrlProfilerTarget :  XrlProfilerTargetBase {
  public:
     XrlProfilerTarget(XrlRouter *r)
