@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,15 +12,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/randomness.cc,v 1.19 2006/03/16 00:06:00 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/randomness.cc,v 1.21 2007/02/16 22:47:24 pavlin Exp $"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "rtrmgr_module.h"
 
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
+#include "libxorp/xorp.h"
+#include "libxorp/xlog.h"
+#include "libxorp/debug.h"
+#include "libxorp/timeval.hh"
+#include "libxorp/timer.hh"
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -29,15 +29,6 @@
 // XXX: Needs HAVE_OPENSSL_MD5_H
 #include <openssl/md5.h>
 
-#include "rtrmgr_module.h"
-
-#include "libxorp/xorp.h"
-#include "libxorp/xlog.h"
-#include "libxorp/debug.h"
-
-#include "libxorp/timeval.hh"
-#include "libxorp/timer.hh"
-
 #include "randomness.hh"
 
 
