@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/test_module_manager.cc,v 1.20 2006/03/16 00:06:03 pavlin Exp $"
+#ident "$XORP: xorp/rtrmgr/test_module_manager.cc,v 1.22 2007/02/16 22:47:26 pavlin Exp $"
 
 
 #include "rtrmgr_module.h"
@@ -101,7 +101,7 @@ Rtrmgr::run()
 	return -1;
     }
 
-    while (eventloop.timers_pending() || waiting == true) {
+    while (eventloop.events_pending() || waiting == true) {
 	printf(".");
 	eventloop.run();
     }
@@ -113,7 +113,7 @@ Rtrmgr::run()
     printf("shutting down\n");
     mmgr.shutdown();
     while ((mmgr.is_shutdown_completed() != true)
-	   && eventloop.timers_pending()) {
+	   && eventloop.events_pending()) {
 	printf(".");
 	eventloop.run();
     }
