@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2006 International Computer Science Institute
+// Copyright (c) 2001-2007 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,15 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/rtrmgr/xrl_rtrmgr_interface.cc,v 1.51 2006/04/26 04:42:31 pavlin Exp $"
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef HAVE_SYS_STAT_H
-#include <sys/stat.h>
-#endif
+#ident "$XORP: xorp/rtrmgr/xrl_rtrmgr_interface.cc,v 1.54 2007/02/16 22:47:28 pavlin Exp $"
 
 #include "rtrmgr_module.h"
 
@@ -29,6 +21,10 @@
 #include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
 
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #include "main_rtrmgr.hh"
 #include "master_conf_tree.hh"
 #include "module_manager.hh"
@@ -318,7 +314,7 @@ XrlRtrmgrInterface::initialize_client_state(uid_t user_id,
     }
     t = _eventloop.new_oneoff_after_ms(delay,
              callback(this, &XrlRtrmgrInterface::send_client_state, 
-		      user_id, user));
+		      user_id, user->clientname()));
     _background_tasks.push_front(t);
 }
 
@@ -332,9 +328,11 @@ XrlRtrmgrInterface::finder_register_done(const XrlError& e, string clientname)
 }
 
 void
-XrlRtrmgrInterface::send_client_state(uid_t user_id, UserInstance *user)
+XrlRtrmgrInterface::send_client_state(uid_t user_id, string client)
 {
-    string client = user->clientname();
+    UserInstance *user = find_user_instance(user_id, client);
+    if (user == NULL)
+	return;
 
     debug_msg("send_client_state %s\n", client.c_str());
     if (!_rtrmgr.ready()) {
